@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -30,39 +30,22 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/processing.h"
-#include "core/undo.h"
 #include "core/OS_utils.h"
-#include "io/single_image.h"
+#include "core/siril_log.h"
 #include "io/image_format_fits.h"
 #include "io/sequence.h"
-#include "algos/statistics.h"
 #include "algos/geometry.h"
 #include "algos/sorting.h"
-#include "opencv/opencv.h"
-#include "gui/utils.h"
-#include "gui/callbacks.h"
-#include "gui/image_display.h"
-#include "gui/image_interactions.h"
-#include "gui/progress_and_log.h"
 #include "gui/message_dialog.h"
-#include "gui/dialogs.h"
-#include "gui/siril_preview.h"
-#include "registration/registration.h"	// for mouse_status
+#include "opencv/opencv.h"
 #include "background_extraction.h"
 
-static gboolean background_computed = FALSE;
-
-
-static void background_startup() {
-	copy_gfit_to_backup();
-}
-
 #define NPARAM_POLY4 15		// Number of parameters used with 4rd order
 #define NPARAM_POLY3 10		// Number of parameters used with 3rd order
 #define NPARAM_POLY2 6		// Number of parameters used with 2nd order
 #define NPARAM_POLY1 3		// Number of parameters used with 1nd order
 
-#define SAMPLE_SIZE 25
+#define SAMPLE_SIZE 25		// must be odd to compute a radius
 
 struct sample {
 	double median[3]; // median of each channel of the sample (if color)
@@ -120,22 +103,22 @@ static double siril_gsl_vector_sum(const gsl_vector *v) {
 	return sum;
 }
 
-static gboolean computeBackground_RBF(GSList *list, double *background, int channel, unsigned int width, unsigned int height, double smoothing, gchar **err, gboolean processing) {
+static gboolean computeBackground_RBF(GSList *list, double *background, int channel, unsigned int width, unsigned int height, double smoothing, gchar **err, int threads, gboolean processing) {
 	/* Implementation of RBF interpolation with a thin-plate Kernel k(r) = r^2 * log(r)
-	
+
 	References:
-	----------  
-	[1] G. B. Wright. Radial Basis Function Interpolation: Numerical and 
+	----------
+	[1] G. B. Wright. Radial Basis Function Interpolation: Numerical and
 		Analytical Developments. PhD thesis, University of Colorado, 2003.
 
 	[2] SciPy version 1.1.0
 		https://github.com/scipy/scipy/blob/v1.1.0/scipy/interpolate/rbf.py
-	
-	[3] S. Rippa. An algorithm for selecting a good value for the parameter 
-		c in radial basis function interpolation. Advances in Computational 
+
+	[3] S. Rippa. An algorithm for selecting a good value for the parameter
+		c in radial basis function interpolation. Advances in Computational
 		Mathematics, 11(2-3):193–210, 1999.
-		
-	[4] J. D. Martin and T. W. Simpson. Use of Kriging Models to Approximate 
+
+	[4] J. D. Martin and T. W. Simpson. Use of Kriging Models to Approximate
 		Deterministic Computer Models. AIAA journal, 43(4):853–863, 2005.
 	*/
 
@@ -161,12 +144,12 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 	int scaling_factor = 4;
 	int width_scaled = round_to_int(width / scaling_factor);
 	int height_scaled = round_to_int(height / scaling_factor);
-	
+
 	double *background_scaled = calloc(width_scaled * height_scaled, sizeof(double));
 	double *kernel_scaled = calloc(width_scaled * height_scaled, sizeof(double));
 	double x_scaling = (double)height_scaled / (double)height;
 	double y_scaling = (double)width_scaled / (double)width;
-	
+
 	/* Copy linked list into array */
 	list_array = malloc(n * sizeof(double[3]));
 	l_i = list;
@@ -175,7 +158,6 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 		list_array[i][0] = round(sample_i->position.x * x_scaling);
 		list_array[i][1] = round(sample_i->position.y * y_scaling);
 		list_array[i][2] = sample_i->median[channel];
-
 		l_i = l_i->next;
 	}
 
@@ -203,9 +185,12 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 		}
 	}
 
+	// Must turn off error handler or it aborts on error
+	gsl_set_error_handler_off();
+
 	/* Smoothing */
 	smoothing = 1e-4 * pow(10.0, (smoothing-0.5) * 3);
-	
+
 	for (int i = 0; i < n; i++) {
 		gsl_matrix_set(K, i, i, smoothing * mean);
 	}
@@ -257,11 +242,14 @@ static gboolean computeBackground_RBF(GSList *list, double *background, int chan
 	/* Calculate background from coefficients coef */
 	double total = height_scaled * width_scaled;
 	int progress = 0;
-
-#pragma omp parallel shared(background_scaled) private(A) num_threads(com.max_thread)
+#ifdef _OPENMP
+#pragma omp parallel shared(background_scaled) private(A) num_threads(threads)
+#endif
 	{
 		A = gsl_vector_calloc(n + 1);
+#ifdef _OPENMP
 #pragma omp for
+#endif
 		for (int i = 0; i < height_scaled; i++) {
 			for (int j = 0; j < width_scaled; j++) {
 				for (int k = 0; k < n; k++) {
@@ -354,9 +342,12 @@ static gboolean computeBackground_Polynom(GSList *list, double *background, int
 		double col = sample->position.x;
 		double row = sample->position.y;
 		pixel = sample->median[channel];
-		// here, it is a bit sketchy in the sense that if there is not value to report in a box (because the threshold is too
-		// low for example, then I just skip the initialization of J and y. gsl automatically discard the non assigned values
-		// during the minimization. I tested it with Matlab and it works fine. The results agree.
+		// here, it is a bit sketchy in the sense that if there is no
+		// value to report in a box (because the threshold is too low
+		// for example), then I just skip the initialization of J and y.
+		// gsl automatically discards the unassigned values during the
+		// minimization. I tested it with Matlab and it works fine. The
+		// results agree.
 		if (pixel < 0)
 			continue;
 
@@ -407,7 +398,6 @@ static gboolean computeBackground_Polynom(GSList *list, double *background, int
 	}
 
 	// Calculation of the background with the same dimension that the input matrix.
-
 	for (unsigned int i = 0; i < height; i++) {
 		for (unsigned int j = 0; j < width; j++) {
 			switch (order) {
@@ -441,26 +431,23 @@ static gboolean computeBackground_Polynom(GSList *list, double *background, int
 
 static background_sample *get_sample(float *buf, const int xx,
 		const int yy, const int w, const int h) {
-	int radius, x, y;
-	double *data;
 	size_t size = SAMPLE_SIZE * SAMPLE_SIZE;
-	background_sample *sample = (background_sample *) g_malloc(sizeof(background_sample));
+	int radius = SAMPLE_SIZE / 2;
+	background_sample *sample = malloc(sizeof(background_sample));
 	if (!sample) {
 		PRINT_ALLOC_ERR;
 		return NULL;
 	}
 
-	radius = (int) (SAMPLE_SIZE / 2);
-
-	int n = 0;
-	data = calloc(size, sizeof(double));
+	double *data = malloc(size * sizeof(double));
 	if (!data) {
 		free(sample);
 		PRINT_ALLOC_ERR;
 		return NULL;
 	}
-	for (y = yy - radius; y <= yy + radius; y ++) {
-		for (x = xx - radius; x <= xx + radius; x ++) {
+	int n = 0;
+	for (int y = yy - radius; y <= yy + radius; y ++) {
+		for (int x = xx - radius; x <= xx + radius; x ++) {
 			if (y >= 0 && y < h) {
 				if (x >= 0 && x < w) {
 					data[n++] = (double)buf[x + y * w];
@@ -468,6 +455,12 @@ static background_sample *get_sample(float *buf, const int xx,
 			}
 		}
 	}
+	if (n != size) {
+		siril_debug_print("sample did not have the expected size (on border?)\n");
+		free(sample);
+		free(data);
+		return NULL;
+	}
 	gsl_stats_minmax(&sample->min, &sample->max, data, 1, size);
 	sample->mean = gsl_stats_mean(data, 1, size);
 	sample->median[RLAYER] = quickmedian_double(data, size);
@@ -480,22 +473,19 @@ static background_sample *get_sample(float *buf, const int xx,
 	return sample;
 }
 
-static double get_sample_median(double *buf, const int xx,
+static double get_sample_median(const double *buf, const int xx,
 		const int yy, const int w, const int h) {
-	int radius, x, y, n;
-	double *data, median;
 	size_t size = SAMPLE_SIZE * SAMPLE_SIZE;
+	int radius = SAMPLE_SIZE / 2;
 
-	radius = get_sample_radius();
-
-	n = 0;
-	data = calloc(size, sizeof(double));
+	int n = 0;
+	double *data = calloc(size, sizeof(double));
 	if (!data) {
 		PRINT_ALLOC_ERR;
 		return -1.0;
 	}
-	for (y = yy - radius; y <= yy + radius; y ++) {
-		for (x = xx - radius; x <= xx + radius; x ++) {
+	for (int y = yy - radius; y <= yy + radius; y ++) {
+		for (int x = xx - radius; x <= xx + radius; x ++) {
 			if (y >= 0 && y < h) {
 				if (x >= 0 && x < w) {
 					data[n++] = buf[x + y * w];
@@ -503,7 +493,7 @@ static double get_sample_median(double *buf, const int xx,
 			}
 		}
 	}
-	median = quickmedian_double(data, size);
+	double median = quickmedian_double(data, size);
 
 	free(data);
 	return median;
@@ -514,14 +504,14 @@ static double get_background_mean(GSList *list, int num_channels) {
 	GSList *l;
 	guint n = g_slist_length(list);
 	double mean = 0.0;
-	
+
 	for (int channel = 0; channel < num_channels; channel++) {
 		for (l = list; l; l = l->next) {
 			background_sample *sample = (background_sample *) l->data;
 			mean += sample->median[channel];
 		}
 	}
-	
+
 	return mean / n / num_channels;
 }
 
@@ -531,15 +521,15 @@ static unsigned int _rand(guint64 *const p_rng) {
 }
 
 static gboolean convert_fits_to_img(fits *fit, double *image, int channel, gboolean add_dither) {
-
 	guint64 seed = time(NULL);
 
+	double invnorm = 1.0 / USHRT_MAX;
 	const int height = fit->ry;
 	const int width = fit->rx;
 	if (fit->type == DATA_USHORT) {
 		for (int y = 0; y < height; ++y) {
 			for (int x = 0; x < width; ++x) {
-				image[y * width + x] = fit->pdata[channel][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
+				image[y * width + x] = fit->pdata[channel][(height - y - 1) * width + x] * invnorm;
 				if (add_dither) {
 					/* add dithering in order to avoid colour banding */
 					image[y * width + x] += (_rand(&seed) % 1048576) * 0.000000000095367431640625f;
@@ -560,8 +550,10 @@ static gboolean convert_fits_to_img(fits *fit, double *image, int channel, gbool
 	return TRUE;
 }
 
-static float* convert_fits_to_luminance(fits *fit) {
+static float* convert_fits_to_luminance(fits *fit, threading_type threads) {
+	g_assert(fit->type == DATA_USHORT || fit->type == DATA_FLOAT);
 	const size_t n = fit->naxes[0] * fit->naxes[1];
+	float invnorm = (float)(1.0 / USHRT_MAX);
 	/* allocating memory to image */
 	float *image = malloc(n * sizeof(float));
 	if (!image) {
@@ -572,28 +564,35 @@ static float* convert_fits_to_luminance(fits *fit) {
 	const int height = fit->ry;
 	const int width = fit->rx;
 
+#ifdef _OPENMP
+	limit_threading(&threads, 200000/width, height);
+#pragma omp parallel for num_threads(threads) schedule(static)
+#endif
 	for (int y = 0; y < height; ++y) {
+		size_t in_idx = (height - y - 1) * width;
+		size_t out_idx = y * width;
 		for (int x = 0; x < width; ++x) {
 			if (fit->naxes[2] > 1) {
 				float r, g, b;
 				if (fit->type == DATA_USHORT) {
-					r = fit->pdata[RLAYER][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
-					g = fit->pdata[GLAYER][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
-					b = fit->pdata[BLAYER][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
-				} else if (fit->type == DATA_FLOAT) {
-					r = fit->fpdata[RLAYER][(height - y - 1) * width + x];
-					g = fit->fpdata[GLAYER][(height - y - 1) * width + x];
-					b = fit->fpdata[BLAYER][(height - y - 1) * width + x];
-				} else
-					return NULL;
-				image[y * width + x] = 0.2126f * r + 0.7152f * g + 0.0722f * b;
+					r = fit->pdata[RLAYER][in_idx] * invnorm;
+					g = fit->pdata[GLAYER][in_idx] * invnorm;
+					b = fit->pdata[BLAYER][in_idx] * invnorm;
+				} else {
+					r = fit->fpdata[RLAYER][in_idx];
+					g = fit->fpdata[GLAYER][in_idx];
+					b = fit->fpdata[BLAYER][in_idx];
+				}
+				image[out_idx] = 0.3333f * r + 0.3333f * g + 0.3333f * b;
 			} else {
 				if (fit->type == DATA_USHORT) {
-					image[y * width + x] = fit->pdata[RLAYER][(height - y - 1) * width + x] / USHRT_MAX_SINGLE;
+					image[out_idx] = fit->pdata[RLAYER][in_idx] * invnorm;
 				} else if (fit->type == DATA_FLOAT) {
-					image[y * width + x] = fit->fpdata[RLAYER][(height - y - 1) * width + x];
+					image[out_idx] = fit->fpdata[RLAYER][in_idx];
 				}
 			}
+			in_idx++;
+			out_idx++;
 		}
 	}
 
@@ -605,64 +604,85 @@ static void convert_img_to_fits(double *image, fits *fit, int channel) {
 	const int width = fit->rx;
 	if (fit->type == DATA_USHORT) {
 		WORD *buf = fit->pdata[channel];
+		size_t out_idx = 0;
 		for (int y = 0; y < height; ++y) {
+			size_t in_idx = (height - y - 1) * width;
 			for (int x = 0; x < width; ++x) {
-				buf[y * width + x] = round_to_WORD(	image[(height - y - 1) * width + x] * USHRT_MAX_SINGLE);
+				buf[out_idx] = round_to_WORD(image[in_idx] * USHRT_MAX);
+				in_idx++;
+				out_idx++;
 			}
 		}
 	} else if (fit->type == DATA_FLOAT) {
 		float *buf = fit->fpdata[channel];
+		size_t out_idx = 0;
 		for (int y = 0; y < height; ++y) {
+			size_t in_idx = (height - y - 1) * width;
 			for (int x = 0; x < width; ++x) {
-				buf[y * width + x] = (float) image[(height - y - 1) * width + x];
+				buf[out_idx] = (float) image[in_idx];
+				in_idx++;
+				out_idx++;
 			}
 		}
 	}
 }
 
-static GSList *generate_samples(fits *fit, int nb_per_line, double tolerance, size_t size) {
+static GSList *generate_samples(fits *fit, int nb_per_line, double tolerance, int size, const char **error, threading_type threads) {
 	int nx = fit->rx;
 	int ny = fit->ry;
-	int dist, starty, startx;
-	unsigned int x, y;
-	float median, mad0, *work;
-	size_t radius;
 	size_t n = fit->naxes[0] * fit->naxes[1];
 	GSList *list = NULL;
 
-	float *image = convert_fits_to_luminance(fit);
-
-	work = malloc(n * sizeof(float));
-	if (!work) {
-		PRINT_ALLOC_ERR;
+	float *image = convert_fits_to_luminance(fit, threads);	// upside down
+	if (!image) {
+		if (error)
+			*error = "out of memory";
+		return NULL;
+	}
+	float median = (float)histogram_median_float(image, n, threads);
+	if (median <= 0.0f) {
+		if (error)
+			*error = "removing the gradient on negative images is not supported";
+		free(image);
 		return NULL;
 	}
 
-	dist = (int) (nx / nb_per_line);
-	radius = size / 2;
-	startx = ((nx - size) % dist) / 2;
-	starty = ((ny - size) % dist) / 2;
-	median = histogram_median_float(image, nx * ny, TRUE);
+	int boxes_width = nb_per_line * size + 2;	// leave a margin of 1 px on the sides
+	float spacing = (nx - boxes_width) / (float)(nb_per_line-1);
+	int radius = size / 2;
+	// solving iteratively n for: ny - 2 = n * size + (n-1) * spacing;
+	int nb_per_column = 1;
+	while (nb_per_column * size + round_to_int((nb_per_column - 1) * spacing) <= (ny - 2))
+		nb_per_column++;
+	nb_per_column--;
+	if (nb_per_column == 0) {
+		if (error)
+			*error = "image is smaller than the sample size of the background extraction";
+		free(image);
+		return NULL;
+	}
 
-	/* create samples */
-	for (y = starty; y <= ny - radius; y = y + dist) {
-		for (x = startx; x <= nx - radius; x = x + dist) {
+	guint nb = nb_per_line * nb_per_column;
+	float *mad = malloc(nb * sizeof(float));
+	int k = 0;
+
+	for (int i = 0; i < nb_per_line; i++) {
+		for (int j = 0; j < nb_per_column; j++) {
+			int x = round_to_int(i * (spacing + size)) + radius + 1;
+			int y = round_to_int(j * (spacing + size)) + radius + 1;
 			background_sample *sample = get_sample(image, x, y, nx, ny);
+			if (sample) {
+				mad[k++] = fabs(sample->median[RLAYER] - median);
 				list = g_slist_prepend(list, sample);
+			}
 		}
 	}
 
 	/* compute mad */
-	guint nb = g_slist_length(list);
-	float *mad = malloc(nb * sizeof(float));
-	int i = 0;
-	for (GSList *l = list; l; l = l->next) {
-		background_sample *sample = (background_sample*) l->data;
-		mad[i] = fabs(sample->median[RLAYER] - median);
-		i++;
-	}
-
-	mad0 = histogram_median_float(mad, nb, TRUE);
+	double mad0 = histogram_median_float(mad, k, TRUE);
+	free(mad);
+	double threshold = median + mad0 * tolerance;
+	siril_debug_print("Background gradient: %d samples per line, threshold %f\n", nb_per_line, threshold);
 
 	/* remove bad samples */
 	GSList *l = list;
@@ -670,20 +690,17 @@ static GSList *generate_samples(fits *fit, int nb_per_line, double tolerance, si
 		background_sample *sample = (background_sample*) l->data;
 		/* Store next element's pointer before removing it */
 		GSList *next = g_slist_next(l);
-		if (sample->median[RLAYER] <= 0.0
-				|| sample->median[RLAYER] >= (mad0 * tolerance) + median) {
-			g_free(sample);
+		if (sample->median[RLAYER] <= 0.0 || sample->median[RLAYER] >= threshold) {
+			free(sample);
 			list = g_slist_delete_link(list, l);
 		}
 		l = next;
 	}
 
 	list = g_slist_reverse(list);
-
-	free(mad);
 	free(image);
-	free(work);
-
+	if (!list && error)
+		*error = "none of the samples matched the thresholds";
 	return list;
 }
 
@@ -691,7 +708,6 @@ static GSList *update_median_samples(GSList *orig, fits *fit) {
 	const int nx = fit->rx;
 	const int ny = fit->ry;
 
-
 	const size_t n = fit->naxes[0] * fit->naxes[1];
 	double *channelData = malloc(n * sizeof(double));
 	if (!channelData) {
@@ -701,6 +717,7 @@ static GSList *update_median_samples(GSList *orig, fits *fit) {
 
 	for (int channel = 0; channel < fit->naxes[2]; channel++) {
 		convert_fits_to_img(fit, channelData, channel, FALSE);
+
 		for (GSList *list = orig; list; list = list->next) {
 			background_sample *sample = (background_sample*) list->data;
 			sample->median[channel] = get_sample_median(channelData,
@@ -709,82 +726,47 @@ static GSList *update_median_samples(GSList *orig, fits *fit) {
 	}
 
 	free(channelData);
-
 	return orig;
 }
 
-static poly_order get_poly_order() {
-	GtkComboBox *combo_box_poly_order;
-
-	combo_box_poly_order = GTK_COMBO_BOX(lookup_widget("box_background_order"));
-	return gtk_combo_box_get_active(combo_box_poly_order);
-}
-
-static int get_correction_type() {
-	GtkComboBox *combo_box_correction;
-
-	combo_box_correction = GTK_COMBO_BOX(lookup_widget("box_background_correction"));
-	return gtk_combo_box_get_active(combo_box_correction);
-}
-
-static int get_nb_samples_per_line() {
-	GtkSpinButton *nb_samples = GTK_SPIN_BUTTON(lookup_widget("spin_background_nb_samples"));
-
-	return gtk_spin_button_get_value_as_int(nb_samples);
-}
-
-static double get_tolerance_value() {
-	GtkRange *tol = GTK_RANGE(lookup_widget("scale_background_nb_samples"));
-
-	return gtk_range_get_value(tol);
-}
-
-static int get_interpolation_method() {
-	GtkComboBox *combo = GTK_COMBO_BOX(lookup_widget("background_extraction_combo"));
-
-	return gtk_combo_box_get_active(combo);
-}
-
-static double get_smoothing_parameter() {
-	GtkSpinButton *spin = GTK_SPIN_BUTTON(lookup_widget("spin_background_smoothing"));
-
-	return gtk_spin_button_get_value(spin);
-}
-
-static void remove_gradient(double *img, const double *background, double background_mean, size_t ndata, int type) {
+static void remove_gradient(double *img, const double *background, double background_mean, size_t ndata, background_correction type, threading_type threads) {
 	size_t i;
 	double mean;
 
 	switch (type) {
-	default:
-	case 0: // Subtraction
-		for (i = 0; i < ndata; i++) {
-			img[i] -= background[i];
-			img[i] += background_mean;
-		}
-		break;
-	case 1: // Division
-		mean = gsl_stats_mean(img, 1, ndata);
-		for (i = 0; i < ndata; i++) {
-			img[i] /= background[i];
-			img[i] *= mean;
-		}
+		default:
+		case BACKGROUND_CORRECTION_SUBTRACT:
+#ifdef _OPENMP
+			limit_threading(&threads, 300000, ndata);
+#pragma omp parallel for num_threads(threads) schedule(static)
+#endif
+			for (i = 0; i < ndata; i++) {
+				img[i] -= background[i];
+				img[i] += background_mean;
+			}
+			break;
+		case BACKGROUND_CORRECTION_DIVIDE:
+			mean = gsl_stats_mean(img, 1, ndata);
+#ifdef _OPENMP
+			limit_threading(&threads, 300000, ndata);
+#pragma omp parallel for num_threads(threads) schedule(static)
+#endif
+			for (i = 0; i < ndata; i++) {
+				img[i] /= background[i];
+				img[i] *= mean;
+			}
 	}
 }
 
 /************* PUBLIC FUNCTIONS *************/
 
-int get_sample_radius() {
-	return (int) (SAMPLE_SIZE / 2);
-}
-
-gboolean is_dither_checked() {
-	return (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lookup_widget("bkg_dither_button"))));
+int get_background_sample_radius() {
+       return SAMPLE_SIZE / 2;
 }
 
 void free_background_sample_list(GSList *list) {
 	if (list == NULL) return;
-	g_slist_free_full(list, g_free);
+	g_slist_free_full(list, free);
 }
 
 GSList *add_background_sample(GSList *orig, fits *fit, point pt) {
@@ -793,7 +775,7 @@ GSList *add_background_sample(GSList *orig, fits *fit, point pt) {
 	int ny = fit->ry;
 	float *image;
 
-	image = convert_fits_to_luminance(fit);
+	image = convert_fits_to_luminance(fit, MULTI_THREADED);
 
 	list = orig;
 
@@ -814,7 +796,7 @@ GSList *remove_background_sample(GSList *orig, fits *fit, point pt) {
 	float *image;
 	double min_radius = DBL_MAX;
 
-	image = convert_fits_to_luminance(fit);
+	image = convert_fits_to_luminance(fit, MULTI_THREADED);
 
 	/* search for the min radius vale */
 	for (list = orig; list; list = list->next) {
@@ -835,6 +817,7 @@ GSList *remove_background_sample(GSList *orig, fits *fit, point pt) {
 		if (radius == min_radius) {
 			orig = g_slist_remove(orig, sample);
 			g_free((background_sample *) sample);
+			break;
 		}
 	}
 	free(image);
@@ -843,50 +826,36 @@ GSList *remove_background_sample(GSList *orig, fits *fit, point pt) {
 }
 
 /* generates samples and stores them in com.grad_samples */
-void generate_background_samples(int nb_of_samples, double tolerance) {
+int generate_background_samples(int nb_of_samples, double tolerance) {
 	free_background_sample_list(com.grad_samples);
-	com.grad_samples = generate_samples(&gfit, nb_of_samples, tolerance, SAMPLE_SIZE);
-	/* If RGB we need to update all local median, not only the first one */
-	if (gfit.naxes[2] > 1) {
+	const char *err;
+	com.grad_samples = generate_samples(&gfit, nb_of_samples, tolerance, SAMPLE_SIZE, &err, MULTI_THREADED);
+	if (!com.grad_samples) {
+		siril_log_color_message(_("Failed to generate background samples for image: %s\n"), "red", _(err));
+		return 1;
+	}
+
+	if (com.grad_samples && gfit.naxes[2] > 1) {
+		/* If RGB we need to update all local median, not only the first one */
 		com.grad_samples = update_median_samples(com.grad_samples, &gfit);
 	}
-
-	redraw(REDRAW_OVERLAY);
-}
-
-static gboolean end_background(gpointer p) {
-	struct background_data *args = (struct background_data *)p;
-	stop_processing_thread();
-	if (args) {
-		invalidate_stats_from_fit(args->fit);
-		background_computed = TRUE;
-		if (!args->from_ui) {
-			free_background_sample_list(com.grad_samples);
-			com.grad_samples = NULL;
-		}
-		gtk_widget_set_sensitive(lookup_widget("background_ok_button"), TRUE);
-		gtk_widget_set_sensitive(lookup_widget("bkg_show_original"), TRUE);
-		adjust_cutoff_from_updated_gfit();
-		redraw(REMAP_ALL);
-		set_cursor_waiting(FALSE);
-		gtk_widget_set_sensitive(lookup_widget("bkg_show_original"), TRUE);
-		free(args);
-	}
-	set_cursor_waiting(FALSE);
-	return FALSE;
+	return 0;
 }
 
+gboolean end_background(gpointer p);	// in gui/background_extraction.c
 
 /* uses samples from com.grad_samples */
 gpointer remove_gradient_from_image(gpointer p) {
 	struct background_data *args = (struct background_data *)p;
-	gchar *error;
-	double *background = (double*)malloc(gfit.ry * gfit.rx * sizeof(double));
-	
-	if (!background && !com.script) {
+	gchar *error = NULL;
+	double *background = malloc(gfit.ry * gfit.rx * sizeof(double));
+
+	if (!background) {
 		PRINT_ALLOC_ERR;
-		set_cursor_waiting(FALSE);
-		return NULL;
+		if (!com.script) {
+			set_cursor_waiting(FALSE);
+		}
+		return GINT_TO_POINTER(1);
 	}
 
 	const size_t n = gfit.naxes[0] * gfit.naxes[1];
@@ -894,9 +863,9 @@ gpointer remove_gradient_from_image(gpointer p) {
 	if (!image) {
 		free(background);
 		PRINT_ALLOC_ERR;
-		return NULL;
+		return GINT_TO_POINTER(1);
 	}
-	
+
 	/* Make sure to update local median. Useful if undo is pressed */
 	update_median_samples(com.grad_samples, &gfit);
 
@@ -906,55 +875,61 @@ gpointer remove_gradient_from_image(gpointer p) {
 	for (int channel = 0; channel < gfit.naxes[2]; channel++) {
 		/* compute background */
 		gboolean interpolation_worked = TRUE;
-		if (args->interpolation_method == INTER_POLY) {
-			interpolation_worked = computeBackground_Polynom(com.grad_samples, background, channel, gfit.rx, gfit.ry, args->degree, &error);
+		if (args->interpolation_method == BACKGROUND_INTER_POLY) {
+			interpolation_worked = computeBackground_Polynom(com.grad_samples, background, channel,
+					gfit.rx, gfit.ry, args->degree, &error);
 		} else {
-			interpolation_worked = computeBackground_RBF(com.grad_samples, background, channel, gfit.rx, gfit.ry, args->smoothing, &error, TRUE);
+			interpolation_worked = computeBackground_RBF(com.grad_samples, background, channel,
+					gfit.rx, gfit.ry, args->smoothing, &error, args->threads, TRUE);
 		}
-		
+
 		if (!interpolation_worked) {
 			free(image);
 			free(background);
-			queue_message_dialog(GTK_MESSAGE_ERROR, _("Not enough samples."),	error);
+			queue_error_message_dialog(_("Not enough samples."), error);
 			free(args);
 			siril_add_idle(end_background, NULL);
-			return NULL;
+			return GINT_TO_POINTER(1);
 		}
 		/* remove background */
-		const char *c_name = vport_number_to_name(channel);
-		siril_log_message(_("Background extraction from channel %s.\n"), c_name);
+		const char *c_name;
+		if (gfit.naxes[2] > 1)
+			c_name = channel_number_to_name(channel);
+		else
+			c_name = _("monochrome");
+		siril_log_message(_("Background extraction from %s channel.\n"), c_name);
 		convert_fits_to_img(&gfit, image, channel, args->dither);
-		remove_gradient(image, background, background_mean, gfit.naxes[0] * gfit.naxes[1], args->correction);
+		remove_gradient(image, background, background_mean, n, args->correction, MULTI_THREADED);
 		convert_img_to_fits(image, &gfit, channel);
 	}
-	siril_log_message(_("Background with %s interpolation computed.\n"), (args->interpolation_method == INTER_POLY) ? "polynomial" : "RBF");
+	siril_log_message(_("Background with %s interpolation computed.\n"),
+			(args->interpolation_method == BACKGROUND_INTER_POLY) ? "polynomial" : "RBF");
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 	/* free memory */
 	free(image);
 	free(background);
 	siril_add_idle(end_background, args);
-	return args;
+	return GINT_TO_POINTER(0);
 }
 
 /** Apply for sequence **/
 
 static int background_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
-		rectangle *_) {
+		rectangle *_, int threads) {
 	struct background_data *b_args = (struct background_data*) args->user;
 
-	gchar *error;
 	double *background = (double*)malloc(fit->naxes[0] * fit->naxes[1] * sizeof(double));
 	if (!background) {
 		PRINT_ALLOC_ERR;
-		error = _("Out of memory - aborting");
-		siril_log_message(error);
-		set_cursor_waiting(FALSE);
+		siril_log_message(_("Out of memory - aborting"));
 		return 1;
 	}
 
-	GSList *samples = generate_samples(fit, b_args->nb_of_samples, b_args->tolerance, SAMPLE_SIZE);
+	const char *err;
+	GSList *samples = generate_samples(fit, b_args->nb_of_samples, b_args->tolerance, SAMPLE_SIZE, &err, (threading_type)threads);
 	if (!samples) {
+		siril_log_color_message(_("Failed to generate background samples for image %d: %s\n"), "red", i, _(err));
 		free(background);
 		return 1;
 	}
@@ -972,23 +947,22 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 		PRINT_ALLOC_ERR;
 		return 1;
 	}
-	
+
 	double background_mean = get_background_mean(samples, fit->naxes[2]);
 	for (int channel = 0; channel < fit->naxes[2]; channel++) {
 		/* compute background */
 		gboolean interpolation_worked = TRUE;
-		if (b_args->interpolation_method == INTER_POLY){
+		gchar *error = NULL;
+		if (b_args->interpolation_method == BACKGROUND_INTER_POLY){
 			interpolation_worked = computeBackground_Polynom(samples, background, channel, fit->rx, fit->ry, b_args->degree, &error);
 		} else {
-			interpolation_worked = computeBackground_RBF(samples, background, channel, fit->rx, fit->ry, b_args->smoothing, &error, FALSE);
+			interpolation_worked = computeBackground_RBF(samples, background, channel, fit->rx, fit->ry, b_args->smoothing, &error, threads, FALSE);
 		}
-		
+
 		if (!interpolation_worked) {
 			if (error) {
 				siril_log_message(error);
 			}
-
-			set_cursor_waiting(FALSE);
 			free(image);
 			free(background);
 			free_background_sample_list(samples);
@@ -996,7 +970,7 @@ static int background_image_hook(struct generic_seq_args *args, int o, int i, fi
 		}
 		/* remove background */
 		convert_fits_to_img(fit, image, channel, b_args->dither);
-		remove_gradient(image, background, background_mean, fit->naxes[0] * fit->naxes[1], b_args->correction);
+		remove_gradient(image, background, background_mean, fit->naxes[0] * fit->naxes[1], b_args->correction, (threading_type)threads);
 		convert_img_to_fits(image, fit, channel);
 	}
 	/* free memory */
@@ -1029,14 +1003,14 @@ static int background_mem_limits_hook(struct generic_seq_args *args, gboolean fo
 		required = MB_per_image + double_channel_size_MB * 2;
 		int thread_limit = MB_avail / required;
 		if (thread_limit > com.max_thread)
-						thread_limit = com.max_thread;
+			thread_limit = com.max_thread;
 
 		if (for_writer) {
-						/* we allow the already allocated thread_limit images,
-						 * plus how many images can be stored in what remains
-						 * unused by the main processing */
-						limit = thread_limit + (MB_avail - required * thread_limit) / MB_per_image;
-				} else limit = thread_limit;
+			/* we allow the already allocated thread_limit images,
+			 * plus how many images can be stored in what remains
+			 * unused by the main processing */
+			limit = thread_limit + (MB_avail - required * thread_limit) / MB_per_image;
+		} else limit = thread_limit;
 
 	}
 	if (limit == 0) {
@@ -1067,13 +1041,20 @@ static int background_mem_limits_hook(struct generic_seq_args *args, gboolean fo
 	return limit;
 }
 
+int bg_extract_finalize_hook(struct generic_seq_args *args) {
+	struct background_data *data = (struct background_data *) args->user;
+	int retval = seq_finalize_hook(args);
+	free(data);
+	return retval;
+}
+
 void apply_background_extraction_to_sequence(struct background_data *background_args) {
 	struct generic_seq_args *args = create_default_seqargs(background_args->seq);
 	args->filtering_criterion = seq_filter_included;
 	args->nb_filtered_images = background_args->seq->selnum;
 	args->compute_mem_limits_hook = background_mem_limits_hook;
 	args->prepare_hook = seq_prepare_hook;
-	args->finalize_hook = seq_finalize_hook;
+	args->finalize_hook = bg_extract_finalize_hook;
 	args->image_hook = background_image_hook;
 	args->stop_on_error = FALSE;
 	args->description = _("Background Extraction");
@@ -1088,7 +1069,7 @@ void apply_background_extraction_to_sequence(struct background_data *background_
 	start_in_new_thread(generic_sequence_worker, args);
 }
 
-/**** getter and setter ***/
+/**** getters ***/
 
 gboolean background_sample_is_valid(background_sample *sample) {
 	return sample->valid;
@@ -1102,201 +1083,3 @@ point background_sample_get_position(background_sample *sample) {
 	return sample->position;
 }
 
-static fits background_backup;
-
-static void copy_gfit_to_bkg_backup() {
-	if (!background_computed) return;
-	if (copyfits(&gfit, &background_backup, CP_ALLOC | CP_COPYA | CP_FORMAT, -1)) {
-		siril_log_message(_("Image copy error in previews\n"));
-		return;
-	}
-}
-
-static int copy_bkg_backup_to_gfit() {
-	if (!background_computed) return 0;
-	int retval = 0;
-	if (!gfit.data && !gfit.fdata)
-		retval = 1;
-	else if (copyfits(&background_backup, &gfit, CP_COPYA, -1)) {
-		siril_log_message(_("Image copy error in previews\n"));
-		retval = 1;
-	}
-	return retval;
-}
-
-/************* CALLBACKS *************/
-
-void on_background_generate_clicked(GtkButton *button, gpointer user_data) {
-	set_cursor_waiting(TRUE);
-	int nb_of_samples;
-	double tolerance;
-
-	nb_of_samples = get_nb_samples_per_line();
-	tolerance = get_tolerance_value();
-
-	generate_background_samples(nb_of_samples, tolerance);
-	set_cursor_waiting(FALSE);
-}
-
-void on_background_clear_all_clicked(GtkButton *button, gpointer user_data) {
-	free_background_sample_list(com.grad_samples);
-	com.grad_samples = NULL;
-
-	redraw(REDRAW_OVERLAY);
-	set_cursor_waiting(FALSE);
-}
-
-void on_bkg_compute_bkg_clicked(GtkButton *button, gpointer user_data) {
-	if (com.grad_samples == NULL) {
-		return;
-	}
-	set_cursor_waiting(TRUE);
-	copy_backup_to_gfit();
-
-	int correction = get_correction_type();
-	poly_order degree = get_poly_order();
-	double smoothing = get_smoothing_parameter();
-	gboolean use_dither = is_dither_checked();
-	int interpolation_method = get_interpolation_method();
-
-	struct background_data *args = malloc(sizeof(struct background_data));
-	args->threads = com.max_thread;
-	args->from_ui = TRUE;
-	args->correction = correction;
-	args->interpolation_method = interpolation_method;
-	args->degree = (poly_order) degree;
-	args->smoothing = smoothing;
-	args->dither = use_dither;
-	args->fit = &gfit;
-
-	start_in_new_thread(remove_gradient_from_image, args);
-}
-
-void on_background_ok_button_clicked(GtkButton *button, gpointer user_data) {
-	GtkToggleButton *seq_button = GTK_TOGGLE_BUTTON(
-			lookup_widget("checkBkgSeq"));
-	if (gtk_toggle_button_get_active(seq_button) && sequence_is_loaded()) {
-		struct background_data *args = malloc(sizeof(struct background_data));
-
-		args->nb_of_samples = get_nb_samples_per_line();
-		args->tolerance = get_tolerance_value();
-		args->correction = get_correction_type();
-		args->degree = get_poly_order();
-		args->smoothing = get_smoothing_parameter();
-		args->dither = is_dither_checked();
-		args->interpolation_method = get_interpolation_method();
-		
-		if (args->interpolation_method == INTER_POLY && args->degree > BACKGROUND_POLY_1) {
-			int confirm = siril_confirm_dialog(_("Polynomial order seems too high."),
-					_("You are about to process a sequence of preprocessed files with "
-							"a polynomial degree greater than 1. This is unlikely because such "
-							"gradients are often linear and a correction with a polynomial "
-							"function of degree 1 is probably enough."), _("Extract Background"));
-			if (!confirm) {
-				free(args);
-				set_cursor_waiting(FALSE);
-				return;
-			}
-		} else if (args->interpolation_method != INTER_POLY) {
-			int confirm = siril_confirm_dialog(_("Using wrong interpolation method"),
-					_("You are about to process a sequence of preprocessed files with an RBF algorithm. "
-							"This algorithm may not be very well suited for automated processing "
-							"and we advise you to use the polynomial algorithm with a "
-							"degree order of 1."), _("Extract Background"));
-			if (!confirm) {
-				free(args);
-				set_cursor_waiting(FALSE);
-				return;
-			}
-		}
-
-		set_cursor_waiting(TRUE);
-
-		args->seqEntry = gtk_entry_get_text(GTK_ENTRY(lookup_widget("entryBkgSeq")));
-		if (args->seqEntry && args->seqEntry[0] == '\0')
-			args->seqEntry = "bkg_";
-		args->seq = &com.seq;
-		/* now we uncheck the button */
-		gtk_toggle_button_set_active(seq_button, FALSE);
-		apply_background_extraction_to_sequence(args);
-	} else {
-		if (background_computed) {
-			int correction = get_correction_type();
-			undo_save_state(get_preview_gfit_backup(), _("Background extraction (Correction: %s)"),
-					correction ? "Division" : "Subtraction");
-			background_computed = FALSE;
-			clear_backup();
-			siril_close_dialog("background_extraction_dialog");
-		} else {
-			siril_message_dialog(GTK_MESSAGE_WARNING, _("No Background model computed"),
-					_("You must first compute the background model."));
-		}
-	}
-}
-
-void apply_background_cancel() {
-	siril_close_dialog("background_extraction_dialog");
-}
-
-void on_background_close_button_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("background_extraction_dialog");
-}
-
-void on_background_extraction_dialog_hide(GtkWidget *widget, gpointer user_data) {
-	free_background_sample_list(com.grad_samples);
-	com.grad_samples = NULL;
-	mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
-	redraw(REDRAW_OVERLAY);
-
-	if (background_computed) {
-		siril_preview_hide();
-		background_computed = FALSE;
-	} else {
-		clear_backup();
-	}
-	gtk_widget_set_sensitive(lookup_widget("background_ok_button"), FALSE);
-	gtk_widget_set_sensitive(lookup_widget("bkg_show_original"), FALSE);
-}
-
-void on_background_extraction_dialog_show(GtkWidget *widget, gpointer user_data) {
-	mouse_status = MOUSE_ACTION_DRAW_SAMPLES;
-	background_startup();
-}
-
-void on_background_extraction_combo_changed(GtkComboBox *combo, gpointer user_data) {
-	GtkNotebook *notebook = GTK_NOTEBOOK(lookup_widget("bkg_notebook_inter"));
-	gtk_notebook_set_current_page(notebook, gtk_combo_box_get_active(combo));
-}
-
-void on_bkg_show_original_button_press_event(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-	GtkStateFlags new_state;
-	new_state = gtk_widget_get_state_flags(widget)
-			& ~(GTK_STATE_FLAG_PRELIGHT | GTK_STATE_FLAG_ACTIVE);
-
-	new_state |= GTK_STATE_FLAG_PRELIGHT;
-    new_state |= GTK_STATE_FLAG_ACTIVE;
-
-	gtk_widget_set_state_flags(widget, new_state, TRUE);
-
-	copy_gfit_to_bkg_backup();
-	copy_backup_to_gfit();
-	adjust_cutoff_from_updated_gfit();
-	redraw(REMAP_ALL);
-	set_cursor_waiting(FALSE);
-}
-
-void on_bkg_show_original_button_release_event(GtkWidget *widget, GdkEvent *event, gpointer user_data) {
-	GtkStateFlags new_state;
-	new_state = gtk_widget_get_state_flags(widget)
-			& ~(GTK_STATE_FLAG_PRELIGHT | GTK_STATE_FLAG_ACTIVE);
-
-	new_state |= GTK_STATE_FLAG_PRELIGHT;
-
-	gtk_widget_set_state_flags(widget, new_state, TRUE);
-
-	copy_bkg_backup_to_gfit();
-	clearfits(&background_backup);
-	adjust_cutoff_from_updated_gfit();
-	redraw(REMAP_ALL);
-	set_cursor_waiting(FALSE);
-}
