@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,9 +26,20 @@ struct tilt_struct {
 	double fwhm_centre;
 };
 
-typedef struct tilt_struct sensor_tilt;
+struct tilt_data {
+	fits *fit;
+	sequence *seq;
+	gboolean draw_polygon;
+	int nbstars;
+	float m, m1, m2, m3, m4, mr1, mr2;
+};
 
 void clear_sensor_tilt();
 int draw_sensor_tilt(fits *fit);
 
+void apply_tilt_to_sequence(struct tilt_data *tilt_args);
+
+void compute_aberration_inspector();
+void redraw_aberration_inspector();
+
 #endif /* SRC_ALGOS_CCD_INSPECTOR_H_ */
