@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,37 +24,33 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
+#include "algos/astrometry_solver.h"
 #include "algos/statistics.h"
 #include "algos/siril_wcs.h"
-#include "core/undo.h"
 #include "core/processing.h"
-#include "gui/utils.h"
-#include "gui/image_display.h"
-#include "gui/image_interactions.h"
-#include "gui/PSF_list.h"
-#include "gui/dialogs.h"
-#include "gui/message_dialog.h"
-#include "gui/registration_preview.h"
-#include "gui/siril_preview.h"
 #include "opencv/opencv.h"
 #include "io/single_image.h"
 #include "io/sequence.h"
 #include "io/image_format_fits.h"
+#include "gui/PSF_list.h"
+#include "gui/utils.h"
 
 #include "geometry.h"
-#include "astrometry_solver.h"
 
 /* this method rotates the image 180 degrees, useful after german mount flip.
  * fit->rx, fit->ry, fit->naxes[2] and fit->pdata[*] are required to be assigned correctly */
 static void fits_rotate_pi_ushort(fits *fit) {
 	int i, line, axis;
-	WORD *line1, *line2, *src, *dst, swap;
+	WORD *line1 = NULL, *line2 = NULL, *src, *dst, swap;
 
 	size_t line_size = fit->rx * sizeof(WORD);
 	line1 = malloc(line_size);
 	line2 = malloc(line_size);
 	if (!line1 || !line2) {
 		PRINT_ALLOC_ERR;
+		if (line1) free(line1);
+		if (line2) free(line2);
 		return;
 	}
 
@@ -94,13 +90,15 @@ static void fits_rotate_pi_ushort(fits *fit) {
 
 static void fits_rotate_pi_float(fits *fit) {
 	int i, line, axis;
-	float *line1, *line2, *src, *dst, swap;
+	float *line1 = NULL, *line2 = NULL, *src, *dst, swap;
 
 	size_t line_size = fit->rx * sizeof(float);
 	line1 = malloc(line_size);
 	line2 = malloc(line_size);
 	if (!line1 || !line2) {
 		PRINT_ALLOC_ERR;
+		if (line1) free(line1);
+		if (line2) free(line2);
 		return;
 	}
 
@@ -146,84 +144,186 @@ static void fits_rotate_pi(fits *fit) {
 	}
 }
 
-void mirrorx_gui(fits *fit) {
-	set_cursor_waiting(TRUE);
-	undo_save_state(fit, _("Mirror X"));
-	mirrorx(fit, TRUE);
-	redraw(REMAP_ALL);
-	redraw_previews();
-	set_cursor_waiting(FALSE);
-}
+static void fit_update_buffer(fits *fit, void *newbuf, int width, int height, int bin_factor) {
+	size_t nbdata = width * height;
 
-void mirrory_gui(fits *fit) {
-	set_cursor_waiting(TRUE);
-	undo_save_state(fit, _("Mirror Y"));
-	mirrory(fit, TRUE);
-	redraw(REMAP_ALL);
-	redraw_previews();
-	set_cursor_waiting(FALSE);
-}
+	full_stats_invalidation_from_fit(fit);
 
-static void rotate_gui(fits *fit) {
-	static GtkToggleButton *crop_rotation = NULL;
-	double angle = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_rotation")));
-	if (angle == 0) return;
-	int interpolation = gtk_combo_box_get_active(
-			GTK_COMBO_BOX(lookup_widget("combo_interpolation_rotation")));
-	int cropped;
-
-	if (crop_rotation == NULL) {
-		crop_rotation = GTK_TOGGLE_BUTTON(
-				lookup_widget("checkbutton_rotation_crop"));
+	if (fit->type == DATA_USHORT) {
+		if (fit->data)
+			free(fit->data);
+		fit->data = (WORD *)newbuf;
+		fit->pdata[RLAYER] = fit->data;
+		fit->pdata[GLAYER] = fit->naxes[2] == 3 ? fit->data + nbdata : fit->data;
+		fit->pdata[BLAYER] = fit->naxes[2] == 3 ? fit->data + nbdata * 2 : fit->data;
 	}
-	cropped = gtk_toggle_button_get_active(crop_rotation);
+	else if (fit->type == DATA_FLOAT) {
+		if (fit->fdata)
+			free(fit->fdata);
+		fit->fdata = (float *)newbuf;
+		fit->fpdata[RLAYER] = fit->fdata;
+		fit->fpdata[GLAYER] = fit->naxes[2] == 3 ? fit->fdata + nbdata : fit->fdata;
+		fit->fpdata[BLAYER] = fit->naxes[2] == 3 ? fit->fdata + nbdata * 2 : fit->fdata;
+	}
+	/* update size */
+	fit->naxes[0] = width;
+	fit->naxes[1] = height;
+	fit->rx = width;
+	fit->ry = height;
 
-	set_cursor_waiting(TRUE);
-	undo_save_state(fit, _("Rotation (%.1lfdeg, cropped=%s)"), angle,
-			cropped ? "TRUE" : "FALSE");
-	verbose_rotate_image(fit, angle, interpolation, cropped);
-	
-	update_zoom_label();
-	redraw(REMAP_ALL);
-	redraw_previews();
-	set_cursor_waiting(FALSE);
+	if (fit->binning_x == 0 || fit->binning_x == 1) {
+		fit->binning_x = bin_factor;
+		fit->binning_y = bin_factor;
+	} else {
+		fit->binning_x *= bin_factor;
+		fit->binning_y *= bin_factor;
+	}
+	if (!com.pref.binning_update) {
+		fit->pixel_size_x *= bin_factor;
+		fit->pixel_size_y *= bin_factor;
+	}
+}
+
+static void fits_binning_float(fits *fit, int bin_factor, gboolean mean) {
+	if (bin_factor == 0) // Bin 0 would be nonsensical.
+		return;
+	int width = fit->rx;
+	int height = fit->ry;
+	int new_width = width / bin_factor;
+	int new_height = height / bin_factor;
+
+	size_t npixels = new_width * new_height;
+
+	float *newbuf = malloc(npixels * fit->naxes[2] * sizeof(float));
+	if (!newbuf) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
+
+	for (int channel = 0; channel < fit->naxes[2]; channel++) {
+		float *buf = fit->fdata + (width * height) * channel;
+
+		long k = 0 + channel * npixels;
+		for (int row = 0, nrow = 0; row < height - bin_factor + 1; row += bin_factor, nrow++) {
+			for (int col = 0, ncol = 0; col < width - bin_factor + 1; col += bin_factor, ncol++) {
+				int c = 0;
+				newbuf[k] = 0;
+				for (int i = 0; i < bin_factor; i++) {
+					for (int j = 0; j < bin_factor; j++) {
+						newbuf[k] += buf[i + col + (j + row) * width];
+						c++;
+					}
+				}
+				if (mean) newbuf[k] /= c;
+				k++;
+			}
+		}
+	}
+	fit_update_buffer(fit, newbuf, new_width, new_height, bin_factor);
+	return;
+}
+
+static void fits_binning_ushort(fits *fit, int bin_factor, gboolean mean) {
+	if (bin_factor == 0) // Bin 0 would be nonsensical.
+		return;
+	int width = fit->rx;
+	int height = fit->ry;
+	int new_width = width / bin_factor;
+	int new_height = height / bin_factor;
+
+	size_t npixels = new_width * new_height;
+
+	WORD *newbuf = malloc(npixels * fit->naxes[2] * sizeof(WORD));
+	if (!newbuf) {
+		PRINT_ALLOC_ERR;
+		return;
+	}
+
+	for (int channel = 0; channel < fit->naxes[2]; channel++) {
+		WORD *buf = fit->data + (width * height) * channel;
+
+		long k = 0 + channel * npixels;
+		for (int row = 0, nrow = 0; row < height - bin_factor + 1; row += bin_factor, nrow++) {
+			for (int col = 0, ncol = 0; col < width - bin_factor + 1; col += bin_factor, ncol++) {
+				int c = 0;
+				int tmp = 0;
+				for (int i = 0; i < bin_factor; i++) {
+					for (int j = 0; j < bin_factor; j++) {
+						tmp += (buf[i + col + (j + row) * width]);
+						c++;
+					}
+				}
+				if (mean) tmp /= c;
+				newbuf[k] = truncate_to_WORD(tmp);
+				k++;
+			}
+		}
+	}
+	fit_update_buffer(fit, newbuf, new_width, new_height, bin_factor);
+}
+
+int fits_binning(fits *fit, int factor, gboolean mean) {
+	struct timeval t_start, t_end;
+
+	siril_log_color_message(_("Binning x%d: processing...\n"), "green", factor);
+	gettimeofday(&t_start, NULL);
+
+	if (fit->type == DATA_USHORT) {
+		fits_binning_ushort(fit, factor, mean);
+	} else if (fit->type == DATA_FLOAT) {
+		fits_binning_float(fit, factor, mean);
+	}
+
+	free_wcs(fit, TRUE); // we keep RA/DEC to initialize platesolve
+	load_WCS_from_memory(fit);
+
+	gettimeofday(&t_end, NULL);
+	show_time(t_start, t_end);
+
+	char log[90];
+	sprintf(log, "Binned x%d (%s)", factor, mean ? "mean" : "sum");
+	fit->history = g_slist_append(fit->history, strdup(log));
+
+	siril_log_message(_("New image size: %dx%d pixels.\n"), fit->rx, fit->ry);
+
+	return 0;
+}
+
+const char *interp_to_str(opencv_interpolation interpolation) {
+	switch (interpolation) {
+		case OPENCV_NEAREST:
+			return _("Nearest-Neighbor");
+		default:
+		case OPENCV_LINEAR:
+			return _("Bilinear");
+		case OPENCV_AREA:
+			return _("Pixel Area Relation");
+		case OPENCV_CUBIC:
+			return _("Bicubic");
+		case OPENCV_LANCZOS4:
+			return _("Lanczos4");
+	}
 }
 
 /* These functions do not more than resize_gaussian and rotate_image
  * except for console outputs.
  * Indeed, siril_log_message seems not working in a cpp file */
-int verbose_resize_gaussian(fits *image, int toX, int toY, int interpolation) {
+int verbose_resize_gaussian(fits *image, int toX, int toY, opencv_interpolation interpolation, gboolean clamp) {
 	int retvalue;
-	const char *str_inter;
 	struct timeval t_start, t_end;
-
-	switch (interpolation) {
-	case OPENCV_NEAREST:
-		str_inter = _("Nearest-Neighbor");
-		break;
-	default:
-	case OPENCV_LINEAR:
-		str_inter = _("Bilinear");
-		break;
-	case OPENCV_AREA:
-		str_inter = _("Pixel Area Relation");
-		break;
-	case OPENCV_CUBIC:
-		str_inter = _("Bicubic");
-		break;
-	case OPENCV_LANCZOS4:
-		str_inter = _("Lanczos4");
-		break;
-	}
+	float factor_X = (float)image->rx / (float)toX;
+	float factor_Y = (float)image->ry / (float)toY;
 
 	siril_log_color_message(_("Resample (%s interpolation): processing...\n"),
-			"green", str_inter);
+			"green", interp_to_str(interpolation));
 
 	gettimeofday(&t_start, NULL);
 
-	retvalue = cvResizeGaussian(image, toX, toY, interpolation);
-	free_wcs(image);
+	retvalue = cvResizeGaussian(image, toX, toY, interpolation, clamp);
+	if (image->pixel_size_x > 0) image->pixel_size_x *= factor_X;
+	if (image->pixel_size_y > 0) image->pixel_size_y *= factor_Y;
+	free_wcs(image, TRUE); // we keep RA/DEC to initialize platesolve
+	load_WCS_from_memory(image);
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
@@ -231,31 +331,77 @@ int verbose_resize_gaussian(fits *image, int toX, int toY, int interpolation) {
 	return retvalue;
 }
 
-int verbose_rotate_image(fits *image, double angle, int interpolation,
-		int cropped) {
+// computes H matrix for rotation and crop
+static void GetMatrixReframe(fits *image, rectangle area, double angle, int cropped, int *target_rx, int *target_ry, Homography *H) {
+	*target_rx = area.w;
+	*target_ry = area.h;
+	double orig_x = (double)area.x;
+	double orig_y = (double)area.y;
+	if (!cropped) {
+		point center = {orig_x + (double)*target_rx * 0.5, orig_y + (double)*target_ry * 0.5 };
+		cvGetBoundingRectSize(image, center, angle, target_rx, target_ry);
+		orig_x = (double)((int)image->rx - *target_rx) * 0.5;
+		orig_y = (double)((int)image->ry - *target_ry) * 0.5;
+	}
+	cvGetMatrixReframe(orig_x, orig_y, *target_rx, *target_ry, angle, H);
+}
+
+// wraps cvRotateImage to update WCS data as well
+int verbose_rotate_fast(fits *image, int angle) {
+	if (angle % 90 != 0) return 1;
+	struct timeval t_start, t_end;
+	gettimeofday(&t_start, NULL);
+	siril_log_color_message(
+			_("Rotation (%s interpolation, angle=%g): processing...\n"), "green",
+			_("No"), (double)angle);
+
+#ifdef HAVE_WCSLIB // needs to be done prior to modifying the image
+	int orig_ry = image->ry; // required to compute flips afterwards
+	int target_rx, target_ry;
+	Homography H = { 0 };
+	rectangle area = {0, 0, image->rx, image->ry};
+	GetMatrixReframe(image, area, (double)angle, 0, &target_rx, &target_ry, &H);
+#endif
+
+	if (cvRotateImage(image, angle)) return 1;
+
+	gettimeofday(&t_end, NULL);
+	show_time(t_start, t_end);
+#ifdef HAVE_WCSLIB
+	if (has_wcs(image)) {
+		cvApplyFlips(&H, orig_ry, target_ry);
+		reframe_astrometry_data(image, H);
+		load_WCS_from_memory(image);
+	}
+#endif
+	return 0;
+}
+
+int verbose_rotate_image(fits *image, rectangle area, double angle, int interpolation,
+		int cropped, gboolean clamp) {
 	const char *str_inter;
 	struct timeval t_start, t_end;
 
 	switch (interpolation) {
-	case -1:
-		str_inter = _("No");
-		break;
-	case OPENCV_NEAREST:
-		str_inter = _("Nearest-Neighbor");
-		break;
-	default:
-	case OPENCV_LINEAR:
-		str_inter = _("Bilinear");
-		break;
-	case OPENCV_AREA:
-		str_inter = _("Pixel Area Relation");
-		break;
-	case OPENCV_CUBIC:
-		str_inter = _("Bicubic");
-		break;
-	case OPENCV_LANCZOS4:
-		str_inter = _("Lanczos4");
-		break;
+		case -1:
+			str_inter = _("No");
+			break;
+		case OPENCV_NEAREST:
+			str_inter = _("Nearest-Neighbor");
+			break;
+		default:
+		case OPENCV_LINEAR:
+			str_inter = _("Bilinear");
+			break;
+		case OPENCV_AREA:
+			str_inter = _("Pixel Area Relation");
+			break;
+		case OPENCV_CUBIC:
+			str_inter = _("Bicubic");
+			break;
+		case OPENCV_LANCZOS4:
+			str_inter = _("Lanczos4");
+			break;
 	}
 
 	siril_log_color_message(
@@ -263,16 +409,21 @@ int verbose_rotate_image(fits *image, double angle, int interpolation,
 			str_inter, angle);
 	gettimeofday(&t_start, NULL);
 
-	point center = {image->rx / 2.0, image->ry / 2.0};
-
-	cvRotateImage(image, center, angle, interpolation, cropped);
+#ifdef HAVE_WCSLIB
+	int orig_ry = image->ry; // required to compute flips afterwards
+#endif
+	int target_rx, target_ry;
+	Homography H = { 0 };
+	GetMatrixReframe(image, area, angle, cropped, &target_rx, &target_ry, &H);
+	if (cvTransformImage(image, target_rx, target_ry, H, FALSE, interpolation, clamp)) return 1;
 
 	gettimeofday(&t_end, NULL);
 	show_time(t_start, t_end);
 
 #ifdef HAVE_WCSLIB
-	if (image->wcslib) {
-		rotate_astrometry_data(image, center, angle, cropped);
+	if (has_wcs(image)) {
+		cvApplyFlips(&H, orig_ry, target_ry);
+		reframe_astrometry_data(image, H);
 		load_WCS_from_memory(image);
 	}
 #endif
@@ -353,9 +504,20 @@ void mirrorx(fits *fit, gboolean verbose) {
 	} else if (fit->type == DATA_FLOAT) {
 		mirrorx_float(fit, verbose);
 	}
+	if (!strcmp(fit->row_order, "BOTTOM-UP"))
+		sprintf(fit->row_order, "TOP-DOWN");
+	else { //if (!strcmp(fit->row_order, "TOP-DOWN"))
+		// let's create the keyword in all cases
+		sprintf(fit->row_order, "BOTTOM-UP");
+	}
+	fit->history = g_slist_append(fit->history, strdup("TOP-DOWN mirror"));
 #ifdef HAVE_WCSLIB
-	if (fit->wcslib) {
-		flip_bottom_up_astrometry_data(fit);
+	if (has_wcs(fit)) {
+		Homography H = { 0 };
+		cvGetEye(&H);
+		H.h11 = -1.;
+		H.h12 = (double)fit->ry - 1.;
+		reframe_astrometry_data(fit, H);
 		load_WCS_from_memory(fit);
 	}
 #endif
@@ -377,9 +539,14 @@ void mirrory(fits *fit, gboolean verbose) {
 		show_time(t_start, t_end);
 	}
 
+	fit->history = g_slist_append(fit->history, strdup("Left-right mirror"));
 #ifdef HAVE_WCSLIB
-	if (fit->wcslib) {
-		flip_left_right_astrometry_data(fit);
+	if (has_wcs(fit)) {
+		Homography H = { 0 };
+		cvGetEye(&H);
+		H.h00 = -1.;
+		H.h02 = (double)fit->rx - 1.;
+		reframe_astrometry_data(fit, H);
 		load_WCS_from_memory(fit);
 	}
 #endif
@@ -393,25 +560,17 @@ void mirrory(fits *fit, gboolean verbose) {
 static int crop_ushort(fits *fit, rectangle *bounds) {
 	int i, j, layer;
 	int newnbdata;
-	struct timeval t_start = { 0 }, t_end = { 0 };
 	rectangle bounds_cpy = { 0 };
 
-	if (bounds->x >= fit->rx) return -1;
-	if (bounds->y >= fit->ry) return -1;
 	bounds_cpy.x = bounds->x;
 	bounds_cpy.y = bounds->y;
 	bounds_cpy.w = (bounds->x + bounds->w > fit->rx) ? fit->rx - bounds->x : bounds->w;
 	bounds_cpy.h = (bounds->y + bounds->h > fit->ry) ? fit->ry - bounds->y : bounds->h;
 
-	if (fit == &gfit) {
-		siril_log_color_message(_("Crop: processing...\n"), "red");
-		gettimeofday(&t_start, NULL);
-	}
-
 	newnbdata = bounds_cpy.w * bounds_cpy.h;
 	for (layer = 0; layer < fit->naxes[2]; ++layer) {
 		WORD *from = fit->pdata[layer]
-				+ (fit->ry - bounds_cpy.y - bounds_cpy.h) * fit->rx + bounds_cpy.x;
+			+ (fit->ry - bounds_cpy.y - bounds_cpy.h) * fit->rx + bounds_cpy.x;
 		fit->pdata[layer] = fit->data + layer * newnbdata;
 		WORD *to = fit->pdata[layer];
 		int stridefrom = fit->rx - bounds_cpy.w;
@@ -425,38 +584,23 @@ static int crop_ushort(fits *fit, rectangle *bounds) {
 	}
 	fit->rx = fit->naxes[0] = bounds_cpy.w;
 	fit->ry = fit->naxes[1] = bounds_cpy.h;
-
-	if (fit == &gfit) {
-		clear_stars_list();
-		gettimeofday(&t_end, NULL);
-		show_time(t_start, t_end);
-	}
-	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
 static int crop_float(fits *fit, rectangle *bounds) {
 	int i, j, layer;
 	int newnbdata;
-	struct timeval t_start = { 0 }, t_end = { 0 };
 	rectangle bounds_cpy = { 0 };
 
-	if (bounds->x >= fit->rx) return -1;
-	if (bounds->y >= fit->ry) return -1;
 	bounds_cpy.x = bounds->x;
 	bounds_cpy.y = bounds->y;
 	bounds_cpy.w = (bounds->x + bounds->w > fit->rx) ? fit->rx - bounds->x : bounds->w;
 	bounds_cpy.h = (bounds->y + bounds->h > fit->ry) ? fit->ry - bounds->y : bounds->h;
 
-	if (fit == &gfit) {
-		siril_log_color_message(_("Crop: processing...\n"), "green");
-		gettimeofday(&t_start, NULL);
-	}
-
 	newnbdata = bounds_cpy.w * bounds_cpy.h;
 	for (layer = 0; layer < fit->naxes[2]; ++layer) {
 		float *from = fit->fpdata[layer]
-				+ (fit->ry - bounds_cpy.y - bounds_cpy.h) * fit->rx + bounds_cpy.x;
+			+ (fit->ry - bounds_cpy.y - bounds_cpy.h) * fit->rx + bounds_cpy.x;
 		fit->fpdata[layer] = fit->fdata + layer * newnbdata;
 		float *to = fit->fpdata[layer];
 		int stridefrom = fit->rx - bounds_cpy.w;
@@ -470,20 +614,22 @@ static int crop_float(fits *fit, rectangle *bounds) {
 	}
 	fit->rx = fit->naxes[0] = bounds_cpy.w;
 	fit->ry = fit->naxes[1] = bounds_cpy.h;
-
-	if (fit == &gfit) {
-		clear_stars_list();
-		gettimeofday(&t_end, NULL);
-		show_time(t_start, t_end);
-	}
-	invalidate_stats_from_fit(fit);
 	return 0;
 }
 
 int crop(fits *fit, rectangle *bounds) {
-	point shift; //need to be computed before fit rx/ry are altered by crop
-	shift.x = (double)(bounds->x);
-	shift.y = fit->ry - (double)(bounds->h) - (double)(bounds->y) - 1; // for top-bottom flip
+	if (bounds->w <= 0 || bounds->h <= 0 || bounds->x < 0 || bounds->y < 0) return -1;
+	if (bounds->x + bounds->w > fit->rx) return -1;
+	if (bounds->y + bounds->h > fit->ry) return -1;
+#ifdef HAVE_WCSLIB
+	int orig_ry = fit->ry; // required to compute flips afterwards
+	int target_rx, target_ry;
+	Homography H = { 0 };
+	gboolean wcs = has_wcs(fit);
+	if (wcs)
+		GetMatrixReframe(fit, *bounds, 0., 1, &target_rx, &target_ry, &H);
+#endif
+
 	if (fit->type == DATA_USHORT) {
 		if (crop_ushort(fit, bounds)) {
 			return -1;
@@ -495,139 +641,18 @@ int crop(fits *fit, rectangle *bounds) {
 	} else {
 		return -1;
 	}
+
+	invalidate_stats_from_fit(fit);
 #ifdef HAVE_WCSLIB
-	if (fit->wcslib) {
-		crop_astrometry_data(fit, shift);
+	if (wcs) {
+		cvApplyFlips(&H, orig_ry, target_ry);
+		reframe_astrometry_data(fit, H);
 		load_WCS_from_memory(fit);
 	}
 #endif
 	return 0;
 }
 
-/************************* CALLBACKS *************************************/
-
-/**
- *  ROTATION
- */
-void siril_rotate90() {
-		set_cursor_waiting(TRUE);
-		undo_save_state(&gfit, _("Rotation (90.0deg)"));
-		verbose_rotate_image(&gfit, 90.0, -1, 0);	// fast rotation, no interpolation, no crop
-		update_zoom_label();
-		redraw(REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
-}
-
-void siril_rotate270() {
-		set_cursor_waiting(TRUE);
-		undo_save_state(&gfit, _("Rotation (-90.0deg)"));
-		verbose_rotate_image(&gfit, 270.0, -1, 0);// fast rotation, no interpolation, no crop
-		update_zoom_label();
-		redraw(REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
-}
-
-void on_button_rotation_close_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("rotation_dialog");
-}
-
-void on_button_rotation_ok_clicked(GtkButton *button, gpointer user_data) {
-	rotate_gui(&gfit);
-}
-
-/*************
- * RESAMPLE
- */
-
-/* Resample */
-
-void on_button_resample_ok_clicked(GtkButton *button, gpointer user_data) {
-	if (confirm_delete_wcs_keywords(&gfit)) {
-		double sample[2];
-		sample[0] = gtk_spin_button_get_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")));
-		sample[1] = gtk_spin_button_get_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")));
-		int interpolation = gtk_combo_box_get_active(
-				GTK_COMBO_BOX(lookup_widget("combo_interpolation")));
-
-		set_cursor_waiting(TRUE);
-		int toX = round_to_int((sample[0] / 100.0) * gfit.rx);
-		int toY = round_to_int((sample[1] / 100.0) * gfit.ry);
-		undo_save_state(&gfit, _("Resample (%g - %g)"), sample[0] / 100.0,
-				sample[1] / 100.0);
-		verbose_resize_gaussian(&gfit, toX, toY, interpolation);
-		
-		redraw(REMAP_ALL);
-		redraw_previews();
-		set_cursor_waiting(FALSE);
-	}
-}
-
-void on_button_resample_close_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("resample_dialog");
-}
-
-void on_spinbutton_resample_X_value_changed(GtkSpinButton *spinbutton,
-		gpointer user_data) {
-	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(
-			lookup_widget("button_sample_ratio"));
-	double xvalue = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")));
-
-	if (gtk_toggle_button_get_active(ratio))
-		gtk_spin_button_set_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")),
-				xvalue);
-}
-
-void on_spinbutton_resample_Y_value_changed(GtkSpinButton *spinbutton,
-		gpointer user_data) {
-	GtkToggleButton *ratio = GTK_TOGGLE_BUTTON(
-			lookup_widget("button_sample_ratio"));
-	double yvalue = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")));
-
-	if (gtk_toggle_button_get_active(ratio))
-		gtk_spin_button_set_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")),
-				yvalue);
-}
-
-void on_button_sample_ratio_toggled(GtkToggleButton *button, gpointer user_data) {
-	double xvalue = gtk_spin_button_get_value(
-			GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_X")));
-
-	if (gtk_toggle_button_get_active(button))
-		gtk_spin_button_set_value(
-				GTK_SPIN_BUTTON(lookup_widget("spinbutton_resample_Y")),
-				xvalue);
-}
-
-/**************
- * CROP
- */
-void siril_crop() {
-	undo_save_state(&gfit, _("Crop (x=%d, y=%d, w=%d, h=%d)"),
-			com.selection.x, com.selection.y, com.selection.w,
-			com.selection.h);
-	if (is_preview_active()) {
-		siril_message_dialog(GTK_MESSAGE_INFO, _("A live preview session is active"),
-				_("It is impossible to crop the image when a filter with preview session is active. "
-						"Please consider to close the filter dialog first."));
-		return;
-	}
-	crop(&gfit, &com.selection);
-	delete_selected_area();
-	reset_display_offset();
-	update_zoom_label();
-	adjust_cutoff_from_updated_gfit();
-	redraw(REMAP_ALL);
-	redraw_previews();
-}
-
 gint64 crop_compute_size_hook(struct generic_seq_args *args, int nb_frames) {
 	struct crop_sequence_data *c_args = (struct crop_sequence_data*) args->user;
 	double ratio = (double)(c_args->area.h * c_args->area.w) / (double)(args->seq->rx * args->seq->ry);
@@ -636,12 +661,19 @@ gint64 crop_compute_size_hook(struct generic_seq_args *args, int nb_frames) {
 }
 
 int crop_image_hook(struct generic_seq_args *args, int o, int i, fits *fit,
-		rectangle *_) {
+		rectangle *_, int threads) {
 	struct crop_sequence_data *c_args = (struct crop_sequence_data*) args->user;
 
 	return crop(fit, &(c_args->area));
 }
 
+int crop_finalize_hook(struct generic_seq_args *args) {
+	struct crop_sequence_data *data = (struct crop_sequence_data *) args->user;
+	int retval = seq_finalize_hook(args);
+	free(data);
+	return retval;
+}
+
 /* TODO: should we use the partial image? */
 gpointer crop_sequence(struct crop_sequence_data *crop_sequence_data) {
 	struct generic_seq_args *args = create_default_seqargs(crop_sequence_data->seq);
@@ -649,7 +681,7 @@ gpointer crop_sequence(struct crop_sequence_data *crop_sequence_data) {
 	args->nb_filtered_images = crop_sequence_data->seq->selnum;
 	args->compute_size_hook = crop_compute_size_hook;
 	args->prepare_hook = seq_prepare_hook;
-	args->finalize_hook = seq_finalize_hook;
+	args->finalize_hook = crop_finalize_hook;
 	args->image_hook = crop_image_hook;
 	args->stop_on_error = FALSE;
 	args->description = _("Crop Sequence");
@@ -659,42 +691,8 @@ gpointer crop_sequence(struct crop_sequence_data *crop_sequence_data) {
 	args->load_new_sequence = TRUE;
 	args->user = crop_sequence_data;
 
-	remove_prefixed_sequence_files(crop_sequence_data->seq, crop_sequence_data->prefix);
 	start_in_new_thread(generic_sequence_worker, args);
 
 	return 0;
 }
 
-/*** GUI for crop sequence */
-void on_crop_Apply_clicked(GtkButton *button, gpointer user_data) {
-	if (get_thread_run()) {
-		PRINT_ANOTHER_THREAD_RUNNING;
-		return;
-	}
-
-#ifdef HAVE_FFMS2
-	if (com.seq.type == SEQ_AVI) {
-		siril_log_message(_("Crop does not work with "
-				"avi film. Please, convert your file to SER first.\n"));
-		return;
-	}
-#endif
-	if (com.seq.type == SEQ_INTERNAL) {
-		siril_log_message(_("Not a valid sequence for cropping.\n"));
-	}
-
-	struct crop_sequence_data *args = malloc(sizeof(struct crop_sequence_data));
-
-	GtkEntry *cropped_entry = GTK_ENTRY(lookup_widget("cropped_entry"));
-
-	args->seq = &com.seq;
-	memcpy(&args->area, &com.selection, sizeof(rectangle));
-	args->prefix = gtk_entry_get_text(cropped_entry);
-
-	set_cursor_waiting(TRUE);
-	crop_sequence(args);
-}
-
-void on_crop_close_clicked(GtkButton *button, gpointer user_data) {
-	siril_close_dialog("crop_dialog");
-}
