@@ -1,21 +1,21 @@
 /* @(#)io_wave.c	19.1 (ES0-DMD) 02/25/03 13:34:39 */
 /*===========================================================================
  Copyright (C) 1995 European Southern Observatory (ESO)
- 
+
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License as
  published by the Free Software Foundation; either version 2 of
  the License, or (at your option) any later version.
- 
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
- 
+
  You should have received a copy of the GNU General Public
  License along with this program;
  If not, see <http://www.gnu.org/licenses/>.
- 
+
  Corresponding concerning ESO-MIDAS should be addressed as follows:
  Internet e-mail: midas@eso.org
  Postal address: European Southern Observatory
@@ -83,7 +83,7 @@
  **
  ** Nl, Nc = original image size
  ** Nbr_Plan = number of scales = number of planes
- ** Type_Wave_Transform = choosen wavelet transform algorithm
+ ** Type_Wave_Transform = chosen wavelet transform algorithm
  **
  *******************************************************************************
  **
@@ -97,7 +97,7 @@
  **
  ** Nl, Nc = original image size
  ** Nbr_Plan = number of scales = number of planes
- ** Type_Wave_Transform = choosen wavelet transform algorithm
+ ** Type_Wave_Transform = chosen wavelet transform algorithm
  **
  ******************************************************************************/
 
@@ -134,7 +134,7 @@ int wave_io_size_data(int Nl, int Nc, int Nbr_Plan, int Type_Wave_Transform) {
 int static wave_io_name(char *File_Name_In, char *File_Name_Out) {
 	int L;
 
-	strcpy(File_Name_Out, File_Name_In);
+	strncpy(File_Name_Out, File_Name_In, STRING_SIZE - 1);
 
 	L = strlen(File_Name_In);
 	if ((L < 5) || (File_Name_In[L - 1] != 'e') || (File_Name_In[L - 2] != 'v')
@@ -151,7 +151,7 @@ int wave_io_read(char *File_Name_In, wave_transf_des *Wave_Trans) {
 	FILE *File_Des;
 	int Nbr, Nl, Nc, Nbr_Plan, Size;
 	char *Ptr;
-	string File_Name;
+	string File_Name = { 0 };
 
 	/* find the exact file name which must finish by .wave */
 	wave_io_name(File_Name_In, File_Name);
@@ -176,6 +176,11 @@ int wave_io_read(char *File_Name_In, wave_transf_des *Wave_Trans) {
 	Nc = Wave_Trans->Nbr_Col;
 	Nbr_Plan = Wave_Trans->Nbr_Plan;
 
+	if (Nl < 1 || Nc < 1 || Nl > MAX_IMAGE_DIM || Nc > MAX_IMAGE_DIM) {
+		printf("wave_io_read: file metadata fails validity checking\n");
+		fclose(File_Des);
+		return 1;
+	}
 	switch (Wave_Trans->Type_Wave_Transform) {
 	case TO_PAVE_LINEAR:
 	case TO_PAVE_BSPLINE:
@@ -207,7 +212,7 @@ int wave_io_write(char *File_Name_In, wave_transf_des *Wave_Trans) {
 	FILE *File_Des;
 	int Nbr, Nl, Nc, Nbr_Plan, Size;
 	char *Ptr;
-	string File_Name;
+	string File_Name = { 0 };
 
 	/* find the exact file name */
 	wave_io_name(File_Name_In, File_Name);
