@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "algos/quality.h"
@@ -35,7 +36,7 @@
 static double QualityEstimate_ushort(fits *fit, int layer);
 static int32_t SubSample(WORD *ptr, int img_wid, int x_size, int y_size);
 static void _smooth_image_16(unsigned short *buf, int width, int height);
-static double Gradient(WORD *buf, int width, int height);
+static double Gradient(const WORD *buf, int width, int height);
 
 double QualityEstimate(fits *fit, int layer) {
 	if (fit->type == DATA_USHORT)
@@ -173,7 +174,7 @@ static double QualityEstimate_ushort(fits *fit, int layer) {
 #endif
 		q = Gradient(buf, x_samples, y_samples);
 
-		dval += (q * ((QSUBSAMPLE_MIN * QSUBSAMPLE_MIN)
+		dval += (q * ((double)(QSUBSAMPLE_MIN * QSUBSAMPLE_MIN)
 					/ (subsample * subsample)));
 		//printf("dval val : %f\n", dval);
 
@@ -209,7 +210,7 @@ static int32_t SubSample(WORD *ptr, int img_wid, int x_size, int y_size) {
 	return round_to_WORD((double)val / (double)(x_size * y_size));
 }
 
-static double Gradient(WORD *buf, int width, int height) {
+static double Gradient(const WORD *buf, int width, int height) {
 	int pixels;
 	int x, y;
 	int yborder = (int) ((double) height * QMARGIN) + 1;
