@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -51,12 +51,13 @@ gboolean has_wcs(fits *fit) {
 
 // deal with cases where wcsdata is not NULL but members are set to 0
 gboolean has_wcsdata(fits *fit) {
-	if ((fit->wcsdata.crval[0] == 0.0 && fit->wcsdata.crval[1] == 0.0)) return FALSE;
-		return TRUE;
+	if ((fit->wcsdata.crval[0] == 0.0 && fit->wcsdata.crval[1] == 0.0))
+		return FALSE;
+	return TRUE;
 }
 
 
-void free_wcs(fits *fit) {
+void free_wcs(fits *fit, gboolean keep_RADEC) {
 #ifdef HAVE_WCSLIB
 	if (fit->wcslib) {
 		if (!wcsfree(fit->wcslib))
@@ -64,7 +65,16 @@ void free_wcs(fits *fit) {
 		fit->wcslib = NULL;
 	}
 #endif
-	memset(&fit->wcsdata, 0, sizeof(fit->wcsdata));
+	if (keep_RADEC) {
+		memset(&fit->wcsdata.cdelt, 0, sizeof(fit->wcsdata.cdelt));
+		memset(&fit->wcsdata.crpix, 0, sizeof(fit->wcsdata.crpix));
+		memset(&fit->wcsdata.crval, 0, sizeof(fit->wcsdata.crval));
+		memset(&fit->wcsdata.pc, 0, sizeof(fit->wcsdata.pc));
+		memset(&fit->wcsdata.pltsolvd, 0, sizeof(fit->wcsdata.pltsolvd));
+		memset(&fit->wcsdata.pltsolvd_comment, 0, sizeof(fit->wcsdata.pltsolvd_comment));
+	} else {
+		memset(&fit->wcsdata, 0, sizeof(fit->wcsdata));
+	}
 }
 
 gboolean load_WCS_from_memory(fits *fit) {
@@ -72,6 +82,10 @@ gboolean load_WCS_from_memory(fits *fit) {
 	int status;
 	if (!fit->wcslib) {
 		fit->wcslib = calloc(1, sizeof(struct wcsprm));
+		if(!fit->wcslib) {
+			PRINT_ALLOC_ERR;
+			return FALSE;
+		}
 		fit->wcslib->flag = -1;
 	}
 	wcsinit(1, NAXIS, fit->wcslib, 0, 0, 0);
@@ -80,7 +94,7 @@ gboolean load_WCS_from_memory(fits *fit) {
 	const char CUNIT[2][9] = { "deg", "deg" };
 
 	for (int i = 0; i < NAXIS; i++) {
-		strcpy(fit->wcslib->cunit[i], &CUNIT[i][0]);
+		strncpy(fit->wcslib->cunit[i], &CUNIT[i][0], 71); // 72 char fixed buffer, keep 1 for the NULL
 	}
 
 	double *pcij = fit->wcslib->pc;
@@ -103,7 +117,7 @@ gboolean load_WCS_from_memory(fits *fit) {
 	}
 
 	for (int i = 0; i < NAXIS; i++) {
-		strcpy(fit->wcslib->ctype[i], &CTYPE[i][0]);
+		strncpy(fit->wcslib->ctype[i], &CTYPE[i][0], 71); // 72 byte buffer, leave 1 byte for the NULL
 	}
 
 	fit->wcslib->equinox = fit->wcsdata.equinox;
@@ -111,15 +125,13 @@ gboolean load_WCS_from_memory(fits *fit) {
 	fit->wcslib->latpole = fit->wcsdata.crval[1];
 
 	if ((status = wcsset(fit->wcslib)) != 0) {
-	/* here we do not want to use free_wcs because
-	 * we want to keep original header, just in case */
-#ifdef HAVE_WCSLIB
-	if (fit->wcslib) {
-		if (!wcsfree(fit->wcslib))
-			free(fit->wcslib);
-		fit->wcslib = NULL;
-	}
-#endif
+		/* here we do not want to use free_wcs because
+		 * we want to keep original header, just in case */
+		if (fit->wcslib) {
+			if (!wcsfree(fit->wcslib))
+				free(fit->wcslib);
+			fit->wcslib = NULL;
+		}
 		siril_debug_print("wcsset error %d: %s.\n", status, wcs_errmsg[status]);
 		return FALSE;
 	}
@@ -149,7 +161,7 @@ gboolean load_WCS_from_file(fits* fit) {
 	}
 
 	if (fit->wcslib) {
-		free_wcs(fit);
+		free_wcs(fit, FALSE);
 	}
 
 	ffhdr2str(fit->fptr, 1, NULL, 0, &header, &nkeyrec, &status);
@@ -209,41 +221,61 @@ gboolean load_WCS_from_file(fits* fit) {
 #endif
 }
 
-void pix2wcs(fits *fit, double x, double y, double *r, double *d) {
-	*r = -1.0;
-	*d = -1.0;
 #ifdef HAVE_WCSLIB
+void pix2wcs2(struct wcsprm *wcslib, double x, double y, double *r, double *d) {
+	*r = 0.0;
+	*d = 0.0;
 	int status, stat[NWCSFIX];
 	double imgcrd[NWCSFIX], phi, pixcrd[NWCSFIX], theta, world[NWCSFIX];
 
 	pixcrd[0] = x;
 	pixcrd[1] = y;
 
-	status = wcsp2s(fit->wcslib, 1, 2, pixcrd, imgcrd, &phi, &theta, world, stat);
+	status = wcsp2s(wcslib, 1, 2, pixcrd, imgcrd, &phi, &theta, world, stat);
 	if (status != 0)
 		return;
 
 	*r = world[0];
 	*d = world[1];
+}
+#endif
+
+void pix2wcs(fits *fit, double x, double y, double *r, double *d) {
+	*r = 0.0;
+	*d = 0.0;
+#ifdef HAVE_WCSLIB
+	if (fit->wcslib)
+		pix2wcs2(fit->wcslib, x, y, r, d);
 #endif
 }
 
-void wcs2pix(fits *fit, double r, double d, double *x, double *y) {
-	*x = -1.0;
-	*y = -1.0;
-#ifdef HAVE_WCSLIB
+// ra in degrees
+int wcs2pix(fits *fit, double ra, double dec, double *x, double *y) {
+	if (x) *x = -1.0;
+	if (y) *y = -1.0;
+#ifndef HAVE_WCSLIB
+	return 1;
+#else
 	int status, stat[NWCSFIX];
 	double imgcrd[NWCSFIX], phi, pixcrd[NWCSFIX], theta, world[NWCSFIX];
-
-	world[0] = r;
-	world[1] = d;
+	world[0] = ra;
+	world[1] = dec;
 
 	status = wcss2p(fit->wcslib, 1, 2, world, &phi, &theta, imgcrd, pixcrd, stat);
-	if (status != 0)
-		return;
 
-	*x = pixcrd[0];
-	*y = pixcrd[1];
+	if (!status) {
+		double xx = pixcrd[0];
+		double yy = pixcrd[1];
+		// return values even if outside (required for celestial grid display)
+		if (x) *x = xx;
+		if (y) *y = yy;
+		if (xx < 0.0 || yy < 0.0 || xx > (double)fit->rx || yy > (double)fit->ry) {
+			//siril_debug_print("outside image but valid return\n");
+			// wcss2p returns values between 0 and 9, picking a new one
+			status = 10;
+		}
+	}
+	return status;
 #endif
 }
 
@@ -255,8 +287,8 @@ void center2wcs(fits *fit, double *r, double *d) {
 	int status, stat[NWCSFIX];
 	double imgcrd[NWCSFIX], phi, pixcrd[NWCSFIX], theta, world[NWCSFIX];
 
-	pixcrd[0] = (double)(fit->rx + 1) / 2.;
-	pixcrd[1] = (double)(fit->ry + 1) / 2.;
+	pixcrd[0] = (double)(fit->rx) / 2.;
+	pixcrd[1] = (double)(fit->ry) / 2.;
 
 	status = wcsp2s(fit->wcslib, 1, 2, pixcrd, imgcrd, &phi, &theta, world, stat);
 	if (status != 0)
@@ -267,7 +299,7 @@ void center2wcs(fits *fit, double *r, double *d) {
 #endif
 }
 
-/* get resolution in arcsec/pixel */
+/* get resolution in degree/pixel */
 double get_wcs_image_resolution(fits *fit) {
 	double resolution = -1.0;
 #ifdef HAVE_WCSLIB
@@ -294,8 +326,9 @@ double get_wcs_image_resolution(fits *fit) {
 	}
 #endif
 	if (resolution <= 0.0) {
-		if (gfit.focal_length >= 0.0 && gfit.pixel_size_x >= 0.0 && gfit.pixel_size_y == gfit.pixel_size_x)
-			resolution = (RADCONV / gfit.focal_length * gfit.pixel_size_x) / 3600;
+		if (fit->focal_length >= 0.0 && fit->pixel_size_x >= 0.0 && fit->pixel_size_y == fit->pixel_size_x)
+			resolution = (RADCONV / fit->focal_length * fit->pixel_size_x) / 3600.0;
+		// what about pix size x != y?
 	}
 	return resolution;
 }
