@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,21 +26,77 @@
 #include "core/proto.h"
 #include "core/initfile.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "gui/registration_preview.h"
 #include "gui/image_interactions.h"
 #include "gui/image_display.h"
 #include "gui/callbacks.h"
+#include "gui/preferences.h"
 #include "core/processing.h"
 #include "core/command_list.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
 #include "io/ser.h"
+#include "livestacking/livestacking.h"
 
+#include "command.h"
 #include "command_line_processor.h"
 
-static void parse_line(char *myline, int len, int *nb) {
+static const char *cmd_err_to_str(cmd_errors err) {
+	switch (err) {
+		case CMD_NOT_FOUND:
+			return _("unknown command name");
+		case CMD_NO_WAIT:
+			return _("non-blocking command, no error");
+		case CMD_NO_CWD:
+			return _("current working directory is invalid");
+		case CMD_NOT_SCRIPTABLE:
+			return _("command cannot be used in a script");
+		case CMD_WRONG_N_ARG:
+			return _("invalid number of arguments");
+		case CMD_ARG_ERROR:
+			return _("invalid arguments");
+		case CMD_SELECTION_ERROR:
+			return _("invalid image area selection");
+		case CMD_OK:
+			return _("command succeeded");
+		default:
+		case CMD_GENERIC_ERROR:
+			return _("generic error");
+		case CMD_IMAGE_NOT_FOUND:
+			return _("input image not found");
+		case CMD_SEQUENCE_NOT_FOUND:
+			return _("invalid input sequence");
+		case CMD_INVALID_IMAGE:
+			return _("invalid input image");
+		case CMD_LOAD_IMAGE_FIRST:
+			return _("load an image or sequence first");
+		case CMD_ONLY_SINGLE_IMAGE:
+			return _("command is only for a loaded single image");
+		case CMD_NOT_FOR_SINGLE:
+			return _("command is only for a sequence");
+		case CMD_NOT_FOR_MONO:
+			return _("command is not for monochrome images");
+		case CMD_NOT_FOR_RGB:
+			return _("command is not for color images");
+		case CMD_FOR_CFA_IMAGE:
+			return _("command is only for OSC CFA images");
+		case CMD_FILE_NOT_FOUND:
+			return _("file not found");
+		case CMD_FOR_PLATE_SOLVED:
+			return _("command is only for plate solved images");
+		case CMD_NEED_INIT_FIRST:
+			return _("command requires a preliminary step");
+		case CMD_ALLOC_ERROR:
+			return _("memory allocation error");
+		case CMD_THREAD_RUNNING:
+			return _("a processing is already running");
+	}
+}
+
+void parse_line(char *myline, int len, int *nb) {
 	int i = 0, wordnb = 0;
 	char string_starter = '\0';	// quotes don't split words on spaces
 	word[0] = NULL;
@@ -69,7 +125,7 @@ static void parse_line(char *myline, int len, int *nb) {
 	*nb = wordnb;
 }
 
-static void remove_trailing_cr(char *str) {
+void remove_trailing_cr(char *str) {
 	if (str == NULL || str[0] == '\0')
 		return;
 	int length = strlen(str);
@@ -77,28 +133,64 @@ static void remove_trailing_cr(char *str) {
 		str[length - 1] = '\0';
 }
 
-static int execute_command(int wordnb) {
+int execute_command(int wordnb) {
 	// search for the command in the list
 	if (word[0] == NULL) return 1;
 	int i = G_N_ELEMENTS(commands);
 	while (g_ascii_strcasecmp(commands[--i].name, word[0])) {
 		if (i == 0) {
 			siril_log_message(_("Unknown command: '%s' or not implemented yet\n"), word[0]);
-			return 1 ;
+			return CMD_NOT_FOUND;
 		}
 	}
 
 	// verify argument count
 	if (wordnb - 1 < commands[i].nbarg) {
 		siril_log_message(_("Usage: %s\n"), commands[i].usage);
-		return 1;
+		return CMD_WRONG_N_ARG;
 	}
 
 	// verify if command is scriptable
 	if (com.script) {
 		if (!commands[i].scriptable) {
 			siril_log_message(_("This command cannot be used in a script: %s\n"), commands[i].name);
-			return 1;
+			return CMD_NOT_SCRIPTABLE;
+		}
+	}
+
+	/* verify prerequires */
+	/* we check that (REQ_CMD_SINGLE_IMAGE | REQ_CMD_SEQUENCE) is inside the bitmask */
+	if ((commands[i].prerequires & (REQ_CMD_SINGLE_IMAGE | REQ_CMD_SEQUENCE)) == (REQ_CMD_SINGLE_IMAGE | REQ_CMD_SEQUENCE)) {
+		if (!single_image_is_loaded() && !sequence_is_loaded()) {
+			return CMD_LOAD_IMAGE_FIRST;
+		}
+	} else if ((commands[i].prerequires & REQ_CMD_SINGLE_IMAGE) == REQ_CMD_SINGLE_IMAGE) {
+		if (!single_image_is_loaded()) {
+			return CMD_ONLY_SINGLE_IMAGE;
+		}
+	} else if ((commands[i].prerequires & REQ_CMD_SEQUENCE) == REQ_CMD_SEQUENCE) {
+		if (!sequence_is_loaded()) {
+			return CMD_NOT_FOR_SINGLE;
+		}
+	}
+
+	if ((commands[i].prerequires & (REQ_CMD_FOR_MONO | REQ_CMD_FOR_CFA)) == (REQ_CMD_FOR_MONO | REQ_CMD_FOR_CFA)) {
+		if (isrgb(&gfit)) {
+			return CMD_FOR_CFA_IMAGE;
+		}
+	} else if ((commands[i].prerequires & REQ_CMD_FOR_MONO) != 0) {
+		if (isrgb(&gfit)) {
+			return CMD_NOT_FOR_RGB;
+		}
+	} else if ((commands[i].prerequires & REQ_CMD_FOR_RGB) != 0) {
+		if (!isrgb(&gfit)) {
+			return CMD_NOT_FOR_MONO;
+		}
+	}
+
+	if ((commands[i].prerequires & REQ_CMD_NO_THREAD) != 0) {
+		if (get_thread_run()) {
+			return CMD_THREAD_RUNNING;
 		}
 	}
 
@@ -171,16 +263,42 @@ static gboolean end_script(gpointer p) {
 	update_display_fwhm();
 	display_filename();
 	new_selection_zone();
-	set_GUI_misc();
-	set_GUI_compression();
 	update_spinCPU(0);
 	set_cursor_waiting(FALSE);
 	return FALSE;
 }
 
+int check_requires(gboolean *checked_requires, gboolean is_required) {
+	int retval = CMD_OK;
+	/* check for requires command */
+	if (!g_ascii_strcasecmp(word[0], "requires")) {
+		*checked_requires = TRUE;
+	} else if (is_required && !*checked_requires) {
+		siril_log_color_message(_("The \"requires\" command is missing at the top of the script file."
+					" This command is needed to check script compatibility.\n"), "red");
+		retval = CMD_GENERIC_ERROR;
+	}
+	return retval;
+}
+
+int check_command_mode() {
+	/* until we have a proper implementation of modes, we just forbid other
+	 * commands to be run during live stacking */
+	int retval = 0;
+	if (livestacking_is_started()) {
+		retval = g_ascii_strcasecmp(word[0], "livestack") &&
+			g_ascii_strcasecmp(word[0], "stop_ls") &&
+			g_ascii_strcasecmp(word[0], "exit");
+		if (retval)
+			siril_log_message(_("This command cannot be run while live stacking is active, ignoring.\n"));
+
+	}
+	return retval;
+}
+
 gpointer execute_script(gpointer p) {
 	GInputStream *input_stream = (GInputStream*) p;
-	gboolean check_required = FALSE;
+	gboolean checked_requires = FALSE;
 	gchar *buffer;
 	int line = 0, retval = 0;
 	int wordnb;
@@ -217,34 +335,33 @@ gpointer execute_script(gpointer p) {
 
 		/* in Windows case, remove trailing CR */
 		remove_trailing_cr(buffer);
+		g_strstrip(buffer);
 
 		if (buffer[0] == '\0') {
 			g_free (buffer);
 			continue;
 		}
 
-
 		display_command_on_status_bar(line, buffer);
 		parse_line(buffer, length, &wordnb);
-		/* check for requires command */
-		if (!g_ascii_strcasecmp(word[0], "requires")) {
-			check_required = TRUE;
-		} else {
-			if (com.pref.script_check_requires && !check_required) {
-				siril_log_color_message(_("The \"requires\" command is missing at the top of the script file."
-						" This command is needed to check script compatibility.\n"), "red");
-				retval = 1;
-				g_free (buffer);
-				break;
-			}
+		if (check_requires(&checked_requires, com.pref.script_check_requires)) {
+			g_free (buffer);
+			break;
 		}
-		if ((retval = execute_command(wordnb))) {
-			siril_log_message(_("Error in line %d: '%s'.\n"), line, buffer);
+		if (check_command_mode()) {
+			g_free (buffer);
+			continue;
+		};
+
+		retval = execute_command(wordnb);
+
+		if (retval && retval != CMD_NO_WAIT) {
+			siril_log_message(_("Error in line %d ('%s'): %s.\n"), line, buffer, cmd_err_to_str(retval));
 			siril_log_message(_("Exiting batch processing.\n"));
 			g_free (buffer);
 			break;
 		}
-		if (waiting_for_thread()) {
+		if (retval != CMD_NO_WAIT && waiting_for_thread()) {
 			retval = 1;
 			g_free (buffer);
 			break;	// abort script on command failure
@@ -297,13 +414,23 @@ static void show_command_help_popup(GtkEntry *entry) {
 			if (!g_ascii_strcasecmp(current->name, command_line[0])) {
 				gchar **token;
 
-				token = g_strsplit_set(current->usage, " ", -1);
+				token = g_strsplit_set(current->usage, " \n", -1);
 				GString *str = g_string_new(token[0]);
 				str = g_string_prepend(str, "<span foreground=\"red\" size=\"larger\"><b>");
 				str = g_string_append(str, "</b>");
 				if (token[1] != NULL) {
-					str = g_string_append(str,
-							current->usage + strlen(token[0]));
+					int i = 1;
+					while (token[i]) {
+						str = g_string_append(str, " ");
+						if (!g_ascii_strcasecmp(current->name, token[i])) {
+							str = g_string_append(str, "\n<b>");
+						}
+						str = g_string_append(str, token[i]);
+						if (!g_ascii_strcasecmp(current->name, token[i])) {
+							str = g_string_append(str, "</b>");
+						}
+						i++;
+					}
 				}
 				str = g_string_append(str, "</span>\n\n\t");
 				str = g_string_append(str, _(current->definition));
@@ -446,9 +573,7 @@ int processcommand(const char *line) {
 			g_object_unref(file);
 			return 1;
 		}
-		/* ensure that everything is closed */
-		process_close(0);
-		/* Then, run script */
+		/* Run the script */
 		siril_log_message(_("Starting script %s\n"), filename);
 		com.script_thread = g_thread_new("script", execute_script, input_stream);
 		g_object_unref(file);
@@ -458,10 +583,13 @@ int processcommand(const char *line) {
 
 		gchar *myline = strdup(line);
 		int len = strlen(line);
+		if (len > 0)
+			g_print("input command:%s\n", myline);
 		parse_line(myline, len, &wordnb);
-		if (execute_command(wordnb)) {
-			siril_log_color_message(_("Command execution failed.\n"), "red");
-			if (!com.script && !com.headless) {
+		int ret = execute_command(wordnb);
+		if (ret) {
+			siril_log_color_message(_("Command execution failed: %s.\n"), "red", cmd_err_to_str(ret));
+			if (!com.script && !com.headless && (ret == CMD_WRONG_N_ARG || ret == CMD_ARG_ERROR)) {
 				show_command_help_popup(GTK_ENTRY(lookup_widget("command")));
 			}
 			free(myline);
@@ -474,16 +602,20 @@ int processcommand(const char *line) {
 
 // loads the sequence from com.wd
 sequence *load_sequence(const char *name, char **get_filename) {
-	gchar *file = g_strdup(name);
+	gchar *file = NULL;
 	gchar *altfile = NULL;
-	if (!g_str_has_suffix(name, ".seq")) {
-		str_append(&file, ".seq");
-		if (!g_str_has_suffix(name, "_"))
-			altfile = g_strdup_printf("%s_.seq", name);
+	if (name[0] == '.' && g_utf8_strlen(name, -1) == 1 && sequence_is_loaded())
+		file = g_strdup(com.seq.seqname);
+	else {
+		file = g_strdup(name);
+		if (!g_str_has_suffix(name, ".seq")) {
+			str_append(&file, ".seq");
+			if (!g_str_has_suffix(name, "_"))
+				altfile = g_strdup_printf("%s_.seq", name);
+		}
 	}
-
 	if (!is_readable_file(file) && (!altfile || !is_readable_file(altfile))) {
-		if (check_seq(FALSE)) {
+		if (check_seq()) {
 			siril_log_color_message(_("No sequence `%s' found.\n"), "red", name);
 			g_free(file);
 			g_free(altfile);
@@ -491,7 +623,7 @@ sequence *load_sequence(const char *name, char **get_filename) {
 		}
 	}
 
-	sequence *seq;
+	sequence *seq = NULL;
 	if ((seq = readseqfile(file))) {
 		if (get_filename) {
 			*get_filename = file;
@@ -531,17 +663,11 @@ static gboolean on_match_selected(GtkEntryCompletion *widget, GtkTreeModel *mode
 	gint cur_pos = gtk_editable_get_position(e);
 	gint p = cur_pos;
 	gchar *end;
-	gint del_end_pos = -1;
 
 	gtk_tree_model_get(model, iter, COMPLETION_COLUMN, &cmd, -1);
 
 	end = s + cur_pos;
-
-	if (end) {
-		del_end_pos = end - s + 1;
-	} else {
-		del_end_pos = cur_pos;
-	}
+	gint del_end_pos = end - s + 1;
 
 	gtk_editable_delete_text(e, 0, del_end_pos);
 	gtk_editable_insert_text(e, cmd, -1, &p);
@@ -661,3 +787,16 @@ void on_command_activate(GtkEntry *entry, gpointer user_data) {
 		set_precision_switch();
 	}
 }
+
+void log_several_lines(char *text) {
+	char *line = text;
+	do {
+		char *eol = strchr(line, '\n');
+		if (eol)
+			*eol = '\0';
+		siril_log_message("%s\n", line);
+		if (!eol)
+			break;
+		line = eol+1;
+	} while (line[0] != '\0');
+}
