@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -21,21 +21,26 @@
 #ifndef SRC_CORE_INITFILE_H_
 #define SRC_CORE_INITFILE_H_
 
+#include <glib.h>
+
 enum token_index {
 	WD = 0,		/* Working Directory */
-	RAW = 1,	/* Raw settings */
-	BAY = 2,	/* Bayer settings */
-	PRE = 3,	/* Preprocessing settings */
-	REG = 4,	/* Registration settings */
-	STK = 5,	/* Stacking settings */
-	AST = 6,	/* Astrometry settings */
-	PTM = 7,	/* Photometry settings */
-	MISC = 8,	/* Miscellaneous settings */
-	CMP = 9,	/* Compression settings */
+	BAY = 1,	/* Bayer settings */
+	PRE = 2,	/* Preprocessing settings */
+	REG = 3,	/* Registration settings */
+	STK = 4,	/* Stacking settings */
+	AST = 5,	/* Astrometry settings */
+	PTM = 6,	/* Photometry settings */
+	MISC = 7,	/* Miscellaneous settings */
+	CMP = 8,	/* Compression settings */
 	NOTOK
 };
 
-int	writeinitfile();
-int	checkinitfile();
+int writeinitfile();
+int checkinitfile();
+
+int readinitfile(char *path);	// to import some settings
+
+int read_keyfile(GKeyFile *kf);
 
 #endif /* SRC_CORE_INITFILE_H_ */
