@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 #include "core/arithm.h"
 #include "core/processing.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/statistics.h"
 #include "algos/fix_xtrans_af.h"
 #include "filters/cosmetic_correction.h"
@@ -35,8 +36,9 @@
 #include "gui/progress_and_log.h"
 #include "io/single_image.h"
 #include "io/sequence.h"
-#include "io/conversion.h"
+#include "algos/demosaicing.h"
 #include "io/image_format_fits.h"
+#include "io/path_parse.h"
 #include "io/ser.h"
 
 #include "preprocess.h"
@@ -71,7 +73,7 @@ static float evaluateNoiseOfCalibratedImage(fits *fit, fits *dark,
 
 	for (chan = 0; chan < fit->naxes[2]; chan++) {
 		/* STATS_SIGMEAN computes mean and normvalue */
-		imstats *stat = statistics(NULL, -1, &fit_tmp, chan, &area, STATS_SIGMEAN, FALSE);
+		imstats *stat = statistics(NULL, -1, &fit_tmp, chan, &area, STATS_SIGMEAN, SINGLE_THREADED);
 		if (!stat) {
 			siril_log_message(_("Error: statistics computation failed.\n"));
 			return -1.0;
@@ -125,14 +127,18 @@ static int preprocess(fits *raw, struct preprocessing_data *args) {
 	int ret = 0;
 
 	if (args->use_bias) {
-		if (args->bias_level < FLT_MAX) { // an offset level has been defined
+		if (args->bias_level < FLT_MAX) {
+			// an offset level has been defined
 			ret = soper(raw, args->bias_level, OPER_SUB, args->allow_32bit_output);
-		} else ret = imoper(raw, args->bias, OPER_SUB, args->allow_32bit_output);
+		}
+		else ret = imoper(raw, args->bias, OPER_SUB, args->allow_32bit_output);
 	}
 
 	/* if dark optimization, the master-dark has already been subtracted */
 	if (!ret && args->use_dark && !args->use_dark_optim) {
 		ret = imoper(raw, args->dark, OPER_SUB, args->allow_32bit_output);
+		if (raw->neg_ratio > 0.2f)
+			siril_log_message(_("After dark subtraction, the image contains many negative pixels (%d%%), calibration frames are probably incorrect\n"), (int)(100.f*raw->neg_ratio));
 #ifdef SIRIL_OUTPUT_DEBUG
 		image_find_minmax(raw);
 		fprintf(stdout, "after dark: min=%f, max=%f\n", raw->mini, raw->maxi);
@@ -154,7 +160,7 @@ static int preprocess(fits *raw, struct preprocessing_data *args) {
 	return ret;
 }
 
-static int darkOptimization(fits *raw, struct preprocessing_data *args) {
+static int darkOptimization(fits *raw, struct preprocessing_data *args, int in_index, GSList *hist) {
 	float k0;
 	float lo = 0.f, up = 2.f;
 	int ret = 0;
@@ -167,19 +173,26 @@ static int darkOptimization(fits *raw, struct preprocessing_data *args) {
 	}
 
 	// TODO: avoid duplicating with soper+imoper together
+	// https://gitlab.com/free-astro/siril/-/issues/671
 	if (copyfits(dark, &dark_tmp, CP_ALLOC | CP_COPYA | CP_FORMAT, 0))
 		return 1;
 
 	/* Minimization of background noise to find better k */
 	k0 = goldenSectionSearch(raw, &dark_tmp, lo, up, 0.001f, args->allow_32bit_output);
 	if (k0 < 0.f) {
+		siril_log_message(_("Dark optimization of image %d failed\n"), in_index);
 		ret = -1;
 	} else {
-		siril_log_message(_("Dark optimization: k0=%.3f\n"), k0);
 		/* Multiply coefficient to master-dark */
 		ret = soper(&dark_tmp, k0, OPER_MUL, args->allow_32bit_output);
 		if (!ret)
 			ret = imoper(raw, &dark_tmp, OPER_SUB, args->allow_32bit_output);
+		if (!ret) {
+			siril_log_message(_("Dark optimization of image %d: k0=%.3f\n"), in_index, k0);
+			if (hist)
+				hist = g_slist_append(hist, g_strdup_printf("Calibrated with an optimized master dark (factor: %.3f)", k0));
+		}
+		else siril_log_message(_("Dark optimization of image %d failed\n"), in_index);
 	}
 	clearfits(&dark_tmp);
 	return ret;
@@ -202,9 +215,9 @@ static int prepro_compute_mem_hook(struct generic_seq_args *args, gboolean for_w
 	if (prepro->use_dark && prepro->dark) nb_masters++;
 	if (prepro->use_bias && prepro->bias) nb_masters++;
 	int input_is_float = get_data_type(args->seq->bitpix) == DATA_FLOAT;
-	int output_is_float = input_is_float || !com.pref.force_to_16bit;
+	int output_is_float = input_is_float || !com.pref.force_16bit;
 	/*
-	 * are the masters float? yes if !com.pref.force_to_16bit, raw images too
+	 * are the masters float? yes if !com.pref.force_16bit, raw images too
 	 *
 	 * if (prepro->use_dark_optim && prepro->use_dark)
 	 *	image gets three copies
@@ -241,7 +254,7 @@ static int prepro_compute_mem_hook(struct generic_seq_args *args, gboolean for_w
                          * plus how many images can be stored in what remains
                          * unused by the main processing */
                         limit = thread_limit + (MB_avail - required * thread_limit) / MB_per_output_image;
-			siril_debug_print("%d MB avail for writer\n", MB_avail - required * thread_limit);
+			siril_debug_print("%u MB avail for writer\n", MB_avail - required * thread_limit);
                 } else limit = thread_limit;
 	}
 	if (limit == 0) {
@@ -272,8 +285,9 @@ static int prepro_compute_mem_hook(struct generic_seq_args *args, gboolean for_w
 	return limit;
 }
 
-static int prepro_prepare_hook(struct generic_seq_args *args) {
+int prepro_prepare_hook(struct generic_seq_args *args) {
 	struct preprocessing_data *prepro = args->user;
+	gboolean set_hist = prepro->output_seqtype != SEQ_SER;
 
 	if (prepro->seq) {
 		// handling SER and FITSEQ
@@ -281,6 +295,19 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 			return 1;
 	}
 
+	if (set_hist) {
+		if (prepro->use_dark && !prepro->use_dark_optim)
+			prepro->history = g_slist_prepend(prepro->history, g_strdup("Calibrated with a master dark"));
+		if (prepro->use_bias) {
+			if (prepro->bias_level < FLT_MAX) {
+				int bitpix = prepro->seq ? prepro->seq->bitpix : gfit.orig_bitpix;
+				prepro->history = g_slist_prepend(prepro->history, g_strdup_printf("Calibrated with a synthetic bias of %d", roundf_to_int(prepro->bias_level * ((bitpix == BYTE_IMG) ? UCHAR_MAX_SINGLE : USHRT_MAX_SINGLE))));
+			} else {
+				prepro->history = g_slist_prepend(prepro->history, g_strdup("Calibrated with a master bias"));
+			}
+		}
+	}
+
 	// precompute flat levels
 	if (prepro->use_flat) {
 		if (prepro->equalize_cfa) {
@@ -298,7 +325,7 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 
 			rectangle selection = { startx, starty, startx, starty };
 
-			imstats *stat = statistics(NULL, -1, prepro->flat, RLAYER, &selection, STATS_BASIC, FALSE);
+			imstats *stat = statistics(NULL, -1, prepro->flat, RLAYER, &selection, STATS_BASIC, MULTI_THREADED);
 			if (!stat) {
 				siril_log_message(_("Error: statistics computation failed.\n"));
 				return 1;
@@ -322,19 +349,26 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 			siril_log_message(_("Normalisation value auto evaluated: %.3f\n"),
 					prepro->normalisation);
 		}
+		if (set_hist) {
+			prepro->history = g_slist_append(prepro->history, g_strdup_printf("Calibrated with a master flat, normalization of %.3f", prepro->normalisation));
+		}
 	}
 
 	/** FIX XTRANS AC ISSUE **/
 	if (prepro->fix_xtrans && prepro->use_dark) {
 		fix_xtrans_ac(prepro->dark);
+		if (set_hist)
+			prepro->history = g_slist_append(prepro->history, g_strdup("Fixed X-Trans sensor artifact on the master dark"));
 	}
 
 	if (prepro->fix_xtrans && prepro->use_bias && prepro->bias) { // check for existence of bias in case of synth offset
 		fix_xtrans_ac(prepro->bias);
+		if (set_hist)
+			prepro->history = g_slist_append(prepro->history, g_strdup("Fixed X-Trans sensor artifact on the master bias"));
 	}
 
 	// proceed to cosmetic correction
-	if (prepro->use_cosmetic_correction && prepro->use_dark) {
+	if (prepro->use_cosmetic_correction && prepro->use_dark && prepro->cc_from_dark) {
 		if (strlen(prepro->dark->bayer_pattern) > 4) {
 			siril_log_color_message(_("Cosmetic correction cannot be applied on X-Trans files.\n"), "red");
 			prepro->use_cosmetic_correction = FALSE;
@@ -346,6 +380,8 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 				str = g_strdup_printf(str, prepro->icold + prepro->ihot, prepro->icold, prepro->ihot);
 				siril_log_message(str);
 				g_free(str);
+				if (set_hist)
+					prepro->history = g_slist_append(prepro->history, g_strdup_printf("Cosmetic correction of %ld cold pixels and %ld hot pixels", prepro->icold, prepro->ihot));
 			} else
 				siril_log_message(_("Darkmap cosmetic correction is only supported with single channel images\n"));
 		}
@@ -354,20 +390,24 @@ static int prepro_prepare_hook(struct generic_seq_args *args) {
 	return 0;
 }
 
-static int prepro_image_hook(struct generic_seq_args *args, int out_index, int in_index, fits *fit, rectangle *_) {
+int prepro_image_hook(struct generic_seq_args *args, int out_index, int in_index, fits *fit, rectangle *_, int threads) {
 	struct preprocessing_data *prepro = args->user;
+	GSList *history = g_slist_copy_deep(prepro->history, (GCopyFunc)g_strdup, NULL);
 
-	/******/
 	if (prepro->use_dark_optim && prepro->use_dark) {
-		if (darkOptimization(fit, prepro))
+		if (darkOptimization(fit, prepro, in_index, history)) {
+			g_slist_free_full(history, g_free);
 			return 1;
+		}
 	}
 
-	if (preprocess(fit, prepro))
+	if (preprocess(fit, prepro)) {
+		g_slist_free_full(history, g_free);
 		return 1;
+	}
 
 	if (prepro->use_cosmetic_correction && prepro->use_dark
-			&& prepro->dark->naxes[2] == 1) {
+			&& prepro->dark->naxes[2] == 1 && prepro->cc_from_dark) {
 		cosmeticCorrection(fit, prepro->dev, prepro->icold + prepro->ihot, prepro->is_cfa);
 #ifdef SIRIL_OUTPUT_DEBUG
 		image_find_minmax(fit);
@@ -377,8 +417,12 @@ static int prepro_image_hook(struct generic_seq_args *args, int out_index, int i
 #endif
 	}
 
+	if (prepro->use_cosmetic_correction && !prepro->cc_from_dark && prepro->bad_pixel_map_file) {
+		apply_cosme_to_image(fit, prepro->bad_pixel_map_file, prepro->is_cfa);
+	}
+
 	if (prepro->debayer) {
-		// debayering SER is now allowed - https://gitlab.com/free-astro/siril/-/issues/549	
+		// debayering SER is now allowed - https://gitlab.com/free-astro/siril/-/issues/549
 		if (!prepro->seq || prepro->seq->type == SEQ_REGULAR || prepro->seq->type == SEQ_FITSEQ || prepro->seq->type == SEQ_SER ) {
 			debayer_if_needed(TYPEFITS, fit, TRUE);
 		}
@@ -398,10 +442,14 @@ static int prepro_image_hook(struct generic_seq_args *args, int out_index, int i
 	 * sequence, which may be done automatically by the caller.
 	 */
 	full_stats_invalidation_from_fit(fit);
+	fit->history = g_slist_concat(fit->history, history);
+	fit->lo = 0;
 	return 0;
 }
 
-static void clear_preprocessing_data(struct preprocessing_data *prepro) {
+void clear_preprocessing_data(struct preprocessing_data *prepro) {
+	if (prepro->bad_pixel_map_file)
+		g_object_unref(prepro->bad_pixel_map_file);
 	if (prepro->use_bias && prepro->bias)
 		clearfits(prepro->bias);
 	if (prepro->use_dark && prepro->dark)
@@ -414,24 +462,19 @@ static void clear_preprocessing_data(struct preprocessing_data *prepro) {
 
 static int prepro_finalize_hook(struct generic_seq_args *args) {
 	int retval = seq_finalize_hook(args);
-	struct preprocessing_data *prepro = args->user;
-	clear_preprocessing_data(prepro);
+	clear_preprocessing_data((struct preprocessing_data *)args->user);
 	free(args->user);
 	return retval;
 }
 
-gpointer prepro_worker(gpointer p) {
-	gpointer retval = generic_sequence_worker(p);
-
-	struct generic_seq_args *args = (struct generic_seq_args *)p;
-	free_sequence(args->seq, TRUE);
-	free(args);
-	return retval;
-}
-
 void start_sequence_preprocessing(struct preprocessing_data *prepro) {
 	struct generic_seq_args *args = create_default_seqargs(prepro->seq);
-	args->force_float = !com.pref.force_to_16bit && prepro->output_seqtype != SEQ_SER;
+	args->force_float = !com.pref.force_16bit && prepro->output_seqtype != SEQ_SER;
+	if (!prepro->ignore_exclusion) {
+		siril_debug_print("Ignoring exclusions: frames marked as excluded will still be processed.\n");
+		args->filtering_criterion = seq_filter_included;
+		args->nb_filtered_images = prepro->seq->selnum;
+	}
 	args->compute_size_hook = prepro_compute_size_hook;
 	args->compute_mem_limits_hook = prepro_compute_mem_hook;
 	args->prepare_hook = prepro_prepare_hook;
@@ -439,28 +482,20 @@ void start_sequence_preprocessing(struct preprocessing_data *prepro) {
 	args->finalize_hook = prepro_finalize_hook;
 	args->description = _("Preprocessing");
 	args->has_output = TRUE;
-	args->new_seq_prefix = prepro->ppprefix;
+	args->new_seq_prefix = strdup(prepro->ppprefix);
 	args->load_new_sequence = TRUE;
 	args->force_ser_output = prepro->seq->type != SEQ_SER && prepro->output_seqtype == SEQ_SER;
 	args->force_fitseq_output = prepro->seq->type != SEQ_FITSEQ && prepro->output_seqtype == SEQ_FITSEQ;
 	args->user = prepro;
 	// float output is always used in case of FITS sequence
-	args->output_type = (args->force_ser_output || com.pref.force_to_16bit ||
+	args->output_type = (args->force_ser_output || com.pref.force_16bit ||
 			(args->seq->type == SEQ_SER && !args->force_fitseq_output)) ?
 		DATA_USHORT : DATA_FLOAT;
 
-	remove_prefixed_sequence_files(prepro->seq, prepro->ppprefix);
-
-	if (com.script) {
-		args->already_in_a_thread = TRUE;
-		start_in_new_thread(prepro_worker, args);
-	} else {
-		args->already_in_a_thread = FALSE;
-		start_in_new_thread(generic_sequence_worker, args);
-	}
+	start_in_new_thread(generic_sequence_worker, args);
 }
 
-/********** SINGLE IMAGE ************/
+/********** SINGLE IMAGE (from com.uniq) ************/
 int preprocess_single_image(struct preprocessing_data *args) {
 	gchar *msg;
 	fits fit = { 0 };
@@ -469,14 +504,14 @@ int preprocess_single_image(struct preprocessing_data *args) {
 	msg = g_strdup_printf(_("Pre-processing image %s"), com.uniq->filename);
 	set_progress_bar_data(msg, 0.5);
 	g_free(msg);
-	struct generic_seq_args generic = { .user = args };
 
 	copyfits(com.uniq->fit, &fit, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
 	copy_fits_metadata(com.uniq->fit, &fit);
 
+	struct generic_seq_args generic = { .user = args };
 	ret = prepro_prepare_hook(&generic);
 	if (!ret)
-		ret = prepro_image_hook(&generic, 0, 0, &fit, NULL);
+		ret = prepro_image_hook(&generic, 0, 0, &fit, NULL, com.max_thread);
 	clear_preprocessing_data(args);
 
 	if (!ret) {
@@ -489,15 +524,18 @@ int preprocess_single_image(struct preprocessing_data *args) {
 		ret = savefits(dest_filename, &fit);
 
 		if (!ret) {
-			// open the new image?
-			copyfits(&fit, com.uniq->fit, CP_ALLOC | CP_FORMAT | CP_COPYA, 0);
-			com.uniq->nb_layers = fit.naxes[2];
+			// make the result the open image
+			clearfits(com.uniq->fit);
 			if (com.uniq->filename)
 				free(com.uniq->filename);
-			com.uniq->filename = strdup(dest_filename);
-		}
 
-		clearfits(&fit);
+			memcpy(com.uniq->fit, &fit, sizeof(fits));
+			com.uniq->nb_layers = fit.naxes[2];
+			com.uniq->filename = strdup(dest_filename);
+			// this way of opening it will not create gfit.header
+		}
+		else clearfits(&fit);
+
 		free(filename_noext);
 		g_free(dest_filename);
 		g_free(msg);
@@ -506,12 +544,47 @@ int preprocess_single_image(struct preprocessing_data *args) {
 	return ret;
 }
 
+/* single image preprocess, headless */
+int preprocess_given_image(char *file, struct preprocessing_data *args) {
+	fits fit = { 0 };
+	int ret = 0;
+
+	siril_log_message(_("Pre-processing image %s\n"), file);
+	struct generic_seq_args generic = { .user = args };
+
+	if (readfits(file, &fit, NULL, !com.pref.force_16bit)) {
+		siril_log_message(_("Could not load the image, aborting.\n"));
+		return 1;
+	}
+
+	ret = prepro_prepare_hook(&generic);
+	if (!ret)
+		ret = prepro_image_hook(&generic, 0, 0, &fit, NULL, com.max_thread);
+	clear_preprocessing_data(args);
+
+	if (!ret) {
+		gchar *filename = g_path_get_basename(file);
+		char *filename_noext = remove_ext_from_filename(filename);
+		g_free(filename);
+		gchar *dest_filename = g_strdup_printf("%s%s%s", args->ppprefix, filename_noext, com.pref.ext);
+		siril_log_message(_("Saving image %s\n"), filename_noext);
+		ret = savefits(dest_filename, &fit);
+		free(filename_noext);
+		g_free(dest_filename);
+		clearfits(&fit);
+	}
+	return ret;
+}
+
+
 int evaluateoffsetlevel(const char* expression, fits *fit) {
 	// try to find an occurence of *
 	// If none -> the level is just an integer to evaluate
 	// If found -> Try to find $ sign to read the offset value and its multiplier
 
 	gchar *expressioncpy = g_strdup(expression);
+	if (!expressioncpy)
+		return 0;
 	gchar *end = NULL;
 	remove_spaces_from_str(expressioncpy);
 	gchar *mulsignpos = g_strrstr(expressioncpy, "*");
@@ -533,7 +606,7 @@ int evaluateoffsetlevel(const char* expression, fits *fit) {
 		if (!g_str_equal(mulsignpos,"$OFFSET")) goto free_on_error;
 	}
 	if (!multiplier) goto free_on_error; // multiplier not parsed
-	if (!(end[0] == '\0')) goto free_on_error; // some characters were found after the multiplier
+	if (end[0] != '\0') goto free_on_error; // some characters were found after the multiplier
 	offsetlevel = (int)(multiplier * fit->key_offset);
 	if (expressioncpy) g_free(expressioncpy);
 	return offsetlevel;
@@ -542,11 +615,57 @@ free_on_error:
 	return 0;
 }
 
+gboolean check_for_cosme_file_sanity(GFile *file) {
+	GError *error = NULL;
+	gchar *line;
+
+	GInputStream *input_stream = (GInputStream *)g_file_read(file, NULL, &error);
+
+	if (input_stream == NULL) {
+		if (error != NULL) {
+			g_clear_error(&error);
+			siril_log_message(_("File [%s] does not exist\n"), g_file_peek_path(file));
+		}
+		g_object_unref(file);
+		return FALSE;
+	}
+
+	GDataInputStream *data_input = g_data_input_stream_new(input_stream);
+	while ((line = g_data_input_stream_read_line_utf8(data_input, NULL,
+				NULL, NULL))) {
+		if (!g_str_has_prefix(line, "P ") && !g_str_has_prefix(line, "C ")
+				&& !g_str_has_prefix(line, "C ")
+				&& !g_str_has_prefix(line, "#")) {
+			g_free(line);
+			g_object_unref(data_input);
+			g_object_unref(input_stream);
+			return FALSE;
+		}
+		g_free(line);
+	}
+	g_object_unref(data_input);
+	g_object_unref(input_stream);
+	return TRUE;
+}
+
 static gboolean test_for_master_files(struct preprocessing_data *args) {
 	GtkToggleButton *tbutton;
 	GtkEntry *entry;
 	gboolean has_error = FALSE;
 	args->bias_level = FLT_MAX;
+	args->bad_pixel_map_file = NULL;
+	fits reffit = { 0 };
+	gboolean isseq = (args->seq != NULL);
+	if (isseq) {
+		// loading the sequence reference image's metadata in case it's needed
+		int image_to_load = sequence_find_refimage(args->seq);
+		if (seq_read_frame_metadata(args->seq, image_to_load, &reffit)) {
+			siril_log_message(_("Could not load the reference image of the sequence, aborting.\n"));
+			has_error = TRUE;
+		}
+	} else {
+		reffit = gfit;
+	}
 
 	tbutton = GTK_TOGGLE_BUTTON(lookup_widget("useoffset_button"));
 	if (gtk_toggle_button_get_active(tbutton)) {
@@ -576,28 +695,37 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 					}
 				}
 			} else {
-				args->bias = calloc(1, sizeof(fits));
-				set_progress_bar_data(_("Opening offset image..."), PROGRESS_NONE);
-				if (!readfits(filename, args->bias, NULL, !com.pref.force_to_16bit)) {
-					if (args->bias->naxes[2] != gfit.naxes[2]) {
-						error = _("NOT USING OFFSET: number of channels is different");
-					} else if (args->bias->naxes[0] != gfit.naxes[0] ||
-							args->bias->naxes[1] != gfit.naxes[1]) {
-						error = _("NOT USING OFFSET: image dimensions are different");
-					} else {
-						args->use_bias = TRUE;
-						// if input is 8b, we assume 32b master needs to be rescaled
-						if ((args->bias->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
-							soper(args->bias, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+				int status;
+				gchar *expression = path_parse(&reffit, filename, PATHPARSE_MODE_READ, &status);
+				if (status) {
+					error = _("NOT USING OFFSET: could not parse the expression\n");
+					has_error = TRUE;
+				} else {
+					args->bias = calloc(1, sizeof(fits));
+					set_progress_bar_data(_("Opening offset image..."), PROGRESS_NONE);
+					if (!readfits(expression, args->bias, NULL, !com.pref.force_16bit)) {
+						if (args->bias->naxes[2] != gfit.naxes[2]) {
+							error = _("NOT USING OFFSET: number of channels is different");
+						} else if (args->bias->naxes[0] != gfit.naxes[0] ||
+								args->bias->naxes[1] != gfit.naxes[1]) {
+							error = _("NOT USING OFFSET: image dimensions are different");
+						} else {
+							args->use_bias = TRUE;
+							// if input is 8b, we assume 32b master needs to be rescaled
+							if ((args->bias->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
+								soper(args->bias, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+							}
 						}
-					}
-				} else error = _("NOT USING OFFSET: cannot open the file");
+					} else
+						error = _("NOT USING OFFSET: cannot open the file");
+				}
+				g_free(expression);
 			}
 			if (error) {
 				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
-				free(args->bias);
-				gtk_entry_set_text(entry, "");
+				if (args->bias)
+					free(args->bias);
 				args->use_bias = FALSE;
 				has_error = TRUE;
 			}
@@ -613,32 +741,41 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 			gtk_toggle_button_set_active(tbutton, FALSE);
 		} else {
 			const char *error = NULL;
-			set_progress_bar_data(_("Opening dark image..."), PROGRESS_NONE);
-			args->dark = calloc(1, sizeof(fits));
-			if (!readfits(filename, args->dark, NULL, !com.pref.force_to_16bit)) {
-				if (args->dark->naxes[2] != gfit.naxes[2]) {
-					error = _("NOT USING DARK: number of channels is different");
-				} else if (args->dark->naxes[0] != gfit.naxes[0] ||
-						args->dark->naxes[1] != gfit.naxes[1]) {
-					error = _("NOT USING DARK: image dimensions are different");
-				} else {
-					args->use_dark = TRUE;
-					// if input is 8b, we assume 32b master needs to be rescaled
-					if ((args->dark->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
-						soper(args->dark, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+			int status;
+			gchar *expression = path_parse(&reffit, filename, PATHPARSE_MODE_READ, &status);
+			if (status) {
+				error = _("NOT USING DARK: could not parse the expression");
+				has_error = TRUE;
+			} else {
+				set_progress_bar_data(_("Opening dark image..."), PROGRESS_NONE);
+				args->dark = calloc(1, sizeof(fits));
+				if (!readfits(expression, args->dark, NULL, !com.pref.force_16bit)) {
+					if (args->dark->naxes[2] != gfit.naxes[2]) {
+						error = _("NOT USING DARK: number of channels is different");
+					} else if (args->dark->naxes[0] != gfit.naxes[0] ||
+							args->dark->naxes[1] != gfit.naxes[1]) {
+						error = _("NOT USING DARK: image dimensions are different");
+					} else {
+						args->use_dark = TRUE;
+						// if input is 8b, we assume 32b master needs to be rescaled
+						if ((args->dark->type == DATA_FLOAT) && (gfit.orig_bitpix == BYTE_IMG)) {
+							soper(args->dark, USHRT_MAX_SINGLE / UCHAR_MAX_SINGLE, OPER_MUL, TRUE);
+						}
 					}
-				}
 
-			} else error = _("NOT USING DARK: cannot open the file");
+				} else error = _("NOT USING DARK: cannot open the file");
+			}
 			if (error) {
 				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
-				clearfits(args->dark);
+				if (args->dark)
+					clearfits(args->dark);
 				args->dark = NULL; // in order to be sure it is freed
-				gtk_entry_set_text(entry, "");
 				args->use_dark = FALSE;
 				has_error = TRUE;
 			}
+			g_free(expression);
+			
 		}
 
 		if (args->use_dark) {
@@ -674,10 +811,36 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 					GtkSpinButton *sigHot = GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeHotBox"));
 					args->sigma[1] = gtk_spin_button_get_value(sigHot);
 				} else args->sigma[1] = -1.0;
+
+				/* Using Bad Pixel Map ? */
+				const gchar *bad_pixel_f;
+				entry = GTK_ENTRY(lookup_widget("pixelmap_entry"));
+				bad_pixel_f = gtk_entry_get_text(entry);
+				/* test for file */
+				if (bad_pixel_f[0] != '\0') {
+					args->bad_pixel_map_file = g_file_new_for_path(bad_pixel_f);
+					if (!check_for_cosme_file_sanity(args->bad_pixel_map_file)) {
+						siril_debug_print("cosme file sanity check failed...\n");
+						has_error = TRUE;
+					}
+				}
 			}
 		}
 	}
 
+	/* now we want to know which cosmetic correction we choose */
+	GtkStack *stack = GTK_STACK(lookup_widget("stack_cc"));
+	GtkWidget *w = gtk_stack_get_visible_child(stack);
+	args->cc_from_dark = TRUE; // default value
+	if (w) {
+		GValue value = G_VALUE_INIT;
+		g_value_init(&value, G_TYPE_INT);
+		gtk_container_child_get_property(GTK_CONTAINER(stack), w, "position", &value);
+		gint position = g_value_get_int(&value);
+		args->cc_from_dark = position == 0 ? TRUE : FALSE;
+		g_value_unset(&value);
+	}
+
 	tbutton = GTK_TOGGLE_BUTTON(lookup_widget("useflat_button"));
 	if (gtk_toggle_button_get_active(tbutton)) {
 		const char *filename;
@@ -687,25 +850,33 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 			gtk_toggle_button_set_active(tbutton, FALSE);
 		} else {
 			const char *error = NULL;
-			set_progress_bar_data(_("Opening flat image..."), PROGRESS_NONE);
-			args->flat = calloc(1, sizeof(fits));
-			if (!readfits(filename, args->flat, NULL, !com.pref.force_to_16bit)) {
-				if (args->flat->naxes[2] != gfit.naxes[2]) {
-					error = _("NOT USING FLAT: number of channels is different");
-				} else if (args->flat->naxes[0] != gfit.naxes[0] ||
-						args->flat->naxes[1] != gfit.naxes[1]) {
-					error = _("NOT USING FLAT: image dimensions are different");
-				} else {
-					args->use_flat = TRUE;
-					// no need to deal with bitdepth conversion as flat is just a division (unlike darks which need to be on same scale)
-				}
+			int status;
+			gchar *expression = path_parse(&reffit, filename, PATHPARSE_MODE_READ, &status);
+			if (status) {
+				error = _("NOT USING FLAT: could not parse the expression");
+				has_error = TRUE;
+			} else {
+				set_progress_bar_data(_("Opening flat image..."), PROGRESS_NONE);
+				args->flat = calloc(1, sizeof(fits));
+				if (!readfits(filename, args->flat, NULL, !com.pref.force_16bit)) {
+					if (args->flat->naxes[2] != gfit.naxes[2]) {
+						error = _("NOT USING FLAT: number of channels is different");
+					} else if (args->flat->naxes[0] != gfit.naxes[0] ||
+							args->flat->naxes[1] != gfit.naxes[1]) {
+						error = _("NOT USING FLAT: image dimensions are different");
+					} else {
+						args->use_flat = TRUE;
+						// no need to deal with bitdepth conversion as flat is just a division (unlike darks which need to be on same scale)
+					}
 
-			} else error = _("NOT USING FLAT: cannot open the file");
+				} else error = _("NOT USING FLAT: cannot open the file");
+				g_free(expression);
+			}
 			if (error) {
 				siril_log_color_message("%s\n", "red", error);
 				set_progress_bar_data(error, PROGRESS_DONE);
-				free(args->flat);
-				gtk_entry_set_text(entry, "");
+				if (args->flat)
+					free(args->flat);
 				args->use_flat = FALSE;
 				has_error = TRUE;
 			}
@@ -720,6 +891,8 @@ static gboolean test_for_master_files(struct preprocessing_data *args) {
 			}
 		}
 	}
+	if (isseq)
+		clearfits(&reffit);
 	return has_error;
 }
 
@@ -736,22 +909,25 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 	GtkToggleButton *CFA = GTK_TOGGLE_BUTTON(lookup_widget("cosmCFACheck"));
 	GtkToggleButton *debayer = GTK_TOGGLE_BUTTON(lookup_widget("checkButton_pp_dem"));
 	GtkToggleButton *equalize_cfa = GTK_TOGGLE_BUTTON(lookup_widget("checkbutton_equalize_cfa"));
+	GtkToggleButton *preprocess_excluded = GTK_TOGGLE_BUTTON(lookup_widget("toggle_preprocess_excluded"));
 	GtkComboBox *output_type = GTK_COMBO_BOX(lookup_widget("prepro_output_type_combo"));
 
 	struct preprocessing_data *args = calloc(1, sizeof(struct preprocessing_data));
 	if (test_for_master_files(args)) {
 		siril_log_color_message(_("Some errors have been detected, Please check the logs.\n"), "red");
+		free(args);
 		return;
 	}
-	if (!args->use_bias && !args->use_dark && !args->use_flat)
+	if (!args->use_bias && !args->use_dark && !args->use_flat) {
+		free(args);
 		return;
-
+	}
 	siril_log_color_message(_("Preprocessing...\n"), "green");
-	gettimeofday(&args->t_start, NULL);
 
 	// set output filename (preprocessed file name prefix)
-	args->ppprefix = gtk_entry_get_text(entry);
+	args->ppprefix = strdup(gtk_entry_get_text(entry));
 
+	args->ignore_exclusion = gtk_toggle_button_get_active(preprocess_excluded);
 	args->is_cfa = gtk_toggle_button_get_active(CFA);
 	args->debayer = gtk_toggle_button_get_active(debayer);
 	args->equalize_cfa = gtk_toggle_button_get_active(equalize_cfa);
@@ -765,14 +941,15 @@ void on_prepro_button_clicked(GtkButton *button, gpointer user_data) {
 		args->output_seqtype = gtk_combo_box_get_active(output_type);
 		if (args->output_seqtype < 0 || args->output_seqtype > SEQ_FITSEQ)
 			args->output_seqtype = SEQ_REGULAR;
-		args->allow_32bit_output = !com.pref.force_to_16bit && args->output_seqtype != SEQ_SER;
+		args->allow_32bit_output = !com.pref.force_16bit && args->output_seqtype != SEQ_SER;
 		set_cursor_waiting(TRUE);
 		control_window_switch_to_tab(OUTPUT_LOGS);
 		start_sequence_preprocessing(args);
 	} else {
 		int retval;
 		args->is_sequence = FALSE;
-		args->allow_32bit_output = !com.pref.force_to_16bit;
+		args->output_seqtype = SEQ_REGULAR;
+		args->allow_32bit_output = !com.pref.force_16bit;
 		set_cursor_waiting(TRUE);
 		control_window_switch_to_tab(OUTPUT_LOGS);
 
@@ -801,6 +978,20 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 	long icold = 0L, ihot = 0L;
 	double rate, total;
 	fits fit = { 0 };
+	fits reffit = { 0 };
+	int status;
+	gboolean isseq = FALSE;
+	if (sequence_is_loaded()) {
+		// loading the sequence reference image's metadata
+		int image_to_load = sequence_find_refimage(&com.seq);
+		if (seq_read_frame_metadata(&com.seq, image_to_load, &reffit)) {
+			siril_log_message(_("Could not load the reference image of the sequence, aborting.\n"));
+			return;
+		}
+		isseq = TRUE;
+	} else {
+		reffit = gfit;
+	}
 
 	set_cursor_waiting(TRUE);
 	sig[0] = gtk_spin_button_get_value(GTK_SPIN_BUTTON(lookup_widget("spinSigCosmeColdBox")));
@@ -811,12 +1002,14 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 	label[1] = GTK_LABEL(lookup_widget("GtkLabelHotCC"));
 	entry = GTK_ENTRY(lookup_widget("darkname_entry"));
 	filename = gtk_entry_get_text(entry);
-	if (readfits(filename, &fit, NULL, !com.pref.force_to_16bit)) {
+	gchar *expression = path_parse(&reffit, filename, PATHPARSE_MODE_READ, &status);
+	if (status || readfits(expression, &fit, NULL, !com.pref.force_16bit)) {
 		str[0] = g_markup_printf_escaped(_("<span foreground=\"red\">ERROR</span>"));
 		str[1] = g_markup_printf_escaped(_("<span foreground=\"red\">ERROR</span>"));
 		gtk_label_set_markup(label[0], str[0]);
 		gtk_label_set_markup(label[1], str[1]);
 		set_cursor_waiting(FALSE);
+		g_free(expression);
 		return;
 	}
 	find_deviant_pixels(&fit, sig, &icold, &ihot, TRUE);
@@ -825,11 +1018,11 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 	rate = (double)icold / total;
 	/* 1% of cold pixels seems to be a reasonable limit */
 	if (rate > 0.01) {
-		str[0] = g_markup_printf_escaped(_("<span foreground=\"red\">Cold: %ld px</span>"), icold);
+		str[0] = g_markup_printf_escaped("<span foreground=\"red\">%ld px</span>", icold);
 		gtk_widget_set_tooltip_text(widget[0], _("This value may be too high. Please, consider to change sigma value or uncheck the box."));
 	}
 	else {
-		str[0] = g_markup_printf_escaped(_("Cold: %ld px"), icold);
+		str[0] = g_markup_printf_escaped("%ld px", icold);
 		gtk_widget_set_tooltip_text(widget[0], "");
 	}
 	gtk_label_set_markup(label[0], str[0]);
@@ -837,15 +1030,18 @@ void on_GtkButtonEvaluateCC_clicked(GtkButton *button, gpointer user_data) {
 	rate = (double)ihot / total;
 	/* 1% of hot pixels seems to be a reasonable limit */
 	if (rate > 0.01) {
-		str[1] = g_markup_printf_escaped(_("<span foreground=\"red\">Hot: %ld px</span>"), ihot);
+		str[1] = g_markup_printf_escaped("<span foreground=\"red\">%ld px</span>", ihot);
 		gtk_widget_set_tooltip_text(widget[1], _("This value may be too high. Please, consider to change sigma value or uncheck the box."));
 	}
 	else {
-		str[1] = g_markup_printf_escaped(_("Hot: %ld px"), ihot);
+		str[1] = g_markup_printf_escaped("%ld px", ihot);
 		gtk_widget_set_tooltip_text(widget[1], "");
 	}
 	gtk_label_set_markup(label[1], str[1]);
 	g_free(str[0]);
 	g_free(str[1]);
 	set_cursor_waiting(FALSE);
+	g_free(expression);
+	if (isseq)
+		clearfits(&reffit);
 }
