@@ -1,6 +1,7 @@
 #ifndef PROTO_H_
 #define PROTO_H_
 #include <stdint.h>
+#include <sys/time.h>
 #include "core/siril.h"
 #ifdef HAVE_LIBTIFF
 #define uint64 uint64_hack_
@@ -42,9 +43,12 @@ int readpic(const char *name, fits *fit);
 
 /****************** image_formats_libraries.h ******************/
 #ifdef HAVE_LIBTIFF
-int readtif(const char *name, fits *fit, gboolean force_float);
-void get_tif_data_from_ui(gchar **description, gchar **copyright, gboolean *embeded_icc);
-int savetif(const char *name, fits *fit, uint16_t bitspersample, const char *description, const char *copyright, gboolean embeded_icc);
+gboolean get_tiff_compression();
+int readtif(const char *name, fits *fit, gboolean force_float, gboolean verbose);
+void get_tif_data_from_ui(fits *fit, gchar **description, gchar **copyright, gboolean *embeded_icc);
+int savetif(const char *name, fits *fit, uint16_t bitspersample,
+		const char *description, const char *copyright,
+		gboolean tiff_compression, gboolean embeded_icc, gboolean verbose);
 #endif
 
 #ifdef HAVE_LIBJPEG
@@ -73,6 +77,8 @@ WORD round_to_WORD(double x);
 BYTE round_to_BYTE(double x);
 BYTE roundf_to_BYTE(float f);
 WORD roundf_to_WORD(float f);
+signed short roundf_to_short(float f);
+guint float_to_max_range(float f, guint max);
 int round_to_ceiling_multiple(int x, int factor);
 BYTE conv_to_BYTE(double x);
 int truncate_to_int32(uint64_t x);
@@ -85,9 +91,12 @@ float ushort_to_float_range(WORD w);
 float uchar_to_float_range(BYTE w);
 float double_ushort_to_float_range(double d);
 WORD float_to_ushort_range(float f);
+signed short float_to_short_range(float f);
 BYTE float_to_uchar_range(float f);
 float ushort_to_float_bitpix(fits *fit, WORD value);
 WORD *float_buffer_to_ushort(float *buffer, size_t ndata);
+signed short *float_buffer_to_short(float *buffer, size_t ndata);
+signed short *ushort_buffer_to_short(const WORD *buffer, size_t ndata);
 float *uchar_buffer_to_float(BYTE *buffer, size_t ndata);
 float *ushort_buffer_to_float(WORD *buffer, size_t ndata);
 float *ushort8_buffer_to_float(WORD *buffer, size_t ndata);
@@ -108,6 +117,7 @@ uint64_t cpu_to_be64(uint64_t x);
 uint64_t le64_to_cpu(uint64_t x);
 uint64_t be64_to_cpu(uint64_t x);
 gboolean isrgb(fits *fit);
+const char *channel_number_to_name(int channel);
 int get_extension_index(const char *filename);
 image_type get_type_from_filename(const gchar *filename);
 char* remove_ext_from_filename(const char *basename);
@@ -116,6 +126,7 @@ gboolean string_is_a_path(const char *file);
 int is_readable_file(const char *filename);
 gboolean is_forbiden_in_filename(gchar c);
 gboolean file_name_has_invalid_chars(const char *name);
+void replace_invalid_chars(char *name, char repl);
 int stat_file(const char *filename2, image_type *type, char **realname);
 const char* get_filename_ext(const char *filename);
 
@@ -133,35 +144,48 @@ gchar *siril_truncate_str(gchar *str, gint size);
 char **glist_to_array(GList *list, int *arg_count);
 gchar* url_cleanup(const gchar *uri_string);
 void remove_spaces_from_str(gchar *s);
+void remove_trailing_eol(char *str);
+gboolean string_is_a_number(const char *str);
 #if !GLIB_CHECK_VERSION(2,68,0)
 guint g_string_replace(GString *string, const gchar *find, const gchar *replace,
 		guint limit);
 #endif
+char *str_replace(char *orig, char *rep, char *with);
+void replace_spaces_from_str(gchar *s, char c);
+gchar *build_string_from_words(char **words);
+void append_elements_to_array(char **array, char **elements);
+const gchar *get_com_ext(gboolean fz);
+
+int fits_to_display(double fx, double fy, double *dx, double *dy, int ry);
+int display_to_fits(double dx, double dy, double *fx, double *fy, int ry);
+gchar *siril_file_chooser_get_filename(GtkFileChooser *chooser);
+GSList *siril_file_chooser_get_filenames(GtkFileChooser *chooser);
 
 /****************** quantize.h ***************/
 int siril_fits_img_stats_ushort(WORD *array, long nx, long ny, int nullcheck,
 		WORD nullvalue, long *ngoodpix, WORD *minvalue, WORD *maxvalue,
 		double *mean, double *sigma, double *noise1, double *noise2,
-		double *noise3, double *noise5, gboolean multithread, int *status);
+		double *noise3, double *noise5, threading_type threads, int *status);
 
 int siril_fits_img_stats_float(float *array, long nx, long ny, int nullcheck,
 		float nullvalue, long *ngoodpix, float *minvalue, float *maxvalue,
 		double *mean, double *sigma, double *noise1, double *noise2,
-		double *noise3, double *noise5, gboolean multithread, int *status);
+		double *noise3, double *noise5, threading_type threads, int *status);
 
 /****************** siril.h ******************/
 
 int threshlo(fits *fit, WORD level);
 int threshhi(fits *fit, WORD level);
 int nozero(fits *fit, WORD level);
+int gaussian_blur_RT(fits *fit, double sigma, int threads);
 int unsharp(fits*, double sigma, double mult, gboolean verbose);
 float entropy(fits *fit, int layer, rectangle *area, imstats *opt_stats);
 int loglut(fits *fit);
-int ddp(fits *a, int lev, float coef, float sig);
+int ddp(fits *a, float lev, float coef, float sig);
 int visu(fits *fit, int low, int high);
 int fill(fits *fit, int level, rectangle *arearg);
 int off(fits *a, float level);
-double background(fits *fit, int reqlayer, rectangle *selection, gboolean multithread);
+double background(fits* fit, int reqlayer, rectangle *selection, threading_type threads);
 void show_FITS_header(fits*);
 void compute_grey_flat(fits *fit);
 
@@ -174,10 +198,6 @@ int buildseqfile(sequence *seq, int force_recompute);
 /****************** statistics_list.h ******************/
 void computeStat();
 
-/****************** siril_log.h ******************/
-char* siril_log_message(const char* format, ...);
-char* siril_log_color_message(const char* format, const char* color, ...);
-
 #ifdef __cplusplus
 }
 #endif
