@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,18 +22,25 @@
 #include "core/proto.h"
 #include "core/command.h"
 #include "core/undo.h"
+#include "core/command.h"
 #include "core/siril_update.h"
 #include "core/siril_cmd_help.h"
+#include "core/siril_log.h"
 #include "core/initfile.h"
+#include "compositing/align_rgb.h"
 #include "algos/annotate.h"
-#include "algos/colors.h"
+#include "algos/astrometry_solver.h"
 #include "algos/noise.h"
 #include "algos/geometry.h"
 #include "algos/siril_wcs.h"
+#include "algos/ccd-inspector.h"
 #include "compositing/compositing.h"
+#include "filters/synthstar.h"
 #include "gui/about_dialog.h"
 #include "gui/utils.h"
+#include "gui/colors.h"
 #include "gui/callbacks.h"
+#include "gui/documentation.h"
 #include "gui/histogram.h"
 #include "gui/open_dialog.h"
 #include "gui/message_dialog.h"
@@ -46,20 +53,30 @@
 #include "gui/image_interactions.h"
 #include "gui/image_display.h"
 #include "gui/photometric_cc.h"
+#include "gui/menu_gray_geometry.h"
 #include "gui/registration_preview.h"
+#include "gui/remixer.h"
+#include "livestacking/livestacking.h"
 #include "registration/registration.h"
 
 #include "siril_actions.h"
-#include "algos/astrometry_solver.h"
 
 void open_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	header_open_button_clicked();
+	launch_clipboard_survey();
 }
 
 void cwd_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	cwd_btton_clicked();
 }
 
+void livestacking_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	GtkWidget *w = lookup_widget("livestacking_player");
+
+	gtk_widget_show(w);
+	gtk_window_set_keep_above(GTK_WINDOW(w), TRUE);
+}
+
 void save_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	on_header_save_button_clicked();
 }
@@ -109,7 +126,15 @@ void scripts_action_activate(GSimpleAction *action, GVariant *parameter, gpointe
 }
 
 void updates_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+#if defined(HAVE_JSON_GLIB) && defined(HAVE_NETWORKING)
 	siril_check_updates(TRUE);
+#else
+	siril_log_message(_("Cannot check for updates with this version, missing dependency\n"));
+#endif
+}
+
+void doc_action_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_get_documentation();
 }
 
 static gboolean is_extended = FALSE;
@@ -156,8 +181,8 @@ void panel_activate(GSimpleAction *action, GVariant *parameter, gpointer user_da
 	} else {
 		gtk_image_set_from_icon_name(image, "pan-start-symbolic", GTK_ICON_SIZE_BUTTON);
 	}
-	if (com.pref.remember_windows) {
-		com.pref.is_extended = !is_visible;
+	if (com.pref.gui.remember_windows) {
+		com.pref.gui.is_extended = !is_visible;
 		writeinitfile();
 	}
 }
@@ -260,8 +285,8 @@ void negative_view_activate(GSimpleAction *action, GVariant *parameter, gpointer
 void photometry_state(GSimpleAction *action, GVariant *state, gpointer user_data) {
 	mouse_status = g_variant_get_boolean(state) ? MOUSE_ACTION_PHOTOMETRY : MOUSE_ACTION_SELECT_REG_AREA;
 	g_simple_action_set_state(action, state);
-	free(com.qphot);
-	com.qphot = NULL;
+	free(gui.qphot);
+	gui.qphot = NULL;
 	redraw(REDRAW_OVERLAY);
 }
 
@@ -282,9 +307,18 @@ void color_map_state(GSimpleAction *action, GVariant *state, gpointer user_data)
 }
 
 void color_map_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
-	GVariant *state;
+	GVariant *state = g_action_get_state(G_ACTION(action));
+	g_action_change_state(G_ACTION(action), g_variant_new_boolean(!g_variant_get_boolean(state)));
+	g_variant_unref(state);
+}
 
-	state = g_action_get_state(G_ACTION(action));
+void chain_channels_state_change(GSimpleAction *action, GVariant *state, gpointer user_data) {
+	g_simple_action_set_state(action, state);
+	set_unlink_channels(!g_variant_get_boolean(state));
+}
+
+void chain_channels_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	GVariant *state = g_action_get_state(G_ACTION(action));
 	g_action_change_state(G_ACTION(action), g_variant_new_boolean(!g_variant_get_boolean(state)));
 	g_variant_unref(state);
 }
@@ -303,17 +337,24 @@ void pick_star_activate(GSimpleAction *action, GVariant *parameter, gpointer use
 
 void psf_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	psf_star *result = NULL;
-	int layer = match_drawing_area_widget(gui.view[gui.cvport].drawarea, FALSE);
+	int layer = select_vport(gui.cvport);
 
 	if (layer == -1)
 		return;
 	if (!(com.selection.h && com.selection.w))
 		return;
-	result = psf_get_minimisation(&gfit, layer, &com.selection, TRUE, com.pref.phot_set.force_radius, TRUE, TRUE);
+	if (com.selection.w > 300 || com.selection.h > 300) {
+		siril_message_dialog(GTK_MESSAGE_WARNING, _("Current selection is too large"),
+				_("To determine the PSF, please make a selection around a star."));
+		return;
+	}
+	struct phot_config *ps = phot_set_adjusted_for_image(&gfit);
+	result = psf_get_minimisation(&gfit, layer, &com.selection, TRUE, ps, TRUE, com.pref.starfinder_conf.profile, NULL);
+	free(ps);
 	if (!result)
 		return;
 
-	popup_psf_result(result, &com.selection);
+	popup_psf_result(result, &com.selection, &gfit);
 	free_psf(result);
 }
 
@@ -334,21 +375,27 @@ void search_object_activate(GSimpleAction *action, GVariant *parameter, gpointer
 		siril_open_dialog("search_objects");
 }
 
+void search_object_solar_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	if (has_wcs(&gfit))
+		process_sso();
+}
+
 void annotate_object_state(GSimpleAction *action, GVariant *state, gpointer user_data) {
 	if (g_variant_get_boolean(state)) {
 		if (has_wcs(&gfit)) {
 			com.found_object = find_objects(&gfit);
 		}
 	} else {
-		g_slist_free_full(com.found_object, (GDestroyNotify) free_catalogue_object);
+		g_slist_free(com.found_object);
 		com.found_object = NULL;
+		purge_temp_user_catalogue();
 	}
 	g_simple_action_set_state(action, state);
 	redraw(REDRAW_OVERLAY);
 }
 
 void wcs_grid_state(GSimpleAction *action, GVariant *state, gpointer user_data) {
-	com.show_wcs_grid = g_variant_get_boolean(state);
+	gui.show_wcs_grid = g_variant_get_boolean(state);
 	g_simple_action_set_state(action, state);
 	redraw(REDRAW_OVERLAY);
 }
@@ -369,6 +416,21 @@ void wcs_grid_activate(GSimpleAction *action, GVariant *parameter, gpointer user
 	g_variant_unref(state);
 }
 
+void regframe_state(GSimpleAction *action, GVariant *state, gpointer user_data) {
+	GtkToggleButton *drawframe;
+	drawframe = GTK_TOGGLE_BUTTON(lookup_widget("drawframe_check"));
+	gtk_toggle_button_set_active(drawframe, g_variant_get_boolean(state));
+	g_simple_action_set_state(action, state);
+	redraw(REDRAW_OVERLAY);
+}
+
+void regframe_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	GVariant *state;
+	state = g_action_get_state(G_ACTION(action));
+	g_action_change_state(G_ACTION(action), g_variant_new_boolean(!g_variant_get_boolean(state)));
+	g_variant_unref(state);
+}
+
 void seq_list_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	if (gtk_widget_get_visible(lookup_widget("seqlist_dialog"))) {
 		siril_close_dialog("seqlist_dialog");
@@ -400,6 +462,10 @@ void noise_activate(GSimpleAction *action, GVariant *parameter, gpointer user_da
 	evaluate_noise_in_image();
 }
 
+void ccd_inspector_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	compute_aberration_inspector();
+}
+
 void image_information_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	siril_open_dialog("file_information");
 }
@@ -426,6 +492,7 @@ void color_calib_activate(GSimpleAction *action, GVariant *parameter, gpointer u
 void pcc_activate(GSimpleAction *action, GVariant *parameter,gpointer user_data) {
 	initialize_photometric_cc_dialog();
 	siril_open_dialog("ImagePlateSolver_Dial");
+	on_GtkButton_IPS_metadata_clicked(NULL, NULL);	// fill it automatically
 }
 
 void split_channel_activate(GSimpleAction *action, GVariant *parameter,gpointer user_data) {
@@ -437,7 +504,7 @@ void negative_activate(GSimpleAction *action, GVariant *parameter, gpointer user
 }
 
 void histo_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
-	toggle_histogram_window_visibility();
+	toggle_histogram_window_visibility(1);
 }
 
 void fix_banding_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
@@ -456,12 +523,19 @@ void asinh_activate(GSimpleAction *action, GVariant *parameter, gpointer user_da
 	siril_open_dialog("asinh_dialog");
 }
 
+void starnet_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_open_dialog("starnet_dialog");
+}
 void deconvolution_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
-	siril_open_dialog("deconvolution_dialog");
+	siril_open_dialog("bdeconv_dialog");
 }
 
 void payne_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
-	siril_open_dialog("payne_dialog");
+	toggle_histogram_window_visibility(2);
+}
+
+void binning_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_open_dialog("binxy_dialog");
 }
 
 void resample_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
@@ -469,7 +543,11 @@ void resample_activate(GSimpleAction *action, GVariant *parameter, gpointer user
 }
 
 void rotation_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	if (com.selection.w == 0 || com.selection.h == 0) {
+		com.selection = (rectangle){ 0, 0, gfit.rx, gfit.ry };
+	}
 	siril_open_dialog("rotation_dialog");
+	redraw(REDRAW_OVERLAY);
 }
 
 void rotation90_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
@@ -526,6 +604,10 @@ void rgb_compositing_activate(GSimpleAction *action, GVariant *parameter, gpoint
 	open_compositing_window();
 }
 
+void star_remix_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	toggle_remixer_window_visibility(CALL_FROM_MENU, NULL, NULL);
+}
+
 void pixel_math_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	siril_open_dialog("pixel_math_dialog");
 }
@@ -533,3 +615,41 @@ void pixel_math_activate(GSimpleAction *action, GVariant *parameter, gpointer us
 void split_cfa_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
 	siril_open_dialog("split_cfa_dialog");
 }
+
+void nina_lc_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_open_dialog("nina_light_curve");
+}
+
+void denoise_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_open_dialog("denoise_dialog");
+}
+
+void merge_cfa_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	siril_open_dialog("merge_cfa_dialog");
+}
+
+void star_desaturate_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	undo_save_state(&gfit, "Synthetic stars: desaturate clipped stars");
+	control_window_switch_to_tab(OUTPUT_LOGS);
+	start_in_new_thread(fix_saturated_stars, NULL);
+}
+
+void star_synthetic_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	undo_save_state(&gfit, "Synthetic stars: full replacement");
+	control_window_switch_to_tab(OUTPUT_LOGS);
+	start_in_new_thread(do_synthstar, NULL);
+}
+
+void align_dft_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	undo_save_state(&gfit, _("RGB alignment (DFT)"));
+	rgb_align(1);
+}
+
+void align_psf_activate(GSimpleAction *action, GVariant *parameter, gpointer user_data) {
+	undo_save_state(&gfit, _("RGB alignment (PSF)"));
+	if (com.selection.w > 300 || com.selection.h > 300) {
+		siril_log_message(_("Current selection is too large. To determine the PSF, please make a selection around a single star.\n"));
+		return;
+	}
+	rgb_align(0);
+}
