@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -20,16 +20,19 @@
 #ifndef SRC_CORE_SIRIL_UPDATE_H_
 #define SRC_CORE_SIRIL_UPDATE_H_
 
+#if defined(HAVE_JSON_GLIB) && defined(HAVE_NETWORKING)
 
 typedef struct {
 	guint major_version;
 	guint minor_version;
 	guint micro_version;
 	guint patched_version;
+	gboolean beta_version;
 	gboolean rc_version;
 } version_number;
 
-//void siril_check_updates(gboolean verbose);
 void siril_check_updates(gboolean verbose);
 
+#endif
+
 #endif /* SRC_CORE_SIRIL_UPDATE_H_ */
