@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,8 +25,10 @@
 #include <string.h>
 
 #include "core/siril.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
+#include "gui/dialogs.h"
 #include "gui/image_display.h"
 #include "gui/histogram.h"
 #include "gui/progress_and_log.h"
@@ -87,7 +89,8 @@ static int undo_build_swapfile(fits *fit, char **filename) {
 
 static int undo_remove_item(historic *histo, int index) {
 	if (histo[index].filename) {
-		g_unlink(histo[index].filename);
+		if (g_unlink(histo[index].filename))
+			siril_debug_print("g_unlink() failed\n");
 		g_free(histo[index].filename);
 		histo[index].filename = NULL;
 		memset(&histo[index].wcsdata, 0, sizeof(wcs_info));
@@ -132,24 +135,24 @@ static void undo_add_item(fits *fit, char *filename, char *histo) {
 	com.hist_display = com.hist_current;
 }
 
-static int undo_get_data_ushort(fits *fit, historic hist) {
+static int undo_get_data_ushort(fits *fit, historic *hist) {
 	int fd;
 
-	if ((fd = g_open(hist.filename, O_RDONLY | O_BINARY, 0)) == -1) {
-		printf("Error opening swap file : %s\n", hist.filename);
+	if ((fd = g_open(hist->filename, O_RDONLY | O_BINARY, 0)) == -1) {
+		printf("Error opening swap file : %s\n", hist->filename);
 		return 1;
 	}
 
 	errno = 0;
-	fit->rx = fit->naxes[0] = hist.rx;
-	fit->ry = fit->naxes[1] = hist.ry;
+	fit->rx = fit->naxes[0] = hist->rx;
+	fit->ry = fit->naxes[1] = hist->ry;
 
 	size_t n = fit->naxes[0] * fit->naxes[1];
 	size_t size = n * fit->naxes[2] * sizeof(WORD);
 	WORD *buf = calloc(1, size);
 	// read the data from temporary file
 	if ((read(fd, buf, size)) < size) {
-		printf("Undo Read of [%s], failed with error [%s]\n", hist.filename, strerror(errno));
+		printf("Undo Read of [%s], failed with error [%s]\n", hist->filename, strerror(errno));
 		free(buf);
 		g_close(fd, NULL);
 		return 1;
@@ -169,11 +172,13 @@ static int undo_get_data_ushort(fits *fit, historic hist) {
 	if (fit->naxes[2] > 1) {
 		fit->pdata[GLAYER] = fit->data + n;
 		fit->pdata[BLAYER] = fit->data + n * 2;
+	} else {
+		fit->pdata[GLAYER] = fit->pdata[BLAYER] = fit->pdata[RLAYER];
 	}
-	memcpy(&fit->wcsdata, &hist.wcsdata, sizeof(wcs_info));
-	fit->focal_length = hist.focal_length;
+	memcpy(&fit->wcsdata, &hist->wcsdata, sizeof(wcs_info));
+	fit->focal_length = hist->focal_length;
 	if (!has_wcsdata(fit)) {
-		free_wcs(fit);
+		free_wcs(fit, TRUE);
 	} else {
 		load_WCS_from_memory(fit);
 	}
@@ -185,24 +190,24 @@ static int undo_get_data_ushort(fits *fit, historic hist) {
 	return 0;
 }
 
-static int undo_get_data_float(fits *fit, historic hist) {
+static int undo_get_data_float(fits *fit, historic *hist) {
 	int fd;
 
-	if ((fd = g_open(hist.filename, O_RDONLY | O_BINARY, 0)) == -1) {
-		printf("Error opening swap file : %s\n", hist.filename);
+	if ((fd = g_open(hist->filename, O_RDONLY | O_BINARY, 0)) == -1) {
+		printf("Error opening swap file : %s\n", hist->filename);
 		return 1;
 	}
 
 	errno = 0;
-	fit->rx = fit->naxes[0] = hist.rx;
-	fit->ry = fit->naxes[1] = hist.ry;
+	fit->rx = fit->naxes[0] = hist->rx;
+	fit->ry = fit->naxes[1] = hist->ry;
 
 	size_t n = fit->naxes[0] * fit->naxes[1];
 	size_t size = n * fit->naxes[2] * sizeof(float);
 	float *buf = calloc(1, size);
 	// read the data from temporary file
 	if ((read(fd, buf, size) < size)) {
-		printf("Undo Read of [%s], failed with error [%s]\n", hist.filename, strerror(errno));
+		printf("Undo Read of [%s], failed with error [%s]\n", hist->filename, strerror(errno));
 		free(buf);
 		g_close(fd, NULL);
 		return 1;
@@ -222,11 +227,13 @@ static int undo_get_data_float(fits *fit, historic hist) {
 	if (fit->naxes[2] > 1) {
 		fit->fpdata[GLAYER] = fit->fdata + n;
 		fit->fpdata[BLAYER] = fit->fdata + n * 2;
+	} else {
+		fit->fpdata[GLAYER] = fit->fpdata[BLAYER] = fit->fpdata[RLAYER];
 	}
-	memcpy(&fit->wcsdata, &hist.wcsdata, sizeof(wcs_info));
-	fit->focal_length = hist.focal_length;
+	memcpy(&fit->wcsdata, &hist->wcsdata, sizeof(wcs_info));
+	fit->focal_length = hist->focal_length;
 	if (!has_wcsdata(fit)) {
-		free_wcs(fit);
+		free_wcs(fit, TRUE);
 	} else {
 		load_WCS_from_memory(fit);
 	}
@@ -237,15 +244,15 @@ static int undo_get_data_float(fits *fit, historic hist) {
 	return 0;
 }
 
-static int undo_get_data(fits *fit, historic hist) {
-	if (hist.type == DATA_USHORT) {
+static int undo_get_data(fits *fit, historic *hist) {
+	if (hist->type == DATA_USHORT) {
 		if (gfit.type != DATA_USHORT) {
 			size_t ndata = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 			fit_replace_buffer(fit, float_buffer_to_ushort(fit->fdata, ndata), DATA_USHORT);
 			set_precision_switch();
 		}
 		return undo_get_data_ushort(fit, hist);
-	} else if (hist.type == DATA_FLOAT) {
+	} else if (hist->type == DATA_FLOAT) {
 		if (gfit.type != DATA_FLOAT) {
 			size_t ndata = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 			fit_replace_buffer(fit, ushort_buffer_to_float(fit->data, ndata), DATA_FLOAT);
@@ -297,13 +304,15 @@ int undo_display_data(int dir) {
 	switch (dir) {
 	case UNDO:
 		if (is_undo_available()) {
+			/* first we close all liveview dialog to avoid issue, especially with crop */
+			siril_close_preview_dialogs();
 			if (com.hist_current == com.hist_display) {
 				undo_save_state(&gfit, NULL);
 				com.hist_display--;
 			}
 			com.hist_display--;
 			siril_log_message(_("Undo: %s\n"), com.history[com.hist_display].history);
-			undo_get_data(&gfit, com.history[com.hist_display]);
+			undo_get_data(&gfit, &com.history[com.hist_display]);
 			invalidate_gfit_histogram();
 			invalidate_stats_from_fit(&gfit);
 			update_gfit_histogram_if_needed();
@@ -313,9 +322,11 @@ int undo_display_data(int dir) {
 		break;
 	case REDO:
 		if (is_redo_available()) {
+			/* first we close all liveview dialog to avoid issue, especially with crop */
+			siril_close_preview_dialogs();
 			siril_log_message(_("Redo: %s\n"), com.history[com.hist_display].history);
 			com.hist_display++;
-			undo_get_data(&gfit, com.history[com.hist_display]);
+			undo_get_data(&gfit, &com.history[com.hist_display]);
 			invalidate_gfit_histogram();
 			invalidate_stats_from_fit(&gfit);
 			update_gfit_histogram_if_needed();
