@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -39,6 +39,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_app_dirs.h"
+#include "core/siril_log.h"
 #include "core/exif.h"
 #include "io/conversion.h"
 #include "io/ser.h"
@@ -70,8 +71,8 @@ int round_to_int(double x) {
  * @return an integer
  */
 int roundf_to_int(float x) {
-	if (x <= INT_MIN + 0.5f) return INT_MIN;
-	if (x >= INT_MAX - 0.5f) return INT_MAX;
+	if (x <= (float)INT_MIN + 0.5f) return INT_MIN;
+	if (x >= (float)INT_MAX - 0.5f) return INT_MAX;
 	if (x >= 0.0f)
 		return (int)(x + 0.5f);
 	return (int)(x - 0.5f);
@@ -125,6 +126,31 @@ WORD roundf_to_WORD(float f) {
 	return (WORD)(f + 0.5f);
 }
 
+/**
+ * Round float value to a short
+ * @param f value to round
+ * @return a truncated and rounded short
+ */
+signed short roundf_to_short(float f) {
+	if (f < SHRT_MIN + 0.5f) return SHRT_MIN;
+	if (f >= SHRT_MAX - 0.5f) return SHRT_MAX;
+	return (signed short)(f + 0.5f);
+}
+
+/**
+ * Scale float value to a maximum value up to 2^32-1
+ * and return as guint32
+ * @param f value to scale
+ * @param max float range [0f..1f] scales to guint32 range [0..max]
+ * @return a guint32
+ */
+guint float_to_max_range(float f, guint max) {
+	f *= max;
+	if (f < 0.5f) return 0;
+	if (f >= max - 0.5f) return max;
+	return (guint)(f + 0.5f);
+}
+
 /**
  * Compute a ceiling factor
  * @param x the number to test
@@ -246,6 +272,16 @@ WORD float_to_ushort_range(float f) {
 	return roundf_to_WORD(f * USHRT_MAX_SINGLE);
 }
 
+/**
+ * convert a siril float [0, 1] to a signed short
+ * @param f value to convert
+ * @return the signed short equivalent
+ * (-SHRT_MAX - 1)
+ */
+signed short float_to_short_range(float f) {
+	return roundf_to_short((f * USHRT_MAX_SINGLE) - SHRT_MAX_SINGLE - 1);
+}
+
 /**
  * convert a siril float [0, 1] to an unsigned char
  * @param f value to convert
@@ -275,6 +311,7 @@ float ushort_to_float_bitpix(fits *fit, WORD value) {
  * @return
  */
 WORD *float_buffer_to_ushort(float *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
 	WORD *buf = malloc(ndata * sizeof(WORD));
 	if (!buf) {
 		PRINT_ALLOC_ERR;
@@ -286,6 +323,44 @@ WORD *float_buffer_to_ushort(float *buffer, size_t ndata) {
 	return buf;
 }
 
+/**
+ * convert a float type buffer into a signed short buffer
+ * @param buffer in float
+ * @param ndata
+ * @return
+ */
+signed short *float_buffer_to_short(float *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
+	signed short *buf = malloc(ndata * sizeof(signed short));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = float_to_short_range(buffer[i]);
+		}
+	}
+	return buf;
+}
+
+/**
+ * convert a ushort type buffer into a signed short buffer
+ * @param buffer in WORD
+ * @param ndata
+ * @return
+ */
+signed short *ushort_buffer_to_short(const WORD *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
+	signed short *buf = malloc(ndata * sizeof(signed short));
+	if (!buf) {
+		PRINT_ALLOC_ERR;
+	} else {
+		for (size_t i = 0; i < ndata; i++) {
+			buf[i] = (buffer[i] - SHRT_MAX_SINGLE - 1);
+		}
+	}
+	return buf;
+}
+
 /**
  * convert a BYTE type buffer into a float buffer
  * @param buffer in BYTE
@@ -293,6 +368,7 @@ WORD *float_buffer_to_ushort(float *buffer, size_t ndata) {
  * @return
  */
 float *uchar_buffer_to_float(BYTE *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
 	float *buf = malloc(ndata * sizeof(float));
 	if (!buf) {
 		PRINT_ALLOC_ERR;
@@ -311,6 +387,7 @@ float *uchar_buffer_to_float(BYTE *buffer, size_t ndata) {
  * @return
  */
 float *ushort_buffer_to_float(WORD *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
 	float *buf = malloc(ndata * sizeof(float));
 	if (!buf) {
 		PRINT_ALLOC_ERR;
@@ -329,6 +406,7 @@ float *ushort_buffer_to_float(WORD *buffer, size_t ndata) {
  * @return
  */
 float *ushort8_buffer_to_float(WORD *buffer, size_t ndata) {
+	if (!buffer) { siril_debug_print("buffer is NULL in data format conversion\n"); return NULL; }
 	float *buf = malloc(ndata * sizeof(float));
 	if (!buf) {
 		PRINT_ALLOC_ERR;
@@ -481,9 +559,9 @@ uint64_t change_endianness64(uint64_t x) {
  */
 uint64_t cpu_to_le64(uint64_t x) {
 #ifdef __BIG_ENDIAN__
-    return change_endianness64(x);
+	return change_endianness64(x);
 #else
-    return x;
+	return x;
 #endif
 }
 
@@ -494,9 +572,9 @@ uint64_t cpu_to_le64(uint64_t x) {
  */
 uint64_t cpu_to_be64(uint64_t x) {
 #ifdef __BIG_ENDIAN__
-    return x;
+	return x;
 #else
-    return change_endianness64(x);
+	return change_endianness64(x);
 #endif
 }
 
@@ -506,7 +584,7 @@ uint64_t cpu_to_be64(uint64_t x) {
  * @return value
  */
 uint64_t le64_to_cpu(uint64_t x) {
-    return cpu_to_le64(x);
+	return cpu_to_le64(x);
 }
 
 /**
@@ -515,7 +593,7 @@ uint64_t le64_to_cpu(uint64_t x) {
  * @return value
  */
 uint64_t be64_to_cpu(uint64_t x) {
-    return cpu_to_be64(x);
+	return cpu_to_be64(x);
 }
 
 /**
@@ -527,6 +605,25 @@ gboolean isrgb(fits *fit) {
 	return (fit->naxis == 3);
 }
 
+/**
+ * Converts a channel number from a color image to its name.
+ * @param channel the channel number [0, 2]
+ * @return the string containing the name of the channel's color
+ */
+const char *channel_number_to_name(int channel) {
+	switch (channel) {
+		case 0:
+			return _("red");
+		case 1:
+			return _("green");
+		case 2:
+			return _("blue");
+		default:
+			g_warning("unknown channel number %d\n", channel);
+			return "";
+	}
+}
+
 /**
  *  Searches for an extension '.something' in filename from the end
  *  @param filename input filename or path
@@ -559,6 +656,14 @@ const char *get_filename_ext(const char *filename) {
 	g_free(basename);
 
 	p = filename + len;
+	if (g_str_has_suffix(p, ".fz")) {
+		int l = strlen(p);
+		for (int i = l - 1 - 3; i >= 0; i--) {
+			if ((p[i] == '.')) {
+				return (p + i + 1);
+			}
+		}
+	}
 	dot = strrchr(p, '.');
 	if (!dot || dot == p) {
 		return NULL;
@@ -652,7 +757,9 @@ int is_readable_file(const char *filename) {
 	return 0;
 }
 
-static gchar forbidden_char[] = { '/', '\\', '"', '\'' };
+// https://en.wikipedia.org/wiki/Filename#Reserved_characters_and_words
+// we still allow for '.' though
+static gchar forbidden_char[] = { '/', '\\', '"', '\'' , '?', '%', '*', ':', '|', '<', '>', ';', '='};
 
 gboolean is_forbiden_in_filename(gchar c) {
 	for (int i = 0; i < G_N_ELEMENTS(forbidden_char); i++) {
@@ -665,20 +772,31 @@ gboolean is_forbiden_in_filename(gchar c) {
 gboolean file_name_has_invalid_chars(const char *name) {
 	if (!name)
 		return TRUE;	// NULL is kind of invalid
-	for (int i = 0; i < strlen(name); i++)
+	int l = strlen(name);
+	for (int i = 0; i < l; i++)
 		if (is_forbiden_in_filename(name[i]))
 			return TRUE;
 	return FALSE;
 }
 
+void replace_invalid_chars(char *name, char repl) {
+	if (!name)
+		return;	// NULL is kind of invalid
+	int l = strlen(name);
+	for (int i = 0; i < l; i++)
+		if (is_forbiden_in_filename(name[i]))
+			name[i] = repl;
+	return;
+}
+
 /** Tests if filename is the canonical name of a known file type
  *  If filename contains an extension, only this file name is tested, else all
  *  extensions are tested for the file name until one is found.
  * @param[in] filename the filename to test for.
  * @param[in] type is set according to the result of the test.
- * @param[out] realname (optionnal) is set according to the found file name: it
+ * @param[out] realname (optional) is set according to the found file name: it
  *  must be freed with when no longer needed.
- * @return 0 if sucess, 1 if error
+ * @return 0 if success, 1 if error
  */
 int stat_file(const char *filename, image_type *type, char **realname) {
 	int k;
@@ -926,6 +1044,7 @@ int siril_change_dir(const char *dir, gchar **err) {
 	}
 	return retval;
 }
+
 /**
  * If Windows OS, converts a filename from UTF-8 to the system codepage. Do nothing on other system
  * @param path to convert
@@ -936,7 +1055,7 @@ gchar *get_locale_filename(const gchar *path) {
 #ifdef _WIN32
 	str = g_win32_locale_filename_from_utf8(path);
 	if (!str) {
-		siril_log_color_message("Conversion of the filename to system codepage failed. Please consider removing all wide chars.\n", "red");
+		siril_log_color_message(_("Conversion of the filename to system codepage failed. Please consider removing all wide chars.\n"), "red");
 		return g_strdup(path);
 	}
 #else // _WIN32
@@ -945,134 +1064,6 @@ gchar *get_locale_filename(const gchar *path) {
 	return str;
 }
 
-#ifdef _WIN32
-static int ListSequences(const gchar *sDir, const char *sequence_name_to_select,
-		GtkComboBoxText *seqcombo, int *index_of_seq_to_load) {
-	WIN32_FIND_DATAW fdFile;
-	HANDLE hFind = NULL;
-	char sPath[2048];
-	char filename[256];
-	int number_of_loaded_sequences = 0;
-	wchar_t *wpath;
-
-	//Specify a file mask. *.seq = We want only seq file!
-	sprintf(sPath, "%s\\*.seq", sDir);
-
-	wpath = g_utf8_to_utf16(sPath, -1, NULL, NULL, NULL);
-	if (wpath == NULL)
-		return 1;
-
-	if ((hFind = FindFirstFileW(wpath, &fdFile)) == INVALID_HANDLE_VALUE) {
-		fprintf(stderr, "Path not found: [%s]\n", sDir);
-		g_free(wpath);
-		return 1;
-	}
-	g_free(wpath);
-
-	do {
-		//Is the entity a File or Folder?
-		if (!(fdFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
-			gchar *cFileName = g_utf16_to_utf8(fdFile.cFileName, -1, NULL, NULL, NULL);
-			if (cFileName == NULL) {
-				return 1;
-			}
-			sequence *seq = readseqfile(cFileName);
-			if (seq != NULL) {
-				strncpy(filename, cFileName, 255);
-				free_sequence(seq, TRUE);
-				gtk_combo_box_text_append_text(seqcombo, filename);
-				if (sequence_name_to_select
-						&& !strncmp(filename, sequence_name_to_select,
-								strlen(filename))) {
-					*index_of_seq_to_load = number_of_loaded_sequences;
-				}
-				++number_of_loaded_sequences;
-			}
-			g_free(cFileName);
-		}
-	} while (FindNextFileW(hFind, &fdFile)); //Find the next file.
-
-	FindClose(hFind);
-
-	return number_of_loaded_sequences;
-}
-#endif
-
-/** This method populates the sequence combo box with the sequences found in the CWD.
- *  If only one sequence is found, or if a sequence whose name matches the
- *  possibly NULL argument is found, it is automatically selected, which triggers
- *  its loading
- *  @param sequence_name_to_select the name of the input sequence
- *  @return 0 if success
- */
-int update_sequences_list(const char *sequence_name_to_select) {
-	GtkComboBoxText *seqcombo;
-	int number_of_loaded_sequences = 0;
-	int index_of_seq_to_load = -1;
-	char *seqname = NULL;
-
-	// clear the previous list
-	seqcombo = GTK_COMBO_BOX_TEXT(lookup_widget("sequence_list_combobox"));
-	gtk_combo_box_text_remove_all(seqcombo);
-
-	if (sequence_name_to_select) {
-	       if (g_str_has_suffix(sequence_name_to_select, ".seq"))
-		       seqname = strdup(sequence_name_to_select);
-	       else {
-		       seqname = malloc(strlen(sequence_name_to_select) + 5);
-		       sprintf(seqname, "%s.seq", sequence_name_to_select);
-	       }
-	}
-
-#ifdef _WIN32
-	number_of_loaded_sequences = ListSequences(com.wd, seqname, seqcombo, &index_of_seq_to_load);
-#else
-	struct dirent **list;
-	int i, n;
-
-	n = scandir(com.wd, &list, 0, alphasort);
-	if (n < 0)
-		perror("scandir");
-
-	for (i = 0; i < n; ++i) {
-		char *suf;
-
-		if ((suf = strstr(list[i]->d_name, ".seq")) && strlen(suf) == 4) {
-			sequence *seq = readseqfile(list[i]->d_name);
-			if (seq != NULL) {
-				free_sequence(seq, TRUE);
-				char *filename = list[i]->d_name;
-				gtk_combo_box_text_append_text(seqcombo, filename);
-				if (seqname && !strcmp(filename, seqname))
-					index_of_seq_to_load = number_of_loaded_sequences;
-				++number_of_loaded_sequences;
-			}
-		}
-	}
-	for (i = 0; i < n; i++)
-		free(list[i]);
-	free(list);
-#endif
-
-	if (seqname) free(seqname);
-
-	if (!number_of_loaded_sequences) {
-		fprintf(stderr, "No valid sequence found in CWD.\n");
-		return -1;
-	} else {
-		fprintf(stdout, "Loaded %d %s\n", number_of_loaded_sequences,
-				ngettext("sequence", "sequences", number_of_loaded_sequences));
-	}
-
-	if (number_of_loaded_sequences > 1 && index_of_seq_to_load < 0) {
-		gtk_combo_box_popup(GTK_COMBO_BOX(seqcombo));
-	} else if (index_of_seq_to_load >= 0)
-		gtk_combo_box_set_active(GTK_COMBO_BOX(seqcombo), index_of_seq_to_load);
-	else
-		gtk_combo_box_set_active(GTK_COMBO_BOX(seqcombo), 0);
-	return 0;
-}
-
 /**
  * Expands the ~ in filenames
  * @param[in] filename input filename
@@ -1155,6 +1146,8 @@ char *format_basename(char *root, gboolean can_free) {
 	}
 
 	char *appended = malloc(len + 2);
+	if (!appended)
+		return NULL;
 	sprintf(appended, "%s_", root);
 	if (can_free)
 		free(root);
@@ -1177,7 +1170,7 @@ float compute_slope(WORD *lo, WORD *hi) {
  * Try to get file info, i.e width and height
  * @param filename name of the file
  * @param pixbuf
- * @return a newly allocated and formated string containing dimension information or NULL
+ * @return a newly allocated and formatted string containing dimension information or NULL
  */
 gchar* siril_get_file_info(const gchar *filename, GdkPixbuf *pixbuf) {
 	int width, height;
@@ -1238,7 +1231,7 @@ char **glist_to_array(GList *list, int *arg_count) {
 		if (arg_count)
 			*arg_count = count;
 	}
-	char **array = malloc(count * sizeof(char *));
+	char **array = malloc((count + 1) * sizeof(char *));
 	if (!array) {
 		PRINT_ALLOC_ERR;
 		return NULL;
@@ -1246,6 +1239,7 @@ char **glist_to_array(GList *list, int *arg_count) {
 	GList *orig_list = list;
 	for (int i = 0; i < count && list; list = list->next, i++)
 		array[i] = g_strdup(list->data);
+	array[count] = NULL;
 	g_list_free_full(orig_list, g_free);
 	return array;
 }
@@ -1296,6 +1290,35 @@ void remove_spaces_from_str(gchar *s) {
 	} while((*s++ = *d++));
 }
 
+/**
+ * Removing trailing carriage return and newline characters in-place
+ * @param the string that will be modified, allocation unchanged
+ */
+void remove_trailing_eol(char *str) {
+	int i = strlen(str) - 1;
+	while (i >= 0 && (str[i] == '\r' || str[i] == '\n'))
+		str[i--] = '\0';
+}
+
+gboolean string_is_a_number(const char *str) {
+	if (str[0] != '-' && str[0] != '.' && (str[0] < '0' || str[0] > '9'))
+		return FALSE;
+	int i = 0;
+	gboolean had_a_dot = FALSE;
+	while (str[i] != '\0') {
+		if (str[i] == '.') {
+			if (had_a_dot)
+				return FALSE;
+			had_a_dot = TRUE;
+			i++;
+		}
+		else if (str[i] >= '0' && str[i] <= '9')
+			i++;
+		else return FALSE;
+	}
+	return TRUE;
+}
+
 #if !GLIB_CHECK_VERSION(2,68,0)
 /**
  * g_string_replace:
@@ -1360,3 +1383,185 @@ g_string_replace (GString     *string,
   return n;
 }
 #endif
+
+/*
+ * str_replace()
+ *
+ * Substring replacement utility function for use with basic null
+ * terminated char* strings that can't be handled with the glib
+ * functions of similar purpose.
+ *
+ * Calling function must initialize a char* to hold the result.
+ * result is malloc()ed here and is the responsibility of the calling
+ * function to free.
+ */
+
+char *str_replace(char *orig, char *rep, char *with) {
+    char *result; // the return string
+    char *ins;    // the next insert point
+    char *tmp;    // varies
+    int len_rep;  // length of rep (the string to remove)
+    int len_with; // length of with (the string to replace rep with)
+    int len_front; // distance between rep and end of last rep
+    int count;    // number of replacements
+
+
+    // sanity checks and initialization
+    if (!orig || !rep)
+        return NULL;
+    len_rep = strlen(rep);
+    if (len_rep == 0)
+        return NULL; // empty rep causes infinite loop during count
+    if (!with)
+        with = "";
+    len_with = strlen(with);
+
+    // count the number of replacements needed
+    ins = orig;
+    for (count = 0; (tmp = strstr(ins, rep)); ++count) {
+        ins = tmp + len_rep;
+    }
+
+    tmp = result = malloc(strlen(orig) + (len_with - len_rep) * count + 1);
+
+    if (!result)
+        return NULL;
+
+    // first time through the loop, all the variable are set correctly
+    // from here on,
+    //    tmp points to the end of the result string
+    //    ins points to the next occurrence of rep in orig
+    //    orig points to the remainder of orig after "end of rep"
+    while (count--) {
+        ins = strstr(orig, rep);
+        len_front = ins - orig;
+        tmp = strncpy(tmp, orig, len_front) + len_front;
+        tmp = strcpy(tmp, with) + len_with;
+        orig += len_front + len_rep; // move to next "end of rep"
+    }
+    strcpy(tmp, orig);
+    return result;
+}
+
+/**
+ * Deblanks a string and replace spaces with char c
+ * Multiple adjacent spaces are replaced only once
+ * @param s string to be deblanked
+ * @param c character to replace spaces
+ */
+void replace_spaces_from_str(gchar *s, gchar c) {
+	gchar *d = s;
+	do {
+		while (g_ascii_isspace(*d)) {
+			++d;
+		}
+		if ((d > s) && g_ascii_isspace(*(d - 1))) {
+			*(d-1) = c;
+			--d;
+		}
+	} while((*s++ = *d++));
+}
+
+/**
+ * Recomposes a string from words, with a space between each.
+ * @param words a NULL-terminated array of words
+ * @return a string to be freed with g_free()
+ */
+gchar *build_string_from_words(char **words) {
+	GString *str = g_string_new(words[0]);
+	int i = 1;
+	while (words[i]) {
+		g_string_append_printf(str, " %s", words[i]);
+		i++;
+	}
+	return g_string_free(str, FALSE);
+}
+
+/**
+ * Appends elements to an existing array.
+ * @param array an NULL-terminated array sufficiently allocated to contain the
+ * extra elements at its end. It will be NULL-terminated after append.
+ * @param elements a NULL-terminated array of elements to add to array
+ */
+void append_elements_to_array(char **array, char **elements) {
+	int i = 0, j = 0;
+	while (array[i]) i++;
+	while (elements[j])
+		array[i++] = elements[j++];
+	array[i] = NULL;
+}
+
+/**
+ * Get the file extension following the fz flag. If the file is
+ * compressed, fz is appended to the file extension.
+ * @param fz flag to know if the fz extension must be appended.
+ * @return a string that must not be freed
+ */
+static const gchar *ext[] = { ".fit.fz", ".fits.fz", ".fts.fz" };
+const gchar *get_com_ext(gboolean fz) {
+    if (fz) {
+        for (int i = 0; i < G_N_ELEMENTS(ext); i++) {
+            if (g_str_has_prefix(ext[i], com.pref.ext)) return ext[i];
+        }
+    }
+    return com.pref.ext;
+}
+
+/* converts FITS or WCS coordinates to display coordinates */
+int fits_to_display(double fx, double fy, double *dx, double *dy, int ry) {
+       if (fx < 0.0 || fy < 1.0 || fy > ry)
+               return 1;
+       // TODO: does ROWORDER change this?
+       *dx = fx;
+       *dy = ry - fy;
+       return 0;
+}
+
+/* converts display coordinates to FITS or WCS coordinates */
+int display_to_fits(double dx, double dy, double *fx, double *fy, int ry) {
+       if (dx < 0.0 || dy < 0.0 || dy > ry - 1)
+               return 1;
+       // TODO: does ROWORDER change this?
+       *fx = dx;
+       *fy = ry - dy;
+       return 0;
+}
+
+gchar *siril_file_chooser_get_filename(GtkFileChooser *chooser) {
+	gchar *filename = NULL;
+    gchar *uri = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+
+    if (uri != NULL) {
+        filename = g_filename_from_uri(uri, NULL, NULL);
+        if (filename != NULL) {
+        	char *scheme = g_uri_parse_scheme(uri);
+            if (g_strcmp0(scheme, "file") == 0) {
+                printf("The URI points to a local file.\n");
+            } else {
+                printf("The URI is non-local (scheme: %s).\n", uri);
+            }
+            g_free(scheme);
+        }
+        g_free(uri);
+    }
+    return filename;
+}
+
+GSList *siril_file_chooser_get_filenames(GtkFileChooser *chooser) {
+    GSList *filenames = NULL;
+    GSList *uris = gtk_file_chooser_get_uris(GTK_FILE_CHOOSER(chooser));
+
+    for (GSList *iter = uris; iter != NULL; iter = g_slist_next(iter)) {
+        const gchar *uri = (const gchar *)iter->data;
+        gchar *filename = g_filename_from_uri(uri, NULL, NULL);
+
+        if (filename != NULL) {
+        	printf("filename=%s\n", filename);
+            filenames = g_slist_append(filenames, filename);
+        }
+    }
+
+    g_slist_free(uris);
+
+    return filenames;
+}
