@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/command.h"
 #include "core/processing.h"
 #include "core/undo.h"
 #include "core/siril_world_cs.h"
@@ -41,6 +42,9 @@
 
 mouse_status_enum mouse_status;
 
+// caching widgets
+static GtkWidget *rotation_dlg = NULL;
+
 /* mouse callbacks */
 static double margin_size = 10;
 
@@ -52,13 +56,12 @@ static gboolean is_over_the_left_side_of_sel(pointi zoomed, double zoom) {
 				&& zoomed.y < com.selection.y + com.selection.h + s)
 			return TRUE;
 	}
-
 	return FALSE;
 }
 
 static gboolean is_over_the_right_side_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
-	double s = margin_size / zoom;
+	int s = round_to_int(margin_size / zoom);
 	if (zoomed.x > com.selection.x + com.selection.w - s
 				&& zoomed.x < com.selection.x + com.selection.w + s) {
 		if (zoomed.y > com.selection.y - s
@@ -70,7 +73,7 @@ static gboolean is_over_the_right_side_of_sel(pointi zoomed, double zoom) {
 
 static gboolean is_over_the_bottom_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
-	double s = margin_size / zoom;
+	int s = round_to_int(margin_size / zoom);
 	if (zoomed.y > com.selection.y + com.selection.h - s
 				&& zoomed.y < com.selection.y + com.selection.h + s) {
 		if (zoomed.x > com.selection.x - s
@@ -82,7 +85,7 @@ static gboolean is_over_the_bottom_of_sel(pointi zoomed, double zoom) {
 
 static gboolean is_over_the_top_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
-	double s = margin_size / zoom;
+	int s = round_to_int(margin_size / zoom);
 	if (zoomed.y > com.selection.y - s && zoomed.y < com.selection.y + s) {
 		if (zoomed.x > com.selection.x - s
 				&& zoomed.x < com.selection.x + com.selection.w + s)
@@ -93,7 +96,7 @@ static gboolean is_over_the_top_of_sel(pointi zoomed, double zoom) {
 
 static gboolean is_inside_of_sel(pointi zoomed, double zoom) {
 	if (com.selection.w == 0 && com.selection.h == 0) return FALSE;
-	double s = margin_size / zoom;
+	int s = round_to_int(margin_size / zoom);
 	if (zoomed.x >= com.selection.x + s && zoomed.x <= com.selection.x + com.selection.w - s) {
 		if (zoomed.y >= com.selection.y + s && zoomed.y <= com.selection.y + com.selection.h - s)
 			return TRUE;
@@ -106,21 +109,21 @@ static gboolean is_inside_of_sel(pointi zoomed, double zoom) {
 */
 static gboolean clamp2image(pointi* pt) {
 	gboolean x_inside = FALSE;
-	gboolean y_inside = FALSE;
 	if (pt->x < 0) {
 		pt->x = 0;
-	} else if (pt->x >= gfit.rx) {
+	} else if (pt->x > gfit.rx) {
 		pt->x = gfit.rx - 1;
 	} else {
-		x_inside = TRUE;
+		x_inside = pt->x < gfit.rx;
 	}
 
+	gboolean y_inside = FALSE;
 	if (pt->y < 0) {
 		pt->y = 0;
-	} else if (pt->y >= gfit.ry) {
+	} else if (pt->y > gfit.ry) {
 		pt->y = gfit.ry - 1;
 	} else {
-		y_inside = TRUE;
+		y_inside = pt->y < gfit.ry;
 	}
 	return x_inside && y_inside;
 }
@@ -137,7 +140,7 @@ void register_selection_update_callback(selection_update_callback f) {
 	}
 }
 
-void unregister_selection_update_callback(selection_update_callback f) {
+void unregister_selection_update_callback(const selection_update_callback f) {
 	int i;
 	for (i = 0; i < _nb_selection_callbacks; ++i) {
 		if (_registered_selection_callbacks[i] == f) {
@@ -192,31 +195,31 @@ void update_zoom_fit_button() {
  * Button events
  */
 
-static void do_popup_rgbmenu(GtkWidget *my_widget, GdkEventButton *event) {
-	static GtkMenu *menu = NULL;
-
-	if (!menu) {
-		menu = GTK_MENU(gtk_builder_get_object(gui.builder, "menurgb"));
-		gtk_menu_attach_to_widget(GTK_MENU(menu), my_widget, NULL);
-	}
-
-#if GTK_CHECK_VERSION(3, 22, 0)
-	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
-#else
-	int button, event_time;
-
-	if (event) {
-		button = event->button;
-		event_time = event->time;
-	} else {
-		button = 0;
-		event_time = gtk_get_current_event_time();
-	}
-
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
-			event_time);
-#endif
-}
+//static void do_popup_rgbmenu(GtkWidget *my_widget, GdkEventButton *event) {
+//	static GtkMenu *menu = NULL;
+//
+//	if (!menu) {
+//		menu = GTK_MENU(gtk_builder_get_object(gui.builder, "menurgb"));
+//		gtk_menu_attach_to_widget(GTK_MENU(menu), my_widget, NULL);
+//	}
+//
+//#if GTK_CHECK_VERSION(3, 22, 0)
+//	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
+//#else
+//	int button, event_time;
+//
+//	if (event) {
+//		button = event->button;
+//		event_time = event->time;
+//	} else {
+//		button = 0;
+//		event_time = gtk_get_current_event_time();
+//	}
+//
+//	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, button,
+//			event_time);
+//#endif
+//}
 
 /* Gray popup menu */
 
@@ -245,10 +248,10 @@ static void do_popup_graymenu(GtkWidget *my_widget, GdkEventButton *event) {
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_9_16")), gui.ratio == 9.0 / 16.0);
 	gtk_widget_set_sensitive(lookup_widget("menuitem_selection_preserve"), is_a_single_image_loaded || sequence_is_loaded());
 	gtk_widget_set_sensitive(lookup_widget("menuitem_selection_all"), is_a_single_image_loaded || sequence_is_loaded());
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_0")), com.pref.selection_guides == 0);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_2")), com.pref.selection_guides == 2);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_3")), com.pref.selection_guides == 3);
-	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_5")), com.pref.selection_guides == 5);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_0")), com.pref.gui.selection_guides == 0);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_2")), com.pref.gui.selection_guides == 2);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_3")), com.pref.gui.selection_guides == 3);
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(lookup_widget("menuitem_selection_guides_5")), com.pref.gui.selection_guides == 5);
 
 #if GTK_CHECK_VERSION(3, 22, 0)
 	gtk_menu_popup_at_pointer(GTK_MENU(menu), NULL);
@@ -267,10 +270,10 @@ static void do_popup_graymenu(GtkWidget *my_widget, GdkEventButton *event) {
 #endif
 }
 
-gboolean rgb_area_popup_menu_handler(GtkWidget *widget) {
-	do_popup_rgbmenu(widget, NULL);
-	return TRUE;
-}
+//gboolean rgb_area_popup_menu_handler(GtkWidget *widget) {
+//	do_popup_rgbmenu(widget, NULL);
+//	return TRUE;
+//}
 
 void init_mouse() {
 	mouse_status = MOUSE_ACTION_SELECT_REG_AREA;
@@ -315,10 +318,12 @@ void enforce_ratio_and_clamp() {
 	com.selection.y = set_int_in_interval(com.selection.y, 0, gfit.ry - com.selection.h);
 }
 
-
 gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 		GdkEventButton *event, gpointer user_data) {
 
+	if (!rotation_dlg) {
+		rotation_dlg = lookup_widget("rotation_dialog");
+	}
 	/* when double clicking on drawing area (if no images loaded)
 	 * you can load an image This feature is in GIMP and I really
 	 * love it: lazy world :).
@@ -327,6 +332,7 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 		if (event->button == GDK_BUTTON_PRIMARY
 				&& event->type == GDK_DOUBLE_BUTTON_PRESS) {
 			header_open_button_clicked();
+			launch_clipboard_survey();
 		}
 		return FALSE;
 	}
@@ -354,18 +360,6 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 			}
 		}
 
-		/* click on RGB image */
-		if (widget == gui.view[RGB_VPORT].drawarea) {
-			if (event->button == GDK_BUTTON_PRIMARY) {	// left click
-				siril_message_dialog(GTK_MESSAGE_INFO, _("Only for visualization"),
-						_("The RGB tab is only for visualization. Operations must be done on R, G, and B channels"));
-			} else if (event->button == GDK_BUTTON_SECONDARY) {	// right click
-				do_popup_rgbmenu(widget, event);
-				return TRUE;
-			}
-			return FALSE;
-		}
-
 		/* else, click on gray image */
 		if (event->button == GDK_BUTTON_PRIMARY) {	// left click
 			if (mouse_status == MOUSE_ACTION_SELECT_REG_AREA) {
@@ -418,7 +412,7 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 				redraw(REDRAW_OVERLAY);
 			} else if (mouse_status == MOUSE_ACTION_DRAW_SAMPLES) {
 				point pt;
-				int radius = get_sample_radius();
+				int radius = get_background_sample_radius();
 
 				pt.x = (gdouble) zoomed.x;
 				pt.y = (gdouble) zoomed.y;
@@ -431,19 +425,22 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 					redraw_previews();
 				}
 			} else if (mouse_status == MOUSE_ACTION_PHOTOMETRY) {
-				int s = com.pref.phot_set.outer;
+				int s = com.pref.phot_set.outer * 1.2;
 				rectangle area = { zoomed.x - s, zoomed.y - s, s * 2, s * 2 };
-				if (area.x - area.w > 0 && area.x + area.w < gfit.rx
-						&& area.y - area.h > 0 && area.y + area.h < gfit.ry) {
-					com.qphot = psf_get_minimisation(&gfit, gui.cvport, &area, TRUE, com.pref.phot_set.force_radius, TRUE, TRUE);
-					if (com.qphot) {
-						com.qphot->xpos = com.qphot->x0 + area.x;
+				if (zoomed.x - area.w > 0 && zoomed.x + area.w < gfit.rx
+						&& zoomed.y - area.h > 0 && zoomed.y + area.h < gfit.ry) {
+
+					struct phot_config *ps = phot_set_adjusted_for_image(&gfit);
+					gui.qphot = psf_get_minimisation(&gfit, select_vport(gui.cvport), &area, TRUE, ps, TRUE, com.pref.starfinder_conf.profile, NULL);
+					free(ps);
+					if (gui.qphot) {
+						gui.qphot->xpos = gui.qphot->x0 + area.x;
 						if (gfit.top_down)
-							com.qphot->ypos = com.qphot->y0 + area.y;
+							gui.qphot->ypos = gui.qphot->y0 + area.y;
 						else
-							com.qphot->ypos = area.y + area.h - com.qphot->y0;
+							gui.qphot->ypos = area.y + area.h - gui.qphot->y0;
 						redraw(REDRAW_OVERLAY);
-						popup_psf_result(com.qphot, &area);
+						popup_psf_result(gui.qphot, &area, &gfit);
 					}
 				}
 			}
@@ -462,6 +459,17 @@ gboolean on_drawingarea_button_press_event(GtkWidget *widget,
 					redraw(REDRAW_OVERLAY);
 					redraw_previews();
 				}
+			} else if (mouse_status == MOUSE_ACTION_PHOTOMETRY) {
+				if (sequence_is_loaded()) {
+					int s = com.pref.phot_set.outer * 1.2;
+					rectangle area = { zoomed.x - s, zoomed.y - s, s * 2, s * 2 };
+					if (zoomed.x - area.w > 0 && zoomed.x + area.w < gfit.rx
+							&& zoomed.y - area.h > 0 && zoomed.y + area.h < gfit.ry) {
+						memcpy(&com.selection, &area, sizeof(rectangle));
+						process_seq_psf(0);
+						delete_selected_area();
+					}
+				}
 			}
 		}
 	}
@@ -485,7 +493,7 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 			gui.drawing = FALSE;
 			/* finalize selection rectangle coordinates */
 			if (!gui.freezeX) {
-				if (zoomed.x > gui.start.x) {
+				if (zoomed.x >= gui.start.x) {
 					com.selection.x = gui.start.x;
 					com.selection.w = zoomed.x - com.selection.x;
 				} else {
@@ -494,7 +502,7 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 				}
 			}
 			if (!gui.freezeY) {
-				if (zoomed.y > gui.start.y) {
+				if (zoomed.y >= gui.start.y) {
 					com.selection.y = gui.start.y;
 					com.selection.h = zoomed.y - com.selection.y;
 				} else {
@@ -502,6 +510,18 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 					com.selection.h = gui.start.y - zoomed.y;
 				}
 			}
+			// Clicking in displayed psf selects star in list if the DynamicPSF dialog is open
+			GtkWidget *dynpsf_dlg = lookup_widget("stars_list_window");
+			if (((com.selection.w == 0 || com.selection.w == 1) && (com.selection.h == 0 || com.selection.h == 1)) && gtk_widget_is_visible(dynpsf_dlg)) {
+				set_iter_of_clicked_psf((double) com.selection.x, (double) com.selection.y);
+			}
+
+			// never let selection be null if rotation_dlg is visible
+			// reinstate full image instead
+			if (!gui.freezeX && com.selection.w == 0 && gtk_widget_is_visible(rotation_dlg))
+				com.selection.w = gfit.rx;
+			if (!gui.freezeY && com.selection.h == 0 && gtk_widget_is_visible(rotation_dlg))
+				com.selection.h = gfit.ry;
 
 			if (gui.freezeX && gui.freezeY) { // Move selection
 				com.selection.x = (zoomed.x - gui.start.x) + gui.origin.x;
@@ -529,12 +549,10 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 		}
 	} else if (event->button == GDK_BUTTON_MIDDLE) {	// middle click
 		if (inside) {
-			double dX, dY, w, h;
-
-			dX = 1.5 * com.pref.phot_set.outer;
-			dY = dX;
-			w = 3 * com.pref.phot_set.outer;
-			h = w;
+			double dX = 1.5 * com.pref.phot_set.outer;
+			double dY = dX;
+			double w = 3 * com.pref.phot_set.outer;
+			double h = w;
 
 			if ((dX <= zoomed.x) && (dY <= zoomed.y)
 					&& (zoomed.x - dX + w < gfit.rx)
@@ -550,7 +568,7 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 		}
 
 	} else if (event->button == GDK_BUTTON_SECONDARY) {	// right click
-		if (mouse_status != MOUSE_ACTION_DRAW_SAMPLES) {
+		if (mouse_status != MOUSE_ACTION_DRAW_SAMPLES && mouse_status != MOUSE_ACTION_PHOTOMETRY) {
 			do_popup_graymenu(widget, NULL);
 		}
 	}
@@ -561,6 +579,9 @@ gboolean on_drawingarea_button_release_event(GtkWidget *widget,
 
 gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 		GdkEventMotion *event, gpointer user_data) {
+	if (!rotation_dlg) {
+		rotation_dlg = lookup_widget("rotation_dialog");
+	}
 	if ((!single_image_is_loaded() && !sequence_is_loaded())
 			|| gfit.type == DATA_UNSUPPORTED) {
 		return FALSE;
@@ -579,98 +600,113 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 	//		event->x, event->y, zoomed.x, zoomed.y, inside ? "" : " not");
 
 	const gchar *label_density_names[] = { "labeldensity_red", "labeldensity_green", "labeldensity_blue", "labeldensity_rgb" };
-	const gchar *label_wcs_names[] = { "labelwcs_red", "labelwcs_green", "labelwcs_blue" };
-	static GtkLabel *labels_wcs[sizeof label_wcs_names / sizeof(gchar *)] = { 0 };
-	static GtkLabel *labels_density[sizeof label_density_names / sizeof(gchar *)] = { 0 };
+	const gchar *label_wcs_names[] = { "labelwcs_red", "labelwcs_green", "labelwcs_blue", "labelwcs_rgb" };
+	static GtkLabel *labels_wcs[G_N_ELEMENTS(label_wcs_names)] = { 0 };
+	static GtkLabel *labels_density[G_N_ELEMENTS(label_density_names)] = { 0 };
 	if (!labels_wcs[0]) {
-		for (int i = 0; i < sizeof label_wcs_names / sizeof(gchar *); i++)
+		for (int i = 0; i < G_N_ELEMENTS(label_wcs_names); i++)
 			labels_wcs[i] = GTK_LABEL(lookup_widget(label_wcs_names[i]));
-		for (int i = 0; i < sizeof label_density_names / sizeof(gchar *); i++)
+		for (int i = 0; i < G_N_ELEMENTS(label_density_names); i++)
 			labels_density[i] = GTK_LABEL(lookup_widget(label_density_names[i]));
 	}
 
 	gboolean blank_density_cvport = TRUE, blank_wcs_cvport = TRUE;
+	gtk_label_set_text(GTK_LABEL(lookup_widget("label-rgb")), "");
 
-	if (gui.cvport < RGB_VPORT) {
-		if (inside) {
+	if (inside) {
+		if (gui.cvport == RGB_VPORT) {
 			static gchar buffer[256] = { 0 };
-			static gchar wcs_buffer[256] = { 0 };
-			static gchar format[256] = { 0 };
-			int coords_width = 3;
-
-			if (gfit.rx >= 1000 || gfit.ry >= 1000)
-				coords_width = 4;
-			if (gfit.rx >= 10000 || gfit.ry >= 10000)
-				coords_width = 5;
-			if (gfit.type == DATA_USHORT && gfit.pdata[gui.cvport] != NULL) {
-				int val_width = 3;
-				char *format_base_ushort = "x: %%.%dd y: %%.%dd (=%%.%dd)";
-				if (gfit.hi >= 1000)
-					val_width = 4;
-				if (gfit.hi >= 10000)
-					val_width = 5;
-				g_sprintf(format, format_base_ushort,
-						coords_width, coords_width, val_width);
-				g_sprintf(buffer, format, zoomed.x, zoomed.y,
-						gfit.pdata[gui.cvport][gfit.rx * (gfit.ry - zoomed.y - 1)
-						+ zoomed.x]);
-			} else if (gfit.type == DATA_FLOAT && gfit.fpdata[gui.cvport] != NULL) {
-				char *format_base_float = "x: %%.%dd y: %%.%dd (=%%f)";
-				g_sprintf(format, format_base_float,
-						coords_width, coords_width);
-				g_sprintf(buffer, format, zoomed.x, zoomed.y,
-						gfit.fpdata[gui.cvport][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x]);
+			if (gfit.type == DATA_USHORT) {
+				g_sprintf(buffer, "<span foreground=\"#FF0000\"><b>R=%.3lf%%</b></span>\n<span foreground=\"#00FF00\"><b>G=%.3lf%%</b></span>\n<span foreground=\"#0054FF\"><b>B=%.3lf%%</b></span>",
+						gfit.pdata[RLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0,
+						gfit.pdata[GLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0,
+						gfit.pdata[BLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0);
+			} else if (gfit.type == DATA_FLOAT) {
+				g_sprintf(buffer, "<span foreground=\"#FF0000\"><b>R=%.3lf%%</b></span>\n<span foreground=\"#00FF00\"><b>G=%.3lf%%</b></span>\n<span foreground=\"#0054FF\"><b>B=%.3lf%%</b></span>",
+						gfit.fpdata[RLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0,
+						gfit.fpdata[GLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0,
+						gfit.fpdata[BLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0);
 			}
-			if (has_wcs(&gfit)) {
-				double world_x, world_y;
-				pix2wcs(&gfit, (double) zoomed.x, (double) (gfit.ry - zoomed.y - 1), &world_x, &world_y);
-				if (world_x >= 0.0 && !isnan(world_x) && !isnan(world_y)) {
-					SirilWorldCS *world_cs;
+			gtk_label_set_markup(GTK_LABEL(lookup_widget("label-rgb")), buffer);
+		}
+		static gchar buffer[256] = { 0 };
+		static gchar format[256] = { 0 };
+		int coords_width = 3;
+		int vport = select_vport(gui.cvport);
 
-					world_cs = siril_world_cs_new_from_a_d(world_x, world_y);
-					if (world_cs) {
-						gchar *ra = siril_world_cs_alpha_format(world_cs, "%02dh%02dm%02ds");
-						gchar *dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%02d\"");
-						g_sprintf(wcs_buffer, "α: %s δ: %s", ra, dec);
-
-						gtk_label_set_text(labels_wcs[gui.cvport], wcs_buffer);
-						blank_wcs_cvport = FALSE;
-
-						g_free(ra);
-						g_free(dec);
-						siril_world_cs_unref(world_cs);
-					}
-				}
+		if (gfit.rx >= 1000 || gfit.ry >= 1000)
+			coords_width = 4;
+		if (gfit.rx >= 10000 || gfit.ry >= 10000)
+			coords_width = 5;
+		if (gfit.type == DATA_USHORT && gfit.pdata[vport] != NULL) {
+			int val_width = 3;
+			char *format_base_ushort;
+			if (gui.cvport < RGB_VPORT) {
+				format_base_ushort = "x: %%.%dd y: %%.%dd (=%%.%dd)";
+			} else {
+				format_base_ushort = "x: %%.%dd y: %%.%dd";
 			}
-
-			if (buffer[0] != '\0') {
-				gtk_label_set_text(labels_density[gui.cvport], buffer);
-				blank_density_cvport = FALSE;
+			if (gfit.hi >= 1000)
+				val_width = 4;
+			if (gfit.hi >= 10000)
+				val_width = 5;
+			g_sprintf(format, format_base_ushort,
+					coords_width, coords_width, val_width);
+			if (gui.cvport < RGB_VPORT) {
+				g_sprintf(buffer, format, zoomed.x, zoomed.y, gfit.pdata[vport][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x]);
+			} else {
+				g_sprintf(buffer, format, zoomed.x, zoomed.y);
+			}
+		} else if (gfit.type == DATA_FLOAT && gfit.fpdata[vport] != NULL) {
+			char *format_base_float;
+			if (gui.cvport < RGB_VPORT) {
+				format_base_float = "x: %%.%dd y: %%.%dd (=%%f)";
+			} else {
+				format_base_float = "x: %%.%dd y: %%.%dd";
+			}
+			g_sprintf(format, format_base_float, coords_width, coords_width);
+			if (gui.cvport < RGB_VPORT) {
+				g_sprintf(buffer, format, zoomed.x, zoomed.y, gfit.fpdata[vport][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x]);
+			} else {
+				g_sprintf(buffer, format, zoomed.x, zoomed.y);
 			}
 		}
-		if (blank_wcs_cvport && gtk_label_get_text(labels_wcs[gui.cvport])[0] != '\0')
-			gtk_label_set_text(labels_wcs[gui.cvport], " ");
-	} else {
-		static gchar buffer[256] = { 0 };
 
-		if (inside) {
-			if (gfit.type == DATA_USHORT) {
-				g_sprintf(buffer, "R=%.3lf%%/G=%.3lf%%/B=%.3lf%%",
-										gfit.pdata[RLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0,
-										gfit.pdata[GLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0,
-										gfit.pdata[BLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] / USHRT_MAX_DOUBLE * 100.0
-										);
-			} else if (gfit.type == DATA_FLOAT) {
-				g_sprintf(buffer, "R=%.3lf%%/G=%.3lf%%/B=%.3lf%%",
-										gfit.fpdata[RLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0,
-										gfit.fpdata[GLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0,
-										gfit.fpdata[BLAYER][gfit.rx * (gfit.ry - zoomed.y - 1) + zoomed.x] * 100.0
-										);
-			}
+		if (buffer[0] != '\0') {
 			gtk_label_set_text(labels_density[gui.cvport], buffer);
 			blank_density_cvport = FALSE;
 		}
+
+		static gchar wcs_buffer[256] = { 0 };
+		if (has_wcs(&gfit)) {
+			double world_x, world_y;
+			pix2wcs(&gfit, (double) zoomed.x, (double) (gfit.ry - zoomed.y - 1), &world_x, &world_y);
+			if (world_x >= 0.0 && !isnan(world_x) && !isnan(world_y)) {
+				SirilWorldCS *world_cs = siril_world_cs_new_from_a_d(world_x, world_y);
+				if (world_cs) {
+					gchar *ra, *dec;
+					if (com.pref.gui.show_deciasec) {
+						ra = siril_world_cs_alpha_format(world_cs, "%02dh%02dm%04.1lfs");
+						dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%04.1lf\"");
+					} else {
+						ra = siril_world_cs_alpha_format(world_cs, "%02dh%02dm%02ds");
+						dec = siril_world_cs_delta_format(world_cs, "%c%02d°%02d\'%02d\"");
+					}
+					g_sprintf(wcs_buffer, "α: %s δ: %s", ra, dec);
+
+					gtk_label_set_text(labels_wcs[gui.cvport], wcs_buffer);
+					blank_wcs_cvport = FALSE;
+
+					g_free(ra);
+					g_free(dec);
+					siril_world_cs_unref(world_cs);
+				}
+			}
+		}
 	}
+
+	if (blank_wcs_cvport && gtk_label_get_text(labels_wcs[gui.cvport])[0] != '\0')
+		gtk_label_set_text(labels_wcs[gui.cvport], " ");
 	if (blank_density_cvport && gtk_label_get_text(labels_density[gui.cvport])[0] != '\0')
 		gtk_label_set_text(labels_density[gui.cvport], " ");
 
@@ -718,19 +754,20 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 	}
 
 	/* don't change cursor if thread is running */
-	if (com.run_thread) return FALSE;
+	if (get_thread_run()) return FALSE;
 
-	if (inside && gui.cvport < RGB_VPORT) {
+	if (inside) {
 		if (mouse_status == MOUSE_ACTION_DRAW_SAMPLES) {
 			set_cursor("cell");
 		} else {
+			// The order matters if the selection is so small that edge detection overlaps
+			// and need to be the same as in the on_drawingarea_button_press_event()
+			gboolean right = is_over_the_right_side_of_sel(zoomed, zoom);
+			gboolean left = is_over_the_left_side_of_sel(zoomed, zoom);
+			gboolean bottom = is_over_the_bottom_of_sel(zoomed, zoom);
+			gboolean top = is_over_the_top_of_sel(zoomed, zoom);
+
 			if (!gui.drawing && !gui.translating) {
-				// The order matters if the selection is so small that edge detection overlaps
-				// and need to be the same as in the on_drawingarea_button_press_event()
-				gboolean right = is_over_the_right_side_of_sel(zoomed, zoom);
-				gboolean left = is_over_the_left_side_of_sel(zoomed, zoom);
-				gboolean bottom = is_over_the_bottom_of_sel(zoomed, zoom);
-				gboolean top = is_over_the_top_of_sel(zoomed, zoom);
 				if (bottom && right) {
 					set_cursor("se-resize");
 				} else if (top && right) {
@@ -753,7 +790,7 @@ gboolean on_drawingarea_motion_notify_event(GtkWidget *widget,
 					set_cursor("crosshair");
 				}
 			} else {
-				if ((event->state & get_primary()) || is_inside_of_sel(zoomed, zoom)) {
+				if ((event->state & get_primary()) || (is_inside_of_sel(zoomed, zoom))) {
 					set_cursor("all-scroll");
 				} else {
 					set_cursor("crosshair");
@@ -779,17 +816,17 @@ void on_drawingarea_leave_notify_event(GtkWidget *widget, GdkEvent *event,
 	}
 }
 
-static const gchar *label_zoom[] = { "labelzoom_red", "labelzoom_green", "labelzoom_blue" };
+static const gchar *label_zoom[] = { "labelzoom_red", "labelzoom_green", "labelzoom_blue", "labelzoom_rgb" };
 
 static gboolean set_label_zoom_text_idle(gpointer p) {
 	const gchar *txt = (const gchar *) p;
 	static GtkLabel *labels[sizeof label_zoom] = { NULL };
 	if (!labels[0]) {
-		for (int i = 0; i < sizeof label_zoom / sizeof (gchar *); i++)
+		for (int i = 0; i < G_N_ELEMENTS(label_zoom); i++)
 			labels[i] = GTK_LABEL(lookup_widget(label_zoom[i]));
 	}
 	if (gfit.naxes[2] == 3)
-		for (int i = 0; i < sizeof label_zoom / sizeof (gchar *); i++)
+		for (int i = 0; i < G_N_ELEMENTS(label_zoom); i++)
 			gtk_label_set_text(labels[i], txt);
 	else gtk_label_set_text(labels[0], txt);
 	return FALSE;
@@ -798,7 +835,7 @@ static gboolean set_label_zoom_text_idle(gpointer p) {
 void update_zoom_label() {
 	static gchar zoom_buffer[256] = { 0 };
 	double zoom = gui.zoom_value;
-	if ((single_image_is_loaded() || sequence_is_loaded()) && gui.cvport < RGB_VPORT) {
+	if ((single_image_is_loaded() || sequence_is_loaded())) {
 		if (zoom < 0)
 			zoom = get_zoom_val();
 		g_sprintf(zoom_buffer, "%d%%", (int) (zoom * 100.0));
