@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -41,6 +41,7 @@
 #include "gui/message_dialog.h"
 #include "gui/dialog_preview.h"
 #include "gui/conversion.h"
+#include "gui/dialogs.h"
 
 #include "open_dialog.h"
 
@@ -62,6 +63,10 @@ static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
 		gtk_file_chooser_set_filter(file_chooser, f);
 }
 
+static void set_single_filter_dialog(GtkFileChooser *chooser, const gchar *name, const gchar *filter) {
+	gtk_filter_add(chooser, name, filter, TRUE);
+}
+
 static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 	GString *all_filter = NULL;
 	gchar *fits_filter = FITS_EXTENSIONS;
@@ -69,8 +74,8 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 	gchar *pic_filter = "*.pic;*.PIC";
 	gchar *ser_filter = "*.ser;*.SER";
 	if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
-		gtk_filter_add(chooser, _("FITS Files (*.fit, *.fits, *.fts, *.fits.fz)"),
-				fits_filter, gui.filter == TYPEFITS);
+		gtk_filter_add(chooser, _("FITS Files (*.fit, *.fits, *.fts, *.fit.fz, *.fits.fz, *.fts.fz)"),
+				fits_filter, gui.file_ext_filter == TYPEFITS);
 	} else {
 		all_filter = g_string_new(fits_filter);
 	}
@@ -96,13 +101,10 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 			g_free(upcase);
 		}
 		raw[strlen(raw) - 1] = '\0';
-		if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
-			gtk_filter_add(chooser, _("RAW DSLR Camera Files"), raw,
-				gui.filter == TYPERAW);
-		} else {
-			all_filter = g_string_append(all_filter, ";");
-			all_filter = g_string_append(all_filter, raw);
-		}
+
+		all_filter = g_string_append(all_filter, ";");
+		all_filter = g_string_append(all_filter, raw);
+
 
 		free(raw);
 
@@ -138,18 +140,18 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 		graphics_filter = g_string_free(s_pattern, FALSE);
 		if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
 			gtk_filter_add(chooser, graphics_supported, graphics_filter,
-					gui.filter == TYPEBMP || gui.filter == TYPEJPG
-							|| gui.filter == TYPEPNG || gui.filter == TYPETIFF);
+					gui.file_ext_filter == TYPEBMP || gui.file_ext_filter == TYPEJPG ||
+					gui.file_ext_filter == TYPEPNG || gui.file_ext_filter == TYPETIFF);
 
 			/* NETPBM FILES */
 			gtk_filter_add(chooser, _("Netpbm Files (*.ppm, *.pnm, *.pgm)"),
-					netpbm_filter, gui.filter == TYPEPNM);
+					netpbm_filter, gui.file_ext_filter == TYPEPNM);
 			/* IRIS FILES */
 			gtk_filter_add(chooser, _("IRIS PIC Files (*.pic)"), pic_filter,
-					gui.filter == TYPEPIC);
+					gui.file_ext_filter == TYPEPIC);
 			/* SER FILES */
 			gtk_filter_add(chooser, _("SER files (*.ser)"), ser_filter,
-					gui.filter == TYPESER);
+					gui.file_ext_filter == TYPESER);
 		} else {
 			all_filter = g_string_append(all_filter, ";");
 			all_filter = g_string_append(all_filter, graphics_filter);
@@ -186,7 +188,7 @@ static void set_filters_dialog(GtkFileChooser *chooser, int whichdial) {
 
 		if (whichdial != OD_CONVERT && whichdial != OD_OPEN) {
 		gtk_filter_add(chooser, _("Film Files (*.avi, *.mpg, ...)"), film_filter,
-				gui.filter == TYPEAVI);
+				gui.file_ext_filter == TYPEAVI);
 		} else {
 			all_filter = g_string_append(all_filter, ";");
 			all_filter = g_string_append(all_filter, film_filter);
@@ -237,9 +239,32 @@ static void opendial(int whichdial) {
 	case OD_FLAT:
 	case OD_DARK:
 	case OD_OFFSET:
+	case OD_FLATLIB:
+	case OD_DARKLIB:
+	case OD_OFFSETLIB:
 		widgetdialog = siril_file_chooser_open(control_window, GTK_FILE_CHOOSER_ACTION_OPEN);
 		dialog = GTK_FILE_CHOOSER(widgetdialog);
 		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gtk_file_chooser_set_local_only(dialog, FALSE);
+		if (whichdial == OD_FLATLIB) {
+			if (com.pref.prepro.flat_lib != NULL) {
+				gchar *path = g_path_get_dirname(com.pref.prepro.flat_lib);
+				gtk_file_chooser_set_current_folder(dialog, path);
+				g_free(path);
+			}
+		} else if (whichdial == OD_DARKLIB) {
+			if (com.pref.prepro.dark_lib != NULL) {
+				gchar *path = g_path_get_dirname(com.pref.prepro.dark_lib);
+				gtk_file_chooser_set_current_folder(dialog, path);
+				g_free(path);
+			}
+		} else if (whichdial == OD_OFFSETLIB) {
+			if (com.pref.prepro.bias_lib != NULL) {
+				gchar *path = g_path_get_dirname(com.pref.prepro.bias_lib);
+				gtk_file_chooser_set_current_folder(dialog, path);
+				g_free(path);
+			}
+		}
 		gtk_file_chooser_set_select_multiple(dialog, FALSE);
 		set_filters_dialog(dialog, whichdial);
 		siril_file_chooser_add_preview(dialog, preview);
@@ -248,12 +273,14 @@ static void opendial(int whichdial) {
 		widgetdialog = siril_file_chooser_open(control_window, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
 		dialog = GTK_FILE_CHOOSER(widgetdialog);
 		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gtk_file_chooser_set_local_only(dialog, FALSE);
 		gtk_file_chooser_set_select_multiple(dialog, FALSE);
 		break;
 	case OD_OPEN:
 		widgetdialog = siril_file_chooser_open(control_window, GTK_FILE_CHOOSER_ACTION_OPEN);
 		dialog = GTK_FILE_CHOOSER(widgetdialog);
 		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gtk_file_chooser_set_local_only(dialog, FALSE);
 		gtk_file_chooser_set_select_multiple(dialog, FALSE);
 		set_filters_dialog(dialog, whichdial);
 		siril_file_chooser_add_preview(dialog, preview);
@@ -263,8 +290,17 @@ static void opendial(int whichdial) {
 		widgetdialog = siril_file_chooser_add(control_window, GTK_FILE_CHOOSER_ACTION_OPEN);
 		dialog = GTK_FILE_CHOOSER(widgetdialog);
 		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gtk_file_chooser_set_local_only(dialog, FALSE);
 		gtk_file_chooser_set_select_multiple(dialog, TRUE);
 		set_filters_dialog(dialog, whichdial);
+		break;
+	case OD_BADPIXEL:
+		widgetdialog = siril_file_chooser_add(control_window, GTK_FILE_CHOOSER_ACTION_OPEN);
+		dialog = GTK_FILE_CHOOSER(widgetdialog);
+		gtk_file_chooser_set_current_folder(dialog, com.wd);
+		gtk_file_chooser_set_local_only(dialog, FALSE);
+		gtk_file_chooser_set_select_multiple(dialog, FALSE);
+		set_single_filter_dialog(dialog, _("Cosmetic correction file (*.lst)"), "*.lst;*.LST");
 	}
 
 	if (!dialog)
@@ -278,11 +314,12 @@ static void opendial(int whichdial) {
 		gchar *filename, *err;
 		gboolean anything_loaded;
 		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
-		GtkEntry *flat_entry, *dark_entry, *bias_entry;
+		GtkEntry *flat_entry, *dark_entry, *bias_entry, *bad_pixel_entry;
+		GtkEntry *flatlib_entry, *darklib_entry, *biaslib_entry;
 		GtkToggleButton *flat_button, *dark_button, *bias_button;
 		GtkWidget *pbutton;
 
-		filename = gtk_file_chooser_get_filename(chooser);
+		filename = siril_file_chooser_get_filename(chooser);
 		if (!filename)
 			goto wait;
 
@@ -297,6 +334,10 @@ static void opendial(int whichdial) {
 		flat_entry = GTK_ENTRY(lookup_widget("flatname_entry"));
 		dark_entry = GTK_ENTRY(lookup_widget("darkname_entry"));
 		bias_entry = GTK_ENTRY(lookup_widget("offsetname_entry"));
+		flatlib_entry = GTK_ENTRY(lookup_widget("flatlib_entry"));
+		darklib_entry = GTK_ENTRY(lookup_widget("darklib_entry"));
+		biaslib_entry = GTK_ENTRY(lookup_widget("biaslib_entry"));
+		bad_pixel_entry = GTK_ENTRY(lookup_widget("pixelmap_entry"));
 
 		flat_button = GTK_TOGGLE_BUTTON(lookup_widget("useflat_button"));
 		dark_button = GTK_TOGGLE_BUTTON(lookup_widget("usedark_button"));
@@ -323,8 +364,23 @@ static void opendial(int whichdial) {
 			gtk_widget_set_sensitive(pbutton, anything_loaded);
 			break;
 
+		case OD_FLATLIB:
+			gtk_entry_set_text(flatlib_entry, filename);
+			break;
+
+		case OD_DARKLIB:
+			gtk_entry_set_text(darklib_entry, filename);
+			break;
+
+		case OD_OFFSETLIB:
+			gtk_entry_set_text(biaslib_entry, filename);
+			break;
+
 		case OD_CWD:
 			if (!siril_change_dir(filename, &err)) {
+				if (com.pref.wd)
+					g_free(com.pref.wd);
+				com.pref.wd = g_strdup(com.wd);
 				writeinitfile();
 				set_GUI_CWD();
 			} else {
@@ -340,11 +396,16 @@ static void opendial(int whichdial) {
 			break;
 
 		case OD_CONVERT:
-			list = gtk_file_chooser_get_filenames(chooser);
+			list = siril_file_chooser_get_filenames(chooser);
 			list = g_slist_sort(list, (GCompareFunc) strcompare);
 			fill_convert_list(list);
 			g_slist_free(list);
 			break;
+
+		case OD_BADPIXEL:
+			gtk_entry_set_text(bad_pixel_entry, filename);
+			break;
+
 		}
 		g_free(filename);
 	}
@@ -352,10 +413,6 @@ static void opendial(int whichdial) {
 	siril_widget_destroy(widgetdialog);
 }
 
-void on_darkfile_button_clicked(GtkButton *button, gpointer user_data) {
-	opendial(OD_DARK);
-}
-
 void cwd_btton_clicked() {
 	opendial(OD_CWD);
 }
@@ -364,10 +421,26 @@ void on_offsetfile_button_clicked(GtkButton *button, gpointer user_data) {
 	opendial(OD_OFFSET);
 }
 
+void on_darkfile_button_clicked(GtkButton *button, gpointer user_data) {
+	opendial(OD_DARK);
+}
+
 void on_flatfile_button_clicked(GtkButton *button, gpointer user_data) {
 	opendial(OD_FLAT);
 }
 
+void on_offsetlibfile_button_clicked(GtkButton *button, gpointer user_data) {
+	opendial(OD_OFFSETLIB);
+}
+
+void on_darklibfile_button_clicked(GtkButton *button, gpointer user_data) {
+	opendial(OD_DARKLIB);
+}
+
+void on_flatlibfile_button_clicked(GtkButton *button, gpointer user_data) {
+	opendial(OD_FLATLIB);
+}
+
 void header_open_button_clicked() {
 	opendial(OD_OPEN);
 }
@@ -376,6 +449,10 @@ void on_select_convert_button_clicked(GtkToolButton *button, gpointer user_data)
 	opendial(OD_CONVERT);
 }
 
+void on_pixelmap_button_clicked(GtkToggleButton *button, gpointer user_data) {
+	opendial(OD_BADPIXEL);
+}
+
 /** callback function for recent document opened
  *
  * @param chooser
@@ -401,3 +478,4 @@ void on_open_recent_action_item_activated(GtkRecentChooser *chooser,
 	g_free(uri);
 	g_free(path);
 }
+
