@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -20,22 +20,13 @@
 #ifndef SRC_GUI_PREFERENCES_H_
 #define SRC_GUI_PREFERENCES_H_
 
-typedef enum {
-	RICE_COMP,
-	GZIP1_COMP,
-	GZIP2_COMP,
-	HCOMPRESS_COMP
-} compression_mode;
-
 void update_libraw_and_debayer_interface();
 void update_photometry_interface();
-void set_GUI_photometry();
 void initialize_path_directory(const gchar *path);
 void set_libraw_settings_menu_available(gboolean activate);
 void initialize_compression_param();
-void set_GUI_compression();
 void initialize_default_preferences();
-void set_preferences_dialog_from_global();
+void update_preferences_from_model();
 gchar *get_swap_dir();
 
 #endif /* SRC_GUI_PREFERENCES_H_ */
