@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -24,8 +24,8 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
-#include "core/OS_utils.h"
 #include "core/processing.h"
+#include "core/siril_log.h"
 #include "core/siril_date.h"
 #include "gui/progress_and_log.h"
 #include "gui/utils.h"
@@ -34,6 +34,8 @@
 #include "gui/utils.h"
 #include "gui/image_display.h"
 #include "gui/callbacks.h"
+#include "gui/dialogs.h"
+#include "io/Astro-TIFF.h"
 #include "io/conversion.h"
 #include "io/sequence.h"
 #include "io/single_image.h"
@@ -61,7 +63,7 @@ static void gtk_filter_add(GtkFileChooser *file_chooser, const gchar *title,
 
 static void set_filters_save_dialog(GtkFileChooser *chooser) {
 	GString *all_filter = NULL;
-	const gchar *fits_filter = "*.fit;*.FIT;*.fits;*.FITS;*.fts;*.FTS";
+	const gchar *fits_filter = "*.fit;*.FIT;*.fits;*.FITS;*.fts;*.FTS;*.fit.fz;*.FIT.fz;*.fits.fz;*.FITS.fz;*.fts.fz;*.FTS.fz";
 	const gchar *bmp_filter = "*.bmp;*.BMP";
 	const gchar *netpbm_filter = "*.ppm;*.PPM;*.pnm;*.PNM;*.pgm;*.PGM";
 
@@ -177,26 +179,37 @@ static void set_description_in_TIFF() {
 
 	gtk_text_buffer_get_bounds(tbuf, &itStart, &itEnd);
 	gtk_text_buffer_delete(tbuf, &itStart, &itEnd);
-	/* History already written in header */
-	if (gfit.history) {
-		GSList *list;
-		for (list = gfit.history; list; list = list->next) {
-			gtk_text_buffer_get_end_iter(tbuf, &itEnd);
-			gtk_text_buffer_insert(tbuf, &itEnd, (gchar *)list->data, -1);
-			gtk_text_buffer_get_end_iter(tbuf, &itEnd);
-			gtk_text_buffer_insert(tbuf, &itEnd, "\n", 1);
-		}
-	}
-	/* New history */
-	if (com.history) {
-		for (int i = 0; i < com.hist_display; i++) {
-			if (com.history[i].history[0] != '\0') {
+
+	if (gtk_combo_box_get_active(GTK_COMBO_BOX(lookup_widget("combo_type_of_tiff"))) == 0) {
+		gchar *astro_tiff = AstroTiff_build_header(&gfit);
+
+		gtk_text_buffer_get_end_iter(tbuf, &itEnd);
+		gtk_text_buffer_insert(tbuf, &itEnd, astro_tiff, -1);
+		gtk_text_buffer_get_end_iter(tbuf, &itEnd);
+		gtk_text_buffer_insert(tbuf, &itEnd, "\n", 1);
+		g_free(astro_tiff);
+	} else {
+		/* History already written in header */
+		if (gfit.history) {
+			GSList *list;
+			for (list = gfit.history; list; list = list->next) {
 				gtk_text_buffer_get_end_iter(tbuf, &itEnd);
-				gtk_text_buffer_insert(tbuf, &itEnd, com.history[i].history, strlen(com.history[i].history));
+				gtk_text_buffer_insert(tbuf, &itEnd, (gchar *)list->data, -1);
 				gtk_text_buffer_get_end_iter(tbuf, &itEnd);
 				gtk_text_buffer_insert(tbuf, &itEnd, "\n", 1);
 			}
 		}
+		/* New history */
+		if (com.history) {
+			for (int i = 0; i < com.hist_display; i++) {
+				if (com.history[i].history[0] != '\0') {
+					gtk_text_buffer_get_end_iter(tbuf, &itEnd);
+					gtk_text_buffer_insert(tbuf, &itEnd, com.history[i].history, strlen(com.history[i].history));
+					gtk_text_buffer_get_end_iter(tbuf, &itEnd);
+					gtk_text_buffer_insert(tbuf, &itEnd, "\n", 1);
+				}
+			}
+		}
 	}
 }
 
@@ -214,8 +227,6 @@ static void prepare_savepopup() {
 
 	GtkWindow *parent = GTK_WINDOW(GTK_APPLICATION_WINDOW(lookup_widget("control_window")));
 
-	gtk_window_set_transient_for(GTK_WINDOW(savepopup), parent);
-
 	switch (type_of_image) {
 	case TYPEBMP:
 		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving BMP"));
@@ -245,6 +256,8 @@ static void prepare_savepopup() {
 		tab = PAGE_FITS;
 	}
 
+	gtk_window_set_transient_for(GTK_WINDOW(savepopup), parent);
+
 	gtk_widget_set_visible(savetxt, FALSE);
 	gtk_notebook_set_current_page(notebookFormat, tab);
 }
@@ -281,6 +294,7 @@ static gchar *get_filename_and_replace_ext() {
 		char *file_no_ext = remove_ext_from_filename(basename);
 		g_free(basename);
 		basename = g_strdup_printf("%s%s", file_no_ext, com.pref.ext);
+		free(file_no_ext);
 	}
 
 	return basename;
@@ -293,7 +307,7 @@ static image_type get_image_type_from_filter(GtkFileFilter *filter) {
 static void filter_changed(gpointer user_data) {
 	GtkFileChooser *chooser = GTK_FILE_CHOOSER(user_data);
 	GtkFileFilter *filter = gtk_file_chooser_get_filter(chooser);
-	gchar *filename = gtk_file_chooser_get_filename(chooser);
+	gchar *filename = siril_file_chooser_get_filename(chooser);
 	char *file_no_ext = remove_ext_from_filename(filename);
 	image_type format = get_image_type_from_filter(filter);
 	gchar *new_filename = NULL;
@@ -337,6 +351,7 @@ static void filter_changed(gpointer user_data) {
 		g_free(new_filename);
 	}
 
+	g_free(file_no_ext);
 	g_free(filename);
 }
 
@@ -349,6 +364,7 @@ static int save_dialog() {
 
 	gtk_file_chooser_set_current_name(chooser, fname);
 	gtk_file_chooser_set_do_overwrite_confirmation(chooser, TRUE);
+	gtk_file_chooser_set_local_only(chooser, FALSE);
 	set_filters_save_dialog(chooser);
 	siril_file_chooser_add_preview(chooser, preview);
 	gtk_file_chooser_unselect_all(chooser);
@@ -357,7 +373,7 @@ static int save_dialog() {
 
 	gint res = siril_dialog_run(saveDialog);
 	if (res == GTK_RESPONSE_ACCEPT) {
-		gchar *filename = gtk_file_chooser_get_filename(chooser);
+		gchar *filename = siril_file_chooser_get_filename(chooser);
 		type_of_image = get_type_from_filename(filename);
 
 		GtkEntry *savetext = GTK_ENTRY(lookup_widget("savetxt"));
@@ -375,7 +391,7 @@ static int save_dialog() {
 }
 
 // idle function executed at the end of the Save Data processing
-gboolean end_save(gpointer p) {
+static gboolean end_save(gpointer p) {
 	struct savedial_data *args = (struct savedial_data *) p;
 	stop_processing_thread();
 
@@ -389,17 +405,19 @@ gboolean end_save(gpointer p) {
 	set_cursor_waiting(FALSE);
 	close_dialog();	// is this different from the hide above?
 	update_MenuItem();
-	
+
+	g_free(args->copyright);
+	g_free(args->description);
 	free(args);
 	return FALSE;
 }
 
 static gboolean test_for_viewer_mode() {
 	gboolean confirm = TRUE;
-	if (get_display_mode_from_menu() != LINEAR_DISPLAY) {
-		confirm = siril_confirm_dialog(_("Viewer mode is not linear"),
+	if (get_display_mode_from_menu() != LINEAR_DISPLAY && !com.pref.gui.silent_linear) {
+		confirm = siril_confirm_dialog_and_remember(_("Viewer mode is not linear"),
 				_("You are saving an image that is displayed in a non linear mode. "
-						"What you see is not what you will get. Switch the viewer mode to linear to save what you see."), _("Save Anyway"));
+						"What you see is not what you will get. Switch the viewer mode to linear to save what you see."), _("Save Anyway"), &com.pref.gui.silent_linear);
 	}
 	return confirm;
 }
@@ -419,7 +437,8 @@ static gboolean initialize_data(gpointer p) {
 	GtkToggleButton *button_8 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton8bits"));
 	GtkToggleButton *button_32 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton32bits"));
 	args->bitspersamples = gtk_toggle_button_get_active(button_8) ? 8 : gtk_toggle_button_get_active(button_32) ? 32 : 16;
-	get_tif_data_from_ui(&args->description, &args->copyright, &args->embeded_icc);
+	get_tif_data_from_ui(&gfit, &args->description, &args->copyright, &args->embeded_icc);
+	args->tiff_compression = get_tiff_compression();
 #endif
 	args->entry = GTK_ENTRY(lookup_widget("savetxt"));
 	args->filename = gtk_entry_get_text(args->entry);
@@ -457,7 +476,7 @@ static gpointer mini_save_dialog(gpointer p) {
 #endif
 #ifdef HAVE_LIBTIFF
 		case TYPETIFF:
-			args->retval = savetif(args->filename, &gfit, args->bitspersamples, args->description, args->copyright, args->embeded_icc);
+			args->retval = savetif(args->filename, &gfit, args->bitspersamples, args->description, args->copyright, args->tiff_compression, args->embeded_icc, TRUE);
 			break;
 #endif
 #ifdef HAVE_LIBPNG
@@ -520,135 +539,6 @@ void set_entry_filename() {
 	}
 }
 
-void on_menu_rgb_savefits_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		GtkToggleButton *b16bitu = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton_save_fit16"));
-		GtkToggleButton *b32bits = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton_save_fit32f"));
-		gtk_toggle_button_set_active(b32bits, gfit.type == DATA_FLOAT);
-		gtk_toggle_button_set_active(b16bitu, gfit.type == DATA_USHORT);
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving FITS"));
-		set_entry_filename();
-		type_of_image = TYPEFITS;
-		gtk_notebook_set_current_page(notebookFormat, PAGE_FITS);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
-void on_menu_rgb_savetiff_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		GtkToggleButton *b16 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton16bits"));
-		GtkToggleButton *b32 = GTK_TOGGLE_BUTTON(lookup_widget("radiobutton32bits"));
-		gtk_toggle_button_set_active(b32, gfit.type == DATA_FLOAT);
-		gtk_toggle_button_set_active(b16, gfit.type == DATA_USHORT);
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving TIFF"));
-		set_entry_filename();
-		type_of_image = TYPETIFF;
-		set_copyright_in_TIFF(); //Write "Siril Version X.Y" (or user defined) in Copyright_txt
-		set_description_in_TIFF();
-		gtk_notebook_set_current_page(notebookFormat, PAGE_TIFF);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
-void on_menu_rgb_savepng_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving PNG"));
-		set_entry_filename();
-		type_of_image = TYPEPNG;
-		gtk_notebook_set_current_page(notebookFormat, PAGE_MISC);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
-void on_menu_rgb_save8ppm_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving Netpbm"));
-		set_entry_filename();
-		type_of_image = TYPEPNM;
-		gtk_notebook_set_current_page(notebookFormat, PAGE_MISC);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
-void on_menu_rgb_savebmp_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving BMP"));
-		set_entry_filename();
-		type_of_image = TYPEBMP;
-		gtk_notebook_set_current_page(notebookFormat, PAGE_MISC);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
-void on_menu_rgb_savejpg_activate(GtkMenuItem *menuitem, gpointer user_data) {
-	static GtkNotebook* notebookFormat = NULL;
-	static GtkWidget *savepopup = NULL;
-	GtkWidget *savetxt = lookup_widget("filenameframe");
-
-	if (notebookFormat == NULL) {
-		notebookFormat = GTK_NOTEBOOK(lookup_widget("notebookFormat"));
-		savepopup = lookup_widget("savepopup");
-	}
-
-	if (single_image_is_loaded() || sequence_is_loaded()) {
-		gtk_window_set_title(GTK_WINDOW(savepopup), _("Saving JPG"));
-		set_entry_filename();
-		type_of_image = TYPEJPG;
-		gtk_notebook_set_current_page(notebookFormat, PAGE_JPG);
-		gtk_widget_set_visible(savetxt, TRUE);
-		gtk_widget_show(savepopup);
-	}
-}
-
 void on_savetxt_changed(GtkEditable *editable, gpointer user_data) {
 	GtkEntry *entry = GTK_ENTRY(editable);
 	GtkWidget *button = lookup_widget("button_savepopup");
@@ -658,23 +548,29 @@ void on_savetxt_changed(GtkEditable *editable, gpointer user_data) {
 }
 
 void on_button_savepopup_clicked(GtkButton *button, gpointer user_data) {
-	struct savedial_data *args = malloc(sizeof(struct savedial_data));
+	struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 	set_cursor_waiting(TRUE);
 	if (initialize_data(args)) {
 		start_in_new_thread(mini_save_dialog, args);
 	} else {
+		g_free(args->copyright);
+		g_free(args->description);
+		free(args);
 		siril_add_idle(end_generic, NULL);
 	}
 }
 
 void on_savetxt_activate(GtkEntry *entry, gpointer user_data) {
-	struct savedial_data *args = malloc(sizeof(struct savedial_data));
+	struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 	set_cursor_waiting(TRUE);
 	if (initialize_data(args)) {
 		start_in_new_thread(mini_save_dialog, args);
 	} else {
+		g_free(args->copyright);
+		g_free(args->description);
+		free(args);
 		siril_add_idle(end_generic, NULL);
 	}
 }
@@ -690,12 +586,15 @@ void on_header_save_as_button_clicked() {
 		if (save_dialog() == GTK_RESPONSE_ACCEPT) {
 			/* now it is not needed for some formats */
 			if (type_of_image & (TYPEBMP | TYPEPNG | TYPEPNM)) {
-				struct savedial_data *args = malloc(sizeof(struct savedial_data));
+				struct savedial_data *args = calloc(1, sizeof(struct savedial_data));
 
 				set_cursor_waiting(TRUE);
 				if (initialize_data(args)) {
 					start_in_new_thread(mini_save_dialog, args);
 				} else {
+					g_free(args->copyright);
+					g_free(args->description);
+					free(args);
 					siril_add_idle(end_generic, NULL);
 				}
 			} else {
@@ -787,8 +686,8 @@ void on_header_snapshot_button_clicked(gboolean clipboard) {
 			gtk_clipboard_set_image(cb, pixbuf);
 			gtk_clipboard_store(cb);
 
-			GtkWidget *widget = lookup_widget("header_snapshot_button");
-			GtkWidget *popover = snapshot_notification(widget, NULL, pixbuf);
+			GtkWidget *w = lookup_widget("header_snapshot_button");
+			GtkWidget *popover = snapshot_notification(w, NULL, pixbuf);
 			g_timeout_add(5000, (GSourceFunc) snapshot_notification_close, (gpointer) popover);
 		} else {
 			GFile *file = g_file_new_build_filename(com.wd, filename, NULL);
@@ -852,3 +751,7 @@ void on_savepopup_show(GtkWidget *widget, gpointer user_data) {
 	gtk_scrolled_window_set_min_content_height(scrolled_window, height);
 	gtk_scrolled_window_set_min_content_width(scrolled_window, width);
 }
+
+void on_combo_type_of_tiff_changed(GtkComboBox* box, gpointer user_data) {
+	set_description_in_TIFF();
+}
