@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -18,8 +18,13 @@
  * along with Siril. If not, see <http://www.gnu.org/licenses/>.
 */
 
+#ifdef _WIN32
+#include <windows.h>
+#endif
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/siril_app_dirs.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
 #include "gui/dialogs.h"
@@ -33,6 +38,7 @@
 #include "algos/PSF.h"
 #include "registration/registration.h"	// for update_reg_interface
 #include "stacking/stacking.h"	// for update_stack_interface
+#include <dirent.h>
 
 #include "sequence_list.h"
 
@@ -94,11 +100,13 @@ static void set_sensitive_layers(GtkCellLayout *cell_layout,
 
 	if (sequence_is_loaded() && !use_photometry) {
 		GtkTreePath *path = gtk_tree_model_get_path(tree_model, iter);
-		gint *index = gtk_tree_path_get_indices(path); // search by index to avoid translation problems
+		if (!path) return;
+		const gint *index = gtk_tree_path_get_indices(path); // search by index to avoid translation problems
 		if (!(com.seq.regparam))
 			return;
 		if (com.seq.regparam[*index] == NULL)
 			sensitive = FALSE;
+		gtk_tree_path_free(path);
 	}
 	g_object_set(cell, "sensitive", sensitive, NULL);
 }
@@ -181,12 +189,14 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 
 	if (!use_photometry) { // reporting registration data
 		if (seq->regparam && seq->regparam[layer]) {
-			shiftx = roundf_to_int(seq->regparam[layer][index].shiftx);
-			shifty = roundf_to_int(seq->regparam[layer][index].shifty);
+			double dx, dy;
+			translation_from_H(seq->regparam[layer][index].H, &dx, &dy);
+			shiftx = round_to_int(dx);
+			shifty = round_to_int(dy);
 			switch (selected_source) {
 				case r_FWHM:
 					if (is_arcsec) {
-						bin = gfit.unbinned ? (double) gfit.binning_x : 1.0;
+						bin = com.pref.binning_update ? (double) gfit.binning_x : 1.0;
 						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].fwhm, bin, (double) gfit.pixel_size_x, gfit.focal_length, &fwhm);
 					} else {
 						fwhm = seq->regparam[layer][index].fwhm;
@@ -194,7 +204,7 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 					break;
 				case r_WFWHM:
 					if (is_arcsec) {
-						bin = gfit.unbinned ? (double) gfit.binning_x : 1.0;
+						bin = com.pref.binning_update ? (double) gfit.binning_x : 1.0;
 						convert_single_fwhm_to_arcsec_if_possible(seq->regparam[layer][index].weighted_fwhm, bin, (double) gfit.pixel_size_x, gfit.focal_length, &fwhm);
 					} else {
 						fwhm = seq->regparam[layer][index].weighted_fwhm;
@@ -206,7 +216,13 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 				case r_QUALITY:
 					fwhm = seq->regparam[layer][index].quality;
 					break;
-				default: 
+				case r_BACKGROUND:
+					fwhm = seq->regparam[layer][index].background_lvl;
+					break;
+				case r_NBSTARS:
+					fwhm = seq->regparam[layer][index].number_of_stars;
+					break;
+				default:
 					break;
 			}
 		}
@@ -224,7 +240,6 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 						fwhm_to_arcsec_if_needed(&gfit, psfs[index]);
 						fwhm = psfs[index]->fwhmx_arcsec < 0 ? psfs[index]->fwhmx : psfs[index]->fwhmx_arcsec;
 					} else {
-						fwhm_to_pixels(psfs[index]);
 						fwhm = psfs[index]->fwhmx;
 					}
 					break;
@@ -252,7 +267,7 @@ static void add_image_to_sequence_list(sequence *seq, int index, int layer) {
 		}
 	}
 
-	color = (com.pref.combo_theme == 0) ? 1 : 0;
+	color = (com.pref.gui.combo_theme == 0) ? 1 : 0;
 	basename = g_path_get_basename(seq_get_image_filename(seq, index, imname));
 	gtk_list_store_append (list_store, &iter);
 	gtk_list_store_set (list_store, &iter,
@@ -314,7 +329,7 @@ static void unselect_select_frame_from_list(GtkTreeView *tree_view) {
 	for (list = references; list; list = list->next) {
 		GtkTreePath *path = gtk_tree_row_reference_get_path((GtkTreeRowReference*)list->data);
 		if (path) {
-			gint *new_index = gtk_tree_path_get_indices(path);
+			const gint *new_index = gtk_tree_path_get_indices(path);
 			GtkTreeIter iter;
 			gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &iter, path);
 			gint real_index = get_real_index_from_index_in_list(model, &iter);
@@ -326,6 +341,10 @@ static void unselect_select_frame_from_list(GtkTreeView *tree_view) {
 			gtk_tree_path_free(path);
 		}
 	}
+	if (!com.seq.imgparam[com.seq.reference_image].incl) {
+		com.seq.reference_image = -1; // reinit to -1 in order to find new reference
+		com.seq.reference_image = sequence_find_refimage(&com.seq);
+	}
 	g_list_free(references);
 	update_reg_interface(FALSE);
 	update_stack_interface(FALSE);
@@ -409,12 +428,14 @@ void on_treeview1_cursor_changed(GtkTreeView *tree_view, gpointer user_data) {
 		idx = g_value_get_int(&value) - 1;
 		if (idx != com.seq.current) {
 			fprintf(stdout, "loading image %d\n", idx);
-			seq_load_image(&com.seq, idx, TRUE);
+			if (seq_load_image(&com.seq, idx, TRUE)) // if loading fails, we fall back reloading the reference image
+				seq_load_image(&com.seq, com.seq.reference_image, TRUE); 
 		}
 		g_value_unset(&value);
 	}
 	g_list_free_full(list, (GDestroyNotify) gtk_tree_path_free);
 	display_status();
+	update_reg_interface(TRUE);
 }
 
 void on_seqlist_dialog_combo_changed(GtkComboBoxText *widget, gpointer user_data) {
@@ -461,14 +482,19 @@ void fill_sequence_list(sequence *seq, int layer, gboolean as_idle) {
 static gboolean fill_sequence_list_idle(gpointer p) {
 	int i;
 	struct _seq_list *args = (struct _seq_list *)p;
-
+	if (!args)
+		return FALSE;
+	if (!args->seq) {
+		g_free(args);
+		return FALSE;
+	}
 	if (combo == NULL) combo = lookup_widget("plotCombo");
 	if (sourceCombo == NULL) sourceCombo = lookup_widget("plotSourceCombo");
 	if (arcsec == NULL) arcsec = lookup_widget("arcsecPhotometry");
 	selected_source = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
 	is_arcsec = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(arcsec));
 	use_photometry = (gboolean)gtk_combo_box_get_active(GTK_COMBO_BOX(sourceCombo));
-	qualfmt = (args->seq && use_photometry && (selected_source == BACKGROUND) && (get_data_type(args->seq->bitpix) == DATA_FLOAT)) ? ("%.5f") : ("%.3f");
+	qualfmt = (args->seq && ((use_photometry && (selected_source == BACKGROUND)) || (!use_photometry && (selected_source == r_BACKGROUND))) && (get_data_type(args->seq->bitpix) == DATA_FLOAT)) ? ("%.5f") : ("%.3f");
 
 	if (list_store) gtk_list_store_clear(list_store);
 	get_list_store();
@@ -487,7 +513,13 @@ static gboolean fill_sequence_list_idle(gpointer p) {
 				case r_QUALITY:
 					gtk_tree_view_column_set_title(GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn5")), _("Quality"));
 					break;
-				default: 
+				case r_BACKGROUND:
+					gtk_tree_view_column_set_title(GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn5")), _("Background"));
+					break;
+				case r_NBSTARS:
+					gtk_tree_view_column_set_title(GTK_TREE_VIEW_COLUMN(gtk_builder_get_object(gui.builder, "treeviewcolumn5")), _("#Stars"));
+					break;
+				default:
 					break;
 			}
 		} else {
@@ -550,7 +582,7 @@ static gboolean fill_sequence_list_idle(gpointer p) {
 	return FALSE;
 }
 
-void exclude_single_frame(int index) {
+void exclude_include_single_frame(int index) {
 	siril_log_message(_("%s image %d in sequence %s\n"),
 			com.seq.imgparam[index].incl ? _("Excluding") : _("Including"),
 			index + 1, com.seq.seqname);
@@ -558,7 +590,19 @@ void exclude_single_frame(int index) {
 	com.seq.imgparam[index].incl = !com.seq.imgparam[index].incl;
 	if (com.seq.imgparam[index].incl)
 		com.seq.selnum++;
-	else 	com.seq.selnum--;
+	else {
+		com.seq.selnum--;
+	}
+	if (com.seq.selnum == 1 || com.seq.reference_image == index) {
+		if (com.seq.reference_image == index) {
+			com.seq.reference_image = -1;
+			com.seq.reference_image = sequence_find_refimage(&com.seq);
+		} else {
+			com.seq.reference_image = index;
+		}
+		adjust_refimage(index);
+		sequence_list_change_reference();
+	}
 	update_reg_interface(FALSE);
 	update_stack_interface(FALSE);
 	redraw(REDRAW_OVERLAY);
@@ -567,6 +611,31 @@ void exclude_single_frame(int index) {
 	writeseqfile(&com.seq);
 }
 
+void select_unselect_frames_from_list(gboolean *selected, gboolean keep) {
+	gboolean need_ref_update = FALSE;
+
+	for (int i = 0; i < com.seq.number; i++) { // use real index
+		com.seq.imgparam[i].incl = (keep) ? selected[i] : !selected[i] && com.seq.imgparam[i].incl;
+		if (!com.seq.imgparam[i].incl && com.seq.reference_image == i) { // reference image is not included anymore
+			need_ref_update = TRUE;
+		}
+	}
+	if (need_ref_update) {
+		com.seq.reference_image = -1;
+		com.seq.reference_image = sequence_find_refimage(&com.seq);
+		adjust_refimage(com.seq.reference_image);
+		sequence_list_change_reference();
+	}
+
+	redraw(REDRAW_OVERLAY);
+	fix_selnum(&com.seq, FALSE);
+	update_reg_interface(FALSE);
+	update_stack_interface(FALSE);
+	adjust_sellabel();
+	writeseqfile(&com.seq);
+	siril_log_message(_("Selection update finished, %d images are selected in the sequence\n"), com.seq.selnum);
+}
+
 void on_seqlist_image_selection_toggled(GtkCellRendererToggle *cell_renderer,
 		gchar *char_path, gpointer user_data) {
 	GtkTreePath *path = gtk_tree_path_new_from_string(char_path);
@@ -576,7 +645,7 @@ void on_seqlist_image_selection_toggled(GtkCellRendererToggle *cell_renderer,
 	fprintf(stdout, "toggle selection index = %d\n", index);
 
 	sequence_list_change_selection(char_path, !com.seq.imgparam[index].incl);
-	exclude_single_frame(index);
+	exclude_include_single_frame(index);
 }
 
 void toggle_image_selection(int index_in_list, int real_index, gboolean initvalue) {
@@ -585,7 +654,7 @@ void toggle_image_selection(int index_in_list, int real_index, gboolean initvalu
 	if (initvalue) {
 		com.seq.imgparam[real_index].incl = FALSE;
 		if (before_change != com.seq.imgparam[real_index].incl) { // decrement only on value change
-			--com.seq.selnum; 
+			--com.seq.selnum;
 			msg = g_strdup_printf(_("Image %d has been unselected from sequence\n"), real_index + 1);
 			if (com.seq.reference_image == real_index) {
 				com.seq.reference_image = -1;  // invalidate to trigger new reference search if ref frame is deselected
@@ -664,6 +733,7 @@ void on_ref_frame_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
 	}
 
 	g_list_free(references);
+	redraw(REDRAW_OVERLAY);
 	sequence_list_change_reference();
 	update_stack_interface(FALSE);// get stacking info and enable the Go button
 	adjust_sellabel();	// reference image is named in the label
@@ -671,6 +741,12 @@ void on_ref_frame_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
 	drawPlot();		// update plots
 }
 
+void on_draw_frame_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
+	if (!sequence_is_loaded())
+		return;
+	redraw(REDRAW_OVERLAY);
+}
+
 /****************** modification of the list store (tree model) ******************/
 
 void sequence_list_change_selection_index(int index_in_list, int real_index) {
@@ -706,7 +782,7 @@ void sequence_list_change_reference() {
 	gboolean valid;
 	int color;
 
-	color = (com.pref.combo_theme == 0) ? 1 : 0;
+	color = (com.pref.gui.combo_theme == 0) ? 1 : 0;
 
 	get_list_store();
 	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list_store), &iter);
@@ -749,7 +825,7 @@ void sequence_list_select_row_from_index(int index, gboolean do_load_image) {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	gboolean valid, found = FALSE;
-	
+
 	tree_view = lookup_widget("treeview1");
 	get_list_store();
 	valid = gtk_tree_model_get_iter_first(GTK_TREE_MODEL(list_store), &iter);
@@ -764,15 +840,173 @@ void sequence_list_select_row_from_index(int index, gboolean do_load_image) {
 		}
 	}
 	model = gtk_tree_view_get_model(GTK_TREE_VIEW(tree_view));
+	if (!model) return;
 	GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(tree_view)); //clear current selection
 	gtk_tree_selection_unselect_all(selection);
 	GtkTreePath *path = gtk_tree_model_get_path(model, &iter);
+	if (!path) return;
 	gtk_tree_selection_select_path(selection, path);
 	gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW(tree_view), path, NULL, FALSE, FALSE, FALSE);
 	gtk_tree_path_free(path);
 
 	if (do_load_image) {
-		seq_load_image(&com.seq, index, TRUE);
+		if (seq_load_image(&com.seq, index, TRUE)) // if loading fails, we fall back reloading the reference image
+			seq_load_image(&com.seq, com.seq.reference_image, TRUE); 
+		update_reg_interface(FALSE);
 		redraw(REDRAW_OVERLAY);
 	}
 }
+
+void update_icons_sequence_list(gboolean is_dark) {
+	gchar *image;
+	GtkWidget *w;
+	if (is_dark) {
+		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "frame_dark.svg", NULL);
+		w = gtk_image_new_from_file(image);
+	} else {
+		image = g_build_filename(siril_get_system_data_dir(), "pixmaps", "frame.svg", NULL);
+		w = gtk_image_new_from_file(image);
+	}
+	g_free(image);
+	gtk_button_set_image(GTK_BUTTON(GTK_TOGGLE_BUTTON(lookup_widget("drawframe_check"))), w);
+	gtk_widget_show(w);
+}
+
+/*****************************************************************************
+ *    Managing the list of sequence (the combo box) from the sequence tab
+ *****************************************************************************/
+
+#ifdef _WIN32
+static int ListSequences(const gchar *sDir, const char *sequence_name_to_select,
+		GtkComboBoxText *seqcombo, int *index_of_seq_to_load, gboolean *found) {
+	WIN32_FIND_DATAW fdFile;
+	HANDLE hFind = NULL;
+	char sPath[2048];
+	char filename[256];
+	int number_of_loaded_sequences = 0;
+	wchar_t *wpath;
+
+	//Specify a file mask. *.seq = We want only seq file!
+	sprintf(sPath, "%s\\*.seq", sDir);
+
+	wpath = g_utf8_to_utf16(sPath, -1, NULL, NULL, NULL);
+	if (wpath == NULL)
+		return 1;
+
+	if ((hFind = FindFirstFileW(wpath, &fdFile)) == INVALID_HANDLE_VALUE) {
+		fprintf(stderr, "Path not found: [%s]\n", sDir);
+		g_free(wpath);
+		return 1;
+	}
+	g_free(wpath);
+
+	do {
+		//Is the entity a File or Folder?
+		if (!(fdFile.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)) {
+			gchar *cFileName = g_utf16_to_utf8(fdFile.cFileName, -1, NULL, NULL, NULL);
+			if (cFileName == NULL) {
+				return 1;
+			}
+			sequence *seq = readseqfile(cFileName);
+			if (seq != NULL) {
+				strncpy(filename, cFileName, 255);
+				free_sequence(seq, TRUE);
+				gtk_combo_box_text_append_text(seqcombo, filename);
+				if (sequence_name_to_select
+						&& !strncmp(filename, sequence_name_to_select,
+							strlen(filename))) {
+					*index_of_seq_to_load = number_of_loaded_sequences;
+					*found = TRUE;
+				}
+				++number_of_loaded_sequences;
+			}
+			g_free(cFileName);
+		}
+	} while (FindNextFileW(hFind, &fdFile)); //Find the next file.
+
+	FindClose(hFind);
+
+	return number_of_loaded_sequences;
+}
+#endif
+
+/** This method populates the sequence combo box with the sequences found in the CWD.
+ *  If only one sequence is found, or if a sequence whose name matches the
+ *  possibly NULL argument is found, it is automatically selected, which triggers
+ *  its loading
+ *  @param sequence_name_to_select the name of the input sequence
+ *  @return 0 if success
+ */
+int update_sequences_list(const char *sequence_name_to_select) {
+	GtkComboBoxText *seqcombo;
+	int number_of_loaded_sequences = 0;
+	int index_of_seq_to_load = -1;
+	char *seqname = NULL;
+	gboolean found = FALSE;
+
+	// clear the previous list
+	seqcombo = GTK_COMBO_BOX_TEXT(lookup_widget("sequence_list_combobox"));
+	gtk_combo_box_text_remove_all(seqcombo);
+
+	if (sequence_name_to_select) {
+		if (g_str_has_suffix(sequence_name_to_select, ".seq"))
+			seqname = strdup(sequence_name_to_select);
+		else {
+			seqname = malloc(strlen(sequence_name_to_select) + 5);
+			sprintf(seqname, "%s.seq", sequence_name_to_select);
+		}
+	}
+
+#ifdef _WIN32
+	number_of_loaded_sequences = ListSequences(com.wd, seqname, seqcombo, &index_of_seq_to_load, &found);
+#else
+	struct dirent **list;
+	int i, n;
+
+
+	n = scandir(com.wd, &list, 0, alphasort);
+	if (n < 0)
+		perror("scandir");
+
+	for (i = 0; i < n; ++i) {
+		char *suf;
+
+		if ((suf = strstr(list[i]->d_name, ".seq")) && strlen(suf) == 4) {
+			sequence *seq = readseqfile(list[i]->d_name);
+			if (seq != NULL) {
+				free_sequence(seq, TRUE);
+				char *filename = list[i]->d_name;
+				gtk_combo_box_text_append_text(seqcombo, filename);
+				if (seqname && !strcmp(filename, seqname)) {
+					index_of_seq_to_load = number_of_loaded_sequences;
+					found = TRUE;
+				}
+				++number_of_loaded_sequences;
+			}
+		}
+	}
+	for (i = 0; i < n; i++)
+		free(list[i]);
+	free(list);
+#endif
+
+	if (!number_of_loaded_sequences) {
+		fprintf(stderr, "No valid sequence found in CWD.\n");
+		if (seqname) free(seqname);
+		return -1;
+	} else if (!seqname || found) {
+		fprintf(stdout, "Loaded %d %s\n", number_of_loaded_sequences,
+				ngettext("sequence", "sequences", number_of_loaded_sequences));
+	} else return -1;
+
+	if (seqname) free(seqname);
+
+	if (number_of_loaded_sequences > 1 && index_of_seq_to_load < 0) {
+		gtk_combo_box_popup(GTK_COMBO_BOX(seqcombo));
+	} else if (index_of_seq_to_load >= 0)
+		gtk_combo_box_set_active(GTK_COMBO_BOX(seqcombo), index_of_seq_to_load);
+	else
+		gtk_combo_box_set_active(GTK_COMBO_BOX(seqcombo), 0);
+	return 0;
+}
+
