@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -20,6 +20,7 @@
 
 #include "core/siril.h"
 #include "core/proto.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/callbacks.h"
 #include "gui/message_dialog.h"
@@ -77,10 +78,13 @@ void on_drawingarea_drag_data_received(GtkWidget *widget,
 					if (!strncmp(src_ext, "seq", 4)) {
 						gchar *sequence_dir = g_path_get_dirname(filename);
 						if (!siril_change_dir(sequence_dir, NULL)) {
-							if (check_seq(FALSE)) {
+							if (check_seq()) {
 								siril_log_message(_("No sequence `%s' found.\n"), filename);
 							} else {
 								set_seq(filename);
+								if (com.pref.wd)
+									g_free(com.pref.wd);
+								com.pref.wd = g_strdup(com.wd);
 								if (!com.script) {
 									populate_seqcombo(filename);
 									set_GUI_CWD();
@@ -97,5 +101,6 @@ void on_drawingarea_drag_data_received(GtkWidget *widget,
 			}
 		}
 		g_free(filename);
+		g_strfreev(uris);
 	}
 }
