@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2020 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -21,12 +21,14 @@
 #define SRC_GUI_SIRIL_WINDOW_H_
 
 void siril_window_enable_image_actions(GtkApplicationWindow *window, gboolean enable);
+void siril_window_autostretch_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_rgb_proc_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_any_proc_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_any_mono_proc_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_single_proc_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_none_proc_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_if_selection_actions(GtkApplicationWindow *window, gboolean enable);
+void siril_window_enable_if_selection_rgb_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_enable_if_selection_sequence_actions(GtkApplicationWindow *window, gboolean enable);
 void siril_window_map_actions(GtkApplicationWindow *window);
 
