@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -19,7 +19,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #ifdef _WIN32
@@ -39,6 +39,7 @@
 #include "core/proto.h"
 #include "core/processing.h"
 #include "core/OS_utils.h"
+#include "core/siril_log.h"
 #include "algos/demosaicing.h"
 #include "io/films.h"
 #include "io/fits_sequence.h"
@@ -131,27 +132,6 @@ void list_format_available() {
 #endif
 }
 
-/* This function sets all default values of libraw settings in the com.raw_set
- * struct, as defined in the glade file.
- * When the ini file is read, the values of com.raw_set are overwritten, but if the
- * file is missing, like the first time Siril is launched, we don't want to have the
- * GUI states reset to zero by set_GUI_LIBRAW() because the data in com.raw_set had
- * not been initialized with the default GUI values (= initialized to 0).
- */
-static void initialize_libraw_settings() {
-	com.pref.raw_set.bright = 1.0;		// brightness
-	com.pref.raw_set.mul[0] = 1.0;		// multipliers: red
-	com.pref.raw_set.mul[1] = 1.0;		// multipliers: green, not used because always equal to 1
-	com.pref.raw_set.mul[2] = 1.0;		// multipliers: blue
-	com.pref.raw_set.auto_mul = 1;		// multipliers are Either read from file, or calculated on the basis of file data, or taken from hardcoded constants
-	com.pref.raw_set.user_black = 0;		// black point correction
-	com.pref.raw_set.use_camera_wb = 0;	// if possible, use the white balance from the camera.
-	com.pref.raw_set.use_auto_wb = 0;		// use automatic white balance obtained after averaging over the entire image
-	com.pref.raw_set.user_qual = 2;		// type of interpolation. VNG by default
-	com.pref.raw_set.gamm[0] = 1.0;		// gamma curve: linear by default
-	com.pref.raw_set.gamm[1] = 1.0;
-}
-
 static void initialize_ser_debayer_settings() {
 	com.pref.debayer.open_debayer = FALSE;
 	com.pref.debayer.use_bayer_header = TRUE;
@@ -183,6 +163,9 @@ gchar *initialize_converters() {
 	supported_extensions[count_ext++] = ".fit";
 	supported_extensions[count_ext++] = ".fits";
 	supported_extensions[count_ext++] = ".fts";
+	supported_extensions[count_ext++] = ".fit.fz";
+	supported_extensions[count_ext++] = ".fits.fz";
+	supported_extensions[count_ext++] = ".fts.fz";
 	supported_extensions[count_ext++] = ".bmp";
 	supported_extensions[count_ext++] = ".ppm";
 	supported_extensions[count_ext++] = ".pgm";
@@ -197,7 +180,6 @@ gchar *initialize_converters() {
 	supported_filetypes |= TYPERAW;
 	string = g_string_append(string, ", ");
 	string = g_string_append(string, _("RAW images"));
-	initialize_libraw_settings();	// below in the file
 
 	nb_raw = get_nb_raw_supported();
 	for (i = 0; i < nb_raw; i++) {
@@ -314,7 +296,10 @@ image_type get_type_for_extension(const char *extension) {
 	} else if ((supported_filetypes & TYPESER) && !g_ascii_strcasecmp(extension, "ser")) {
 		return TYPESER;
 	} else if (!g_ascii_strcasecmp(extension, "fit") || !g_ascii_strcasecmp(extension, "fits") ||
-			!g_ascii_strcasecmp(extension, "fts") || !g_ascii_strcasecmp(extension, "fz")) {
+			!g_ascii_strcasecmp(extension, "fts")) {
+		return TYPEFITS;
+	} else if (!g_ascii_strcasecmp(extension, "fit.fz") || !g_ascii_strcasecmp(extension, "fits.fz") ||
+			!g_ascii_strcasecmp(extension, "fts.fz")) {
 		return TYPEFITS;
 	}
 	return TYPEUNDEF; // not recognized or not supported
@@ -337,7 +322,7 @@ int any_to_fits(image_type imagetype, const char *source, fits *dest,
 			break;
 #ifdef HAVE_LIBTIFF
 		case TYPETIFF:
-			retval = (readtif(source, dest, force_float) < 0);
+			retval = (readtif(source, dest, force_float, TRUE) < 0);
 			break;
 #endif
 		case TYPEPNM:
@@ -430,6 +415,7 @@ typedef enum {
 
 	/* for next file opening */
 	OPEN_ERROR,
+	OPEN_ERROR_AND_STOP,
 	OPEN_OK,
 	OPEN_SEQ,
 	OPEN_NOT_A_SEQ,
@@ -547,6 +533,10 @@ static int compute_nb_images_fit_mem(fits *fit);
 static void print_reader(struct reader_data *reader);
 static void print_writer(struct writer_data *writer);
 
+static void init_report(struct _convert_data *args);
+static void report_file_conversion(struct _convert_data *args, struct readwrite_data *rwarg);
+static void write_conversion_report(struct _convert_data *args);
+
 static gboolean end_convert_idle(gpointer p) {
 	struct _convert_data *args = (struct _convert_data *) p;
 	struct timeval t_end;
@@ -561,13 +551,14 @@ static gboolean end_convert_idle(gpointer p) {
 					converted_seqname = malloc(extidx + 5);
 					strncpy(converted_seqname, args->destroot, extidx);
 					strcpy(converted_seqname+extidx, ".seq");
+				} else {
+					converted_seqname = strdup(args->destroot);
 				}
 			} else {
-				converted_seqname = malloc(strlen(args->destroot) + 5);
-				sprintf(converted_seqname, "%s.seq", args->destroot);
+				converted_seqname = strdup(args->destroot);
 			}
+			check_seq();
 		}
-		check_seq(0);
 		if (converted_seqname) {
 			update_sequences_list(converted_seqname);
 			free(converted_seqname);
@@ -579,16 +570,18 @@ static gboolean end_convert_idle(gpointer p) {
 	gettimeofday(&t_end, NULL);
 	show_time(args->t_start, t_end);
 	stop_processing_thread();
-	g_free(args->destroot);
+	free(args->destroot);
+	free(args->report);
 	free(args);
 	return FALSE;
 }
 
 gpointer convert_thread_worker(gpointer p) {
 	struct _convert_data *args = (struct _convert_data *) p;
+	struct writer_data *writer = NULL;
 	args->nb_converted_files = 0;
 	args->retval = 0;
-	gboolean allow_symlink = args->output_type == SEQ_REGULAR && test_if_symlink_is_ok();
+	gboolean allow_symlink = args->output_type == SEQ_REGULAR && test_if_symlink_is_ok(TRUE);
 	args->make_link &= allow_symlink;
 	if (args->multiple_output && args->output_type != SEQ_SER && args->output_type != SEQ_FITSEQ) {
 		siril_log_message(_("disabling incompatible multiple output option in conversion\n"));
@@ -600,11 +593,20 @@ gpointer convert_thread_worker(gpointer p) {
 		seqwriter_set_max_active_blocks(initial_wqueue_limit);
 	}
 	set_progress_bar_data(_("Converting files"), PROGRESS_RESET);
+	init_report(args);
 
-	/* remove the target .seq to avoid errors */
-	gchar *seqname = replace_ext(args->destroot, ".seq");
-	g_unlink(seqname);
+	char *newdestroot = normalize_seqname(args->destroot, args->output_type == SEQ_REGULAR);
+	free(args->destroot);
+	args->destroot = strdup(newdestroot);
+	gchar *seqname = g_strdup_printf("%s%s", args->destroot, ".seq");
+	if (g_unlink(seqname))
+		siril_debug_print("Error in g_unlink()\n");
+	if (args->output_type == SEQ_REGULAR) { // to make sure destroot has an extension (will be removed when creating the filenames)
+		free(args->destroot);
+		args->destroot = strdup(seqname);
+	}
 	g_free(seqname);
+	free(newdestroot);
 
 	convert_status convert = { 0 };
 	convert.args = args;
@@ -612,7 +614,7 @@ gpointer convert_thread_worker(gpointer p) {
 	convert.number_of_threads = com.max_thread;
 	convert.threads = calloc(com.max_thread, sizeof(void *));
 	convert.allow_link = args->make_link;
-	convert.allow_32bits = args->output_type != SEQ_SER && !com.pref.force_to_16bit;
+	convert.allow_32bits = args->output_type != SEQ_SER && !com.pref.force_16bit;
 	GThreadPool *pool = g_thread_pool_new(pool_worker, &convert, args->input_has_a_film ? 1 : com.max_thread, FALSE, NULL);
 	open_next_input_seq(&convert);
 	open_next_output_seq(args, &convert);
@@ -627,7 +629,7 @@ gpointer convert_thread_worker(gpointer p) {
 		print_reader(reader);
 		g_atomic_int_inc(&convert.nb_input_images);
 
-		struct writer_data *writer = calloc(1, sizeof(struct writer_data));
+		writer = calloc(1, sizeof(struct writer_data));
 		seqwrite_status wstatus = get_next_write_details(args, &convert, writer,
 				rstatus == GOT_OK_LAST_IN_SEQ || rstatus == GOT_OK_LAST_IN_SEQ_LAST_FILE,
 				rstatus == GOT_OK_LAST_IN_SEQ_LAST_FILE || rstatus == GOT_OK_LAST_FILE);
@@ -635,14 +637,21 @@ gpointer convert_thread_worker(gpointer p) {
 			siril_debug_print("got writer error\n");
 			free(reader);
 			free(writer);
+			writer = NULL;
 			break;
 		}
 		print_writer(writer);
 		struct readwrite_data *rwarg = malloc(sizeof(struct readwrite_data));
 		rwarg->reader = reader;
 		rwarg->writer = writer;
+		report_file_conversion(args, rwarg);
+
 		if (!g_thread_pool_push(pool, rwarg, NULL)) {
 			siril_log_message(_("Failed to queue image conversion task, aborting"));
+			free(reader);
+			free(writer);
+			writer = NULL;
+			free(rwarg);
 			break;
 		}
 		if (rstatus == GOT_OK_LAST_FILE || rstatus == GOT_OK_LAST_IN_SEQ_LAST_FILE)
@@ -651,7 +660,7 @@ gpointer convert_thread_worker(gpointer p) {
 			siril_debug_print("last image of the sequence reached, opening next sequence\n");
 			open_next_input_seq(&convert);
 		}
-
+		// reader is freed elsewhere
 	} while (com.run_thread);
 	siril_debug_print("conversion scheduling loop finished, waiting for conversion tasks to finish\n");
 	g_thread_pool_free(pool, FALSE, TRUE);
@@ -662,13 +671,36 @@ gpointer convert_thread_worker(gpointer p) {
 		g_free(args->list[i]);
 	free(args->list);
 	args->nb_converted_files = convert.converted_files;
-	if (convert.fatal_error)
-		siril_log_message(_("Conversion ended with error, %d/%d input files converted\n"), args->nb_converted_files, args->total);
-	else {
-		if (convert.nb_input_images == convert.converted_images)
-			siril_log_message(_("Conversion succeeded, %d file(s) created for %d input file(s) (%d image(s) converted, %d failed)\n"), args->nb_converted_files, args->total, convert.converted_images, convert.failed_images);
-		else siril_log_message(_("Conversion aborted, %d file(s) created for %d input file(s) (%d image(s) converted, %d failed)\n"), args->nb_converted_files, args->total, convert.converted_images, convert.failed_images);
+	if (args->output_type == SEQ_REGULAR) {
+		if (convert.fatal_error)
+			siril_log_message(_("Conversion ended with error, %d/%d input files converted\n"), args->nb_converted_files, args->total);
+		else {
+			if (convert.nb_input_images == convert.converted_images)
+				siril_log_message(_("Conversion succeeded, %d file(s) created for %d input file(s) (%d image(s) converted, %d failed)\n"), args->nb_converted_files, args->total, convert.converted_images, convert.failed_images);
+			else siril_log_message(_("Conversion aborted, %d file(s) created for %d input file(s) (%d image(s) converted, %d failed)\n"), args->nb_converted_files, args->total, convert.converted_images, convert.failed_images);
+			write_conversion_report(args);
+		}
+	} else {
+		if (convert.fatal_error)
+			siril_log_message(_("Conversion ended with error, %d output files created\n"), args->nb_converted_files);
+		else {
+			gboolean success = TRUE;
+			if (!args->multiple_output && args->nb_converted_files == 1)
+				siril_log_message(_("Conversion succeeded, %d file(s) created for %d input file(s) (%d image(s) converted, %d failed)\n"), args->nb_converted_files, args->total, convert.converted_images, convert.failed_images);
+			else if (args->multiple_output && convert.nb_input_images == args->nb_converted_files)
+				siril_log_message(_("Conversion succeeded, %d file(s) created for %d input file(s)\n"), args->nb_converted_files, args->total);
+			else {
+				siril_log_message(_("Conversion aborted, %d file(s) created for %d input file(s)\n"), args->nb_converted_files, args->total);
+				success = FALSE;
+			}
+			if (success)
+				write_conversion_report(args);
+		}
 	}
+	// TODO still need to understand why, in case of error while writing a frame,
+	// sometimes we do create the file and sometimes the error is caught and we get convert.fatal_error to 1...
+	free(convert.output_fitseq);
+	free(convert.output_ser);
 	siril_add_idle(end_convert_idle, args);
 	return NULL;
 }
@@ -726,16 +758,16 @@ static void finish_write_seq(struct writer_data *writer, gboolean success) {
 		if (writer->ser) {
 			siril_debug_print("closing write SER sequence%s\n", success ? "" : " and deleting the file");
 			if (success) {
-				ser_write_and_close(writer->ser);
-				g_atomic_int_inc(writer->converted_files);
+				if(!ser_write_and_close(writer->ser))
+					g_atomic_int_inc(writer->converted_files);
 			}
 			else ser_close_and_delete_file(writer->ser);
 		}
 		else if (writer->fitseq) {
 			siril_debug_print("closing write FITS sequence%s\n", success ? "" : " and deleting the file");
 			if (success) {
-				fitseq_close_file(writer->fitseq);
-				g_atomic_int_inc(writer->converted_files);
+				if (!fitseq_close_file(writer->fitseq))
+					g_atomic_int_inc(writer->converted_files);
 			}
 			else fitseq_close_and_delete_file(writer->fitseq);
 		}
@@ -822,8 +854,11 @@ static void open_next_input_seq(convert_status *conv) {
 	do {
 		const char *filename = conv->args->list[conv->next_file];
 		status = open_next_input_sequence(filename, conv, FALSE);
-		if (status == OPEN_ERROR) {
+		if (status == OPEN_ERROR || status == OPEN_ERROR_AND_STOP) {
 			siril_log_color_message(_("File %s was not recognised as readable by Siril, skipping\n"), "salmon", filename);
+			g_atomic_int_inc(&conv->failed_images);
+			g_atomic_int_set(&conv->fatal_error, 1);
+			if (status == OPEN_ERROR_AND_STOP) break;
 		}
 		else if (status == OPEN_OK) {
 			conv->next_file++;
@@ -972,12 +1007,23 @@ static void pool_worker(gpointer data, gpointer user_data) {
 			g_atomic_int_inc(&conv->converted_files);
 		}
 		free(rwdata);	// reader and writer are freed in their function
+		if (fit) {
+			clearfits(fit);
+			free(fit);
+		}
 		return;
 	}
 	else if (!fit || read_status == NOT_READ || read_status == READ_FAILED) {
 		siril_debug_print("read error, ignoring image\n");
 		g_atomic_int_inc(&conv->failed_images);
 		finish_write_seq(rwdata->writer, FALSE);
+		if (rwdata->writer->have_seqwriter)
+			seqwriter_release_memory();
+		free(rwdata);
+		if (fit) {
+			clearfits(fit);
+			free(fit);
+		}
 		return;
 	}
 	readjust_memory_limits(conv, fit);
@@ -985,7 +1031,9 @@ static void pool_worker(gpointer data, gpointer user_data) {
 	if (!get_thread_run() || g_atomic_int_get(&conv->fatal_error)) {
 		rwdata->reader = NULL;
 		clearfits(fit);
-		handle_error(rwdata);
+		free(fit);
+		if (rwdata->writer->have_seqwriter)
+			handle_error(rwdata);
 		return;
 	}
 
@@ -1001,6 +1049,7 @@ static void pool_worker(gpointer data, gpointer user_data) {
 	double percent = (double)g_atomic_int_get(&conv->converted_images) /
 		(double)g_atomic_int_get(&conv->nb_input_images);
 	set_progress_bar_data(NULL, percent);
+	return;
 }
 
 static seqwrite_status get_next_write_details(struct _convert_data *args, convert_status *conv,
@@ -1011,11 +1060,13 @@ static seqwrite_status get_next_write_details(struct _convert_data *args, conver
 			if (!conv->output_ser) {
 				conv->output_ser = malloc(sizeof(struct ser_struct));
 				ser_init_struct(conv->output_ser);
-				gchar *dest = g_str_has_suffix(args->destroot, ".ser") ? args->destroot : g_strdup_printf("%s.ser", args->destroot);
+				gchar *dest = g_str_has_suffix(args->destroot, ".ser") ? g_strdup(args->destroot) : g_strdup_printf("%s.ser", args->destroot);
 				if (ser_create_file(dest, conv->output_ser, TRUE, NULL)) {
 					siril_log_message(_("Creating the SER file `%s' failed, aborting.\n"), args->destroot);
+					g_free(dest);
 					return GOT_WRITE_ERROR;
 				}
+				g_free(dest);
 				conv->next_image_in_output = 0;
 				conv->writeseq_count = get_new_write_counter();
 			}
@@ -1118,7 +1169,10 @@ static seqread_status open_next_input_sequence(const char *src_filename, convert
 	}
 #ifdef HAVE_FFMS2
 	if (imagetype == TYPEAVI) {
-		if (test_only) return OPEN_SEQ;
+		if (test_only) {
+			g_free(name);
+			return OPEN_SEQ;
+		}
 		if (convert->current_film) {
 			siril_debug_print("error: opening a film while the previous was still here\n");
 			g_free(name);
@@ -1134,11 +1188,15 @@ static seqread_status open_next_input_sequence(const char *src_filename, convert
 			return OPEN_ERROR;
 		}
 		convert->readseq_count = get_new_read_counter();
+		g_free(name);
 		return OPEN_OK;
 	}
 #endif
 	else if (imagetype == TYPESER) {
-		if (test_only) return OPEN_SEQ;
+		if (test_only) {
+			g_free(name);
+			return OPEN_SEQ;
+		}
 		if (convert->current_ser) {
 			siril_debug_print("error: opening a SER while the previous was still here\n");
 			g_free(name);
@@ -1152,9 +1210,10 @@ static seqread_status open_next_input_sequence(const char *src_filename, convert
 			free(convert->current_ser);
 			convert->current_ser = NULL;
 			g_free(name);
-			return OPEN_ERROR;
+			return OPEN_ERROR_AND_STOP;
 		}
 		convert->readseq_count = get_new_read_counter();
+		g_free(name);
 		return OPEN_OK;
 	}
 	else if (imagetype == TYPEFITS && fitseq_is_fitseq(name, NULL)) {
@@ -1181,11 +1240,12 @@ static seqread_status open_next_input_sequence(const char *src_filename, convert
 			fitseq_close_file(convert->current_fitseq);
 			free(convert->current_fitseq);
 			convert->current_fitseq = NULL;
-			return OPEN_ERROR;
+			return OPEN_ERROR_AND_STOP;
 		}
 		convert->readseq_count = get_new_read_counter();
 		return OPEN_OK;
 	}
+	g_free(name);
 	return OPEN_NOT_A_SEQ;
 }
 
@@ -1207,6 +1267,7 @@ static gchar *create_sequence_filename(sequence_type output_type, const char *de
 		default:
 			siril_log_color_message(_("output sequence type unknown, aborting\n"), "red");
 	}
+	free(destroot_noext);
 	return output;
 }
 
@@ -1233,7 +1294,12 @@ static seqwrite_status write_image(fits *fit, struct writer_data *writer) {
 	}
 	else if (writer->filename) {
 		if (savefits(writer->filename, fit)) {
-			siril_log_color_message(_("Error while converting to FITS (no space left?)\n"), "red");
+			siril_log_color_message(_("Error while converting to FITS\n"), "red");
+			/* We test if there's any space left. If not, we remove the image */
+			if (!is_space_disk_available(com.wd)) {
+				siril_log_color_message(_("No space left!\n"), "red");
+				g_unlink(writer->filename);
+			}
 		}
 		else {
 			retval = WRITE_OK;
@@ -1247,6 +1313,7 @@ static seqwrite_status write_image(fits *fit, struct writer_data *writer) {
 		siril_log_color_message(_("Error while converting, unknown output\n"), "red");
 	}
 
+
 	free(writer);
 	return retval;
 }
@@ -1300,3 +1367,50 @@ static void print_writer(struct writer_data *writer) {
 				writer->seq_count->close_sequence_after_write ? " (close after write)" : "");
 	else siril_debug_print("O> writer: %s\n", writer->filename);
 }
+
+static void init_report(struct _convert_data *args) {
+	args->report = malloc(args->total * sizeof(char *));
+	args->report_length = 0;
+}
+
+static void report_file_conversion(struct _convert_data *args, struct readwrite_data *rwarg) {
+	gchar *str = NULL;
+	if (rwarg->reader->filename) {
+		if (rwarg->writer->filename) {
+			str = g_strdup_printf("'%s' -> '%s'\n", rwarg->reader->filename, rwarg->writer->filename);
+		}
+		else if (rwarg->writer->fitseq) {
+			str = g_strdup_printf("'%s' -> '%s' image %d\n", rwarg->reader->filename, rwarg->writer->fitseq->filename, rwarg->writer->index);
+		}
+		else if (rwarg->writer->ser) {
+			str = g_strdup_printf("'%s' -> '%s' image %d\n", rwarg->reader->filename, rwarg->writer->ser->filename, rwarg->writer->index);
+		}
+	}
+	if (str) {
+		args->report[args->report_length++] = str;
+	}
+}
+
+static void write_conversion_report(struct _convert_data *args) {
+	if (args->report_length <= 0)
+		return;
+	gchar *filename;
+	char *filename_noext = remove_ext_from_filename(args->destroot);
+	if (g_str_has_suffix(filename_noext, "_"))
+		filename = g_strdup_printf("%sconversion.txt", filename_noext);
+	else filename = g_strdup_printf("%s_conversion.txt", filename_noext);
+	FILE *fd = g_fopen(filename, "w+");
+	g_free(filename);
+	free(filename_noext);
+	if (!fd)
+		return;
+
+	for (int i = 0; i < args->report_length; i++)
+		if (fputs(args->report[i], fd) == EOF)
+			break;
+
+	for (int i = 0; i < args->report_length; i++)
+		g_free(args->report[i]);
+	fclose(fd);
+}
+
