@@ -20,13 +20,15 @@ struct _convert_data {
 	gboolean input_has_a_seq;
 	gboolean input_has_a_film;
 	gboolean make_link;
-	gchar *destroot;
+	char *destroot;
 	gboolean debayer;
 	sequence_type output_type;
 	gboolean multiple_output;	// multiple SER output
 
 	int retval;
 	int nb_converted_files;
+	gchar **report;			// for file to file conversion report
+	int report_length;		// number of entries in the report
 };
 
 #define MAX_EXTENSIONS 50	// actual size of supported_extensions
@@ -39,7 +41,6 @@ void list_format_available();
 image_type get_type_for_extension(const char *extension);
 gchar *initialize_converters();
 gpointer convert_thread_worker(gpointer p);
-int debayer_if_needed(image_type imagetype, fits *fit, gboolean force_debayer);
 int any_to_fits(image_type imagetype, const char *source, fits *dest, gboolean interactive, gboolean force_float, gboolean debayer);
 #ifdef HAVE_FFMS2
 int convert_single_film_to_ser(sequence *seq);
