@@ -3,7 +3,7 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#if defined(HAVE_FFMS2_1) || defined(HAVE_FFMS2_2)
+#ifdef HAVE_FFMS2
 
 #include <ffms.h>
 
@@ -37,7 +37,6 @@ int check_for_film_extensions(const char *extension);
 int film_open_file(const char *sourcefile, struct film_struct *film);
 void film_close_file(struct film_struct *film);
 int film_read_frame(struct film_struct *film, int frame_no, fits *fit);
-void film_init_struct(struct film_struct *film);
 void film_display_info(struct film_struct *film);
 
 #endif
