@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -39,7 +39,8 @@ static int fitseq_multiple_close(fitseq *fitseq);
 static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 	int status = 0;
 	int nb_hdu, ref_naxis = -1, ref_bitpix = 0, nb_images = 0;
-	long ref_naxes[3] = { 0l };
+	long ref_naxes[3] = { 0L };
+	gboolean homogeneous = TRUE;
 
 	fits_get_num_hdus(fptr, &nb_hdu, &status);
 	if (status || !nb_im)
@@ -63,7 +64,7 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 
 		if (type != IMAGE_HDU) continue;
 
-		long naxes[3];
+		long naxes[3] = { 0L };
 		int naxis;
 		int bitpix;
 		fits_get_img_param(fptr, 3, &bitpix, &naxis, naxes, &status);
@@ -79,10 +80,21 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 				memcpy(ref_naxes, naxes, sizeof naxes);
 				siril_debug_print("found reference HDU %ldx%ldx%d (%d)\n", naxes[0], naxes[1], naxis, bitpix);
 			} else {
-				if (naxis != ref_naxis || naxes[0] != ref_naxes[0] || naxes[1] != ref_naxes[1] || bitpix != ref_bitpix) {
-					fprintf(stderr, "another image was found in the FITS file but does not has the same parameters as the first one\n");
+				printf("naxes[2]=%ld, ref_naxes[2]=%ld\n", naxes[2], ref_naxes[2]);
+				if (naxes[2] != ref_naxes[2]) {
+					siril_log_message(_("Several images were found in the FITS file but they have different number of layers, which is not allowed.\n"));
+					status = 1;
 					break;
 				}
+				if (naxes[0] != ref_naxes[0] || naxes[1] != ref_naxes[1] || bitpix != ref_bitpix) {
+					if (com.pref.allow_heterogeneous_fitseq)
+						homogeneous = FALSE;
+					else {
+						siril_log_message(_("Several images were found in the FITS file but they have different parameters, which is not allowed.\n"));
+						status = 1;
+						break;
+					}
+				}
 			}
 			if (hdus)
 				(*hdus)[nb_images] = i + 1;
@@ -97,8 +109,11 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 		}
 	}
 	else {
+		if (!homogeneous)
+			siril_log_message(_("Several images were found in the FITS file but they have different parameters.\n"));
+		// this is printed too often, maybe we can add a verbose flag?
 		*nb_im = nb_images;
-		siril_debug_print("found %d images with same params in the FITS sequence\n", nb_images);
+		siril_debug_print("found %d images in the FITS sequence\n", nb_images);
 		// we could realloc *hdus, but it's not much useful
 	}
 	return status;
@@ -109,7 +124,7 @@ static int _find_hdus(fitsfile *fptr, int **hdus, int *nb_im) {
 int fitseq_is_fitseq(const char *filename, int *frames) {
 	fitsfile *fptr;
 	int status = 0;
-	if (siril_fits_open_diskfile(&fptr, filename, READONLY, &status))
+	if (siril_fits_open_diskfile_img(&fptr, filename, READONLY, &status))
 		return 0;
 
 	int nb_images;
@@ -142,7 +157,7 @@ int fitseq_open(const char *filename, fitseq *fitseq) {
 	}
 
 	int status = 0;
-	siril_fits_open_diskfile(&(fitseq->fptr), filename, READONLY, &status);
+	siril_fits_open_diskfile_img(&(fitseq->fptr), filename, READONLY, &status);
 	if (status) {
 		report_fits_error(status);
 		siril_log_color_message(_("Cannot open FITS file %s\n"), "red", filename);
@@ -298,24 +313,30 @@ int fitseq_read_partial(fitseq *fitseq, int layer, int index, void *buffer, cons
 
 /* create a fits sequence with the given name into the given struct */
 int fitseq_create_file(const char *filename, fitseq *fitseq, int frame_count) {
-	g_unlink(filename); /* Delete old file if it already exists */
+	gchar *new_filename = set_right_extension(filename);
+
+	if (g_unlink(new_filename))
+		siril_debug_print("g_unlink() failed\n");/* Delete old file if it already exists */
 	fitseq_init_struct(fitseq);
 
 	int status = 0;
-	if (siril_fits_create_diskfile(&fitseq->fptr, filename, &status)) { /* create new FITS file */
+	if (siril_fits_create_diskfile(&fitseq->fptr, new_filename, &status)) { /* create new FITS file */
 		report_fits_error(status);
+		g_free(new_filename);
 		return 1;
 	}
 
-	fitseq->filename = strdup(filename);
+	fitseq->filename = strdup(new_filename);
 	fitseq->frame_count = frame_count;
 	fitseq->writer = malloc(sizeof(struct seqwriter_data));
 	fitseq->writer->write_image_hook = fitseq_write_image_for_writer;
 	fitseq->writer->sequence = fitseq;
+	fitseq->writer->output_type = SEQ_FITSEQ;
 	siril_debug_print("Successfully created the FITS sequence file %s, for %d images, waiting for data\n",
 			fitseq->filename, fitseq->frame_count);
 
 	start_writer(fitseq->writer, frame_count);
+	g_free(new_filename);
 	return 0;
 }
 
@@ -356,25 +377,32 @@ int fitseq_write_image(fitseq *fitseq, fits *image, int index) {
 
 static int fitseq_destroy(fitseq *fitseq, gboolean abort) {
 	int retval = 0;
+	int frame_count = -1;
 	if (fitseq->writer) {
 		retval = stop_writer(fitseq->writer, abort);
+		frame_count = fitseq->writer->frame_count;
 		free(fitseq->writer);
 		fitseq->writer = NULL;
 	}
+	if (frame_count == -1)
+		frame_count = fitseq->frame_count;
 	retval |= fitseq_multiple_close(fitseq);
 	int status = 0;
 	fits_close_file(fitseq->fptr, &status);
+	if ((retval || !frame_count) && fitseq->filename) {
+		siril_log_message(_("Removing failed FITS sequence file: %s\n"), fitseq->filename);
+		if (g_unlink(fitseq->filename))
+			siril_debug_print("g_unlink() failed\n");
+	}
 	if (fitseq->filename)
 		free(fitseq->filename);
+	if (fitseq->hdu_index)
+		free(fitseq->hdu_index);
 	return retval;
 }
 
 void fitseq_close_and_delete_file(fitseq *fitseq) {
-	char *filename = fitseq->filename;
-	fitseq->filename = NULL;
 	fitseq_destroy(fitseq, TRUE);
-	siril_log_message(_("Removing failed FITS sequence file: %s\n"), filename);
-	g_unlink(filename);
 }
 
 int fitseq_close_file(fitseq *fitseq) {
@@ -389,7 +417,7 @@ static int fitseq_prepare_for_multiple_read(fitseq *fitseq) {
 	fitseq->thread_fptr = malloc(fitseq->num_threads * sizeof(fitsfile *));
 	for (guint i = 0; i < fitseq->num_threads; i++) {
 		int status = 0;
-		if (siril_fits_open_diskfile(&fitseq->thread_fptr[i], fitseq->filename, READONLY, &status)) {
+		if (siril_fits_open_diskfile_img(&fitseq->thread_fptr[i], fitseq->filename, READONLY, &status)) {
 			report_fits_error(status);
 			return -1;
 		}
