@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -51,13 +51,31 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/icc_profile.h"
+#include "core/siril_log.h"
+#include "core/exif.h"
 #include "algos/geometry.h"
 #include "algos/siril_wcs.h"
+#include "algos/demosaicing.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "single_image.h"
 #include "image_format_fits.h"
 
+static void fill_date_obs_if_any(fits *fit, const char *file) {
+	gchar *date_time = NULL;
+	int year = 1, month = 1, day = 1, h = 0, m = 0, s = 0;
+	date_time = siril_get_date_from_exif(file);
+	if (date_time) {
+		int n = sscanf(date_time, "%04d:%02d:%02d %02d:%02d:%02d", &year, &month, &day, &h, &m, &s);
+		if (n == 6) {
+			GTimeZone *tz = g_time_zone_new_utc();
+			fit->date_obs = g_date_time_new(tz, year, month, day, h, m, (double) s);
+			g_time_zone_unref(tz);
+		}
+		g_free(date_time);
+	}
+}
+
 /********************* TIFF IMPORT AND EXPORT *********************/
 
 #ifdef HAVE_LIBTIFF
@@ -363,13 +381,13 @@ static int readtif8bits(TIFF* tif, uint32_t width, uint32_t height, uint16_t nsa
 	return retval;
 }
 
-static uint16_t get_compression_mode() {
+gboolean get_tiff_compression() {
 	if (!com.headless) {
 		GtkToggleButton *button = GTK_TOGGLE_BUTTON(lookup_widget("radiobuttonCompDeflate"));
 		if (gtk_toggle_button_get_active(button))
-			return (uint16_t) COMPRESSION_ADOBE_DEFLATE;
+			return TRUE;
 	}
-	return (uint16_t) COMPRESSION_NONE;
+	return FALSE;
 }
 
 static TIFF* Siril_TIFFOpen(const char *name, const char *mode) {
@@ -392,7 +410,7 @@ static TIFF* Siril_TIFFOpen(const char *name, const char *mode) {
 /* reads a TIFF file and stores it in the fits argument.
  * If file loading fails, the argument is untouched.
  */
-int readtif(const char *name, fits *fit, gboolean force_float) {
+int readtif(const char *name, fits *fit, gboolean force_float, gboolean verbose) {
 	int retval = 0;
 	uint32_t height, width;
 	uint16_t nbits, nsamples, color, orientation;
@@ -421,7 +439,7 @@ int readtif(const char *name, fits *fit, gboolean force_float) {
 
 	// Retrieve the Date/Time as in the TIFF TAG
 	gchar *date_time = NULL;
-	int year, month, day, h, m, s;
+	int year = 1, month = 1, day = 1, h = 0, m = 0, s = 0;
 
 	if (TIFFGetField(tif, TIFFTAG_DATETIME, &date_time)) {
 		sscanf(date_time, "%04d:%02d:%02d %02d:%02d:%02d", &year, &month, &day, &h, &m, &s);
@@ -445,7 +463,7 @@ int readtif(const char *name, fits *fit, gboolean force_float) {
 	// Retrieve Description field
 	char *desc = NULL;
 	if (TIFFGetField(tif, TIFFTAG_IMAGEDESCRIPTION, &desc)) {
-		description = strdup(desc);
+		description = g_strdup(desc);
 	}
 
 	size_t npixels = width * height;
@@ -479,13 +497,14 @@ int readtif(const char *name, fits *fit, gboolean force_float) {
 	if (retval < 0) {
 		free(data);
 		free(fdata);
+		g_free(description);
 		return OPEN_IMAGE_ERROR;
 	}
 	/* We clear fits. Everything written above is erased */
 	clearfits(fit);
 	if (date_time) {
 		GTimeZone *tz = g_time_zone_new_utc();
-		fit->date_obs = g_date_time_new(tz, year, month, day, h, m, s);
+		fit->date_obs = g_date_time_new(tz, year, month, day, h, m, (double) s);
 		g_time_zone_unref(tz);
 	}
 	fit->rx = width;
@@ -593,46 +612,46 @@ int readtif(const char *name, fits *fit, gboolean force_float) {
 	retval = nsamples;
 
 	gchar *basename = g_path_get_basename(name);
-	siril_log_message(_("Reading TIFF: %d-bit file %s, %ld layer(s), %ux%u pixels\n"),
+	if (verbose)
+		siril_log_message(_("Reading TIFF: %d-bit file %s, %ld layer(s), %ux%u pixels\n"),
 						nbits, basename, fit->naxes[2], fit->rx, fit->ry);
 	g_free(basename);
 
 	return retval;
 }
 
-void get_tif_data_from_ui(gchar **description, gchar **copyright, gboolean *embeded_icc) {
+void get_tif_data_from_ui(fits *fit, gchar **description, gchar **copyright, gboolean *embeded_icc) {
 	if (!com.script && !com.headless) {
 		/*******************************************************************
 		 * If the user saves a tif from the graphical menu, he can set
 		 * the Description and the Copyright of the Image
 		 ******************************************************************/
-		GtkToggleButton *icc_toggle = GTK_TOGGLE_BUTTON(lookup_widget("check_button_icc_profile"));
-		GtkTextView *description_txt_view = GTK_TEXT_VIEW(lookup_widget("Description_txt"));
-		GtkTextView *copyright_txt_view = GTK_TEXT_VIEW(lookup_widget("Copyright_txt"));
-		GtkTextBuffer *desbuf = gtk_text_view_get_buffer(description_txt_view);
-		GtkTextBuffer *copybuf = gtk_text_view_get_buffer(copyright_txt_view);
+
 		GtkTextIter itDebut;
 		GtkTextIter itFin;
 
+		GtkTextView *description_txt_view = GTK_TEXT_VIEW(lookup_widget("Description_txt"));
+		GtkTextBuffer *desbuf = gtk_text_view_get_buffer(description_txt_view);
 		gtk_text_buffer_get_start_iter(desbuf, &itDebut);
 		gtk_text_buffer_get_end_iter(desbuf, &itFin);
 		*description = gtk_text_buffer_get_text(desbuf, &itDebut, &itFin, TRUE);
-		gtk_text_buffer_get_bounds(desbuf, &itDebut, &itFin);
-		gtk_text_buffer_delete(desbuf, &itDebut, &itFin);
 
+		GtkTextView *copyright_txt_view = GTK_TEXT_VIEW(lookup_widget("Copyright_txt"));
+		GtkTextBuffer *copybuf = gtk_text_view_get_buffer(copyright_txt_view);
 		gtk_text_buffer_get_start_iter(copybuf, &itDebut);
 		gtk_text_buffer_get_end_iter(copybuf, &itFin);
 		*copyright = gtk_text_buffer_get_text(copybuf, &itDebut, &itFin, TRUE);
-		gtk_text_buffer_get_bounds(copybuf, &itDebut, &itFin);
-		gtk_text_buffer_delete(copybuf, &itDebut, &itFin);
 
+		GtkToggleButton *icc_toggle = GTK_TOGGLE_BUTTON(lookup_widget("check_button_icc_profile"));
 		*embeded_icc = gtk_toggle_button_get_active(icc_toggle);
 	}
 }
 
 /*** This function save the current image into a uncompressed 8- or 16-bit file *************/
 
-int savetif(const char *name, fits *fit, uint16_t bitspersample, const char *description, const char *copyright, gboolean embeded_icc){
+int savetif(const char *name, fits *fit, uint16_t bitspersample,
+		const char *description, const char *copyright,
+		gboolean tiff_compression, gboolean embeded_icc, gboolean verbose) {
 	int retval = 0;
 	float norm;
 	gchar *filename = g_strdup(name);
@@ -653,7 +672,7 @@ int savetif(const char *name, fits *fit, uint16_t bitspersample, const char *des
 	const uint16_t nsamples = (uint16_t) fit->naxes[2];
 	const uint32_t width = (uint32_t) fit->rx;
 	const uint32_t height = (uint32_t) fit->ry;
-	
+
 
 	/*******************************************************************/
 
@@ -666,7 +685,7 @@ int savetif(const char *name, fits *fit, uint16_t bitspersample, const char *des
 	TIFFSetField(tif, TIFFTAG_ROWSPERSTRIP, TIFFDefaultStripSize(tif, -1));
 	TIFFSetField(tif, TIFFTAG_PLANARCONFIG, PLANARCONFIG_CONTIG);
 	TIFFSetField(tif, TIFFTAG_SAMPLESPERPIXEL, nsamples);
-	TIFFSetField(tif, TIFFTAG_COMPRESSION, get_compression_mode());
+	TIFFSetField(tif, TIFFTAG_COMPRESSION, tiff_compression ? COMPRESSION_ADOBE_DEFLATE : COMPRESSION_NONE);
 	if (description) {
 		TIFFSetField(tif, TIFFTAG_IMAGEDESCRIPTION, description);
 	}
@@ -808,9 +827,11 @@ int savetif(const char *name, fits *fit, uint16_t bitspersample, const char *des
 	if (!write_ok) {
 		siril_log_color_message(_("Saving TIFF: Cannot write TIFF file.\n"), "red");
 		retval = OPEN_IMAGE_ERROR;
-		g_remove(filename);
+		if (g_remove(filename))
+			fprintf(stderr, "Error removing file\n");
 	} else {
-		siril_log_message(_("Saving TIFF: %d-bit file %s, %ld layer(s), %ux%u pixels\n"),
+		if (verbose)
+			siril_log_message(_("Saving TIFF: %d-bit file %s, %ld layer(s), %ux%u pixels\n"),
 				bitspersample, filename, nsamples, width, height);
 	}
 
@@ -879,6 +900,7 @@ int readjpg(const char* name, fits *fit){
 	fit->binning_x = fit->binning_y = 1;
 	fit->type = DATA_USHORT;
 	mirrorx(fit, FALSE);
+	fill_date_obs_if_any(fit, name);
 	gchar *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading JPG: file %s, %ld layer(s), %ux%u pixels\n"),
 			basename, fit->naxes[2], fit->rx, fit->ry);
@@ -998,13 +1020,18 @@ int readpng(const char *name, fits* fit) {
 	}
 
 	png_infop info = png_create_info_struct(png);
-	if (!info)
+	png_infop end_info = png_create_info_struct(png);
+	if (!info || !end_info)
 		return OPEN_IMAGE_ERROR;
 
 	if (setjmp(png_jmpbuf(png)))
 		return OPEN_IMAGE_ERROR;
 
 	FILE *f = g_fopen(name, "rb");
+	if (!f) {
+		siril_log_color_message(_("Error opening the file %s\n"), "red", name);
+		return OPEN_IMAGE_ERROR;
+	}
 	png_init_io(png, f);
 
 	png_read_info(png, info);
@@ -1019,6 +1046,7 @@ int readpng(const char *name, fits* fit) {
 	if (!data) {
 		PRINT_ALLOC_ERR;
 		fclose(f);
+		png_destroy_read_struct(&png, &info, &end_info);
 		return OPEN_IMAGE_ERROR;
 	}
 	WORD *buf[3] = { data, data + npixels, data + npixels * 2 };
@@ -1052,6 +1080,7 @@ int readpng(const char *name, fits* fit) {
 	png_read_image(png, row_pointers);
 
 	fclose(f);
+	png_destroy_read_struct(&png, &info, &end_info);
 
 	if (bit_depth == 16) {			//in 16-bit: it is stored as RRGGBB
 		for (int y = height - 1; y > -1; y--) {
@@ -1072,6 +1101,7 @@ int readpng(const char *name, fits* fit) {
 				*buf[GLAYER]++ = ptr[1];
 				*buf[BLAYER]++ = ptr[2];
 			}
+
 		}
 	}
 	// We define the number of channel we have
@@ -1110,10 +1140,12 @@ int readpng(const char *name, fits* fit) {
 		fit->orig_bitpix = fit->bitpix;
 		fit->data = data;
 		fit->pdata[RLAYER] = fit->data;
-		fit->pdata[GLAYER] = fit->data + npixels;
-		fit->pdata[BLAYER] = fit->data + npixels * 2;
+		fit->pdata[GLAYER] = fit->naxes[2] == 3 ? fit->data + npixels : fit->data;
+		fit->pdata[BLAYER] = fit->naxes[2] == 3 ? fit->data + npixels * 2 : fit->data;
+
 		fit->binning_x = fit->binning_y = 1;
 		g_snprintf(fit->row_order, FLEN_VALUE, "%s", "TOP-DOWN");
+		fill_date_obs_if_any(fit, name);
 	}
 	gchar *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading PNG: %d-bit file %s, %ld layer(s), %ux%u pixels\n"),
@@ -1254,9 +1286,6 @@ int savepng(const char *name, fits *fit, uint32_t bytes_per_sample,
 				PNG_FILTER_TYPE_DEFAULT);
 		profile = get_sRGB_profile_data(&profile_len);
 
-		if (profile_len > 0) {
-			png_set_iCCP(png_ptr, info_ptr, "icc", 0, (png_const_bytep) profile, profile_len);
-		}
 	} else {
 		png_set_IHDR(png_ptr, info_ptr, width, height, bytes_per_sample * 8,
 				PNG_COLOR_TYPE_GRAY,
@@ -1281,16 +1310,19 @@ int savepng(const char *name, fits *fit, uint32_t bytes_per_sample,
 		samples_per_pixel = 1;
 	}
 
+	WORD *data = NULL;
+	uint8_t *data8 = NULL;
+
 	if (bytes_per_sample == 2) {
 		/* swap bytes of 16 bit files to most significant bit first */
 		png_set_swap(png_ptr);
-		WORD *data = convert_data(fit);
+		data = convert_data(fit);
 		for (unsigned i = 0, j = height - 1; i < height; i++)
 			row_pointers[j--] = (png_bytep) ((uint16_t*) data + (size_t) samples_per_pixel * i * width);
 	} else {
-		uint8_t *data = convert_data8(fit);
+		data8 = convert_data8(fit);
 		for (unsigned i = 0, j = height - 1; i < height; i++)
-			row_pointers[j--] = (uint8_t*) data + (size_t) samples_per_pixel * i * width;
+			row_pointers[j--] = (uint8_t*) data8 + (size_t) samples_per_pixel * i * width;
 	}
 
 	png_write_image(png_ptr, row_pointers);
@@ -1304,6 +1336,8 @@ int savepng(const char *name, fits *fit, uint32_t bytes_per_sample,
 
 	/* Close the file */
 	fclose(p_png_file);
+	if (data) free(data);
+	if (data8) free(data8);
 	free(row_pointers);
 	free(filename);
 	return 0;
@@ -1381,178 +1415,6 @@ static int siril_libraw_open_file(libraw_data_t* rawdata, const char *name) {
 #endif
 }
 
-static int readraw(const char *name, fits *fit) {
-	libraw_data_t *raw = libraw_init(0);
-
-	int ret = siril_libraw_open_file(raw, name);
-	if (ret) {
-		siril_log_color_message(_("Error in libraw %s.\n"), "red", libraw_strerror(ret));
-		libraw_recycle(raw);
-		libraw_close(raw);
-		return OPEN_IMAGE_ERROR;
-	}
-	
-	if (raw->other.shutter > 1.0)
-		siril_log_message(_("Decoding %s %s file (ISO=%g, Exposure=%gs)\n"),
-				raw->idata.make, raw->idata.model, raw->other.iso_speed, raw->other.shutter);
-	else
-		siril_log_message(_("Decoding %s %s file (ISO=%g, Exposure=1/%gs)\n"),
-				raw->idata.make, raw->idata.model, raw->other.iso_speed, 1/raw->other.shutter);
-		
-	raw->params.output_bps = 16;						/* 16-bits files                           */
-	raw->params.four_color_rgb = 0;						/* If == 1, interpolate RGB as four colors.*/
-	raw->params.no_auto_bright = 1;						/* no auto_bright                          */
-	raw->params.gamm[0] = 1.0 / com.pref.raw_set.gamm[0];    /* Gamma curve set by the user             */
-	raw->params.gamm[1] = com.pref.raw_set.gamm[1];
-	raw->params.bright = com.pref.raw_set.bright;			/* Brightness                              */
-	raw->params.user_flip = 0;							/* no flip                                 */
-	raw->params.use_camera_wb = com.pref.raw_set.use_camera_wb;
-	raw->params.use_auto_wb = com.pref.raw_set.use_auto_wb;
-	if (com.pref.raw_set.user_black == 1)
-		raw->params.user_black = 0;						/* user black level equivalent to dcraw -k 0 */
-	raw->params.output_color = 0;						/* output colorspace, 0=raw, 1=sRGB, 2=Adobe, 3=Wide, 4=ProPhoto, 5=XYZ*/
-		
-	if (!(com.pref.raw_set.auto_mul)) { /* 4 multipliers (r,g,b,g) of the user's white balance.    */
-		raw->params.user_mul[0] = (float) com.pref.raw_set.mul[0];
-		raw->params.user_mul[1] = raw->params.user_mul[3] = 1.0f;
-		raw->params.user_mul[2] = (float) com.pref.raw_set.mul[2];
-		siril_log_message(_("Daylight multipliers: %f, %f, %f\n"),
-				raw->params.user_mul[0], raw->params.user_mul[1],
-				raw->params.user_mul[2]);
-	} else {
-		float mul[4]; /* 3 multipliers (r,g,b) from the camera white balance.  */
-		mul[0] = raw->color.pre_mul[0] / raw->color.pre_mul[1];
-		mul[1] = 1.0; /* raw->color.pre_mul[1]/raw->color.pre_mul[1]; */
-		mul[2] = raw->color.pre_mul[2] / raw->color.pre_mul[1];
-		mul[3] = raw->color.pre_mul[3] / raw->color.pre_mul[1];
-		siril_log_message(_("Daylight multipliers: %f, %f, %f\n"), mul[0],
-				mul[1], mul[2]);
-	}
-
-	if (raw->idata.filters == 9) {
-		siril_log_color_message(_("XTRANS Sensor detected.\n"), "salmon");
-	}
-
-	switch (com.pref.raw_set.user_qual) { /* Set interpolation                                        */
-	case 0: /* bilinear interpolaton */
-		raw->params.user_qual = 0;
-		siril_log_message(_("Bilinear interpolation...\n"));
-		break;
-	case 2: /* VNG interpolaton */
-		raw->params.user_qual = 1;
-		siril_log_message(_("VNG interpolation...\n"));
-		break;
-	case 3: /* PPG interpolaton */
-		raw->params.user_qual = 2;
-		siril_log_message(_("PPG interpolation...\n"));
-		break;
-	default:
-	case 1: /* AHD interpolaton */
-		raw->params.user_qual = 3;
-		siril_log_message(_("AHD interpolation...\n"));
-		break;
-	}
-	
-	const ushort width = raw->sizes.iwidth;
-	const ushort height = raw->sizes.iheight;
-	const float pitch = estimate_pixel_pitch(raw);
-	size_t npixels = width * height;
-
-	WORD *data = malloc(npixels * sizeof(WORD) * 3);
-	if (!data) {
-		PRINT_ALLOC_ERR;
-		return OPEN_IMAGE_ERROR;
-	}
-	WORD *buf[3] = { data, data + npixels, data + npixels * 2 };
-	ret = libraw_unpack(raw);
-	if (ret) {
-		printf("Error in libraw %s\n", libraw_strerror(ret));
-		free(data);
-		libraw_recycle(raw);
-		libraw_close(raw);
-		return OPEN_IMAGE_ERROR;
-	}
-
-	ret = libraw_dcraw_process(raw);
-	if (ret) {
-		printf("Error in libraw %s\n", libraw_strerror(ret));
-		free(data);
-		libraw_recycle(raw);
-		libraw_close(raw);
-		return OPEN_IMAGE_ERROR;
-	}
-
-	libraw_processed_image_t *image = libraw_dcraw_make_mem_image(raw, &ret);
-	if (ret) {
-		printf("Error in libraw %s\n", libraw_strerror(ret));
-		free(data);
-		libraw_dcraw_clear_mem(image);
-		libraw_recycle(raw);
-		libraw_close(raw);
-		return OPEN_IMAGE_ERROR;
-	}
-
-	int nbplanes = image->colors;
-	if (nbplanes != 3) {
-		free(data);
-		libraw_dcraw_clear_mem(image);
-		libraw_recycle(raw);
-		libraw_close(raw);
-		return OPEN_IMAGE_ERROR;
-	}
-	// only for 16-bits because of endianness. Are there 8-bits RAW ???
-
-	for (unsigned int i = 0; i < image->data_size; i += 6) {
-		*buf[RLAYER]++ = (image->data[i + 0]) + (image->data[i + 1] << 8);
-		*buf[GLAYER]++ = (image->data[i + 2]) + (image->data[i + 3] << 8);
-		*buf[BLAYER]++ = (image->data[i + 4]) + (image->data[i + 5] << 8);
-	}
-
-	/*  Here we compute the correct size of the output image (imgdata.sizes.iwidth and imgdata.sizes.iheight) for the following cases:
-    	- Files from Fuji cameras (with a 45-degree rotation)
-    	- Files from cameras with non-square pixels
-    	- Images shot by a rotated camera.
-	 */
-	libraw_adjust_sizes_info_only(raw);
-	
-	if (data != NULL) {
-		clearfits(fit);
-		fit->bitpix = fit->orig_bitpix = USHORT_IMG;
-		fit->type = DATA_USHORT;
-		fit->rx = (unsigned int) width;
-		fit->ry = (unsigned int) height;
-		fit->naxes[0] = (long) width;
-		fit->naxes[1] = (long) height;
-		fit->naxes[2] = nbplanes;
-		fit->naxis = 3;
-		fit->data = data;
-		fit->pdata[RLAYER] = fit->data;
-		fit->pdata[GLAYER] = fit->data + npixels;
-		fit->pdata[BLAYER] = fit->data + npixels * 2;
-		fit->binning_x = fit->binning_y = 1;
-		if (pitch > 0.f)
-			fit->pixel_size_x = fit->pixel_size_y = pitch;
-		if (raw->other.focal_len > 0.f)
-			fit->focal_length = raw->other.focal_len;
-		if (raw->other.iso_speed > 0.f)
-			fit->iso_speed = raw->other.iso_speed;
-		if (raw->other.shutter > 0.f)
-			fit->exposure = raw->other.shutter;
-		if (raw->other.aperture > 0.f)
-			fit->aperture = raw->other.aperture;
-		g_snprintf(fit->instrume, FLEN_VALUE, "%s %s", raw->idata.make,
-				raw->idata.model);
-		fit->date_obs = g_date_time_new_from_unix_utc(raw->other.timestamp);
-		mirrorx(fit, FALSE);
-	}
-
-	libraw_dcraw_clear_mem(image);
-	libraw_recycle(raw);
-	libraw_close(raw);
-
-	return nbplanes;
-}
-
 #define FC(filters, row, col) \
 	(filters >> ((((row) << 1 & 14) + ((col) & 1)) << 1) & 3)
 
@@ -1589,13 +1451,13 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 
 	int ret = siril_libraw_open_file(raw, name);
 	if (ret) {
-		printf("Error in libraw %s\n", libraw_strerror(ret));
+		siril_log_color_message("Error in libraw %s\n", "red", libraw_strerror(ret));
 		return OPEN_IMAGE_ERROR;
 	}
 
 	ret = libraw_unpack(raw);
 	if (ret) {
-		printf("Error in libraw %s\n", libraw_strerror(ret));
+		siril_log_color_message("Error in libraw %s\n", "red", libraw_strerror(ret));
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -1603,8 +1465,8 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 	 * the case for DNG built from lightroom for example */
 	if (raw->rawdata.raw_image == NULL
 			&& (raw->rawdata.color3_image || raw->rawdata.color4_image)) {
-		siril_log_color_message(_("Siril cannot open this file in CFA mode (no data available). "
-				"Try to switch into RGB.\n"), "red");
+		siril_log_color_message(_("Siril cannot open this file in CFA mode: "
+				"no RAW data available.\n"), "red");
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -1629,8 +1491,8 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 	}
 
 	float pitch = estimate_pixel_pitch(raw);
-	size_t npixels = width * height;
-	
+	size_t npixels = (size_t) width * (size_t) height;
+
 	if (raw->other.shutter > 0 && raw->other.shutter < 1)
 		siril_log_message(_("Decoding %s %s file (ISO=%g, Exposure=1/%0.1f sec)\n"),
 						raw->idata.make, raw->idata.model, raw->other.iso_speed, 1/raw->other.shutter);
@@ -1673,6 +1535,12 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 
 	int offset = raw_width * top_margin + left_margin;
 
+	if (!raw->rawdata.raw_image) {
+		libraw_recycle(raw);
+		libraw_close(raw);
+		free(buf);
+		return OPEN_IMAGE_ERROR;
+	}
 	int i = 0;
 	for (int row = height - 1; row > -1; row--) {
 		for (int col = 0; col < width; col++) {
@@ -1718,11 +1586,13 @@ static int readraw_in_cfa(const char *name, fits *fit) {
 }
 
 int open_raw_files(const char *name, fits *fit, gboolean debayer) {
-	int retval = 1;
-	if (debayer)
-		retval = readraw(name, fit);
-	else retval = readraw_in_cfa(name, fit);
+	int retval = readraw_in_cfa(name, fit);
+
 	if (retval >= 0) {
+		if (debayer) {
+			debayer_if_needed(TYPEFITS, fit, TRUE);
+		}
+
 		gchar *basename = g_path_get_basename(name);
 		siril_log_message(_("Reading RAW: file %s, %ld layer(s), %ux%u pixels\n"),
 				basename, fit->naxes[2], fit->rx, fit->ry);
@@ -1733,7 +1603,7 @@ int open_raw_files(const char *name, fits *fit, gboolean debayer) {
 #endif
 
 #ifdef HAVE_LIBHEIF
-#define MAX_THUMBNAIL_SIZE com.pref.thumbnail_size
+#define MAX_THUMBNAIL_SIZE com.pref.gui.thumbnail_size
 
 struct HeifImage {
 	uint32_t ID;
@@ -1985,12 +1855,18 @@ static gboolean heif_dialog(struct heif_context *heif, uint32_t *selected_image)
 
 int readheif(const char* name, fits *fit, gboolean interactive){
 	struct heif_error err;
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+	heif_init(NULL);
+#endif
 
 	struct heif_context *ctx = heif_context_alloc();
 	err = heif_context_read_from_file(ctx, name, NULL);
 	if (err.code) {
 		g_printf("%s\n", err.message);
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+		heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -1999,6 +1875,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 	if (num == 0) {
 		siril_log_color_message(_("Input file contains no readable images.\n"), "red");
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+		heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -2010,6 +1889,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 	if (err.code) {
 		g_printf("%s\n", err.message);
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+		heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -2029,6 +1911,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 		} else {
 			if (!heif_dialog(ctx, &selected_image)) {
 				heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+				heif_deinit();
+#endif
 				return OPEN_IMAGE_CANCEL;
 			}
 		}
@@ -2040,6 +1925,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 	if (err.code) {
 		g_printf("%s\n", err.message);
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+		heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -2053,6 +1941,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 		g_printf("%s\n", err.message);
 		heif_image_handle_release(handle);
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+		heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 
@@ -2068,6 +1959,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 		PRINT_ALLOC_ERR;
 		heif_image_handle_release(handle);
 		heif_context_free(ctx);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+	heif_deinit();
+#endif
 		return OPEN_IMAGE_ERROR;
 	}
 	WORD *buf[3] = { data, data + npixels, data + npixels * 2 };
@@ -2101,6 +1995,9 @@ int readheif(const char* name, fits *fit, gboolean interactive){
 	heif_image_handle_release(handle);
 	heif_context_free(ctx);
 	heif_image_release(img);
+#if LIBHEIF_HAVE_VERSION(1,13,0)
+	heif_deinit();
+#endif
 	gchar *basename = g_path_get_basename(name);
 	siril_log_message(_("Reading HEIF: file %s, %ld layer(s), %ux%u pixels\n"),
 			basename, fit->naxes[2], fit->rx, fit->ry);
