@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -32,6 +32,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/siril_date.h"
+#include "core/siril_log.h"
 #include "gui/utils.h"
 #include "gui/progress_and_log.h"
 #include "algos/demosaicing.h"
@@ -49,7 +50,7 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 /* Output SER timestamp */
 static int display_date(guint64 timestamp, char *txt) {
 	if (timestamp == 0)
-		return -1;
+		return SER_GENERIC_ERROR;
 
 	GDateTime *date = ser_timestamp_to_date_time(timestamp);
 	if (date) {
@@ -58,7 +59,7 @@ static int display_date(guint64 timestamp, char *txt) {
 		free(str);
 		g_date_time_unref(date);
 	}
-	return 0;
+	return SER_OK;
 }
 
 static const char *convert_color_id_to_char(ser_color color_id) {
@@ -94,16 +95,15 @@ static const char *convert_color_id_to_char(ser_color color_id) {
 static int ser_read_timestamp(struct ser_struct *ser_file) {
 	gboolean timestamps_in_order = TRUE;
 	guint64 previous_ts = 0L;
-	gint64 frame_size;
 
 	ser_file->fps = -1.0;	// will be calculated from the timestamps
 
 	if (!ser_file->frame_count || ser_file->image_width <= 0 ||
 			ser_file->image_height <= 0 || ser_file->byte_pixel_depth <= 0 ||
 			!ser_file->number_of_planes)
-		return 0;
+		return SER_OK;
 
-	frame_size = ser_file->image_width *
+	gint64 frame_size = (gint64) ser_file->image_width *
 		ser_file->image_height * ser_file->number_of_planes;
 	gint64 offset = SER_HEADER_LEN + frame_size *
 		(gint64)ser_file->byte_pixel_depth * (gint64)ser_file->frame_count;
@@ -112,17 +112,17 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 		ser_file->ts = calloc(8, ser_file->frame_count);
 		if (!ser_file->ts) {
 			PRINT_ALLOC_ERR;
-			return 0;
+			return SER_OK;
 		}
 		ser_file->ts_alloc = ser_file->frame_count;
 
 		// Seek to start of timestamps
 		for (int i = 0; i < ser_file->frame_count; i++) {
 			if ((gint64) -1 == fseek64(ser_file->file, offset + (i * 8), SEEK_SET))
-				return -1;
+				return SER_GENERIC_ERROR;
 
 			if (8 != fread(&ser_file->ts[i], 1, 8, ser_file->file))
-				return 0;
+				return SER_OK;
 
 			ser_file->ts[i] = le64_to_cpu(ser_file->ts[i]);
 		}
@@ -162,7 +162,7 @@ static int ser_read_timestamp(struct ser_struct *ser_file) {
 	} else {
 		fprintf(stdout, _("Warning: no timestamps stored in the SER sequence.\n"));
 	}
-	return 0;
+	return SER_OK;
 }
 
 static int ser_recompute_frame_count(struct ser_struct *ser_file) {
@@ -170,9 +170,9 @@ static int ser_recompute_frame_count(struct ser_struct *ser_file) {
 	gint64 filesize = ser_file->filesize;
 
 	siril_log_message(_("Trying to fix broken SER file...\n"));
-	gint64 frame_size = ser_file->image_width * ser_file->image_height;
+	gint64 frame_size = (gint64) ser_file->image_width * ser_file->image_height;
 	if (frame_size == 0)
-		return 0;
+		return SER_OK;
 
 	if (ser_file->color_id == SER_RGB || ser_file->color_id == SER_BGR) {
 		frame_size *= 3;  // Color images have twice as many samples
@@ -190,23 +190,23 @@ static int ser_recompute_frame_count(struct ser_struct *ser_file) {
 
 static int ser_read_header(struct ser_struct *ser_file) {
 	char header[SER_HEADER_LEN];
-
+	int ret;
 	if (!ser_file || ser_file->file == NULL)
-		return -1;
+		return SER_GENERIC_ERROR;
 
 	/* Get file size */
-	fseek64(ser_file->file, 0, SEEK_END);
+	ret = fseek64(ser_file->file, 0, SEEK_END);
 	ser_file->filesize = ftell64(ser_file->file);
-	fseek64(ser_file->file, 0, SEEK_SET);
-	if (ser_file->filesize == -1) {
+	ret |= fseek64(ser_file->file, 0, SEEK_SET);
+	if (ser_file->filesize == -1 || ret == -1) {
 		perror("seek");
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
 	/* Read header (size of 178) */
 	if (SER_HEADER_LEN != fread(header, 1, sizeof header, ser_file->file)) {
 		perror("fread");
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
 	// modify this to support big endian
@@ -261,7 +261,7 @@ static int ser_read_header(struct ser_struct *ser_file) {
 
 	ser_read_timestamp(ser_file);
 
-	return 0;
+	return SER_OK;
 }
 
 static int ser_write_timestamps(struct ser_struct *ser_file) {
@@ -270,11 +270,11 @@ static int ser_write_timestamps(struct ser_struct *ser_file) {
 	if (!ser_file->frame_count || ser_file->image_width <= 0 ||
 			ser_file->image_height <= 0 || ser_file->byte_pixel_depth <= 0 ||
 			!ser_file->number_of_planes)
-		return -1;
+		return SER_GENERIC_ERROR;
 
 	if (ser_file->ts) {
 		// Seek to start of timestamps
-		frame_size = ser_file->image_width * ser_file->image_height
+		frame_size = (gint64) ser_file->image_width * ser_file->image_height
 			* ser_file->number_of_planes;
 		gint64 offset = SER_HEADER_LEN + frame_size *
 			(gint64)ser_file->byte_pixel_depth * (gint64)ser_file->frame_count;
@@ -285,18 +285,18 @@ static int ser_write_timestamps(struct ser_struct *ser_file) {
 			if (i >= ser_file->ts_alloc)
 				break;
 			if ((gint64)-1 == fseek64(ser_file->file, offset+(i*8), SEEK_SET)) {
-				return -1;
+				return SER_GENERIC_ERROR;
 			}
 
 			ts = cpu_to_le64(ser_file->ts[i]);
 
 			if (8 != fwrite(&ts, 1, 8, ser_file->file)) {
 				perror("write timestamps:");
-				return -1;
+				return SER_GENERIC_ERROR;
 			}
 		}
 	}
-	return 0;
+	return SER_OK;
 }
 
 /* (over)write the header of the opened file on the disk */
@@ -305,10 +305,10 @@ static int ser_write_header(struct ser_struct *ser_file) {
 	struct ser_struct ser_file_le;
 
 	if (!ser_file || ser_file->file == NULL)
-		return -1;
+		return SER_GENERIC_ERROR;
 	if ((gint64) -1 == fseek64(ser_file->file, 0, SEEK_SET)) {
 		perror("seek");
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
 	memcpy(&ser_file_le, ser_file, sizeof(struct ser_struct));
@@ -334,9 +334,9 @@ static int ser_write_header(struct ser_struct *ser_file) {
 
 	if (sizeof(header) != fwrite(header, 1, sizeof(header), ser_file->file)) {
 		perror("write");
-		return 1;
+		return SER_GENERIC_ERROR;
 	}
-	return 0;
+	return SER_OK;
 }
 
 /* populate fields that are not already set in ser_create_file */
@@ -371,7 +371,7 @@ static int ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		ser_file->bit_pixel_depth = 16;
 	} else {
 		siril_log_message(_("Writing a 32-bit image to SER files is not supported.\n"));
-		return 1;
+		return SER_GENERIC_ERROR;
 	}
 	if (fit->instrume[0] != 0) {
 		memset(ser_file->instrument, 0, 40);
@@ -381,14 +381,14 @@ static int ser_write_header_from_fit(struct ser_struct *ser_file, fits *fit) {
 		memset(ser_file->observer, 0, 40);
 		memcpy(ser_file->observer, fit->observer, 40);
 	}
-	if (fit->instrume[0] != 0) {
+	if (fit->telescop[0] != 0) {
 		memset(ser_file->telescope, 0, 40);
 		memcpy(ser_file->telescope, fit->telescop, 40);
 	}
 
 	if (fit->date_obs)
 		ser_file->date = date_time_to_ser_timestamp(fit->date_obs);
-	return 0;
+	return SER_OK;
 }
 
 static int get_SER_Bayer_Pattern(ser_color pattern) {
@@ -430,7 +430,7 @@ static void ser_manage_endianess_and_depth(struct ser_struct *ser_file,
 }
 
 static int ser_alloc_ts(struct ser_struct *ser_file, int frame_no) {
-	int retval = 0;
+	int retval = SER_OK;
 #ifdef _OPENMP
 	omp_set_lock(&ser_file->ts_lock);
 #endif
@@ -455,9 +455,9 @@ static int ser_alloc_ts(struct ser_struct *ser_file, int frame_no) {
  */
 
 gboolean ser_is_cfa(struct ser_struct *ser_file) {
-	return ser_file && (ser_file->color_id == SER_BAYER_RGGB || 
-			ser_file->color_id == SER_BAYER_GRBG || 
-			ser_file->color_id == SER_BAYER_GBRG || 
+	return ser_file && (ser_file->color_id == SER_BAYER_RGGB ||
+			ser_file->color_id == SER_BAYER_GRBG ||
+			ser_file->color_id == SER_BAYER_GBRG ||
 			ser_file->color_id == SER_BAYER_BGGR);
 	// SER_BAYER_CYYM SER_BAYER_YCMY SER_BAYER_YMCY SER_BAYER_MYYC are not
 	// supported yet so returning false for them here is good
@@ -523,7 +523,7 @@ void ser_display_info(struct ser_struct *ser_file) {
 }
 
 static int ser_end_write(struct ser_struct *ser_file, gboolean abort) {
-	int retval = 0;
+	int retval = SER_OK;
 	if (ser_file->writer) {
 		retval = stop_writer(ser_file->writer, abort);
 		ser_file->frame_count = ser_file->writer->frame_count;
@@ -534,28 +534,36 @@ static int ser_end_write(struct ser_struct *ser_file, gboolean abort) {
 }
 
 int ser_close_and_delete_file(struct ser_struct *ser_file) {
-	if (ser_file == NULL) return -1;
+	if (ser_file == NULL) return SER_GENERIC_ERROR;
 	int retval = ser_end_write(ser_file, TRUE);
 	char *filename = ser_file->filename;
 	ser_file->filename = NULL;
 	ser_close_file(ser_file); // closes, frees and zeroes
 	siril_log_message(_("Removing failed SER file: %s\n"), filename);
-	g_unlink(filename);
+	if (g_unlink(filename))
+		siril_debug_print("Error unlinking file\n");
 	free(filename);
 	return retval;
 }
 
 int ser_write_and_close(struct ser_struct *ser_file) {
-	if (ser_file == NULL) return -1;
+	if (ser_file == NULL) return SER_GENERIC_ERROR;
 	int retval = ser_end_write(ser_file, FALSE);
 	if (!ser_file->frame_count) {
 		siril_log_color_message(_("The SER sequence is being created with no image in it.\n"), "red");
 		ser_close_and_delete_file(ser_file);
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 	ser_write_header(ser_file);	// writes the header
 	ser_write_timestamps(ser_file);	// writes the trailer
+	gchar *file_to_delete = NULL;
+	if (retval)
+		file_to_delete = g_strdup(ser_file->filename);
 	ser_close_file(ser_file);// closes, frees and zeroes
+	if (retval && file_to_delete)
+		if (g_unlink(file_to_delete))
+			siril_debug_print("g_unlink() failed\n");
+	g_free(file_to_delete);
 	return retval;
 }
 
@@ -564,10 +572,11 @@ int ser_write_and_close(struct ser_struct *ser_file) {
 int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		gboolean overwrite, struct ser_struct *copy_from) {
 	if (overwrite)
-		g_unlink(filename);
+		if (g_unlink(filename))
+			siril_debug_print("g_unlink() failed\n");
 	if ((ser_file->file = g_fopen(filename, "w+b")) == NULL) {
 		perror("open SER file for creation");
-		return 1;
+		return SER_GENERIC_ERROR;
 	}
 
 	ser_file->filename = strdup(filename);
@@ -599,7 +608,7 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 		 * before closing in case the number of the image in the new
 		 * SER changes from the copied SER */
 		if (ser_write_header(ser_file))
-			return 1;
+			return SER_GENERIC_ERROR;
 	} else {	// new SER
 		ser_file->file_id = strdup("LUCAM-RECORDER");
 		ser_file->lu_id = 0;
@@ -619,10 +628,11 @@ int ser_create_file(const char *filename, struct ser_struct *ser_file,
 	ser_file->writer = malloc(sizeof(struct seqwriter_data));
 	ser_file->writer->write_image_hook = ser_write_image_for_writer;
 	ser_file->writer->sequence = ser_file;
+	ser_file->writer->output_type = SEQ_SER;
 
 	siril_log_message(_("Created SER file %s\n"), filename);
 	start_writer(ser_file->writer, ser_file->frame_count);
-	return 0;
+	return SER_OK;
 }
 
 int ser_reset_to_monochrome(struct ser_struct *ser_file) {
@@ -641,12 +651,12 @@ static int ser_write_image_for_writer(struct seqwriter_data *writer, fits *image
 int ser_open_file(const char *filename, struct ser_struct *ser_file) {
 	if (ser_file->file) {
 		fprintf(stderr, "SER: file already opened, or badly closed\n");
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 	ser_file->file = g_fopen(filename, "r+b"); // now we can fix broken file, so not O_RDONLY anymore
 	if (ser_file->file == NULL) {
 		perror("SER file open");
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 #ifdef _OPENMP
 	omp_init_lock(&ser_file->fd_lock);
@@ -656,17 +666,18 @@ int ser_open_file(const char *filename, struct ser_struct *ser_file) {
 		fprintf(stderr, "SER: reading header failed, closing file %s\n",
 				filename);
 		ser_close_file(ser_file);
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
+
 	ser_file->filename = strdup(filename);
-	return 0;
+	return SER_OK;
 }
 
 int ser_close_file(struct ser_struct *ser_file) {
-	int retval = 0;
+	int retval = SER_OK;
 	user_warned = FALSE;
 	if (!ser_file)
-		return -1;
+		return SER_GENERIC_ERROR;
 	if (ser_file->file) {
 		retval = fclose(ser_file->file);
 		ser_file->file = NULL;
@@ -710,29 +721,29 @@ int ser_metadata_as_fits(struct ser_struct *ser_file, fits *fit) {
 		fit->naxes[2] = 3;
 		break;
 	default:
-		return 1;
+		return SER_GENERIC_ERROR;
 	}
 	fit->naxes[0] = fit->rx = ser_file->image_width;
 	fit->naxes[1] = fit->ry = ser_file->image_height;
 	fit->bitpix = (ser_file->byte_pixel_depth == SER_PIXEL_DEPTH_8) ? BYTE_IMG : USHORT_IMG;
 	fit->orig_bitpix = fit->bitpix;
 	fit->binning_x = fit->binning_y = 1;
-	return 0;
+	return SER_OK;
 }
 
 /* reads a frame on an already opened SER sequence.
  * frame number starts at 0 */
 int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolean force_float, gboolean open_debayer) {
-	int retval = 0, i, j, swap = 0;
+	int retval = SER_OK, i, j, swap = 0;
 	gint64 offset, frame_size;
 	size_t read_size;
 	WORD *olddata, *tmp;
 	if (!ser_file || ser_file->file == NULL || !ser_file->number_of_planes ||
 			!fit || frame_no < 0 || frame_no >= ser_file->frame_count)
-		return -1;
+		return SER_GENERIC_ERROR;
 
-	frame_size = ser_file->image_width * ser_file->image_height *
-			ser_file->number_of_planes;
+	frame_size = (gint64) ser_file->image_width * (gint64) ser_file->image_height *
+			(gint64) ser_file->number_of_planes;
 	read_size = frame_size * ser_file->byte_pixel_depth;
 
 	olddata = fit->data;
@@ -740,7 +751,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 		PRINT_ALLOC_ERR;
 		if (olddata)
 			free(olddata);
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
 	offset = SER_HEADER_LEN	+ frame_size *
@@ -752,16 +763,16 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 #endif
 	if ((gint64)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
 		perror("fseek in SER");
-		retval = -1;
+		retval = SER_GENERIC_ERROR;
 	} else {
 		if (fread(fit->data, 1, read_size, ser_file->file) != read_size)
-			retval = -1;
+			retval = SER_GENERIC_ERROR;
 	}
 #ifdef _OPENMP
 	omp_unset_lock(&ser_file->fd_lock);
 #endif
 	if (retval)
-		return -1;
+		return SER_GENERIC_ERROR;
 
 	ser_manage_endianess_and_depth(ser_file, fit->data, frame_size);
 
@@ -796,11 +807,10 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 		}
 	} else if (open_debayer && type_ser == SER_MONO && !com.pref.debayer.use_bayer_header) {
 		pattern = filter_pattern[com.pref.debayer.bayer_pattern];
-		type_ser = retrieveBayerPatternFromChar(pattern) + 8;
+		type_ser = get_cfa_pattern_index_from_string(pattern) + 8;
 	}
 	if (pattern) {
-		strcpy(fit->bayer_pattern, pattern);
-		strncpy(fit->row_order, "BOTTOM-UP", FLEN_VALUE - 1);
+		strncpy(fit->bayer_pattern, pattern, 70); // fixed char* length FLEN == 71, leave 1 char for the NULL
 	}
 
 	switch (type_ser) {
@@ -854,7 +864,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 		tmp = malloc(frame_size * sizeof(WORD));
 		if (!tmp) {
 			PRINT_ALLOC_ERR;
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 		memcpy(tmp, fit->data, sizeof(WORD) * frame_size);
 		fit->naxes[0] = fit->rx = ser_file->image_width;
@@ -877,7 +887,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 	case SER_BAYER_MYYC:
 	default:
 		siril_log_message(_("This type of Bayer pattern is not handled yet.\n"));
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
 	/* copy the SER timestamp to the fits */
@@ -902,8 +912,8 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 
 	fits_flip_top_to_bottom(fit);
 	fit->top_down = FALSE;
-
-	return 0;
+	snprintf(fit->row_order, FLEN_VALUE, "BOTTOM-UP");
+	return SER_OK;
 }
 
 /* multi-type cropping, works in constant space if needed */
@@ -949,7 +959,7 @@ int ser_read_frame(struct ser_struct *ser_file, int frame_no, fits *fit, gboolea
 static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 		WORD *outbuf, const rectangle *area, const int layer) {
 	gint64 offset, frame_size;
-	int retval = 0;
+	int retval = SER_OK;
 	WORD *read_buffer;
 	size_t read_size = ser_file->image_width * area->h * ser_file->byte_pixel_depth;
 	if (layer != -1) read_size *= 3;
@@ -959,12 +969,12 @@ static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 		read_buffer = malloc(read_size);
 		if (!read_buffer) {
 			PRINT_ALLOC_ERR;
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 	}
 	else read_buffer = outbuf;
 
-	frame_size = ser_file->image_width * ser_file->image_height *
+	frame_size = (gint64) ser_file->image_width * ser_file->image_height *
 		ser_file->number_of_planes * ser_file->byte_pixel_depth;
 
 #ifdef _OPENMP
@@ -977,10 +987,10 @@ static int read_area_from_image(struct ser_struct *ser_file, const int frame_no,
 
 	if ((gint64)-1 == fseek64(ser_file->file, offset, SEEK_SET)) {
 		perror("fseek in SER");
-		retval = -1;
+		retval = SER_GENERIC_ERROR;
 	} else {
 		if (fread(read_buffer, 1, read_size, ser_file->file) != read_size) {
-			retval = -1;
+			retval = SER_GENERIC_ERROR;
 		}
 	}
 #ifdef _OPENMP
@@ -1029,7 +1039,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 
 	if (!ser_file || ser_file->file == NULL || frame_no < 0
 			|| frame_no >= ser_file->frame_count)
-		return -1;
+		return SER_GENERIC_ERROR;
 
 	type_ser = ser_file->color_id;
 	if (!com.pref.debayer.open_debayer &&
@@ -1039,8 +1049,8 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 	switch (type_ser) {
 	case SER_MONO:
 		if (read_area_from_image(ser_file, frame_no, buffer, area, -1))
-			return -1;
-		ser_manage_endianess_and_depth(ser_file, buffer, area->w * area->h);
+			return SER_GENERIC_ERROR;
+		ser_manage_endianess_and_depth(ser_file, buffer, (gint64) area->w * area->h);
 		break;
 
 	case SER_BAYER_RGGB:
@@ -1074,7 +1084,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		}
 		if (layer < 0 || layer >= 3) {
 			siril_log_message(_("For a demosaiced image, layer has to be R, G or B (0 to 2).\n"));
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 
 		image_area = (rectangle) { .x = 0, .y = 0,
@@ -1085,13 +1095,13 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 		rawbuf = malloc(debayer_area.w * debayer_area.h * sizeof(WORD));
 		if (!rawbuf) {
 			PRINT_ALLOC_ERR;
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 		if (read_area_from_image(ser_file, frame_no, rawbuf, &debayer_area, -1)) {
 			free(rawbuf);
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
-		ser_manage_endianess_and_depth(ser_file, rawbuf, debayer_area.w * debayer_area.h);
+		ser_manage_endianess_and_depth(ser_file, rawbuf, (gint64) debayer_area.w * debayer_area.h);
 
 		/* for performance consideration (and many others) we force the interpolation algorithm
 		 * to be BAYER_BILINEAR
@@ -1100,7 +1110,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 				&debayer_area.h, BAYER_BILINEAR, com.pref.debayer.bayer_pattern, ser_file->bit_pixel_depth);
 		free(rawbuf);
 		if (!demosaiced_buf)
-			return -1;
+			return SER_GENERIC_ERROR;
 
 		/* area is the destination area.
 		 * debayer_area is the demosaiced buf area.
@@ -1109,7 +1119,7 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
         const int nbpixels = debayer_area.w * debayer_area.h;
 		for (y = 0; y < area->h; y++) {
 			for (x = 0; x < area->w; x++) {
-				buffer[y*area->w + x] = demosaiced_buf[layer * nbpixels + (yoffset+y)*debayer_area.w + xoffset+x]; 
+				buffer[y*area->w + x] = demosaiced_buf[layer * nbpixels + (yoffset+y)*debayer_area.w + xoffset+x];
 			}
 		}
 
@@ -1121,21 +1131,21 @@ int ser_read_opened_partial(struct ser_struct *ser_file, int layer,
 	case SER_RGB:
 		g_assert(ser_file->number_of_planes == 3);
 		if (read_area_from_image(ser_file, frame_no, buffer, area, layer))
-			return -1;
-		ser_manage_endianess_and_depth(ser_file, buffer, area->w * area->h);
+			return SER_GENERIC_ERROR;
+		ser_manage_endianess_and_depth(ser_file, buffer, (gint64) area->w * area->h);
 		break;
 	default:
 		siril_log_message(_("This type of Bayer pattern is not handled yet.\n"));
-		return -1;
+		return SER_GENERIC_ERROR;
 	}
 
-	return 0;
+	return SER_OK;
 }
 
 int ser_read_opened_partial_fits(struct ser_struct *ser_file, int layer,
 		int frame_no, fits *fit, const rectangle *area) {
 	if (new_fit_image(&fit, area->w, area->h, 1, DATA_USHORT))
-		return -1;
+		return SER_GENERIC_ERROR;
 	fit->top_down = TRUE;
 	if (ser_file->ts) {
 		GDateTime *timestamp = ser_timestamp_to_date_time(ser_file->ts[frame_no]);
@@ -1158,26 +1168,25 @@ int ser_write_frame_from_fit(struct ser_struct *ser_file, fits *fit, int frame_n
 // frame_no should always be the next image, or frame_count
 static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *fit, int frame_no) {
 	int pixel, plane, dest;
-	int ret, retval = 0;
+	int ret, retval = SER_OK;
 	gint64 offset, frame_size;
 	BYTE *data8 = NULL;	// for 8-bit files
 	WORD *data16 = NULL;	// for 16-bit files
 
 	if (!ser_file || ser_file->file == NULL || !fit)
-		return -1;
+		return SER_GENERIC_ERROR;
 	if (ser_file->number_of_planes == 0) {
 		// adding first frame of a new sequence, use it to populate the header
 		if (ser_write_header_from_fit(ser_file, fit)) {
-			return 1;
+			return SER_GENERIC_ERROR;
 		}
 	}
 	if (fit->rx != ser_file->image_width || fit->ry != ser_file->image_height) {
 		siril_log_message(_("Trying to add an image of different size in a SER\n"));
-		return 1;
+		return SER_GENERIC_ERROR;
 	}
 
-	fits_flip_top_to_bottom(fit);
-	frame_size = ser_file->image_width * ser_file->image_height *
+	frame_size = (gint64) ser_file->image_width * ser_file->image_height *
 		ser_file->number_of_planes;
 
 	offset = SER_HEADER_LEN	+ frame_size *
@@ -1187,13 +1196,13 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 		data8 = malloc(frame_size * ser_file->byte_pixel_depth);
 		if (!data8) {
 			PRINT_ALLOC_ERR;
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 	} else {
 		data16 = malloc(frame_size * ser_file->byte_pixel_depth);
 		if (!data16) {
 			PRINT_ALLOC_ERR;
-			return -1;
+			return SER_GENERIC_ERROR;
 		}
 	}
 
@@ -1221,7 +1230,7 @@ static int ser_write_frame_from_fit_internal(struct ser_struct *ser_file, fits *
 		omp_unset_lock(&ser_file->fd_lock);
 #endif
 		perror("seek");
-		retval = -1;
+		retval = SER_GENERIC_ERROR;
 		goto free_and_quit;
 	}
 
@@ -1280,7 +1289,7 @@ int import_metadata_from_serfile(struct ser_struct *ser_file, fits *to) {
 	strncpy(to->telescop, ser_file->telescope, FLEN_VALUE - 1);
 	if (ser_file->fps > 0.0)
 		to->exposure = 1.0 / ser_file->fps;
-	return 0;
+	return SER_OK;
 }
 
 static GdkPixbufDestroyNotify free_preview_data(guchar *pixels, gpointer data) {
@@ -1296,7 +1305,7 @@ static GdkPixbufDestroyNotify free_preview_data(guchar *pixels, gpointer data) {
  */
 GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 	GdkPixbuf *pixbuf = NULL;
-	int MAX_SIZE = com.pref.thumbnail_size;
+	int MAX_SIZE = com.pref.gui.thumbnail_size;
 	gchar *description = NULL;
 	int i, j, k, l, N, M;
 	int w, h, pixScale, Ws, Hs, n_channels, n_frames, bit;
@@ -1309,7 +1318,7 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 		return NULL;
 	}
 	float *pix = malloc(MAX_SIZE * sizeof(float));
-	float *ima_data = NULL, *ptr, byte, n, max, min, wd, avr;
+	float *ima_data = NULL, *ptr = NULL, byte, n, max, min, wd, avr;
 	guchar *pixbuf_data = NULL;
 
 	w = ser.image_width;
@@ -1330,7 +1339,11 @@ GdkPixbuf* get_thumbnail_from_ser(char *filename, gchar **descr) {
 	i = (int) ceil((float) w / MAX_SIZE);
 	j = (int) ceil((float) h / MAX_SIZE);
 	pixScale = (i > j) ? i : j;	// picture scale factor
-	if (pixScale == 0) return NULL;
+	if (pixScale == 0) {
+		free(ima_data);
+		free(pixbuf_data);
+		return NULL;
+	}
 	Ws = w / pixScale; 			// picture width in pixScale blocks
 	Hs = h / pixScale; 			// -//- height pixScale
 
