@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #include <string.h>
 #include <locale.h>
 #include <unistd.h>
+#include <fftw3.h>
 #ifdef OS_OSX
 #import <AppKit/AppKit.h>
 #if defined(ENABLE_RELOCATABLE_RESOURCES)
@@ -51,6 +52,7 @@
 #include "core/siril_app_dirs.h"
 #include "core/siril_language.h"
 #include "core/siril_update.h"
+#include "core/siril_log.h"
 #include "core/OS_utils.h"
 #include "algos/star_finder.h"
 #include "io/sequence.h"
@@ -66,12 +68,12 @@
 cominfo com;	// the core data struct
 guiinfo gui;	// the gui data struct
 fits gfit;	// currently loaded image
-const gchar *startup_cwd = NULL;
-gboolean forcecwd = FALSE;
 
 static gchar *main_option_directory = NULL;
 static gchar *main_option_script = NULL;
 static gchar *main_option_initfile = NULL;
+static gchar *main_option_rpipe_path = NULL;
+static gchar *main_option_wpipe_path = NULL;
 static gboolean main_option_pipe = FALSE;
 
 static gboolean _print_version_and_exit(const gchar *option_name,
@@ -104,6 +106,8 @@ static GOptionEntry main_option[] = {
 	{ "script", 's', 0, G_OPTION_ARG_FILENAME, &main_option_script, N_("run the siril commands script in console mode. If argument is equal to \"-\", then siril will read stdin input"), NULL },
 	{ "initfile", 'i', 0, G_OPTION_ARG_FILENAME, &main_option_initfile, N_("load configuration from file name instead of the default configuration file"), NULL },
 	{ "pipe", 'p', 0, G_OPTION_ARG_NONE, &main_option_pipe, N_("run in console mode with command and log stream through named pipes"), NULL },
+	{ "inpipe", 'r', 0, G_OPTION_ARG_FILENAME, &main_option_rpipe_path, N_("specify the path for the read pipe, the one receiving commands"), NULL },
+	{ "outpipe", 'w', 0, G_OPTION_ARG_FILENAME, &main_option_wpipe_path, N_("specify the path for the write pipe, the one outputing messages"), NULL },
 	{ "format", 'f', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_list_of_formats_and_exit, N_("print all supported image file formats (depending on installed libraries)" ), NULL },
 	{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_version_and_exit, N_("print the application’s version"), NULL},
 	{ "copyright", 'c', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, _print_copyright_and_exit, N_("print the copyright"), NULL},
@@ -123,59 +127,79 @@ static GActionEntry app_entries[] = {
 void load_glade_file() {
 	GError *err = NULL;
 	gchar* gladefile;
+	gboolean retval;
 
 	gladefile = g_build_filename(siril_get_system_data_dir(), GLADE_FILE, NULL);
 
 	/* try to load the glade file, from the sources defined above */
 	gui.builder = gtk_builder_new();
-
-	if (!gtk_builder_add_from_file(gui.builder, gladefile, &err)) {
+	// TODO: the following gtk_builder_add_from_file call is the source
+	// of libfontconfig memory leaks.
+	retval = gtk_builder_add_from_file(gui.builder, gladefile, &err);
+	if (!retval) {
 		g_error(_("%s was not found or contains errors, "
 					"cannot render GUI:\n%s\n Exiting.\n"), gladefile, err->message);
 		g_clear_error(&err);
 		exit(EXIT_FAILURE);
 	}
-	g_printf(_("Successfully loaded '%s'\n"), gladefile);
+	g_print(_("Successfully loaded '%s'\n"), gladefile);
 	g_free(gladefile);
 }
 
+#if defined (HAVE_FFTW3F_THREADS) && (_OPENMP)
+void parallel_loop(void *(*work)(char *), char *jobdata, size_t elsize, int njobs, void *data)
+{
+#pragma omp parallel for
+	for (int i = 0; i < njobs; ++i)
+		work(jobdata + elsize * i);
+}
+#endif
+
 static void global_initialization() {
-	com.selected_star = -1;
 	com.star_is_seqdata = FALSE;
 	com.stars = NULL;
-	com.qphot = NULL;
 	com.tilt = NULL;
 	com.uniq = NULL;
+	com.child_is_running = EXT_NONE;
+	com.kernel = NULL;
+	com.kernelsize = 0;
+	com.kernelchannels = 0;
+	com.num_gnuplot_handles = 0;
+	com.gnuplot_handles = NULL;
+#ifdef _WIN32
+	com.childhandle = NULL;
+#else
+	com.childpid = 0;
+#endif
 	memset(&com.selection, 0, sizeof(rectangle));
 	memset(com.layers_hist, 0, sizeof(com.layers_hist));
 
+	gui.selected_star = -1;
+	gui.qphot = NULL;
+	gui.draw_extra = NULL;
 	gui.cvport = RED_VPORT;
 	gui.show_excluded = TRUE;
-	//gui.color = NORMAL_COLOR;
 	gui.sliders = MINMAX;
 	gui.zoom_value = ZOOM_DEFAULT;
 	gui.ratio = 0.0;
+	gui.use_hd_remap = FALSE;
+	for (int i = 0; i < 3 ; i++)
+		gui.hd_remap_index[i] = NULL;
 
-	initialize_default_preferences();
-}
-
-static void init_num_procs() {
-	/* Get CPU number and set the number of threads */
-#ifdef _OPENMP
-	int num_proc = (int) g_get_num_processors();
-	int omp_num_proc = omp_get_num_procs();
-	if (num_proc != omp_num_proc) {
-		siril_log_message(_("Questionable parallel processing efficiency - openmp reports %d %s. "
-					"Possibly broken opencv/openblas installation.\n"),	omp_num_proc,
-				ngettext("processor", "processors", omp_num_proc));
-	}
-	siril_log_message(
-			_("Parallel processing %s: Using %d logical %s.\n"),
-			_("enabled"), com.max_thread = num_proc,
-			ngettext("processor", "processors", num_proc));
-#else
-	siril_log_message(_("Parallel processing %s: Using %d logical processor.\n"), _("disabled"), com.max_thread = 1);
+	initialize_default_settings();	// com.pref
+#ifdef HAVE_FFTW3F_MULTITHREAD
+	fprintf(stdout, _("Initializing FFTW multithreading support...\n"));
+	fftwf_init_threads(); // Should really only be called once so do it at startup
 #endif
+#if defined (HAVE_FFTW3F_THREADS) && (_OPENMP)
+	// If we are using FFTW built against pthreads but are using OpenMP for other aspects of the
+	// program, replace the parallel loop to avoid competing threads.
+	// See https://www.fftw.org/fftw3_doc/Usage-of-Multi_002dthreaded-FFTW.html
+	void fftw_threads_set_callback(
+		void (*parallel_loop)(void *(*work)(char *), char *jobdata, size_t elsize, int njobs,
+		void *data), void *data);
+#endif
+
 }
 
 static void siril_app_startup(GApplication *application) {
@@ -197,26 +221,17 @@ static void siril_app_startup(GApplication *application) {
 }
 
 static void siril_app_activate(GApplication *application) {
-	gchar *cwd_forced = NULL;
-
-	memset(&com, 0, sizeof(struct cominf));	// needed? doesn't hurt
-	com.initfile = NULL;
-
 	/* the first thing we need to do is to know if we are headless or not */
 	if (main_option_script || main_option_pipe) {
 		com.script = TRUE;
 		com.headless = TRUE;
-		/* need to force cwd to the current dir if no option -d */
-		if (!forcecwd) {
-			cwd_forced = g_strdup(g_get_current_dir());
-			forcecwd = TRUE;
-		}
 	}
+#if defined(_WIN32) && !defined(SIRIL_UNSTABLE)
+	ShowWindow(GetConsoleWindow(), SW_MINIMIZE); //hiding the console
+#endif
 
 	global_initialization();
 
-	/* initialize peaker variables */
-	init_peaker_default();
 	/* initialize sequence-related stuff */
 	initialize_sequence(&com.seq, TRUE);
 
@@ -224,7 +239,6 @@ static void siril_app_activate(GApplication *application) {
 
 	/* initialize converters (utilities used for different image types importing) */
 	gchar *supported_files = initialize_converters();
-	startup_cwd = g_get_current_dir();
 
 	if (main_option_initfile) {
 		com.initfile = g_strdup(main_option_initfile);
@@ -235,23 +249,33 @@ static void siril_app_activate(GApplication *application) {
 		exit(EXIT_FAILURE);
 	}
 
+	// After this point com.pref is populated
+
 	siril_language_parser_init();
-	if (com.pref.combo_lang)
-		language_init(com.pref.combo_lang);
+	if (com.pref.lang)
+		language_init(com.pref.lang);
 
 	if (main_option_directory) {
-		if (!g_path_is_absolute(main_option_directory)) {
+		gchar *cwd_forced;
+		if (!g_path_is_absolute(main_option_directory))
 			cwd_forced = g_build_filename(g_get_current_dir(), main_option_directory, NULL);
-		} else {
-			cwd_forced = g_strdup(main_option_directory);
-		}
-		forcecwd = TRUE;
-	}
+		else cwd_forced = g_strdup(main_option_directory);
 
-	if (forcecwd && cwd_forced) {
 		siril_change_dir(cwd_forced, NULL);
+		if (com.pref.wd)
+			g_free(com.pref.wd);
+		com.pref.wd = g_strdup(cwd_forced);
+		// if provided to the command line, make it persistent
 		g_free(cwd_forced);
 	}
+	else {
+		if (com.pref.wd && com.pref.wd[0] != '\0')
+			siril_change_dir(com.pref.wd, NULL);
+		else {
+			// no other option
+			siril_change_dir(siril_get_startup_dir(), NULL);
+		}
+	}
 
 	init_num_procs();
 
@@ -283,13 +307,13 @@ static void siril_app_activate(GApplication *application) {
 				exit(EXIT_FAILURE);
 			}
 		} else {
-			pipe_start();
-			read_pipe(NULL);
+			pipe_start(main_option_rpipe_path, main_option_wpipe_path);
+			read_pipe(main_option_rpipe_path);
 		}
 	}
 	if (!com.headless) {
 		/* Load preferred theme */
-		load_prefered_theme(com.pref.combo_theme);
+		load_prefered_theme(com.pref.gui.combo_theme);
 		/* Load the css sheet for general style */
 		load_css_style_sheet();
 		/* Load glade file */
@@ -298,33 +322,17 @@ static void siril_app_activate(GApplication *application) {
 		gtk_window_set_application(GTK_WINDOW(GTK_APPLICATION_WINDOW(lookup_widget("control_window"))), GTK_APPLICATION(application));
 		/* Load state of the main windows (position and maximized) */
 		load_main_window_state();
+#if defined(HAVE_JSON_GLIB) && defined(HAVE_NETWORKING)
 		/* Check for update */
-		if (com.pref.check_update)
+		if (com.pref.check_update) {
 			siril_check_updates(FALSE);
-#if 0 //we need to think about it
-		/* see https://gitlab.gnome.org/GNOME/gtk/issues/2342 */
-		NSEvent *focusevent;
-		g_warning("workaround for the GTK3 #2342 bug");
-		focusevent = [NSEvent
-			otherEventWithType: NSEventTypeAppKitDefined
-			location: NSZeroPoint
-			modifierFlags: 0x40
-			timestamp: 0
-			windowNumber: 0
-			context: nil
-			subtype: NSEventSubtypeApplicationActivated
-			data1: 0
-			data2: 0];
-
-		[NSApp postEvent:focusevent atStart:YES];
+		}
+#else
+		gtk_widget_set_visible(lookup_widget("main_menu_updates"), FALSE);
+		gtk_widget_set_visible(lookup_widget("frame24"), FALSE);
 #endif
 	}
 
-	if (siril_change_dir(com.wd, NULL)) {
-		com.wd = g_strdup(siril_get_startup_dir());
-		siril_change_dir(com.wd, NULL);
-	}
-
 	if (!com.headless) {
 		gtk_builder_connect_signals(gui.builder, NULL);
 		initialize_all_GUI(supported_files);
@@ -344,7 +352,7 @@ static void siril_app_open(GApplication *application, GFile **files, gint n_file
 		if (ext && !strncmp(ext, "seq", 4)) {
 			gchar *sequence_dir = g_path_get_dirname(path);
 			if (!siril_change_dir(sequence_dir, NULL)) {
-				if (check_seq(FALSE)) {
+				if (check_seq()) {
 					siril_log_message(_("No sequence `%s' found.\n"), path);
 				} else {
 					set_seq(path);
@@ -357,12 +365,10 @@ static void siril_app_open(GApplication *application, GFile **files, gint n_file
 			}
 		} else {
 			image_type type = get_type_from_filename(path);
-			if (!forcecwd && type != TYPEAVI && type != TYPESER && type != TYPEUNDEF) {
+			if (!main_option_directory && type != TYPEAVI && type != TYPESER && type != TYPEUNDEF) {
 				gchar *image_dir = g_path_get_dirname(path);
 				siril_change_dir(image_dir, NULL);
 				g_free(image_dir);
-			} else if (startup_cwd) {
-				siril_change_dir(startup_cwd, NULL);
 			}
 			if (!com.script)
 				set_GUI_CWD();
@@ -391,7 +397,7 @@ static void siril_macos_setenv(const char *progname) {
 
 		g_snprintf(tmp, sizeof(tmp), "%s/../Resources", app_dir);
 		if (realpath(tmp, lib_dir) && !stat(lib_dir, &sb) && S_ISDIR(sb.st_mode))
-			g_print("SiriL is started as MacOS application\n");
+			g_print("Siril is started as MacOS application\n");
 		else
 			return;
 
@@ -425,10 +431,12 @@ static void siril_macos_setenv(const char *progname) {
 		g_setenv("GDK_PIXBUF_MODULE_DIR", tmp, TRUE);
 		g_snprintf(tmp, sizeof(tmp), "%s/etc/fonts", lib_dir);
 		g_setenv("FONTCONFIG_PATH", tmp, TRUE);
+		g_snprintf(tmp, sizeof(tmp), "%s/etc/ca-certificates/cacert.pem", lib_dir);
+		g_setenv("CURL_CA_BUNDLE", tmp, TRUE);
 		if (g_getenv("HOME") != NULL) {
-			g_snprintf(tmp, sizeof(tmp), "%s/Library/Application Support", g_getenv("HOME"));
+			g_snprintf(tmp, sizeof(tmp), "%s/Library/Application Support/org.free-astro.Siril", g_getenv("HOME"));
 			g_setenv("XDG_CONFIG_HOME", tmp, TRUE);
-			g_snprintf (tmp, sizeof(tmp), "%s/Library/Application Support/SiriL/1.00/cache",
+			g_snprintf (tmp, sizeof(tmp), "%s/Library/Caches/org.free-astro.Siril",
 					g_getenv("HOME"));
 			g_setenv ("XDG_CACHE_HOME", tmp, TRUE);
 
@@ -466,6 +474,12 @@ int main(int argc, char *argv[]) {
 	SetErrorMode(SEM_FAILCRITICALERRORS | SEM_NOGPFAULTERRORBOX | SEM_NOOPENFILEERRORBOX);
 #endif
 
+#ifdef _WIN32
+	// Turn off buffering for output to console on MSYS terminal on Windows
+	setvbuf(stdout, NULL, _IONBF, 0);
+	setvbuf(stderr, NULL, _IONBF, 0);
+#endif
+
 	initialize_siril_directories();
 
 	dir = siril_get_locale_dir();
@@ -483,6 +497,17 @@ int main(int argc, char *argv[]) {
 	g_application_set_option_context_summary(G_APPLICATION(app), _("Siril - A free astronomical image processing software."));
 	g_application_add_main_option_entries(G_APPLICATION(app), main_option);
 
+	{	/* Setting the 'register-session' property is for macOS
+		 * - to enable DnD via dock icon
+		 * - to enable system menu "Quit"
+		 */
+		GValue value = G_VALUE_INIT;
+		g_value_init(&value, G_TYPE_BOOLEAN);
+		g_value_set_boolean(&value, TRUE);
+		g_object_set_property(G_OBJECT(app), "register-session", &value);
+		g_value_unset(&value);
+	}
+
 	status = g_application_run(G_APPLICATION(app), argc, argv);
 	if (status) {
 		gchar *help_msg;
