@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -33,12 +33,11 @@
 #include <opencv2/calib3d.hpp>
 
 #include "core/siril.h"
-#include "core/proto.h"
+#include "core/siril_log.h"
 #include "registration/registration.h"
 #include "registration/matching/misc.h"
 #include "registration/matching/atpmatch.h"
 #include "opencv.h"
-#include "opencv/ecc/ecc.h"
 
 #ifdef __cplusplus
 extern "C" {
@@ -49,14 +48,21 @@ extern "C" {
 #endif
 
 #define defaultRANSACReprojThreshold 3
+#define CLAMPING_FACTOR 0.98
 
 using namespace cv;
 
+static void convert_H_to_MatH(Homography *from, Mat &to);
+static void convert_MatH_to_H(Mat from, Homography *to);
+
 /* TODO:
  * fix memory leak
+ *
+ * Notice that functions are not static anymore as they are used
+ * in kombat.cpp; and "static" function are only valid within current
+ * translation unit in C++.
  */
-
-static WORD *fits_to_bgrbgr_ushort(fits *image) {
+WORD *fits_to_bgrbgr_ushort(fits *image) {
 	size_t ndata = image->rx * image->ry * 3;
 	WORD *bgrbgr = (WORD *)malloc(ndata * sizeof(WORD));
 	if (!bgrbgr) { PRINT_ALLOC_ERR; return NULL; }
@@ -68,7 +74,7 @@ static WORD *fits_to_bgrbgr_ushort(fits *image) {
 	return bgrbgr;
 }
 
-static float *fits_to_bgrbgr_float(fits *image) {
+float *fits_to_bgrbgr_float(fits *image) {
 	size_t ndata = image->rx * image->ry * 3;
 	float *bgrbgr = (float *)malloc(ndata * sizeof(float));
 	if (!bgrbgr) { PRINT_ALLOC_ERR; return NULL; }
@@ -226,17 +232,38 @@ static int Mat_to_image(fits *image, Mat *in, Mat *out, void *bgr, int target_rx
 	return 0;
 }
 
+// int guide image for clamping
+static void init_guide(fits *image, unsigned int target_rx, unsigned int target_ry, Mat *guide) {
+	if (image->type == DATA_USHORT) {
+		*guide = Mat(target_ry, target_rx, (image->naxes[2] == 1) ? CV_16UC1 : CV_16UC3, Scalar(0));
+	} else {
+		*guide = Mat(target_ry, target_rx, (image->naxes[2] == 1) ? CV_32FC1 : CV_32FC3, Scalar(0.0f));
+	}
+}
+
 /* resizes image to the sizes toX * toY, and stores it back in image */
-int cvResizeGaussian(fits *image, int toX, int toY, int interpolation) {
+int cvResizeGaussian(fits *image, int toX, int toY, int interpolation, gboolean clamp) {
 	Mat in, out;
 	void *bgr = NULL;
-
 	if (image_to_Mat(image, &in, &out, &bgr, toX, toY))
 		return 1;
 
 	// OpenCV function
 	resize(in, out, out.size(), 0, 0, interpolation);
 
+	if ((interpolation == OPENCV_LANCZOS4 || interpolation == OPENCV_CUBIC) && clamp) {
+		Mat guide, tmp1;
+		init_guide(image, toX, toY, &guide);
+		// Create guide image
+		resize(in, guide, out.size(), 0, 0, OPENCV_AREA);
+		tmp1 = (out < CLAMPING_FACTOR * guide);
+		Mat element = getStructuringElement(MORPH_ELLIPSE, Size(3, 3), Point(1,1));
+		dilate(tmp1, tmp1, element);
+
+		copyTo(guide, out, tmp1); // Guide copied to the clamped pixels
+		guide.release();
+		tmp1.release();
+	}
 	return Mat_to_image(image, &in, &out, bgr, toX, toY);
 }
 
@@ -246,126 +273,51 @@ void cvResizeArray(double *in, double *out, int inX, int inY, int outX, int outY
 	resize(in_mat, out_mat, out_mat.size(), 0, 0);
 }
 
-int cvRotateImage(fits *image, point center, double angle, int interpolation, int cropped) {
+// This function is now used only for mod90 rotations w/o interp
+int cvRotateImage(fits *image, int angle) {
 	Mat in, out;
 	void *bgr = NULL;
 	int target_rx = image->rx, target_ry = image->ry;
-	Rect frame;
-	Point2f pt(center.x, center.y);
 
-	/*  the angle will be mapped to the range of [-360, 360] */
-	angle = (fmod((angle / 90), 4)) * 90;
+	gboolean is_fast = (angle % 90 == 0);
+	if (!is_fast) return 1;
 
-	gboolean is_fast = fmod(angle, 90.0) == 0.0;
-	if (interpolation == -1)
-		assert(is_fast);
-
-	if (is_fast && (interpolation == -1 || !cropped)) {
-		if (fmod(angle, 180.0) != 0.0) {
-			target_rx = image->ry;
-			target_ry = image->rx;
-		}
-	}
-	else if (!cropped) {
-		frame = RotatedRect(pt, Size(image->rx, image->ry), angle).boundingRect();
-		target_rx = frame.width;
-		target_ry = frame.height;
-		siril_debug_print("after rotation, new image size will be %d x %d\n", target_rx, target_ry);
+	if (angle % 180 != 0.0) {
+		target_rx = image->ry;
+		target_ry = image->rx;
 	}
 
 	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
 		return 1;
 
-	if (is_fast && (interpolation == -1 || !cropped)) {	// fast rotation
-		/* flip third argument: how to flip the array; 0 means flipping around the
-		 * x-axis and positive value (for example, 1) means flipping around y-axis.
-		 * Negative value (for example, -1) means flipping around both axes. 
-		 */
-		if (angle == 90 || angle == -270) {
-			transpose(in, out);
-			flip(out, out, 0);
-		} else if (angle == 180 || angle == -180) {
-			flip(in, out, -1);
-		}
-		else { // 270, -90
-			transpose(in, out);
-			flip(out, out, 1);
-		}
-	} else {
-		Mat r = getRotationMatrix2D(pt, angle, 1.0);
-		if (cropped == 1) {
-			warpAffine(in, out, r, in.size(), interpolation);
-		} else {
-			// adjust transformation matrix
-			r.at<double>(0, 2) += frame.width / 2.0 - pt.x;
-			r.at<double>(1, 2) += frame.height / 2.0 - pt.y;
-
-			warpAffine(in, out, r, frame.size(), interpolation);
-		}
+	// fast rotation
+	/* flip third argument: how to flip the array; 0 means flipping around the
+	* x-axis and positive value (for example, 1) means flipping around y-axis.
+	* Negative value (for example, -1) means flipping around both axes.
+	*/
+	if (angle == 90 || angle == -270) {
+		transpose(in, out);
+		flip(out, out, 0);
+	} else if (angle == 180 || angle == -180) {
+		flip(in, out, -1);
+	}
+	else { // 270, -90
+		transpose(in, out);
+		flip(out, out, 1);
 	}
-
 	return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
 }
 
-void cvRotateImageRefPoint(fits *image, point center, double angle, int cropped, point refpointin, point *refpointout) {
-	Point2f pt(center.x, center.y);
-	Point3d refptin(refpointin.x, refpointin.y, 1);
+void cvTransformImageRefPoint(Homography Hom, point refpointin, point *refpointout) {
 	Mat refptout;
-	Rect frame;
-
-	if (!cropped) {
-		frame = RotatedRect(pt, Size(center.x *2, center.y *2), angle).boundingRect2f();
-	}
-
-	Mat r = getRotationMatrix2D(pt, angle, 1.0);
-	if (!cropped) {
-		// adjust transformation matrix
-		r.at<double>(0, 2) += frame.width / 2.0 - pt.x;
-		r.at<double>(1, 2) += frame.height / 2.0 - pt.y;
-	}
-	refptout = r * Mat(refptin);
+	Point3d refptin(refpointin.x, refpointin.y, 1);
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(&Hom, H);
+	refptout = H * Mat(refptin);
 	refpointout->x = refptout.at<double>(0, 0);
 	refpointout->y = refptout.at<double>(0, 1);
 }
 
-
-int cvAffineTransformation(fits *image, pointf *refpoints, pointf *curpoints, int nb_points, gboolean upscale2x, int interpolation) {
-	// see https://docs.opencv.org/3.4/d4/d61/tutorial_warp_affine.html
-	std::vector<Point2f> ref;
-	std::vector<Point2f> cur;
-
-	/* build vectors with lists of 3 stars. */
-	for (int i = 0; i < nb_points; i++) {
-		ref.push_back(Point2f(refpoints[i].x, image->ry - refpoints[i].y - 1));
-		cur.push_back(Point2f(curpoints[i].x, image->ry - curpoints[i].y - 1));
-	}
-
-	Mat m = estimateAffinePartial2D(cur, ref);
-	//std::cout << m << std::endl;
-
-	/* test that m is not a zero matrix */
-	if (countNonZero(m) < 1) {
-		siril_log_color_message(_("Singular Matrix. Cannot compute Affine Transformation.\n"), "red");
-		return -1;
-	}
-
-	Mat in, out;
-	void *bgr = NULL;
-	int target_rx = image->rx, target_ry = image->ry;
-	if (upscale2x) {
-		target_rx *= 2;
-		target_ry *= 2;
-		m *= 2;
-	}
-
-	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
-		return 1;
-
-	warpAffine(in, out, m, out.size(), interpolation, BORDER_TRANSPARENT);
-
-	return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
-}
-
 static void convert_H_to_MatH(Homography *from, Mat &to) {
 	to.at<double>(0, 0) = from->h00;
 	to.at<double>(0, 1) = from->h01;
@@ -390,33 +342,74 @@ static void convert_MatH_to_H(Mat from, Homography *to) {
 	to->h22 = from.at<double>(2, 2);
 }
 
+void cvGetEye(Homography *Hom) {
+	Mat M = Mat::eye(3, 3, CV_64FC1);
+	convert_MatH_to_H(M, Hom);
+}
+
+void cvTransfPoint(double *x, double *y, Homography Href, Homography Himg) {
+	Mat_<double> ref(3,1);
+	Mat_<double> dst;
+	Mat H0 = Mat(3, 3, CV_64FC1);
+	Mat H1 = Mat(3, 3, CV_64FC1);
+
+	ref(0,0) = *x;
+	ref(1,0) = *y;
+	ref(2,0) = 1.;
+	convert_H_to_MatH(&Href, H0);
+	convert_H_to_MatH(&Himg, H1);
+	if (cv::determinant(H1) == 0) return;
+	dst = H1.inv() * H0 * ref;
+	*x = dst(0,0);
+	*y = dst(1,0);
+}
+
+void cvTransfH(Homography Href, Homography Himg, Homography *Hres) {
+	Mat_<double> ref(3,1);
+	Mat_<double> dst;
+	Mat H0 = Mat(3, 3, CV_64FC1);
+	Mat H1 = Mat(3, 3, CV_64FC1);
+	Mat H2 = Mat(3, 3, CV_64FC1);
+
+	convert_H_to_MatH(&Href, H0);
+	convert_H_to_MatH(&Himg, H1);
+	H2 = H1.inv() * H0;
+	convert_MatH_to_H(H2, Hres);
+}
+
+// TODO: to be looked into when we sort out the conventions for astrometry (see #1103)
+
 unsigned char *cvCalculH(s_star *star_array_img,
-		struct s_star *star_array_ref, int n, Homography *Hom, transformation_type type) {
+		struct s_star *star_array_ref, int n, Homography *Hom, transformation_type type, float offset) {
 
 	std::vector<Point2f> ref;
 	std::vector<Point2f> img;
 	// needed for shift transform which uses estimateTranslation3D
+#ifdef HAVE_CV44
 	std::vector<Point3f> ref3;
 	std::vector<Point3f> img3;
-
+#endif
 	Mat H, a, mask, s;
 	unsigned char *ret = NULL;
 
-	/* build vectors with lists of stars. */
+	/* 	build vectors with lists of stars.
+		When defined with offset, the -0.5 term comes from the difference 
+		in convention between how we compute PSF
+		and opencv (zero coordinate at edge of pixel vs at center) */
 	switch (type) {
 	case SIMILARITY_TRANSFORMATION:
 	case HOMOGRAPHY_TRANSFORMATION:
-	case AFFINE_TRANSFORMATION:	
+	case AFFINE_TRANSFORMATION:
 		for (int i = 0; i < n; i++) {
-			ref.push_back(Point2f(star_array_ref[i].x - 0.5, star_array_ref[i].y - 0.5));
-			img.push_back(Point2f(star_array_img[i].x - 0.5, star_array_img[i].y - 0.5));
+			ref.push_back(Point2f(star_array_ref[i].x + offset, star_array_ref[i].y + offset));
+			img.push_back(Point2f(star_array_img[i].x + offset, star_array_img[i].y + offset));
 		}
 	break;
 #ifdef HAVE_CV44
 	case SHIFT_TRANSFORMATION:
 		for (int i = 0; i < n; i++) {
-			ref3.push_back(Point3f(star_array_ref[i].x - 0.5, star_array_ref[i].y - 0.5, 0.));
-			img3.push_back(Point3f(star_array_img[i].x - 0.5, star_array_img[i].y - 0.5, 0.));
+			ref3.push_back(Point3f(star_array_ref[i].x + offset, star_array_ref[i].y + offset, 0.));
+			img3.push_back(Point3f(star_array_img[i].x + offset, star_array_img[i].y + offset, 0.));
 		}
 	break;
 #endif
@@ -432,7 +425,7 @@ unsigned char *cvCalculH(s_star *star_array_img,
 		if (!s.cols) return NULL; // exit if could not find a match at all=> s is null
 		H = Mat::eye(3, 3, CV_64FC1);
 		H.at<double>(0,2) = s.at<double>(0);
-		H.at<double>(1,2) = s.at<double>(1);		
+		H.at<double>(1,2) = s.at<double>(1);
 	break;
 #endif
 	case SIMILARITY_TRANSFORMATION:
@@ -472,40 +465,53 @@ unsigned char *cvCalculH(s_star *star_array_img,
 }
 
 // transform an image using the homography.
-int cvTransformImage(fits *image, unsigned int width, unsigned int height, Homography Hom, gboolean upscale2x, int interpolation) {
-    Mat in, out;
-    void *bgr = NULL;
-    int target_rx = width, target_ry = height;
-    int source_ry = image->ry;
+int cvTransformImage(fits *image, unsigned int width, unsigned int height, Homography Hom, gboolean upscale2x, int interpolation, gboolean clamp) {
+	Mat in, out;
+	void *bgr = NULL;
+	int target_rx = width, target_ry = height;
+	int source_ry = image->ry;
 
-    if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
-        return 1;
+	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
+		return 1;
 
-    Mat H = Mat(3, 3, CV_64FC1);
-    convert_H_to_MatH(&Hom, H);
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(&Hom, H);
 
-    if (upscale2x) {
-        Mat S = Mat::eye(3, 3, CV_64FC1);
-        S.at<double>(0,0) = 2.0;
-        S.at<double>(1,1) = 2.0;
-        H = S * H;
-    }
+	if (upscale2x) {
+		Mat S = Mat::eye(3, 3, CV_64FC1);
+		S.at<double>(0,0) = 2.0;
+		S.at<double>(1,1) = 2.0;
+		H = S * H;
+	}
 
-    /* modify matrix for reverse Y axis */
-    Mat F1 = Mat::eye(3, 3, CV_64FC1);
-    F1.at<double>(1,1) = -1.0;
-    F1.at<double>(1,2) = source_ry - 1.0;
+	/* modify matrix for reverse Y axis */
+	Mat F1 = Mat::eye(3, 3, CV_64FC1);
+	F1.at<double>(1,1) = -1.0;
+	F1.at<double>(1,2) = source_ry - 1.0;
 
-    Mat F2 = Mat::eye(3, 3, CV_64FC1);
-    F2.at<double>(1,1) = -1.0;
-    F2.at<double>(1,2) = target_ry - 1.0;
+	Mat F2 = Mat::eye(3, 3, CV_64FC1);
+	F2.at<double>(1,1) = -1.0;
+	F2.at<double>(1,2) = target_ry - 1.0;
 
-    H = F2.inv() * H * F1;
+	H = F2.inv() * H * F1;
 
-    // OpenCV function
-    warpPerspective(in, out, H, Size(target_rx, target_ry), interpolation, BORDER_TRANSPARENT);
+	// OpenCV function
+	warpPerspective(in, out, H, Size(target_rx, target_ry), interpolation, BORDER_TRANSPARENT);
+	if ((interpolation == OPENCV_LANCZOS4 || interpolation == OPENCV_CUBIC) && clamp) {
+		Mat guide, tmp1;
+		init_guide(image, target_rx, target_ry, &guide);
+		// Create guide image
+		warpPerspective(in, guide, H, Size(target_rx, target_ry), OPENCV_AREA, BORDER_TRANSPARENT);
+		tmp1 = (out < guide * CLAMPING_FACTOR);
+		Mat element = getStructuringElement( MORPH_ELLIPSE,
+                       Size(3, 3), Point(-1,-1));
+		dilate(tmp1, tmp1, element);
 
-    return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
+		copyTo(guide, out, tmp1); // Guide copied to the clamped pixels
+		guide.release();
+		tmp1.release();
+	}
+	return Mat_to_image(image, &in, &out, bgr, target_rx, target_ry);
 }
 
 int cvUnsharpFilter(fits* image, double sigma, double amount) {
@@ -516,9 +522,14 @@ int cvUnsharpFilter(fits* image, double sigma, double amount) {
 	if (image_to_Mat(image, &in, &out, &bgr, target_rx, target_ry))
 		return 1;
 
+	//setUseOptimized(false);
+	//std::cout << "---- OpenCV setUseOptimize(false) ----" << std::endl;
+	//std::cout << getBuildInformation();
+
 	/* 3rd argument: Gaussian kernel size. When width and height are zeros
 	 * they are computed from sigma.
 	 */
+	siril_debug_print("using opencv GaussianBlur (CPU)\n");
 	GaussianBlur(in, out, Size(), sigma);
 	if (fabs(amount) > 0.0) {
 		out = in * (1 + amount) + out * (-amount);
@@ -743,4 +754,227 @@ int cvClahe(fits *image, double clip_limit, int size) {
 	return -1;
 }
 
+// https://igl.ethz.ch/projects/ARAP/svd_rot.pdf
+double cvCalculRigidTransform(s_star *star_array_in,
+		struct s_star *star_array_out, int n, Homography *Hom) {
 
+	Mat out = Mat(2, n, CV_64FC1);
+	Mat in = Mat(2, n, CV_64FC1);
+	Mat A = Mat(2, 2, CV_64FC1);
+	Mat B = Mat(2, 2, CV_64FC1);
+	Mat U = Mat(2, 2, CV_64FC1);
+	Mat V = Mat(2, 2, CV_64FC1);
+	Mat S = Mat(2, 2, CV_64FC1);
+	Mat T = Mat(2, 2, CV_64FC1);
+	Mat R = Mat(2, 2, CV_64FC1);
+	Mat H = Mat(3, 3, CV_64FC1);
+	Mat res = Mat(3, 1, CV_64FC1);
+	Mat outC = Mat(2, 1, CV_64FC1);
+	Mat inC = Mat(2, 1, CV_64FC1);
+	Mat shift = Mat(2, 1, CV_64FC1);
+
+	double outCx = 0., outCy = 0., inCx = 0., inCy = 0.;
+	/* the -0.5 term comes from the difference in convention between how we compute PSF
+	/ and opencv (zero coordinate at edge of pixel vs at center) */
+	for (int i = 0; i < n; i++) {
+		outCx += star_array_out[i].x - 0.5;
+		outCy += star_array_out[i].y - 0.5;
+		inCx += star_array_in[i].x - 0.5;
+		inCy += star_array_in[i].y - 0.5;
+		out.at<double>(0,i) = star_array_out[i].x - 0.5;
+		out.at<double>(1,i) = star_array_out[i].y - 0.5;
+		in.at<double>(0,i) = star_array_in[i].x - 0.5;
+		in.at<double>(1,i) = star_array_in[i].y - 0.5;
+	}
+	outCx /= n;
+	outCy /= n;
+	inCx /= n;
+	inCy /= n;
+	outC.at<double>(0,0) = outCx;
+	outC.at<double>(1,0) = outCy;
+	inC.at<double>(0,0) = inCx;
+	inC.at<double>(1,0) = inCy;
+
+	for (int i = 0; i < n; i++) {
+		out.at<double>(0,i) -= outCx;
+		out.at<double>(1,i) -= outCy;
+		in.at<double>(0,i) -= inCx;
+		in.at<double>(1,i) -= inCy;
+	}
+	// std::cout << "out\n" << out << std::endl;
+	// std::cout << "in\n" << in << std::endl;
+
+	A = out * Mat::eye(n, n, CV_64FC1) * in.t();
+	B = A.t() * A;
+
+	// std::cout << "A\n" << A << std::endl;
+	// std::cout << "B\n" << B << std::endl;
+
+	double b = -(B.at<double>(0,0) + B.at<double>(1,1));
+	double c = cv::determinant(B);
+
+	double l1 = (-b + sqrt(b * b - 4 *c)) * 0.5;
+	double l2 = (-b - sqrt(b * b - 4 *c)) * 0.5;
+
+	double v1 = B.at<double>(0,1) / (l1 - B.at<double>(0,0));
+	double v2 = B.at<double>(0,1) / (l2 - B.at<double>(0,0));
+	// std::cout << "l1\n" << l1 << std::endl;
+	// std::cout << "l2\n" << l2 << std::endl;
+
+	double n1 = sqrt( 1 + v1 * v1);
+	double n2 = sqrt( 1 + v2 * v2);
+
+	V.at<double>(0,0) = v1 / n1;
+	V.at<double>(1,0) = 1. / n1;
+	V.at<double>(0,1) = v2 / n2;
+	V.at<double>(1,1) = 1. / n2;
+	// std::cout << "V\n" << V << std::endl;
+
+	if (n == 3) {
+		S = Mat::eye(2, 2, CV_64FC1);
+		S.at<double>(0,0) = 1. / sqrt(l1);
+		S.at<double>(1,1) = 1. / sqrt(l2);
+		// std::cout << "S\n" << S << std::endl;
+		U = (S * (A * V).t()).t();
+	} else {
+		U = A * V / sqrt(l1);
+		U.at<double>(0,1) = U.at<double>(1,0);
+		U.at<double>(1,1) = -U.at<double>(0,0);
+	}
+	// std::cout << "U\n" << U << std::endl;
+
+	T = Mat::eye(2, 2, CV_64FC1);
+	T.at<double>(1,1) = cv:: determinant(V * U.t());
+	// std::cout << "T\n" << T << std::endl;
+
+	R = V * T * U.t();
+	// std::cout << "R\n" << R << std::endl;
+	shift = inC - R * outC;
+	// std::cout << "s\n" << shift << std::endl;
+
+
+	H = Mat::eye(3, 3, CV_64FC1);
+	R.copyTo(H(cv::Rect_<int>(0,0,2,2)));
+	H.at<double>(0,2) = shift.at<double>(0,0);
+	H.at<double>(1,2) = shift.at<double>(1,0);
+	// std::cout << "H\n" << H << std::endl;
+
+	// computing the maximum error
+	double err = 0., norm2;
+	for (int i = 0; i < n; i++) {
+		res = H * (Mat_<double>(3,1) << star_array_out[i].x - 0.5, star_array_out[i].y - 0.5, 1) - (Mat_<double>(3,1) << star_array_in[i].x - 0.5, star_array_in[i].y - 0.5, 1);
+		norm2 = cv::norm(res);
+		if (norm2 > err) err = norm2;
+	}
+
+	Hom->Inliers = n;
+	convert_MatH_to_H(H, Hom);
+
+	return err;
+
+}
+
+void cvMultH(Homography H1, Homography H2, Homography *Hout) {
+	Mat _H1 = Mat(3, 3, CV_64FC1);
+	Mat _H2 = Mat(3, 3, CV_64FC1);
+	Mat _H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(&H1, _H1);
+	convert_H_to_MatH(&H2, _H2);
+	_H = _H1 * _H2;
+	convert_MatH_to_H(_H, Hout);
+}
+
+void cvGetMatrixReframe(double x, double y, int w, int h, double angle, Homography *Hom) {
+	double dx = x + (double)w * 0.5;
+	double dy = y + (double)h * 0.5;
+	Point2f pt(0., 0.);
+
+	// shift to get to the center of the initial image
+	Mat S =  Mat::eye(3, 3, CV_64FC1);
+	S.at<double>(0, 2) = dx;
+	S.at<double>(1, 2) = dy;
+
+	// shift backwards to set the top-left point of the final image
+	Mat S2 =  Mat::eye(3, 3, CV_64FC1);
+	S2.at<double>(0, 2) = -(double)w * 0.5;
+	S2.at<double>(1, 2) = -(double)h * 0.5;
+
+	// get rot matrix about origin {0, 0}
+	Mat r = getRotationMatrix2D(pt, angle, 1.0);
+	Mat H = Mat::eye(3, 3, CV_64FC1);
+	r.copyTo(H(cv::Rect_<int>(0,0,3,2))); //slicing is (x, y, w, h)
+	// std::cout << H << std::endl;
+
+	H = S * H * S2;
+	std::cout << H << std::endl;
+
+	// transform is final to orginal, we need to inverse
+	// to have H from original to final
+	H = H.inv();
+	std::cout << H << std::endl;
+
+	convert_MatH_to_H(H, Hom);
+}
+
+void cvGetMatrixResize(double cxin, double cyin, double cxout, double cyout, double scale, Homography *Hom) {
+
+	// shift to get to the center of the initial image
+	Mat S =  Mat::eye(3, 3, CV_64FC1);
+	S.at<double>(0, 2) = cxin;
+	S.at<double>(1, 2) = cyin;
+
+	// shift backwards to set the top-left point of the final image
+	Mat S2 =  Mat::eye(3, 3, CV_64FC1);
+	S2.at<double>(0, 2) = -cxout;
+	S2.at<double>(1, 2) = -cyout;
+
+	// get scaling matrix about origin {0, 0}
+	Mat H = Mat::eye(3, 3, CV_64FC1);
+	H.at<double>(0, 0) = scale;
+	H.at<double>(1, 1) = scale;
+	// std::cout << H << std::endl;
+
+	H = S * H * S2;
+	std::cout << H << std::endl;
+
+	// transform is final to orginal, we need to inverse
+	// to have H from original to final
+	H = H.inv();
+	std::cout << H << std::endl;
+
+	convert_MatH_to_H(H, Hom);
+}
+
+void cvGetBoundingRectSize(fits *image, point center, double angle, int *w, int *h) {
+	Rect frame;
+	Point2f pt(center.x, center.y);
+	frame = RotatedRect(pt, Size(image->rx, image->ry), angle).boundingRect();
+	siril_debug_print("after rotation, new image size will be %d x %d\n", frame.width, frame.height);
+	*w = frame.width;
+	*h = frame.height;
+}
+
+void cvInvertH(Homography *Hom) {
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Hom, H);
+	H = H.inv();
+	convert_MatH_to_H(H, Hom);
+	// std::cout << "H" << std::endl;
+	// std::cout << H << std::endl;
+}
+
+void cvApplyFlips(Homography *Hom, int source_ry, int target_ry) {
+	Mat H = Mat(3, 3, CV_64FC1);
+	convert_H_to_MatH(Hom, H);
+	/* modify matrix for reverse Y axis */
+	Mat F1 = Mat::eye(3, 3, CV_64FC1);
+	F1.at<double>(1,1) = -1.0;
+	F1.at<double>(1,2) = source_ry - 1.0;
+
+	Mat F2 = Mat::eye(3, 3, CV_64FC1);
+	F2.at<double>(1,1) = -1.0;
+	F2.at<double>(1,2) = target_ry - 1.0;
+
+	H = F2.inv() * H * F1;
+	convert_MatH_to_H(H, Hom);
+}
