@@ -1,8 +1,8 @@
 #ifndef SRC_REGISTRATION_MATCHING_PROJECT_COORDS_H_
 #define SRC_REGISTRATION_MATCHING_PROJECT_COORDS_H_
 
-#include <stdio.h>
+#include <glib.h>
 
-int convert_catalog_coords(const char *fileA, SirilWorldCS *coord, GFile *out);
+int convert_catalog_coords(GFile *fileA, SirilWorldCS *coord, GFile *out);
 
 #endif /* SRC_REGISTRATION_MATCHING_PROJECT_COORDS_H_ */
