@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2022 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2023 team free-astro (see more in AUTHORS file)
  * Reference site is https://free-astro.org/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 #include "core/siril.h"
 #include "core/proto.h"
 #include "core/OS_utils.h"
-#include "core/siril_date.h"
+#include "core/siril_log.h"
 #include "io/sequence.h"
 #include "io/ser.h"
 #include "io/image_format_fits.h"
@@ -35,31 +35,7 @@
 #include "algos/statistics.h"
 #include "stacking/stacking.h"
 #include "stacking/siril_fit_linear.h"
-
-typedef struct {
-	GDateTime *date_obs;
-	gdouble exposure;
-} DateEvent;
-
-static void free_list_date(gpointer data) {
-	DateEvent *item = (DateEvent *)data;
-	g_date_time_unref(item->date_obs);
-	g_slice_free(DateEvent, item);
-}
-
-static DateEvent* new_date_item(GDateTime *dt, gdouble exposure) {
-	DateEvent *item = g_slice_new(DateEvent);
-	item->exposure = exposure;
-	item->date_obs = dt;
-	return item;
-}
-
-static gint list_date_compare(gconstpointer *a, gconstpointer *b) {
-	const DateEvent *dt1 = (const DateEvent *) a;
-	const DateEvent *dt2 = (const DateEvent *) b;
-
-	return g_date_time_compare(dt1->date_obs, dt2->date_obs);
-}
+#include "registration/registration.h"
 
 static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean);
 
@@ -92,8 +68,7 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 	set_progress_bar_data(_("Opening images for stacking"), PROGRESS_NONE);
 
 	if (args->seq->type == SEQ_REGULAR || args->seq->type == SEQ_FITSEQ) {
-		if (args->apply_weight) {
-			int nb_frames = args->nb_images_to_stack;
+		if (args->apply_nbstack_weights) {
 			int nb_layers = args->seq->nb_layers;
 			args->weights = malloc(nb_layers * nb_frames * sizeof(double));
 		}
@@ -147,7 +122,7 @@ int stack_open_all_files(struct stacking_args *args, int *bitpix, int *naxis, lo
 			if (image_index == args->ref_image)
 				import_metadata_from_fitsfile(fptr, fit);
 
-			if (args->apply_weight) {
+			if (args->apply_nbstack_weights) {
 				int nb_layers = args->seq->nb_layers;
 				double weight = (double)stack_count;
 				siril_debug_print("weight for image %d: %d\n", i, stack_count);
@@ -245,7 +220,7 @@ static int refine_blocks_candidate(int nb_threads, int nb_channels, int minimum_
  * threads to work but as big as possible for the available memory.
  */
 int stack_compute_parallel_blocks(struct _image_block **blocksptr, long max_number_of_rows,
-		long naxes[3], int nb_threads, long *largest_block_height, int *nb_blocks) {
+		const long naxes[3], int nb_threads, long *largest_block_height, int *nb_blocks) {
 	int candidate = nb_threads;	// candidate number of blocks
 	if (nb_threads < 1 || max_number_of_rows < 1)
 		return ST_GENERIC_ERROR;
@@ -307,7 +282,7 @@ int stack_compute_parallel_blocks(struct _image_block **blocksptr, long max_numb
 	return ST_OK;
 }
 
-static void stack_read_block_data(struct stacking_args *args, int use_regdata,
+static int stack_read_block_data(struct stacking_args *args,
 		struct _image_block *my_block, struct _data_block *data,
 		long *naxes, data_type itype, int thread_id) {
 
@@ -321,19 +296,20 @@ static void stack_read_block_data(struct stacking_args *args, int use_regdata,
 		/* area in C coordinates, starting with 0, not cfitsio coordinates. */
 		rectangle area = {0, my_block->start_row, naxes[0], my_block->height};
 
-		if (!get_thread_run()) {
-			return;
-		}
-		if (use_regdata && args->reglayer >= 0) {
+		if (!get_thread_run())
+			return ST_CANCEL;
+
+		if (args->reglayer >= 0) {
 			/* Load registration data for current image and modify area.
 			 * Here, only the y shift is managed. If possible, the remaining part
 			 * of the original area is read, the rest is filled with zeros. The x
 			 * shift is managed in the main loop after the read. */
 			regdata *layerparam = args->seq->regparam[args->reglayer];
 			if (layerparam) {
-				int shifty = round_to_int(
-						layerparam[args->image_indices[frame]].shifty *
-						args->seq->upscale_at_stacking);
+				double scale = args->seq->upscale_at_stacking;
+				double dx, dy;
+				translation_from_H(layerparam[args->image_indices[frame]].H, &dx, &dy);
+				int shifty = round_to_int(dy * scale);
 #ifdef STACK_DEBUG
 				fprintf(stdout, "shifty for image %d: %d\n", args->image_indices[frame], shifty);
 #endif
@@ -367,7 +343,7 @@ static void stack_read_block_data(struct stacking_args *args, int use_regdata,
 			}
 		}
 
-		if (!use_regdata || readdata) {
+		if (args->reglayer < 0 || readdata) {
 			// reading pixels from current frame
 			void *buffer;
 			if (itype == DATA_FLOAT)
@@ -381,10 +357,11 @@ static void stack_read_block_data(struct stacking_args *args, int use_regdata,
 				if (tid == 0)
 #endif
 					siril_log_color_message(_("Error reading one of the image areas\n"), "red");
-				break;
+				return ST_SEQUENCE_ERROR;
 			}
 		}
 	}
+	return ST_OK;
 }
 
 static void normalize_to16bit(int bitpix, double *mean) {
@@ -398,8 +375,8 @@ static void normalize_to16bit(int bitpix, double *mean) {
 }
 
 static void norm_to_0_1_range(fits *fit) {
-	float mini = fit->fdata[0];
-	float maxi = fit->fdata[0];
+	float mini = FLT_MAX;
+	float maxi = -1.f * FLT_MAX;
 	long n = fit->naxes[0] * fit->naxes[1] * fit->naxes[2];
 
 	/* search for min / max */
@@ -408,6 +385,8 @@ static void norm_to_0_1_range(fits *fit) {
 #endif
 	for (int i = 1; i < n; i++) {
 		float tmp = fit->fdata[i];
+		if (tmp == 0.f)
+			continue;
 		if (tmp < mini)
 			mini = tmp;
 		if (tmp > maxi)
@@ -418,7 +397,7 @@ static void norm_to_0_1_range(fits *fit) {
 #pragma omp parallel for num_threads(com.max_thread) schedule(static)
 #endif
 	for (int i = 0; i < n; i++) {
-		fit->fdata[i] = (fit->fdata[i] - mini) / (maxi - mini);
+		fit->fdata[i] = (fit->fdata[i] == 0.f) ? 0.f : (fit->fdata[i] - mini) / (maxi - mini);
 	}
 }
 
@@ -428,7 +407,7 @@ static void norm_to_0_1_range(fits *fit) {
  * does a different operation to keep the final pixel values.
  *********************************************************************************/
 
-static int percentile_clipping(WORD pixel, float sig[], float median, guint64 rej[]) {
+static int percentile_clipping(WORD pixel, const float sig[], float median, int rej[]) {
 	float plow = sig[0];
 	float phigh = sig[1];
 
@@ -446,7 +425,7 @@ static int percentile_clipping(WORD pixel, float sig[], float median, guint64 re
 /* Rejection of pixels, following sigma_(high/low) * sigma.
  * The function returns 0 if no rejections are required, 1 if it's a high
  * rejection and -1 for a low-rejection */
-static int sigma_clipping(WORD pixel, float sig[], float sigma, float median, guint64 rej[]) {
+static int sigma_clipping(WORD pixel, const float sig[], float sigma, float median, int rej[]) {
 	float sigmalow = sig[0];
 	float sigmahigh = sig[1];
 
@@ -468,7 +447,7 @@ static void Winsorize(WORD *pixel, WORD m0, WORD m1, int N) {
 	}
 }
 
-static int line_clipping(WORD pixel, float sig[], float sigma, int i, float a, float b, guint64 rej[]) {
+static int line_clipping(WORD pixel, const float sig[], float sigma, int i, float a, float b, int rej[]) {
 	float sigmalow = sig[0];
 	float sigmahigh = sig[1];
 
@@ -523,8 +502,7 @@ int check_G_values(float Gs, float Gc) {
 	return (Gs > Gc);
 }
 
-void confirm_outliers(struct outliers *out, int N, double median, int *rejected,
-		guint64 rej[2]) {
+void confirm_outliers(struct outliers *out, int N, double median, int *rejected, int rej[2]) {
 	int i = N - 1;
 
 	while (i > 1 && !out[i].out) {
@@ -542,7 +520,7 @@ void confirm_outliers(struct outliers *out, int N, double median, int *rejected,
 	}
 }
 
-static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struct stacking_args *args, guint64 crej[2]) {
+static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struct stacking_args *args, int rej[2]) {
 	int N = nb_frames;	// N is the number of pixels kept from the current stack
 	float median = 0.f;
 	int pixel, output, changed, n, r = 0;
@@ -591,7 +569,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 	switch (args->type_of_rejection) {
 		case PERCENTILE:
 			for (int frame = 0; frame < N; frame++) {
-				rejected[frame] = percentile_clipping(stack[frame], args->sig, median, crej);
+				rejected[frame] = percentile_clipping(stack[frame], args->sig, median, rej);
 			}
 
 			for (pixel = 0, output = 0; pixel < N; pixel++) {
@@ -611,7 +589,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 				if (args->type_of_rejection == SIGMA)
 					var = args->sd_calculator(stack, N);
 				else
-					var = args->mad_calculator(stack, N, median, FALSE);
+					var = args->mad_calculator(stack, N, median, SINGLE_THREADED);
 
 				if (!firstloop) {
 					median = quickmedian(stack, N);
@@ -623,7 +601,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 						// no more rejections
 						rejected[frame] = 0;
 					} else {
-						rejected[frame] = sigma_clipping(stack[frame], args->sig, var, median, crej);
+						rejected[frame] = sigma_clipping(stack[frame], args->sig, var, median, rej);
 						if (rejected[frame]) {
 							r++;
 						}
@@ -651,7 +629,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 				}
 				n = 0;
 				for (int frame = 0; frame < N; frame++) {
-					if (sigma_clipping(stack[frame], args->sig, sigma, median, crej)) {
+					if (sigma_clipping(stack[frame], args->sig, sigma, median, rej)) {
 						stack[frame] = median;
 						n++;
 					}
@@ -678,7 +656,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 						rejected[frame] = 0;
 					} else {
 						rejected[frame] = sigma_clipping(stack[frame],
-								args->sig, sigma, median, crej);
+								args->sig, sigma, median, rej);
 						if (rejected[frame] != 0)
 							r++;
 					}
@@ -713,7 +691,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 						rejected[frame] = 0;
 					} else {
 						rejected[frame] =
-							line_clipping(stack[frame], args->sig, sigma, frame, a, b, crej);
+							line_clipping(stack[frame], args->sig, sigma, frame, a, b, rej);
 						if (rejected[frame] != 0)
 							r++;
 					}
@@ -763,7 +741,7 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 				out[iter].i = (max_index == 0) ? cold++ : max_index;
 				remove_element(w_stack, max_index, size);
 			}
-			confirm_outliers(out, max_outliers, median, rejected, crej);
+			confirm_outliers(out, max_outliers, median, rejected, rej);
 			free(out);
 
 			for (pixel = 0, output = 0; pixel < N; pixel++) {
@@ -784,16 +762,16 @@ static int apply_rejection_ushort(struct _data_block *data, int nb_frames, struc
 }
 
 static double mean_and_reject(struct stacking_args *args, struct _data_block *data,
-		int stack_size, data_type itype, guint64 crej[2]) {
+		int stack_size, data_type itype, int rej[2]) {
 	double mean;
 
 	int layer = data->layer;
 	if (itype == DATA_USHORT) {
-		int kept_pixels = apply_rejection_ushort(data, stack_size, args, crej);
+		int kept_pixels = apply_rejection_ushort(data, stack_size, args, rej);
 		if (kept_pixels == 0)
 			mean = quickmedian(data->stack, stack_size);
 		else {
-			if (args->apply_weight) {
+			if (args->apply_noise_weights || args->apply_nbstack_weights || args->apply_wfwhm_weights || args->apply_nbstars_weights) {
 				double *pweights = args->weights + layer * stack_size;
 				WORD pmin = 65535, pmax = 0; /* min and max computed here instead of rejection step to avoid dealing with too many particular cases */
 				for (int frame = 0; frame < kept_pixels; ++frame) {
@@ -811,7 +789,9 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 						norm += pweights[frame];
 					}
 				}
-				mean = sum / norm;
+				if (norm == 0.0)
+					mean = sum / (double)kept_pixels;
+				else mean = sum / norm;
 			} else {
 				gint64 sum = 0L;
 				for (int frame = 0; frame < kept_pixels; ++frame) {
@@ -821,11 +801,11 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 			}
 		}
 	} else {
-		int kept_pixels = apply_rejection_float(data, stack_size, args, crej);
+		int kept_pixels = apply_rejection_float(data, stack_size, args, rej);
 		if (kept_pixels == 0)
 			mean = quickmedian_float(data->stack, stack_size);
 		else {
-			if (args->apply_weight) {
+			if (args->apply_noise_weights || args->apply_nbstack_weights || args->apply_wfwhm_weights || args->apply_nbstars_weights) {
 				double *pweights = args->weights + layer * stack_size;
 				float pmin = 10000.0, pmax = -10000.0; /* min and max computed here instead of rejection step to avoid dealing with too many particular cases */
 				for (int frame = 0; frame < kept_pixels; ++frame) {
@@ -842,7 +822,9 @@ static double mean_and_reject(struct stacking_args *args, struct _data_block *da
 						norm += pweights[frame];
 					}
 				}
-			mean = sum / norm;
+				if (norm == 0.0)
+					mean = sum / (double)kept_pixels;
+				else mean = sum / norm;
 			} else {
 				double sum = 0.0;
 				for (int frame = 0; frame < kept_pixels; ++frame) {
@@ -863,7 +845,7 @@ int stack_median(struct stacking_args *args) {
 	return stack_mean_or_median(args, FALSE);
 }
 
-static int compute_weights(struct stacking_args *args) {
+static int compute_noise_weights(struct stacking_args *args) {
 	int nb_frames = args->nb_images_to_stack;
 	int nb_layers = args->seq->nb_layers;
 
@@ -889,50 +871,106 @@ static int compute_weights(struct stacking_args *args) {
 	return ST_OK;
 }
 
-/* computes the observation date (beginning) and the start of exposure (same
- * but in julian date) and end of exposure (start of last shot + exposure time)
- */
-static void compute_date_time_keywords(GList *list_date, fits *fit) {
-	if (!list_date)
-		return;
-	GDateTime *date_obs;
-	gdouble start, end;
-	/* First we want to sort the list */
-	list_date = g_list_sort(list_date, (GCompareFunc) list_date_compare);
+static int compute_wfwhm_weights(struct stacking_args *args) {
+	int nb_frames = args->nb_images_to_stack;
+	int nb_layers = args->seq->nb_layers;
+	double fwhmmin = DBL_MAX;
+	double fwhmmax = -DBL_MAX;
+	double invdenom, invfwhmax2;
 
-	/* go to the first stacked image and get needed values */
-	list_date = g_list_first(list_date);
-	date_obs = g_date_time_ref(((DateEvent *)list_date->data)->date_obs);
-	start = date_time_to_Julian(((DateEvent *)list_date->data)->date_obs);
+	if (!layer_has_registration(args->seq, args->reglayer)) {
+		siril_log_color_message(_("Sequence does not have registration info, cannot use weighing by %s, aborting\n"), "red", "wFWHM");
+		return ST_GENERIC_ERROR;
+	}
 
-	/* go to the last stacked image and get needed values
-	 * This time we need to add the exposure to the date_obs
-	 * to exactly retrieve the end of the exposure
-	 */
-	list_date = g_list_last(list_date);
-	gdouble last_exp = ((DateEvent *)list_date->data)->exposure;
-	GDateTime *last_date = ((DateEvent *)list_date->data)->date_obs;
-	GDateTime *corrected_last_date = g_date_time_add_seconds(last_date, (gdouble) last_exp);
+	args->weights = malloc(nb_layers * nb_frames * sizeof(double));
+	double *pweights[3];
 
-	end = date_time_to_Julian(corrected_last_date);
+	for (int i = 0; i < args->nb_images_to_stack; ++i) {
+		int idx = args->image_indices[i];
+		if (args->seq->regparam[args->reglayer][idx].weighted_fwhm < fwhmmin && args->seq->regparam[args->reglayer][idx].weighted_fwhm > 0) fwhmmin = args->seq->regparam[args->reglayer][idx].weighted_fwhm;
+		if (args->seq->regparam[args->reglayer][idx].weighted_fwhm > fwhmmax) fwhmmax = args->seq->regparam[args->reglayer][idx].weighted_fwhm;
+	}
+	invdenom = 1. / (1. / (fwhmmin * fwhmmin) - 1. / (fwhmmax * fwhmmax));
+	invfwhmax2 = 1. / (fwhmmax * fwhmmax);
 
-	g_date_time_unref(corrected_last_date);
+	for (int layer = 0; layer < nb_layers; ++layer) {
+		double norm = 0.0;
+		pweights[layer] = args->weights + layer * nb_frames;
+		for (int i = 0; i < args->nb_images_to_stack; ++i) {
+			int idx = args->image_indices[i];
+			if (args->seq->regparam[args->reglayer][idx].weighted_fwhm > 0) {
+				pweights[layer][i] = (1. / (args->seq->regparam[args->reglayer][idx].weighted_fwhm * args->seq->regparam[args->reglayer][idx].weighted_fwhm) - invfwhmax2) * invdenom;
+				norm += pweights[layer][i];
+			} else {
+				pweights[layer][i] = 0.;
+			}
+		}
+		norm /= (double) nb_frames;
 
-	/* we address the computed values to the keywords */
-	fit->date_obs = date_obs;
-	fit->expstart = start;
-	fit->expend = end;
+		for (int i = 0; i < args->nb_images_to_stack; i++) {
+			pweights[layer][i] /= norm;
+			siril_debug_print("Image #%d - Layer %d - wFWHM: %3.2f - weight: %3.2f\n", args->image_indices[i], layer, args->seq->regparam[args->reglayer][args->image_indices[i]].weighted_fwhm, pweights[layer][i]);
+		}
+	}
+	return ST_OK;
+}
+
+static int compute_nbstars_weights(struct stacking_args *args) {
+	int nb_frames = args->nb_images_to_stack;
+	int nb_layers = args->seq->nb_layers;
+	int starmin = INT_MAX;
+	int starmax = 0;
+	double invdenom;
+
+	if (!layer_has_registration(args->seq, args->reglayer)) {
+		siril_log_color_message(_("Sequence does not have registration info, cannot use weighing by %s, aborting\n"), "red", _("number of stars"));
+		return ST_GENERIC_ERROR;
+	}
+
+	args->weights = malloc(nb_layers * nb_frames * sizeof(double));
+	double *pweights[3];
+
+	for (int i = 0; i < args->nb_images_to_stack; ++i) {
+		int idx = args->image_indices[i];
+		if (args->seq->regparam[args->reglayer][idx].number_of_stars < starmin) starmin = args->seq->regparam[args->reglayer][idx].number_of_stars;
+		if (args->seq->regparam[args->reglayer][idx].number_of_stars > starmax) starmax = args->seq->regparam[args->reglayer][idx].number_of_stars;
+	}
+	if (starmax == starmin)
+		invdenom = 1.0;
+	else invdenom = 1. / (double)(starmax - starmin);
+
+	for (int layer = 0; layer < nb_layers; ++layer) {
+		double norm = 0.0;
+		pweights[layer] = args->weights + layer * nb_frames;
+		for (int i = 0; i < args->nb_images_to_stack; ++i) {
+			int idx = args->image_indices[i];
+			pweights[layer][i] = (double)(args->seq->regparam[args->reglayer][idx].number_of_stars - starmin) *
+				(double)(args->seq->regparam[args->reglayer][idx].number_of_stars - starmin) *
+				invdenom * invdenom;
+			norm += pweights[layer][i];
+		}
+		norm /= (double) nb_frames;
+
+		for (int i = 0; i < args->nb_images_to_stack; i++) {
+			pweights[layer][i] /= norm;
+			siril_debug_print("Image #%d - Layer %d - nbstars: %d - weight: %3.2f\n", args->image_indices[i], layer, args->seq->regparam[args->reglayer][args->image_indices[i]].number_of_stars, pweights[layer][i]);
+		}
+	}
+	return ST_OK;
 }
 
 /* How many rows fit in memory, based on image size, number and available memory.
  * It returns at most the total number of rows of the image (naxes[1] * naxes[2]) */
-static long stack_get_max_number_of_rows(long naxes[3], data_type type, int nb_images_to_stack) {
+static long stack_get_max_number_of_rows(long naxes[3], data_type type, int nb_images_to_stack, int nb_rejmaps) {
 	int max_memory = get_max_memory_in_MB();
 	long total_nb_rows = naxes[1] * naxes[2];
 	int elem_size = type == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
 
 	guint64 size_of_result = naxes[0] * naxes[1] * naxes[2] * elem_size;
+	guint64 size_of_rejmaps = naxes[0] * naxes[1] * naxes[2] * sizeof(WORD);
 	max_memory -= size_of_result / BYTES_IN_A_MB;
+	max_memory -= nb_rejmaps * size_of_rejmaps / BYTES_IN_A_MB;
 	if (max_memory < 0)
 		max_memory = 0;
 
@@ -956,7 +994,6 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	// data for mean/rej only
 	guint64 irej[3][2] = {{0,0}, {0,0}, {0,0}};
 	regdata *layerparam = NULL;
-	gboolean use_regdata = is_mean;
 
 	int nb_frames = args->nb_images_to_stack; // number of frames actually used
 	naxes[0] = naxes[1] = 0; naxes[2] = 1;
@@ -970,13 +1007,10 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	}
 	g_assert(nb_frames <= args->seq->number);
 
-	if (use_regdata) {
-		if (args->reglayer < 0) {
-			siril_log_message(_("No registration layer passed, ignoring registration data!\n"));
-			use_regdata = FALSE;
-		}
-		else layerparam = args->seq->regparam[args->reglayer];
+	if (args->reglayer < 0) {
+		siril_log_message(_("No registration layer passed, ignoring registration data!\n"));
 	}
+	else layerparam = args->seq->regparam[args->reglayer];
 
 	set_progress_bar_data(NULL, PROGRESS_RESET);
 
@@ -1017,6 +1051,18 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			fit.orig_bitpix = USHORT_IMG;
 	}
 
+	/* initialize rejection maps */
+	if (args->create_rejmaps) {
+		if ((retval = new_fit_image(&args->rejmap_low, naxes[0], naxes[1], naxes[2], DATA_USHORT))) {
+			goto free_and_close;
+		}
+		if (!args->merge_lowhigh_rejmaps) {
+			if ((retval = new_fit_image(&args->rejmap_high, naxes[0], naxes[1], naxes[2], DATA_USHORT))) {
+				goto free_and_close;
+			}
+		}
+	}
+
 	/* manage threads */
 	int nb_threads;
 #ifdef _OPENMP
@@ -1046,7 +1092,13 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	long largest_block_height;
 	int nb_blocks;
 	data_type itype = get_data_type(bitpix);
-	long max_number_of_rows = stack_get_max_number_of_rows(naxes, itype, args->nb_images_to_stack);
+	int nb_rejmaps = 0;
+	if (args->create_rejmaps) {
+		if (args->merge_lowhigh_rejmaps)
+			nb_rejmaps = 1;
+		else nb_rejmaps = 2;
+	}
+	long max_number_of_rows = stack_get_max_number_of_rows(naxes, itype, args->nb_images_to_stack, nb_rejmaps);
 	/* Compute parallel processing data: the data blocks, later distributed to threads */
 	if ((retval = stack_compute_parallel_blocks(&blocks, max_number_of_rows, naxes, nb_threads,
 					&largest_block_height, &nb_blocks))) {
@@ -1065,7 +1117,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 	g_assert(npixels_in_block > 0);
 	int ielem_size = itype == DATA_FLOAT ? sizeof(float) : sizeof(WORD);
 
-	fprintf(stdout, "allocating data for %d threads (each %'lu MB)\n", pool_size,
+	fprintf(stdout, "allocating data for %d threads (each %lu MB)\n", pool_size,
 			(unsigned long)(nb_frames * npixels_in_block * ielem_size) / BYTES_IN_A_MB);
 	data_pool = calloc(pool_size, sizeof(struct _data_block));
 	size_t bufferSize = ielem_size * nb_frames * (npixels_in_block + 1ul) + 4ul; // buffer for tmp and stack, added 4 byte for alignment
@@ -1145,14 +1197,33 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		args->mad_calculator = siril_stats_ushort_mad;
 	}
 
-	if (args->apply_weight) {
-		siril_log_message(_("Computing weights...\n"));
-		retval = compute_weights(args);
-		if(retval) {
+	if (args->apply_noise_weights) {
+		siril_log_message(_("Computing weights based on noise...\n"));
+		retval = compute_noise_weights(args);
+		if (retval) {
 			retval = ST_GENERIC_ERROR;
 			goto free_and_close;
 		}
 	}
+	if (args->apply_wfwhm_weights) {
+		siril_log_message(_("Computing weights based on wFWHM...\n"));
+		retval = compute_wfwhm_weights(args);
+		if (retval) {
+			retval = ST_GENERIC_ERROR;
+			goto free_and_close;
+		}
+	}
+	if (args->apply_nbstars_weights) {
+		siril_log_message(_("Computing weights based on number of stars...\n"));
+		retval = compute_nbstars_weights(args);
+		if (retval) {
+			retval = ST_GENERIC_ERROR;
+			goto free_and_close;
+		}
+	}
+	if (args->apply_nbstack_weights) {
+		siril_log_message(_("Computing weights based on number of stacked images...\n"));
+	}
 
 	siril_log_message(_("Starting stacking...\n"));
 	if (is_mean)
@@ -1169,9 +1240,10 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		struct _image_block *my_block = blocks+i;
 		struct _data_block *data;
 		int data_idx = 0;
+		guint64 brej[2] = {0, 0}; // rejection counts for the block
 		long x, y;
 
-		if (!get_thread_run()) retval = ST_GENERIC_ERROR;
+		if (!get_thread_run()) retval = ST_CANCEL;
 		if (retval) continue;
 #ifdef _OPENMP
 		data_idx = omp_get_thread_num();
@@ -1184,7 +1256,8 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 		data = &data_pool[data_idx];
 
 		/**** Step 2: load image data for the corresponding image block ****/
-		stack_read_block_data(args, use_regdata, my_block, data, naxes, itype, data_idx);
+		retval = stack_read_block_data(args, my_block, data, naxes, itype, data_idx);
+		if (retval) continue;
 
 #if defined _OPENMP && defined STACK_DEBUG
 		{
@@ -1207,7 +1280,6 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			size_t pdata_idx = (naxes[1] - (my_block->start_row + y) - 1) * naxes[0];
 			/* index of the line in the read data, data->pix[frame] */
 			size_t line_idx = y * naxes[0];
-			guint64 crej[2] = {0, 0};
 			if (retval) break;
 
 			// update progress bar
@@ -1217,7 +1289,7 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 			cur_nb++;
 
 			if (!get_thread_run()) {
-				retval = ST_GENERIC_ERROR;
+				retval = ST_CANCEL;
 				break;
 			}
 			if (!(cur_nb % 16))	// every 16 iterations
@@ -1228,13 +1300,12 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 				 * to optimize caching and improve readability */
 				for (int frame = 0; frame < nb_frames; ++frame) {
 					int pix_idx = line_idx + x;
-					if (use_regdata) {
+					if (layerparam) {
 						int shiftx = 0;
-						if (layerparam) {
-							shiftx = round_to_int(
-									layerparam[args->image_indices[frame]].shiftx *
-									args->seq->upscale_at_stacking);
-						}
+						double scale = args->seq->upscale_at_stacking;
+						double dx, dy;
+						translation_from_H(layerparam[args->image_indices[frame]].H, &dx, &dy);
+						shiftx = round_to_int(dx * scale);
 
 						if (shiftx && (x - shiftx >= naxes[0] || x - shiftx < 0)) {
 							/* outside bounds, images are black. We could
@@ -1301,7 +1372,20 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 
 				double result; // resulting pixel value, either mean or median
 				if (is_mean) {
-					result = mean_and_reject(args, data, nb_frames, itype, crej);
+					int rej[2] = { 0, 0 };
+					result = mean_and_reject(args, data, nb_frames, itype, rej);
+					brej[0] += rej[0];
+					brej[1] += rej[1];
+					if (args->create_rejmaps) {
+						if (args->merge_lowhigh_rejmaps) {
+							WORD nbrej = truncate_to_WORD(rej[0] + rej[1]);
+							args->rejmap_low->pdata[my_block->channel][pdata_idx] = nbrej;
+						}
+						else {
+							args->rejmap_low->pdata[my_block->channel][pdata_idx] = truncate_to_WORD(rej[0]);
+							args->rejmap_high->pdata[my_block->channel][pdata_idx] = truncate_to_WORD(rej[1]);
+						}
+					}
 				} else {
 					if (itype == DATA_USHORT)
 						result = quickmedian(data->stack, nb_frames);
@@ -1309,9 +1393,16 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 				}
 
 				if (args->use_32bit_output) {
+					// if we renormalize afterwards, we keep the data as is
+					// otherwise, we clamp in the [0,1] range
 					if (itype == DATA_USHORT)
-						fit.fpdata[my_block->channel][pdata_idx] = min(double_ushort_to_float_range(result), 1.f);
-					else	fit.fpdata[my_block->channel][pdata_idx] = min((float)result, 1.f);
+						fit.fpdata[my_block->channel][pdata_idx] = (args->output_norm) ?
+																	double_ushort_to_float_range(result) :
+																	set_float_in_interval(double_ushort_to_float_range(result), 0.f, 1.f);
+					else
+						fit.fpdata[my_block->channel][pdata_idx] = (args->output_norm) ?
+																	(float)result :
+																	set_float_in_interval((float)result, 0.f, 1.f);
 				} else {
 					/* in case of 8bit data we may want to normalize to 16bits */
 					if (args->output_norm) {
@@ -1321,18 +1412,6 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 				}
 				pdata_idx++;
 			} // end of for x
-
-			if (is_mean && args->type_of_rejection != NO_REJEC) {
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-				irej[my_block->channel][0] += crej[0];
-#ifdef _OPENMP
-#pragma omp atomic
-#endif
-				irej[my_block->channel][1] += crej[1];
-			}
-
 		} // end of for y
 #if defined _OPENMP && defined STACK_DEBUG
 		{
@@ -1344,6 +1423,18 @@ static int stack_mean_or_median(struct stacking_args *args, gboolean is_mean) {
 					data_idx, i, min, sec);
 		}
 #endif
+		if (is_mean && args->type_of_rejection != NO_REJEC) {
+#ifdef _OPENMP
+#pragma omp atomic
+#endif
+			irej[my_block->channel][0] += brej[0];
+#ifdef _OPENMP
+#pragma omp atomic
+#endif
+			irej[my_block->channel][1] += brej[1];
+		}
+
+
 	} /* end of loop over parallel stacks */
 
 	if (retval)
@@ -1392,7 +1483,9 @@ free_and_close:
 		if (is_mean)
 			set_progress_bar_data(_("Rejection stacking failed. Check the log."), PROGRESS_RESET);
 		else	set_progress_bar_data(_("Median stacking failed. Check the log."), PROGRESS_RESET);
-		siril_log_message(_("Stacking failed.\n"));
+		if (retval == ST_CANCEL)
+			siril_log_message(_("Stacking operation was cancelled.\n"));
+		else siril_log_message(_("Stacking failed.\n"));
 	} else {
 		if (is_mean) {
 			set_progress_bar_data(_("Rejection stacking complete."), PROGRESS_DONE);
